package viptool.base;

import java.awt.Image;

import javax.swing.JComponent;
import javax.swing.JFrame;

import viptool.base.action.IActionHandler;
import viptool.base.editor.IEditor;
import viptool.base.editor.IEditorManager;
import viptool.base.editor.IEditorProvider;
import viptool.base.error.CannotOpenFileException;
import viptool.base.error.EditorException;
import viptool.base.error.VipToolException;
import viptool.base.error.VipToolRuntimeException;
import viptool.base.event.IWorkbenchListener;
import viptool.base.gui.IChildWindow;
import viptool.base.gui.IView;
import viptool.base.gui.dialog.event.IStandardDialogListener;
import viptool.base.gui.menu.IContextMenuManager;
import viptool.base.gui.menu.IMenuBarManager;
import viptool.base.gui.toolbar.IToolBarManager;
import viptool.base.io.IFileInterpreter;
import viptool.base.io.IFileInterpreterManager;
import viptool.base.io.IFileInterpreterProvider;
import viptool.base.wizard.IWizard;
import viptool.job.IJobScheduler;
import viptool.logger.ILogger;
import viptool.project.IAbstractFile;
import viptool.project.IAbstractFolder;
import viptool.project.IAbstractFolderProvider;
import viptool.project.IProject;
import viptool.project.IWorkspace;
import viptool.util.resources.IResourceLoader;

/**
 * The workbench interface is the common interface for communication with the
 * viptool in the current session. It handles all views and editors and other
 * windows. It handles the open workspace.
 */
public interface IWorkbench
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // general methods
    // <><><><><><><><><><><><><><><><><><><><><>
    // TODO: replace this method when a adaquate solution is found
    public IAbstractFolderProvider getAbstractFolderProvider();

    // TODO:TI: comment!
    public void showDialog(IStandardDialogListener listener, String title,
            String mainDescription, JComponent content, Image image,
            int buttons, int defaultButton);

    /**
     * Returns a reference to the main frame window. This is necessary to create
     * dialogs that should be childs of the main frame.
     * 
     * @return reference to the main frame window
     */
    public JFrame getMainFrame();

    /**
     * selects the given window
     * 
     * @param window
     *            the child window that is to be selected
     */
    public void selectWindow(IChildWindow window);
    
    /**
     * checks weather the given window is selected
     * 
     * @param window
     *            the child window that is to be checked
     * @return true if the child window is selected, false otherwise
     */
    public boolean windowIsSelected(IChildWindow window);

    /**
     * Shows the given exception in a standard dialog to informt the user about
     * the error.
     * 
     * @param exception
     *            the exception, that should be shown in the dialog.
     */
    public void showExceptionInDialog(VipToolException exception);

    /**
     * Shows the given exception in a standard dialog to informt the user about
     * the error.
     * 
     * @param exception
     *            the exception, that should be shown in the dialog.
     */
    public void showExceptionInDialog(VipToolRuntimeException exception);

    /**
     * Closes the window. Close means, that the child window is removed from the
     * managing extened pane and is no longer displayed. If force is set to
     * false, only the windows that returns true to a call to canClose() will be
     * closed.
     * 
     * @param window
     *            the window to close
     * @param force
     *            if this parameter is true the window will be closed even if
     *            the view return false to a call to canClose
     */
    public void closeWindow(IChildWindow window, boolean force);

    // <><><><><><><><><><><><><><><><><><><><><>
    // Workspace support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Sets the current workspace. If a workspace is already opened, the current
     * workspace is saved and closed and the new one is opened.
     * 
     * @param workspace
     *            the workspace to open and to load
     */
    public void setCurrentWorkspace(IWorkspace workspace);

    /**
     * Returns the currently opened workspace in the vip tool
     * 
     * @return workspace the workspace that is currently loaded in the vip tool
     */
    public IWorkspace getCurrentWorkspace();

    // <><><><><><><><><><><><><><><><><><><><><>
    // Project support
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Returns the project of the current selected object in the project tree.
     * If no item is selected, null is returned.
     * 
     * @return the project of the current selected object or null
     */
    public IProject getSelectedProject();

    /**
     * Returns the parent folder of the current selected object in the project
     * tree. If a project or no item is selected, null is returned.
     * 
     * @return the parent folder of the current selected object or null
     */
    public IAbstractFolder getSelectedFolder();

    /**
     * Returns the current selected file. Returns null if no file is selected.
     * 
     * @return the current select file or null if no file is selected
     */
    public IAbstractFile getSelectedFile();

    /**
     * Select the given file.
     * 
     * @param file
     *            the file to select
     */
    public void selectFile(IAbstractFile file);

    // <><><><><><><><><><><><><><><><><><><><><>
    // View support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Opens the given view. Open means, that the child window is added to the
     * managing extended pane and is displayed in the main frame of the viptool.
     * 
     * @param view
     *            the view that should be opened
     */
    public void openView(IView view);

    /**
     * Closes the view. Close means, that the child window is removed from the
     * managing extened pane and is no longer displayed. If force is set to
     * false, only the views that returns true to a call to canClose() will be
     * closed.
     * 
     * @param view
     *            the view to close
     * @param force
     *            if this parameter is true the view will be closed even if the
     *            view return false to a call to canClose
     */
    public void closeView(IView view, boolean force);

    /**
     * Closes all open views except of the workspace view. The workspace can be
     * closed by an explicit call to closeView with the workspace view as
     * argument. If force is set to false, only the views that returns true to a
     * call to canClose() will be closed.
     * 
     * @param force
     *            if this parameter is true the views will be closed even if the
     *            views return false to a call to canClose
     */
    public void closeAllViews(boolean force);

    /**
     * Returns an array of all opened views. Even the workspace view is
     * returned. If no view is open, an empty array will be returned.
     * 
     * @return an array with a reference to all open views
     */
    public IView[] getOpenViews();

    // <><><><><><><><><><><><><><><><><><><><><>
    // Editor support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Opens the given editor. Open means, that the child window is added to the
     * managing extended pane and is displayed in the main frame of the viptool.
     * 
     * @param editor
     *            the editor that should be opened
     */
    public void openEditor(IEditor editor);

    /**
     * Returns an array of all editors that have currently opened the given
     * abstract file. If the file was not opened by any editor, an empty array
     * will be returned.
     * 
     * @param file
     *            the editors of this file will be returned
     * @return an array containing all editors that have opened the given file
     */
    public IEditor[] findEditors(IAbstractFile file);

    /**
     * Returns the editor that has currently locked the file or null if the file
     * isn't locked. This method uses the method of the IAbstractFile interface
     * but does a casting to the IEditor interface if the returned file locker
     * is an editor. So this method is very fast.
     * 
     * @param file
     *            the locking editor of this file will be returned
     * @return the editor that has currently locked the given file
     */
    public IEditor findLockingEditor(IAbstractFile file);

    /**
     * Closes the editor. Close means, that the child window is removed from the
     * managing extened pane and is no longer displayed. If force is set to
     * false, only the editors that returns true to a call to canClose() will be
     * closed.
     * 
     * @param editor
     *            the editor to close
     * @param force
     *            if this parameter is true the editor will be closed even if
     *            the editor return false to a call to canClose
     */
    public void closeEditor(IEditor editor, boolean force);

    /**
     * Closes all open editors. If force is set to false, only the editors that
     * returns true to a call to canClose() will be closed.
     * 
     * @param force
     *            if this parameter is true the editors will be closed even if
     *            the editors return false to a call to canClose
     */
    public void closeAllEditors(boolean force);

    /**
     * Closes all editors that have opened the given abstract file. If force is
     * set to false, only the editors that returns true to a call to canClose()
     * will be closed.
     * 
     * @param file
     *            the editors of this file will be closed
     * @param force
     *            if this parameter is true the editors will be closed even if
     *            the editors return false to a call to canClose.
     */
    public void closeAllEditors(IAbstractFile file, boolean force);

    /**
     * Returns an array of all opened editors. If no editor is open, an empty
     * array will be returned.
     * 
     * @return an array with a reference to all open editors
     */
    public IEditor[] getOpenEditors();

    // <><><><><><><><><><><><><><><><><><><><><>
    // File support
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Opens the given file using the default file interpreter and the default
     * editor for displaying.
     * 
     * @param file
     *            the file that should be opened
     * @throws EditorException
     *             - if the file cannot be opened
     */
    public void openFile(IAbstractFile file) throws CannotOpenFileException;

    /**
     * Opens the file that is managed by the given file interpreter using the
     * default editor for this interpreter.
     * 
     * @param fileInterpreter
     *            the file interpreter whose file shpuld be opened
     * @throws CannotOpenFileException
     *             - if the file cannot be opened
     */
    public void openFile(IFileInterpreter fileInterpreter)
            throws CannotOpenFileException;

    /**
     * Opens the given file using the file interpreter that is managed by the
     * given file interpreter provider. The default editor for this interpreter
     * is used. Be sure that the given file interpreter provider will accept the
     * file, otherwise an exception is thrown.
     * 
     * @param file
     *            the file that should be opened
     * @param fileInterpreterProvider
     *            the file interpreter provider managing the file interpreter
     *            that should be used to interprete the file
     * @throws CannotOpenFileException
     *             - if the file could not be opened, e.g. because the file
     *             interpreter provider does not accept the file
     */
    public void openFile(IAbstractFile file,
            IFileInterpreterProvider fileInterpreterProvider)
            throws CannotOpenFileException;

    /**
     * Opens the given file using the file interpreter managed by the given file
     * interpreter provider. The file is displayed using the given editor. Be
     * sure that the editor accepts the file interpreter as well as the file
     * interpreter accepts the file, otherwise an exception is thrown.
     * 
     * @param file
     *            the file to open
     * @param fileInterpreterProvider
     *            the file interpreter provider managing the file interpreter
     *            that should be used to interprete the file
     * @param editorProvider
     *            the editor provider that manages the editor that should be
     *            used for displaying the given file
     * @throws CannotOpenFileException
     *             if the file could not be opened, e.g. because the editor
     *             interpreter provider does not accept the given file
     *             interpreter provider
     */
    public void openFile(IAbstractFile file,
            IFileInterpreterProvider fileInterpreterProvider,
            IEditorProvider editorProvider) throws CannotOpenFileException;

    // <><><><><><><><><><><><><><><><><><><><><>
    // listener support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds the given workbench listener to the workbench. The workbench
     * listener will be notified about events concerning the workbench.
     * 
     * @param listener
     *            the workbench listener that should be added
     */
    public void addWorkbenchListener(IWorkbenchListener listener);

    /**
     * Removes the given workbench listener from the workbench. The listener
     * will no longer be notified about changes in the workbench.
     * 
     * 
     * @param listener
     *            the listener that should be removed from the workbench
     */
    public void removeWorkbenchListener(IWorkbenchListener listener);

    // <><><><><><><><><><><><><><><><><><><><><>
    // Provider support
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Returns the file interpreter manager that manages all file interpreters
     * and their providers.
     * 
     * @return the file interpreter manager
     */
    public IFileInterpreterManager getFileInterpreterManager();

    // <><><><><><><><><><><><><><><><><><><><><>
    // Logger support
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Returns the logger for the viptool.
     * 
     * @return the viptool logger
     */
    public ILogger getLogger();

    public IMenuBarManager getMenuBarManager();

    public IToolBarManager getToolBarManager();

    public IContextMenuManager getContextMenuManager();

    public IActionHandler getActionHandler();

    public IEditorManager getEditorManager();

    public IJobScheduler getJobScheduler();

    public void startWizard(IWizard wizard);

    public IResourceLoader getResourceLoader();

    public String getPropertyPath();

    public String[] getCommandLineOptions();
}
