package viptool.base.action;

import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;

import javax.swing.AbstractAction;
import javax.swing.SwingUtilities;

import viptool.base.event.WorkbenchEvent;

/**
 * ExtendedAction is the base class for all actions in viptool. Extended actions
 * have more abilities than abstract actions, they can be cloned for example. It
 * also has get- methods for the important attributes.<br>
 * ExtendedActions can be assigned to one or more windows. Typically an action
 * is assigned to only one window. But if the action is used in the context
 * menu, it can be assigned to all windows, that belongs to the selected files.
 */
public abstract class ExtendedAction extends AbstractAction
{
    /**
	 * generated
	 */
	private static final long serialVersionUID = -7883055415632230461L;

	/** the key value for the inherit from attribute (not stored in the action) */
    public static final String ATTR_INHERITFROM  = "InheritFrom";

    /** the key value for the insert at attribute (not stored in the action) */
    public static final String ATTR_INSERTAT     = "InsertAt";

    /** the key value for the seek attribute (not stored in the action) */
    public static final String ATTR_SEEK         = "Seek";

    /** the key value for the class attribute */
    public static final String ATTR_CLASS        = "Class";

    /** the key value for the id attribute */
    public static final String ATTR_ID           = "Id";

    /** the key value for the group attribute */
    public static final String ATTR_GROUP        = "Group";

    /** the key value for the group attribute */
    public static final String ATTR_TYPE         = "Type";

    /** the key value for the group attribute */
    public static final String ATTR_USER_DEFINED = "UserDefined";

    public static final String TYPE_RADIO        = "radio";

    public static final String TYPE_CHECKBOX     = "checkbox";
    
    public static final String TYPE_SLIDER	     = "slider";
    
    public static final String TYPE_TEXT	     = "text";
    
    public static final String TYPE_COMBOBOX     = "combobox";

    public static final String INSTANCE_KEY      = "instance";

    // <><><><><><><><><><><><><><><><><><><><><>
    // Initializing
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Is called when the action is created the first time. It can be used by
     * subclasses to do initialisation work. When this method is called, the
     * subclass can be sure, that every property of the action is intialized and
     * every method is working as expected.
     * 
     * Not an abstract method but most children don't need anything here
     */
    public void initialize(){}

    // <><><><><><><><><><><><><><><><><><><><><>
    // state methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the id of the action
     * 
     * @return the action id
     */
    public String getIdKey()
    {
        return (String) getValue(ATTR_ID);
    }

    /**
     * Returns the full class name of the implementing class.
     * 
     * @return the full class name
     */
    public String getClassName()
    {
        return (String) getValue(ATTR_CLASS);
    }

    /**
     * Returns the user defined value associated with this action.
     * 
     * @return the value of the "UserDefined" attribute.
     */
    public String getUserDefined()
    {
        return (String) getValue(ATTR_USER_DEFINED);
    }

    /**
     * Returns the group of the action.
     * 
     * @return the group of the action
     */
    public String getGroup()
    {
        return (String) getValue(ATTR_GROUP);
    }

    /**
     * set the selected property to the given parameter, true or false.
     * 
     * @param selected
     *            true to 'select' the action, false to deselect.
     */
    public void setSelected(boolean selected)
    {
        putValue(SELECTED_KEY, new Boolean(selected));
    }

    /**
     * Returns the state of selection of this action. Returns true if the action
     * is selected and false otherwise.
     * 
     * @return the value of the selected property
     */
    public boolean isSelected()
    {
        Boolean state = (Boolean) getValue(SELECTED_KEY);

        return (state == null ? false : state.booleanValue());
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // mouse event methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns true if the mouse event stores the left mouse button clicked
     * event.
     * 
     * @param mouseEvent
     *            the mouse event to test
     * 
     * @return true if the left mouse button was clicked, false otherwise
     */
    public static boolean isMouseLeftClicked(MouseEvent mouseEvent)
    {
        if ((mouseEvent.getID() == MouseEvent.MOUSE_CLICKED)
                && (SwingUtilities.isLeftMouseButton(mouseEvent)))
        {
            return true;
        }

        return false;
    }

    /**
     * Returns true if the mouse event stores the right mouse button clicked
     * event.
     * 
     * @param mouseEvent
     *            the mouse event to test
     * 
     * @return true if the right mouse button was clicked, false otherwise
     */
    public static boolean isMouseRightClicked(MouseEvent mouseEvent)
    {
        if ((mouseEvent.getID() == MouseEvent.MOUSE_CLICKED)
                && (!SwingUtilities.isLeftMouseButton(mouseEvent)))
        {
            return true;
        }

        return false;
    }

    /**
     * Returns true if the mouse event stores the left mouse button pressed
     * event.
     * 
     * @param mouseEvent
     *            the mouse event to test
     * 
     * @return true if the left mouse button was pressed, false otherwise
     */
    public static boolean isMouseLeftPressed(MouseEvent mouseEvent)
    {
        if ((mouseEvent.getID() == MouseEvent.MOUSE_PRESSED)
                && (SwingUtilities.isLeftMouseButton(mouseEvent)))
        {
            return true;
        }

        return false;
    }

    /**
     * Returns true if the mouse event stores the right mouse button pressed
     * event.
     * 
     * @param mouseEvent
     *            the mouse event to test
     * 
     * @return true if the right mouse button was pressed, false otherwise
     */
    public static boolean isMouseRightPressed(MouseEvent mouseEvent)
    {
        if ((mouseEvent.getID() == MouseEvent.MOUSE_PRESSED)
                && (!SwingUtilities.isLeftMouseButton(mouseEvent)))
        {
            return true;
        }

        return false;
    }

    /**
     * Returns true if the mouse event stores the left mouse button released
     * event.
     * 
     * @param mouseEvent
     *            the mouse event to test
     * 
     * @return true if the left mouse button was released, false otherwise
     */
    public static boolean isMouseLeftReleased(MouseEvent mouseEvent)
    {
        if ((mouseEvent.getID() == MouseEvent.MOUSE_RELEASED)
                && (SwingUtilities.isLeftMouseButton(mouseEvent)))
        {
            return true;
        }

        return false;
    }

    /**
     * Returns true if the mouse event stores the right mouse button released
     * event.
     * 
     * @param mouseEvent
     *            the mouse event to test
     * 
     * @return true if the right mouse button was released, false otherwise
     */
    public static boolean isMouseRightReleased(MouseEvent mouseEvent)
    {
        if ((mouseEvent.getID() == MouseEvent.MOUSE_RELEASED)
                && (!SwingUtilities.isLeftMouseButton(mouseEvent)))
        {
            return true;
        }

        return false;
    }

    /**
     * Returns true if the mouse event stores the left mouse button dragged
     * event.
     * 
     * @param mouseEvent
     *            the mouse event to test
     * 
     * @return true if the mouse was dragged with the left mouse button, false
     *         otherwise
     */
    public static boolean isMouseLeftDragged(MouseEvent mouseEvent)
    {
        if ((mouseEvent.getID() == MouseEvent.MOUSE_DRAGGED)
                && (SwingUtilities.isLeftMouseButton(mouseEvent)))
        {
            return true;
        }

        return false;
    }

    /**
     * Returns true if the mouse event stores the right mouse button dragged
     * event.
     * 
     * @param mouseEvent
     *            the mouse event to test
     * 
     * @return true if the mouse was dragged with the right mouse button, false
     *         otherwise
     */
    public static boolean isMouseRightDragged(MouseEvent mouseEvent)
    {
        if ((mouseEvent.getID() == MouseEvent.MOUSE_DRAGGED)
                && (!SwingUtilities.isLeftMouseButton(mouseEvent)))
        {
            return true;
        }

        return false;
    }

    /**
     * Returns the numbre of points the mouse wheel was moved. A negative value
     * indicates that the user scrolled the mouse wheel up/away from the user.
     * 
     * @param mouseEvent
     *            the mouse event to test
     * 
     * @return number of "clicks" the mouse wheel was rotated. Negative, if the
     *         user moves the wheel upside
     */
    public static int getMouseWheels(MouseEvent mouseEvent)
    {
        if (!(mouseEvent instanceof MouseWheelEvent))
        {
            return 0;
        }

        if (mouseEvent.getID() == MouseEvent.MOUSE_WHEEL)
        {
            MouseWheelEvent mouseWheelEvent = (MouseWheelEvent) mouseEvent;
            return mouseWheelEvent.getWheelRotation();
        }

        return 0;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the class AbstractAction
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see javax.swing.AbstractAction */
    @Override
	public Object clone() throws CloneNotSupportedException
    {
        return super.clone();
    }

       /* @see java.awt.event.ActionListener */
    @Override
	public void actionPerformed(ActionEvent event){}    
    
}
