package viptool.base.action;

import java.net.URI;

import viptool.base.error.XMLException;

/**
 * The action handler manages all actions in viptool. Actions are typically read
 * from xml files into the memory of the action handler. The action handler
 * holds templates from the action he has loaded. Every instance mut request its
 * own action, that will be a clone of the template action.<br>
 * <br>
 * Example:<br>
 * The main window has a menu and a toolbar. It requests a new menu bar from the
 * menu template manager with its hash code as instance. After that it requests
 * a toolbar from the ToolbarTemplateManager with the same instance.<br>
 * The action handler will now give the same action instances to the menu and
 * the toolbar so the actions are linked. That means, if the action is disabled,
 * it will be disabled in the menu and the toolbar.
 */
public interface IActionHandler
{
    /** Name of the actions element in the xml file */
    public static final String ELEMENT_ACTIONS   = "actions";

    /** Name of the action element in the xml file */
    public static final String ELEMENT_ACTION    = "action";

    /** Name of the separator element in the xml file */
    public static final String ELEMENT_SEPARATOR = "separator";

    /** Name of the group element in the xml file */
    public static final String ELEMENT_CONTAINER = "container";

    /**
     * Removes the actions that were created for the specified instance.
     * 
     * @param instance
     *            the instance number
     */
    public void removeActions(int instance);

    /**
     * Returns an action from the action handler. Since the action handler
     * manages the actions for all instances you have to give an instane from
     * which you want to have the action. If the instance does not exist, it
     * will be created and a clone of the specified template action will be
     * returned. The instance 0 is the template instance (all actions that are
     * readed from an xml file are member of this instance).<br>
     * In most cases, the return of the hasCode() method of the using class is a
     * good value for instance.
     * 
     * @param instance
     *            the instance that want to have the action
     * @param id
     *            the id of the action
     * @return the action that belongs to the given id and instance or null if
     *         the given action id was not found
     */
    public ExtendedAction getAction(int instance, String id);

    /**
     * Returns the template action with the given id. The template action must
     * only be used in template menubars or template toolbars. Real menus and
     * toolbars, that ones, that are displayed in the window, must use an
     * instance action (that is an action retrieved by getAction(instance, id).
     * 
     * @param id
     *            the id of the action that should be returned
     * @return the action with the given id or null if the action was not found
     */
    public ExtendedAction getAction(String id);

    /**
     * Adds the given action to the action handler. The given action must have
     * an unique id.
     * 
     * @param action
     *            the action to add to the action handler
     */
    public void addAction(ExtendedAction action);

    /**
     * Registers an action post processor to the specified instance. The action
     * post processor is called every time when an action is created for the
     * given instance.
     * 
     * @param instance
     *            the instance for which the post processor should be used
     * 
     * @param postProcessor
     *            the post processor that should be called
     */
    public void registerActionPostProcessor(int instance,
            IActionPostProcessor postProcessor);
}
