package viptool.base.editor;

import java.awt.Color;

import javax.swing.JOptionPane;

import viptool.base.IWorkbench;
import viptool.base.error.EditorNotAcceptedException;
import viptool.base.error.EditorNotOpenedException;
import viptool.base.error.FileAlreadyLockedException;
import viptool.base.error.FileInterpreterException;
import viptool.base.io.IFileInterpreter;
import viptool.base.io.IFileLocker;
import viptool.project.IAbstractFile;
import viptool.project.event.IAbstractFileListener;
import viptool.util.resources.ResourceConstants;

/**
 * An Editor is a window that gives access to the information from a file.
 * All displayed graphs in the viptool are editors.
 */

public abstract class Editor implements IEditor, IFileLocker,
        IAbstractFileListener
{ 
    /**
     *  the file interpreter used to open the file
     */
    private IFileInterpreter fileInterpreter_;

    /**
     *  flag indicating whether the file is open or not
     */
    private boolean          opened_;

    /**
     *  flag indication whether the editor has opened the file
     */
    private boolean          modified_;

    /**
     *  the current workbench
     */
    private IWorkbench       workbench_;

    private IEditorProvider  provider_;

    private boolean          externModified_;

    private long             timestamp_;

    private Color            oldBackground_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new editor object and initializes it.
     */
    public Editor(IWorkbench workbench, IEditorProvider provider)
    {
        provider_ = provider;
        fileInterpreter_ = null;
        opened_ = false;
        modified_ = false;
        externModified_ = false;
        workbench_ = workbench;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // modification methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Set the modified flag that is returned by the isModified method. If the
     * flag is set to true, the method automatically request the lock for the
     * file. If the flag is set to false, the lock is released. Be sure to call
     * save before setting this flag to false.<br>
     * If the file is already locked by another editor a log message is created
     * but no error is thrown. Call the canModify() method before changing
     * contents.
     * 
     * @param modified
     *            a flag indicating if the file was changed since the last save
     */
    protected void setModified(boolean modified)
    {
        try
        {
            assertOpened();

            if (modified && !modified_)
            {
                // the file was modified the first time
                try
                {
                    // request the lock
                    getAbstractFile().requestLock(this);
                }
                catch (FileAlreadyLockedException exception)
                {
                    workbench_.getLogger().warning(
                            "The file '" + getAbstractFile().getFullPath()
                                    + "' is already locked! "
                                    + exception.getMessage());
                }
            }
            else if (!modified)
            {
                // the file is no longer modified
                getAbstractFile().releaseLock(this);
            }

            modified_ = modified;
        }
        catch (EditorNotOpenedException exception)
        {
            workbench_.getLogger().debug(
                    "Call to 'setModified' on unopened file! "
                            + exception.getMessage());
            return;
        }
    }

    /**
     * Returns true if the editor can modify the file. That is if no other
     * editor has locked the file. If any editor except himself has already
     * locked the file, false is returned.<br>
     * Note: this method does not lock the file. This must be manually done.
     * 
     * @return true if the file can be locked by the editor, false otherwise
     */
    protected boolean canModify()
    {
        IEditor editor;

        try
        {
            // check wether we or another editor has already locked the file
            editor = workbench_.findLockingEditor(getAbstractFile());

            return (editor == this) || (editor == null);
        }
        catch (EditorNotOpenedException exception)
        {
            return false;
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // helper methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Checks whether the file was opened in the editor or not. If the file was
     * not opened yet, an EditorNotOpenedException is thrown.
     * 
     * @throws EditorNotOpenedException
     *             - if the file was not opened in the editor yet
     */
    protected void assertOpened() throws EditorNotOpenedException
    {
        if (fileInterpreter_ == null)
        {
            throw new EditorNotOpenedException(this, "The editor was not yet"
                    + " opened. Call open() first!");
        }
    }

    protected IWorkbench getWorkbench()
    {
        return workbench_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IEditor interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.base.editor.IEditor */
    public void open(IFileInterpreter file) throws EditorNotAcceptedException
    {
        opened_ = false;

        fileInterpreter_ = file;
        if (fileInterpreter_.getAbstractFile().getFileLocker() == this)
        {
            // already locked and opened
            opened_ = true;
            return;
        }

        try
        {
            fileInterpreter_.load();
            fileInterpreter_.getAbstractFile().addAbstractFileListener(this);

            externModified_ = false;
            modified_ = false;
            timestamp_ = fileInterpreter_.getAbstractFile().getLastModified();

            if (fileInterpreter_.getAbstractFile().isLocked()
                    && fileInterpreter_.getAbstractFile().getFileLocker() != this)
            {
                fileLocked();
            }
        }
        catch (FileInterpreterException exception)
        {
            throw new EditorNotAcceptedException(fileInterpreter_, this,
                    "The file cannot be opened!", exception);
        }

        opened_ = true;
    }

    /* @see viptool.base.editor.IEditor */
    public boolean isOpen()
    {
        return opened_;
    }

    public boolean canClose(boolean force)
    {
        if (!isOpen())
        {
            return true;
        }

        try
        {
            if (isModified() || getAbstractFile().getFileLocker() == this)
            {
                getWorkbench().selectWindow(this);

                int key = JOptionPane.showConfirmDialog(
                        workbench_.getMainFrame(),
                        "The file has been modified. Do you want to save the changes?",
                        getAbstractFile().getName(),
                        force ? JOptionPane.YES_NO_OPTION
                                : JOptionPane.YES_NO_CANCEL_OPTION,
                        JOptionPane.QUESTION_MESSAGE);
                if (key == JOptionPane.OK_OPTION)
                {
                    save();
                    return true;
                }
                else if (key == JOptionPane.NO_OPTION)
                {
                    return true;
                }

                return false;
            }
        }
        catch (EditorNotOpenedException e)
        {
            // do nothing
        }

        return true;
    }

    /* @see viptool.base.editor.IEditor */
    public IAbstractFile getAbstractFile() throws EditorNotOpenedException
    {
        assertOpened();

        return fileInterpreter_.getAbstractFile();
    }

    /* @see viptool.base.editor.IEditor */
    public IFileInterpreter getFileInterpreter()
            throws EditorNotOpenedException
    {
        assertOpened();

        return fileInterpreter_;
    }

    /* @see viptool.base.editor.IEditor */
    public boolean isModified() throws EditorNotOpenedException
    {
        assertOpened();

        return modified_;
    }

    public void save() throws EditorNotOpenedException
    {
        assertOpened();

        setModified(false);
        timestamp_ = getAbstractFile().getLastModified();
    }

    public IEditorProvider getProvider()
    {
        return provider_;

    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the ChildWindow interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.base.extendedPane.IChildWindow */
    public String getTitle()
    {
        try
        {
            return getAbstractFile().getName();
        }
        catch (EditorNotOpenedException exception)
        {
            return "no file opened.";
        }
    }

    protected void fileLocked()
    {
        oldBackground_ = getComponent().getBackground();
        getComponent().setBackground(ResourceConstants.COLOR_EDITOR_LOCKED);
    }

    protected void fileUnlocked()
    {
        getComponent().setBackground(oldBackground_);
    }

    protected Color getSavedBackground()
    {
        return oldBackground_;
    }

    protected boolean isExternalModified()
    {
        return externModified_;
    }

    public void fileLocked(IFileLocker locker)
    {
        if (locker != this)
        {
            fileLocked();
        }
    }

    public void fileReleased(IFileLocker locker, boolean modified)
    {
        if (locker != this)
        {
            externModified_ = modified;
            fileUnlocked();
        }
    }

    public void windowActivated()
    {
        if ((isExternalModified() || timestamp_ < fileInterpreter_.getAbstractFile().getLastModified())
                && isOpen())
        {
            try
            {
                int key = JOptionPane.showConfirmDialog(
                        null,
                        "The file has been modified by another editor. Do you want to load the changes?",
                        getAbstractFile().getName(), JOptionPane.YES_NO_OPTION,
                        JOptionPane.QUESTION_MESSAGE);

                if (key == JOptionPane.OK_OPTION)
                {
                    initialize();
                }
            }
            catch (EditorNotOpenedException exception)
            {
                // not important
            }
        }
    }
}
