package viptool.base.editor;

import viptool.base.error.EditorNotOpenedException;
import viptool.base.gui.IChildWindow;
import viptool.base.io.IFileInterpreter;
import viptool.base.io.IFileLocker;
import viptool.project.IAbstractFile;
import viptool.util.resources.ResourceConstants;

/**
 * This interface is for all editors in the viptool. Editors are displayed in a
 * separate area in the extended pane, and cannot be mixed with other child
 * windows (except other editors).<br>
 * The editor interface provides some methods for opening and closing abstract
 * files with the help of a file interpreter.<br>
 * The editor must lock the file
 */
public interface IEditor extends IChildWindow, IFileLocker
{
    /**
     * Name of the editor group
     */
    public static final String GROUP_NAME = ResourceConstants.GROUP_EDITORS;

    /**
     * This method saves the current opened file. If no changes were made, the
     * operation is ignored.
     * 
     * @throws EditorNotOpenedException
     *             - if the editor was not opened
     */
    public void save() throws EditorNotOpenedException;

    /**
     * Called by the framework before the editor is displayed.
     */
    public void initialize();

    /**
     * Return a flag indicating whether the edior is opened or not. The editor
     * is closed after creating and must be opened by a call to open. If the
     * editor is open, isOpen must return true, otherwise false should be
     * returned.
     * 
     * @return true if the editor is opened, false otherwise
     */
    public boolean isOpen();

    /**
     * Returns true when the editor has changed the file and not written the
     * changes yet.
     * 
     * @return true if the editor modified the file, false otherwise
     * @throws EditorNotOpenedException
     *             - if no file was opened in the editor
     */
    public boolean isModified() throws EditorNotOpenedException;

    /**
     * Returns the assigned file interpreter. The file interpreter is the class
     * that has interpreted the file for the editor, so the editor can display
     * the content of the file.
     * 
     * @return the file interpreter associated with this editor
     * @throws EditorNotOpenedException
     *             - if the editor was not opened
     */
    public IFileInterpreter getFileInterpreter()
            throws EditorNotOpenedException;

    /**
     * Returns the abstract file that was loaded into the editor.
     * 
     * @return the abstract file loaded into the editor.
     * @throws EditorNotOpenedException
     *             - if the editor was not opened
     */
    public IAbstractFile getAbstractFile() throws EditorNotOpenedException;

    public IEditorProvider getProvider();
}
