package viptool.base.editor;

import viptool.base.error.EditorException;
import viptool.base.io.IFileInterpreter;
import viptool.base.io.IFileInterpreterProvider;

/**
 * This interface is provides some methods for editor providers and is the base
 * interface for them. Only editors that are managed by an editor provider can
 * be added to the editor manager. In Consequence only registered editor
 * providers can open editors in viptool.
 */
public interface IEditorProvider
{
    /**
     * Creates a new editor that can display files of this editor provider. An
     * editor has one or more abstract file providers assigned to him. These
     * providers describe the file formats the editor can open.
     * 
     * @return the created editor
     * @throws EditorException
     *             - if the editor can't be created
     */
    public IEditor createEditor(IFileInterpreter fileInterpreter)
            throws EditorException;

    /**
     * Tests whether the file matching the given abstract file provider can be
     * opened by the editor of this provider.
     * 
     * @param fileprovider
     *            the provider for the file format
     * @return true if the editor can open such files, false if not
     */
    public boolean accept(IFileInterpreterProvider fileprovider);

    /**
     * Returns an array of abstract file providers that are accepted by the
     * editor. This is typically only one.
     * 
     * @return an array with all accepted abstract file providers
     */
    public IFileInterpreterProvider[] getAbstractFileProviders();

    /**
     * Returns the name of the editor. The name must be unique in the viptool.
     * 
     * @return the name of the editor
     */
    public String getName();

    /**
     * Returns the id of the editor provider. The id must be unique in viptool
     * and follow the specifiactions for package names.
     * 
     * @return id of the editor provider
     */
    public String getId();
}
