package viptool.base.gui.component;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.File;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.NoSuchElementException;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import viptool.base.IWorkbench;
import viptool.base.error.FileAlreadyLockedException;
import viptool.base.error.FileRenameException;
import viptool.base.gui.component.event.FolderTreePanelSelectionEvent;
import viptool.base.gui.component.event.IFolderTreePanelListener;
import viptool.base.io.IFileLocker;
import viptool.project.IAbstractFile;
import viptool.project.IAbstractFolder;
import viptool.project.IProject;
import viptool.project.IWorkspace;
import viptool.project.error.ChildNotAcceptedException;
import viptool.project.event.IAbstractFileListener;
import viptool.project.event.IAbstractFolderListener;
import viptool.util.resources.ResourceConstants;

/**
 * The folder tree panel displays a tree with projects and folders or folders
 * only. The panel supports three modes:
 * <ol>
 * <li>display all projects as root elements and their subfolders</li>
 * <li>display only one project as root and its subfolders</li>
 * <li>display a specific folder as root and its subfolders</li>
 * </ol>
 * The folder tree panel has a method to add virtual folders (that are folders
 * that don't exist on the hard disk) and for getting the selected element. The
 * panel does not create any folder on the hard disk.
 */
public class FolderTreePanel extends JPanel implements FocusListener,
        TreeSelectionListener
{
    // the workbench of the viptool
    private IWorkbench  workbench_;

    // the tree
    private JTree       tree_;

    // the scroll pane for scrolling the tree
    private JScrollPane scrollPane_;

    // a hashmap mapping project or files to tree nodes
    // key: viptool.project.IProject or viptool.project.IAbstractFolder
    // value: javax.swing.tree.DefaultMutableTreeNode the node in the tree
    private HashMap     nodes_;

    // the listeners of the class
    private HashSet     listeners_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This is the renderer class for tree items. It sets the text and the icon
     * depending on the node (project, folder).
     */
    private class TreeCellRenderer extends DefaultTreeCellRenderer
    {
        // <><><><><><><><><><><><><><><><><><><><><>
        // Overridden methods from DefaultTreeCellRenderer
        // <><><><><><><><><><><><><><><><><><><><><>

        /* see javax.swing.tree.DefaultTreeCellRenderer */
        public Component getTreeCellRendererComponent(JTree tree, Object value,
                boolean sel, boolean expanded, boolean leaf, int row,
                boolean hasFocus)
        {
            super.getTreeCellRendererComponent(tree, value, sel, expanded,
                    leaf, row, hasFocus);

            Object obj = ((DefaultMutableTreeNode) value).getUserObject();
            if (obj instanceof IProject)
            {
                IProject project = (IProject) obj;

                setText(project.getAbstractFile().getTitle());
                if (expanded)
                {
                    setIcon(workbench_.getResourceLoader().getIcon(
                            ResourceConstants.ICON_PROJECT_OPEN));
                }
                else
                {
                    setIcon(workbench_.getResourceLoader().getIcon(
                            ResourceConstants.ICON_PROJECT));
                }
            }
            else if (obj instanceof IAbstractFolder)
            {
                IAbstractFolder folder = (IAbstractFolder) obj;

                setText(folder.getTitle());
                if (expanded)
                {
                    setIcon(workbench_.getResourceLoader().getIcon(
                            ResourceConstants.ICON_FOLDER_OPEN));
                }
                else
                {
                    setIcon(workbench_.getResourceLoader().getIcon(
                            ResourceConstants.ICON_FOLDER));
                }
            }
            else if (obj instanceof IAbstractFile)
            {
                IAbstractFile file = (IAbstractFile) obj;

                setText(file.getName());
                setIcon(workbench_.getFileInterpreterManager().getDefaultFileInterpreterProvider(
                        file).getIcon());
            }

            return this;
        }

    }

    /**
     * The expander class expands the given node and selects it if the select
     * flag is set. The class implements the runnable interface so it can be
     * used with invokeLater.
     */
    private class Expander implements Runnable
    {
        // a flag indicating if the node should be selected
        private boolean                select_;

        // the node that should be visible
        private DefaultMutableTreeNode node_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Constructs a new expander. This expander will expand the given node
         * but will not select it.
         * 
         * @param node
         *            the node to expand
         */
        public Expander(DefaultMutableTreeNode node)
        {
            this(node, false);
        }

        /**
         * Constructs a new expander. This expander will expand the given node
         * and select it if select is set to true.
         * 
         * @param node
         *            the node to expand
         * @param select
         *            a flag whether the expanded node should be selected
         */
        public Expander(DefaultMutableTreeNode node, boolean select)
        {
            node_ = node;
            select_ = select;
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the Runnable interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @ see java.lang.Runnable */
        public void run()
        {
            TreePath path = new TreePath(node_.getPath());
            tree_.expandPath(path);
            if (select_)
            {
                tree_.setSelectionPath(path);
            }
        }
    }

    /**
     * The virtual folder is an IAbstractFolder implementation. This folder
     * doesn't exist on the file system but in the folder tree. The virtual
     * folder cannot contain any children.
     */
    private class VirtualFolder implements IAbstractFolder
    {
        // the parent folder
        private IAbstractFolder parent_;

        // the name of the folder
        private String          name_;

        // the project of the folder
        private IProject        project_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Constructs a new virtual folder that is a children of the given
         * project. The folder gets the given name.
         * 
         * @param project
         *            the project of the folder (this is the paren too)
         * @param name
         *            the name of the folder
         */
        public VirtualFolder(IProject project, String name)
        {
            project_ = project;
            name_ = name;
            parent_ = null;
        }

        /**
         * Constructs a new virtual folder that is a children of the given
         * folder. The folder gets the given name.
         * 
         * @param folder
         *            the parent folder of the new virtual folder
         * @param name
         *            the name of the folder
         */
        public VirtualFolder(IAbstractFolder folder, String name)
        {
            project_ = null;
            name_ = name;
            parent_ = folder;
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the IAbstractFolder interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see viptool.project.IAbstractFolder */
        public IAbstractFile add(IAbstractFile file)
                throws ChildNotAcceptedException
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFolder */
        public void addAbstractFolderListener(IAbstractFolderListener listener)
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFolder */
        public Iterator getChildIterator()
        {
            return new Iterator()
            {
                // <><><><><><><><><><><><><><><><><><><><><>
                // Implementation of the Iterator interface
                // <><><><><><><><><><><><><><><><><><><><><>

                /* @see java.util.Iterator */
                public boolean hasNext()
                {
                    return false;
                }

                /* @see java.util.Iterator */
                public Object next()
                {
                    throw new NoSuchElementException();
                }

                /* @see java.util.Iterator */
                public void remove()
                {
                    throw new UnsupportedOperationException();
                }

            };
        }

        /* @see viptool.project.IAbstractFolder */
        public IAbstractFile[] getChilds()
        {
            return new IAbstractFile[0];
        }

        /* @see viptool.project.IAbstractFolder */
        public IAbstractFile[] getChilds(Class filter)
        {
            return new IAbstractFile[0];
        }

        /* @see viptool.project.IAbstractFolder */
        public int getSize()
        {
            return 0;
        }

        /* @see viptool.project.IAbstractFolder */
        public void remove(IAbstractFile file)
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFolder */
        public void removeAbstractFolderListener(
                IAbstractFolderListener listener)
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFolder */
        public IAbstractFile[] update(boolean recursive)
        {
            throw new UnsupportedOperationException();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the IAbstractFile interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see viptool.project.IAbstractFile */
        public File getFile()
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFile */
        public String getFullPath()
        {
            if (getParentFolder() != null)
            {
                return getParentFolder().getFullPath() + File.separator
                        + getName();
            }

            return project_.getAbstractFile().getPathFile().getAbsolutePath()
                    + File.separator + getName();
        }

        /* @see viptool.project.IAbstractFile */
        public long getLastModified()
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFile */
        public String getName()
        {
            return name_;
        }

        /* @see viptool.project.IAbstractFile */
        public IAbstractFolder getParentFolder()
        {
            return parent_;
        }

        /* @see viptool.project.IAbstractFile */
        public File getPathFile()
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFile */
        public IProject getProject()
        {
            if (project_ != null)
            {
                return project_;
            }

            return getParentFolder().getProject();
        }

        /* @see viptool.project.IAbstractFile */
        public String getTitle()
        {
            return getName();
        }

        /* @see viptool.project.IAbstractFile */
        public boolean isLocked()
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFile */
        public void releaseLock(IFileLocker locker)
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFile */
        public IFileLocker getFileLocker()
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFile */
        public void requestLock(IFileLocker locker)
                throws FileAlreadyLockedException
        {
            throw new UnsupportedOperationException();
        }

        /* @see viptool.project.IAbstractFile */
        public void setName(String name, IWorkbench workbench)
                throws FileRenameException
        {
            name_ = name;
        }

        public void addAbstractFileListener(IAbstractFileListener listener)
        {
            throw new UnsupportedOperationException();
        }

        public void removeAbstractFileListener(IAbstractFileListener listener)
        {
            throw new UnsupportedOperationException();
        }
    }

    /**
     * The Project comparator compares two project by their names.
     */
    private class ProjectComparator implements Comparator
    {
        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the Comparator interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see java.util.Comparator */
        public int compare(Object obj1, Object obj2)
        {
            if (!(obj1 instanceof IProject) || !(obj2 instanceof IProject))
            {
                return 0;
            }

            return ((IProject) obj1).getName().compareTo(
                    ((IProject) obj2).getName());
        }
    }

    /**
     * The folder comparator compares to folders by their names.
     */
    private class FolderComparator implements Comparator
    {
        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the Comparator interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see java.util.Comparator */
        public int compare(Object obj1, Object obj2)
        {
            if (!(obj1 instanceof IAbstractFolder)
                    || !(obj2 instanceof IAbstractFolder))
            {
                return 0;
            }

            return ((IAbstractFolder) obj1).getName().compareTo(
                    ((IAbstractFolder) obj2).getName());
        }
    }

    /**
     * This class holds information about the selected element in the folder
     * tree panel. It contains the enclosing project, the parent and the
     * selected folder itself.
     */
    public class FolderTreeSelection
    {
        // the project of the selected element
        private IProject        project_;

        // the parent of the selected element. Is null if the parent is a
        // project
        private IAbstractFolder parent_;

        // the selected folder
        private IAbstractFolder folder_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Constructs a new folder tree selection object with the given
         * information. The project and folder parameter must not be null, thus
         * the parent can be null if the parent of the folder is project.
         * 
         * @param project
         *            the project of the selected element
         * @param parent
         *            the parent of the selected element or null if the parent
         *            is a project
         * @param folder
         *            the selected folder
         */
        public FolderTreeSelection(IProject project, IAbstractFolder parent,
                IAbstractFolder folder)
        {
            project_ = project;
            parent_ = parent;
            folder_ = folder;
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Getter methods
        // <><><><><><><><><><><><><><><><><><><><><>

        /**
         * Returns the project of the selected element.
         * 
         * @return the project of the selected element
         */
        public IProject getProject()
        {
            return project_;
        }

        /**
         * Returns the parent of the selected element or null if the parent is a
         * project.
         * 
         * @return the parent of the selected element or null
         */
        public IAbstractFolder getParent()
        {
            return parent_;
        }

        /**
         * Returns the selected folder.
         * 
         * @return the selected folder
         */
        public IAbstractFolder getFolder()
        {
            return folder_;
        }

        /**
         * Returns true if the selected folder is a virtual folder and does not
         * exist on the hard disk.
         * 
         * @return true of the folder is virtual folder, false otherwise
         */
        public boolean isVirtualFolder()
        {
            return (folder_ instanceof VirtualFolder);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Call this constructor to construct a folder pane, that displays all
     * projects and their subfolders.
     */
    public FolderTreePanel(IWorkbench workbench)
    {
        // create a pseudo root element (a tree must have a root element!)
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("root");

        // initalize
        initialize(workbench, root);

        // hide the pseudo root element so the projects are the first level
        tree_.setRootVisible(false);

        // fill the tree and expand the root otherwise the elements are not
        // visible
        fillTree(root);
        if (root.getChildCount() > 0)
        {
            SwingUtilities.invokeLater(new Expander(
                    (DefaultMutableTreeNode) root.getFirstChild(), true));
        }
    }

    /**
     * Call this constructor to construct a folder pane, that displays the given
     * project and its subfolders.
     * 
     * @param project
     *            the project that should be displayed the the folder pane
     */
    public FolderTreePanel(IWorkbench workbench, IProject project)
    {
        DefaultMutableTreeNode node;
        // create a pseudo root element so we can use the addProject method
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("root");

        // initialize the tree
        initialize(workbench, root);

        // hide the pseudo root element so the project is the first level
        tree_.setRootVisible(false);

        // fill the tree with the current project and expand the root
        // otherwise the project will not be visible
        node = addProject(root, project);
        SwingUtilities.invokeLater(new Expander(node, true));
    }

    /**
     * Call this constructor to display the given folder and all its subfolders.
     * 
     * @param folder
     *            the folder that should be displayed
     */
    public FolderTreePanel(IWorkbench workbench, IAbstractFolder folder)
    {
        DefaultMutableTreeNode node;
        // create a pseudo root element so we can use the addFolder method
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("root");

        // initialize the tree
        initialize(workbench, root);

        // hide the pseudo root element so the folder is the first level
        tree_.setRootVisible(false);

        // add the folder and its subfolders to the tree and expand the root
        // otherwise the folder will not be visible
        node = addFolder(root, folder);
        tree_.expandPath(new TreePath(root));

        // expand the folder, looks nicer
        SwingUtilities.invokeLater(new Expander(node, true));
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Tree manipulation
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Expands the given item. The item must be a project or an abstract folder.
     * If the item is no such object or the item cannot be found in the tree,
     * the operation is ignored. Otherwise the item is expanded, so it is
     * visible in the tree
     * 
     * @param item
     *            the item (project or folder) that should be expanded and
     *            viewable
     */
    public void expandItem(Object item)
    {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) nodes_.get(item);

        if (node != null)
        {
            tree_.expandPath(new TreePath(node.getPath()));
        }
    }

    /**
     * Selects the given item. The item must be a project or an abstract folder.
     * If the item is no such object or the item cannot be found in the tree,
     * the operation is ignored. Otherwise the item is selected (an if necessary
     * expanded).
     * 
     * @param item
     *            the item (project or folder) that should be selected
     */
    public void selectItem(Object item)
    {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) nodes_.get(item);

        if (node != null)
        {
            tree_.setSelectionPath(new TreePath(node.getPath()));
        }
    }

    /**
     * Adds a folder with the given name as child to the currently selected item
     * in the tree. If the selected item is a virtual folder the operation is
     * ignored. A virtual folder is a folder that only exists in the tree (that
     * is a folder created by createFolder).
     * 
     * @param name
     *            the name for the new folder
     */
    public void createFolder(String name)
    {
        int index;
        DefaultTreeModel model = (DefaultTreeModel) tree_.getModel();
        FolderTreeSelection selection = getSelectedItem();
        VirtualFolder folder = null;
        DefaultMutableTreeNode parentNode;
        DefaultMutableTreeNode node = null;
        Enumeration enumeration;
        DefaultMutableTreeNode children;
        IAbstractFolder childrenFolder;

        if (selection == null || selection.isVirtualFolder())
        {
            // only one virtual folder can be created
            return;
        }

        if (selection.getFolder() == null)
        {
            parentNode = (DefaultMutableTreeNode) nodes_.get(selection.getProject());

            if (parentNode != null)
            {
                folder = new VirtualFolder(selection.getProject(), name);
            }
        }
        else
        {
            parentNode = (DefaultMutableTreeNode) nodes_.get(selection.getFolder());

            if (parentNode != null)
            {
                folder = new VirtualFolder(selection.getFolder(), name);
            }
        }

        if (folder != null)
        {
            node = new DefaultMutableTreeNode(folder);

            // find the right index to insert the node (respect alphabetical
            // order)
            index = 0;
            enumeration = parentNode.children();
            while (enumeration.hasMoreElements())
            {
                children = (DefaultMutableTreeNode) enumeration.nextElement();
                // the children must be an abstract folder, because only
                // projects and folders are displayed and projects are only on
                // the first level
                childrenFolder = (IAbstractFolder) children.getUserObject();

                if (childrenFolder.getName().compareTo(name) == 0)
                {
                    // the folder is already here. Don't add it.
                    SwingUtilities.invokeLater(new Expander(children, true));
                    return;
                }

                if (childrenFolder.getName().compareTo(name) > 0)
                {
                    // the children name follows the new name, so this is the
                    // right insert index
                    break;
                }

                index++;
            }

            model.insertNodeInto(node, parentNode, index);
            SwingUtilities.invokeLater(new Expander(node, true));
        }
    }

    /**
     * Returns a folder tree selection object containing information about the
     * currently selected folder. If no folder is selected, null will be
     * returned.
     * 
     * @return a folder tree selection object with information about the
     *         selected folder or null
     */
    public FolderTreeSelection getSelectedItem()
    {
        TreePath path = tree_.getSelectionPath();

        return getSelectedItem(path);
    }

    private FolderTreeSelection getSelectedItem(TreePath path)
    {
        IProject project;
        IAbstractFolder parent;
        IAbstractFolder folder;
        DefaultMutableTreeNode node;

        if (path != null)
        {
            node = ((DefaultMutableTreeNode) path.getLastPathComponent());

            if (node.getUserObject() instanceof IAbstractFolder)
            {
                folder = (IAbstractFolder) node.getUserObject();

                project = folder.getProject();
                parent = folder.getParentFolder();
            }
            else
            {
                // must be a project
                project = (IProject) node.getUserObject();
                folder = null;
                parent = null;
            }

            return new FolderTreeSelection(project, parent, folder);
        }

        // no item selected
        return null;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Initializes the class. The tree and the hosting scroll pane is created.
     * The layout is set. The tree is not filled, but the root element is set to
     * the given value.
     * 
     * @param workbench
     *            the current workbench of viptool
     * @param root
     *            the root item for the tree
     */
    private void initialize(IWorkbench workbench, DefaultMutableTreeNode root)
    {
        workbench_ = workbench;

        listeners_ = new HashSet();
        nodes_ = new HashMap();

        tree_ = new JTree(root);
        tree_.setCellRenderer(new TreeCellRenderer());
        tree_.getSelectionModel().setSelectionMode(
                TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree_.addFocusListener(this);
        tree_.addTreeSelectionListener(this);

        scrollPane_ = new JScrollPane(tree_);

        setLayout(new BorderLayout());
        add(scrollPane_, BorderLayout.CENTER);
    }

    /**
     * Fills the tree with elements. This method is for trees that should show
     * all projects as "pseudo root elements". This means the projects become
     * children of the given root element, but the root element should not be
     * visible, so the projects are the first level in the tree.
     * 
     * @param root
     *            the root element of the tree
     */
    private void fillTree(DefaultMutableTreeNode root)
    {
        IWorkspace workspace = workbench_.getCurrentWorkspace();
        IProject projects[] = workspace.getProjects();

        Arrays.sort(projects, new ProjectComparator());
        for (int i = 0; i < projects.length; i++)
        {
            addProject(root, projects[i]);
        } // end for (iterating projects)
    }

    /**
     * Adds the given project to the given parent. Then all subfolders are
     * recursivly added to the project. This method will expand the project so
     * we have a nice look.
     * 
     * @param parent
     *            the parent for the project
     * @param project
     *            the project to add
     * 
     * @return the created tree node for the project
     */
    private DefaultMutableTreeNode addProject(DefaultMutableTreeNode parent,
            IProject project)
    {
        IAbstractFile[] files;
        DefaultMutableTreeNode item = new DefaultMutableTreeNode(project);

        parent.add(item);
        nodes_.put(project, item);

        files = project.getFiles(IAbstractFolder.class);
        Arrays.sort(files, new FolderComparator());
        for (int i = 0; i < files.length; i++)
        {
            addFolder(item, (IAbstractFolder) files[i]);
        }

        SwingUtilities.invokeLater(new Expander(item));
        return item;
    }

    /**
     * Adds the given folder to the parent node. After that the method is
     * recursivly called to add all subfolders too.
     * 
     * @param parent
     *            the parent node for the folder
     * @param folder
     *            the folder to add
     * 
     * @return the created tree node for the folder
     */
    private DefaultMutableTreeNode addFolder(DefaultMutableTreeNode parent,
            IAbstractFolder folder)
    {
        IAbstractFile[] files;
        DefaultMutableTreeNode item = new DefaultMutableTreeNode(folder);

        parent.add(item);
        nodes_.put(folder, item);

        files = folder.getChilds(IAbstractFolder.class);
        Arrays.sort(files, new FolderComparator());
        for (int i = 0; i < files.length; i++)
        {
            addFolder(item, (IAbstractFolder) files[i]);
        }

        return item;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Listener support
    // <><><><><><><><><><><><><><><><><><><><><>

    public void addFolderTreePanelListener(IFolderTreePanelListener listener)
    {
        if (listener != null)
        {
            listeners_.add(listener);
        }
    }

    public void removeFolderTreePanelListener(IFolderTreePanelListener listener)
    {
        listeners_.remove(listener);
    }

    protected void fireFocusGainedEvent(FocusEvent event)
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IFolderTreePanelListener) it.next()).focusGained(event);
        }
    }

    protected void fireFocusLostEvent(FocusEvent event)
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IFolderTreePanelListener) it.next()).focusLost(event);
        }
    }

    protected void fireSelectionEvent(FolderTreeSelection selection)
    {
        Iterator it;
        FolderTreePanelSelectionEvent event = new FolderTreePanelSelectionEvent(
                this, selection);

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IFolderTreePanelListener) it.next()).itemSelected(event);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the TreeSelectionListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    public void valueChanged(TreeSelectionEvent event)
    {
        fireSelectionEvent(getSelectedItem(event.getNewLeadSelectionPath()));
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the FocusListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    public void focusGained(FocusEvent event)
    {
        fireFocusGainedEvent(event);
    }

    public void focusLost(FocusEvent event)
    {
        fireFocusLostEvent(event);
    }

}
