package viptool.base.gui.dialog;

import java.awt.Image;

import javax.swing.JComponent;

import viptool.base.gui.dialog.event.IStandardDialogListener;

/**
 * This class provides a standard dialog with the following layout:
 * 
 * <pre>
 *   ----------------------------------
 *   | TITLE                    Image |
 *   | Description              Image |
 *   ----------------------------------
 *   | Notice panel (optional)        |
 *   ----------------------------------
 *   |                                |
 *   |                                |
 *   |                                |
 *   |           CONTENT              |
 *   |                                |
 *   |                                |
 *   |                                |
 *   |                                |
 *   ----------------------------------
 *   |                       BUTTONS  |
 *   ----------------------------------
 * </pre>
 * 
 * You can give a JPanel object as content panel to embedd your own Dialog. You
 * can also implement validators and dynamicly show warnings and errors. The
 * button panel Supports various Buttons such as OK, CANCEL, IGNORE, ...
 */
public interface IStandardModalDialog
{
    /**
     * Adds a new listener to the dialog. A listener receives all important
     * events that happens in the dialog except of the content panel, since the
     * 'user' is responsible for the content himself.
     * 
     * @param listener
     *            the listener to add to the dialog
     */
    public void addStandardDialogListener(IStandardDialogListener listener);

    /**
     * Removes the given listener from the dialog. The listener will not receive
     * any notifications of the dialog in the future.
     * 
     * @param listener
     *            the listener to remove from the dialog
     */
    public void removeStandardDialogListener(IStandardDialogListener listener);

    /**
     * Sets a new notice. To remove a notice set it to null. The notice panel is
     * automatically shown if the notice is not null and automatically hidden,
     * if a null notice is applied.<br>
     * The type must be one of the values in DialogConstants. At the moment
     * warning and error is supported and expressed by colours (yellow and red).
     * 
     * @param notice
     *            the new notice to display or null to remove a notice
     * @param type
     *            the new type for the notice (value from DialogConstants)
     */
    public void setNotice(String notice, int type);

    /**
     * Removes the notice from the dialog, so that no notice is shown.
     */
    public void removeNotice();

    /**
     * Enables or disable one or more buttons in the button panel. The Parameter
     * buttons can be a combination of several buttons in DialogConstants.
     * 
     * @param buttons
     *            a combination of buttons from DialogConstants
     * @param enable
     *            set to true to enable the buttons, set to false to disable the
     *            buttons
     */
    public void setButtonsEnabled(int buttons, boolean enable);

    /**
     * Sets the new content for the dialog. The component is displayed in the
     * client area of the dialog. Respect that the dialog can be resized.
     * 
     * @param content
     *            the new content for the dialog. MUST NOT be null.
     */
    public void setContent(JComponent component);

    /**
     * Changes the detailed description in the head of the dialog. The change is
     * shown immediatly and no further update is needed.
     * 
     * @param description
     *            the new description that should be displayed
     */
    public void setDetailedDescription(String description);

    /**
     * Sets the new image for the dialog. To remove an existing image give null
     * as parameter.
     * 
     * @image the new image for the dialog
     */
    public void setImage(Image image);

    /**
     * Sets the main description for the dialog. The main description is
     * displayed on the top left of the dialog in BIG letters so don't use more
     * than two or three words. The main description must not be imageempty or
     * null.<br>
     * The dialog is updated automatically so no extra call is necessary.
     * 
     * @param description
     *            the new main description for the dialog.
     */
    public void setMainDescription(String description);
}
