package viptool.base.gui.menu;

import java.net.URI;
import java.util.ArrayList;

import viptool.base.error.XMLException;
import viptool.base.gui.selection.IAbstractFileSelection;
import viptool.base.gui.selection.IFileTypeFilter;
import viptool.base.gui.selection.ISelectionFilter;
import viptool.base.gui.selection.ISelectionModel;

/**
 * The IContextMenuManager interface provides methods to manage the ContextMenu,
 * like adding or removing a menu entry e.g.
 */
public interface IContextMenuManager {

	/**
	 * Creates and adds the context menu template to the contextmenu manager.
	 */
	public void insertContextMenuTemplate();

	/**
	 * Creates a concrete instance of the template that was specified by the id
	 * parameter. The returned menu bar is a clone of the template with fully
	 * cloned actions, that are registered in the action handler with the same
	 * instance id. This enables access to the same actions as they are in the
	 * menu (good for using the actions somewhere else, e.g. in toolbars).
	 * 
	 * @param selection
	 * 
	 * @return a menu bar based on the template given by the id
	 */
	public IExtendedContextMenu createContextMenu(
			IAbstractFileSelection selection);

	/**
	 * Adds a new entry to the contemporary context menu.
	 * 
	 * @param modelList
	 *            the SelectionModels that determine if the entry shall be
	 *            displayed or not for the current selection
	 * @param actionID
	 *            the id of the action that is added to the menu
	 * @param container
	 *            the id of the containing action, if the added action is a
	 *            submenu of another menu. If it is no submenu, leave container
	 *            empty.
	 * @param insertAt
	 *            the id of the action, that shall serve as an anchor for the
	 *            added action. The position of the menu entry is determined
	 *            relatively to this anchor.
	 * @param seek
	 *            the distance from the inserAt, e.g. 1, if the entry shall be
	 *            displayed directly below the anchor.
	 */
	public void addMenuEntry(ArrayList<ISelectionModel> modelList,
			String actionID, String container, String insertAt, int seek);

	/**
	 * Removes a menu entry from the contemporary context menu.
	 * 
	 * @param actionID
	 *            the id of the action that shall be removed.
	 */
	public void removeMenuEntry(String actionID);

	/**
	 * Creates a new FileTypeFilter. The ContextMenuManager works as a factory
	 * here, to provide an interface for creating FileTypeFilters.
	 * 
	 * @param name
	 *            a regular expression that specifies all file names that are
	 *            accepted by the filter
	 * @param type
	 *            the accepted file types, "f" for normal files, "d" for
	 *            directories, "" for all file types
	 * @param content
	 *            checks weather a given file provider is registered, i.e.
	 *            "viptool.petrinet.io.RensingNetFileProvider", usually this
	 *            field remains empty
	 * @param min
	 *            number of files that have to be selected at least
	 * @param max
	 *            number of files that may maximally be selected, -1 for maxInt
	 * 
	 * @return the new instance of FileTypeFilter
	 */
	public IFileTypeFilter createFileTypeFilter(String name, String type,
			String content, int min, int max);

	/**
	 * Creates a new SelectionModel. The ContextMenuManager works as a factory
	 * here, to provide an interface for creating SelectionModels.
	 * 
	 * @param filterList
	 *            a list of ISelectionFilters, that the SelectionModel shall
	 *            contain.
	 * 
	 * @return the new instance of SelectionModel
	 */
	public ISelectionModel createSelectionModel(
			ArrayList<ISelectionFilter> filterList);

}
