package viptool.base.gui.toolbar;

import java.awt.Component;

import viptool.base.action.IActionFilter;
import viptool.base.action.IActionPostProcessor;

/**
 * The toolbar template is a template for a toolbar. The template is usually
 * created by the ToolBarTemplateManager. If a concrete instance is needed, e.g.
 * for the main windows, the createToolBar method can be called. This method
 * will create a clone of the toolbar template and will ask the action handler
 * for new actions with the specified instance.
 */
public interface IToolBarTemplate
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // template manipulation
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds a new toolbar item with the given id to the template. The toolbar
     * item is inserted as a child of the specified container. The insertAt and
     * seek parameters can be used to determine the position of the inserted
     * item within the container. The toolbar item type is determined by the
     * action that is returned by a request to the action handler using the
     * given id. If the action is a group action, a submenu will be created. If
     * the action is a separator action a separator will be created. Otherwise a
     * button, radio or checkbox item would be created.
     * 
     * The seek parameter can have the following values:
     * <ul>
     * <li>negative value: the element is added seek elements before the
     * InsertAt-element</li>
     * <li>zero: the InsertAt parameter is ignored and the element is added at
     * the end</li>
     * <li>positive value: the element is added seek elements after the
     * InsertAt-element</li>
     * </ul>
     * 
     * @param id
     *            the id of the toolbar item that has to be added
     * @param container
     *            the container in which the toolbar item should be added
     * @param insertAt
     *            the id of the element to which the seek parameter refers
     * @param seek
     *            a value that determines the position of the new toolbar item.
     *            See above for possible values.
     */
    public void addToolBarItem(String id, String container, String insertAt,
            int seek);

    /**
     * Removes the toolbar item with the given id from the toolbar. If the id
     * represents a container, the container will be removed. This method cannot
     * be used to remove separators, because separators have no unique id.
     * Separators cannot be removed at this time.
     * 
     * @param id
     *            the id of the element to be removed.
     */
    public void removeToolBarItem(String id);

    /**
     * Sets the given template as parent. This will delete the templates content
     * and overwrite it with the parent content. To avoid loosing information
     * make sure you call setParent BEFORE you add specific child content. Do
     * not change the parent in the past (after adding child content).
     * 
     * @param parent
     *            the parent template. This template will inherit from the given
     *            parent.
     */
    public void setParent(IToolBarTemplate parent);

    // <><><><><><><><><><><><><><><><><><><><><>
    // state information
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the toolbar item with the given id. If the id represents a
     * container a JToolBar will be returned. Separators cannot be returned,
     * because they have no unique id.
     * 
     * @param id
     *            the id of the element to be returned
     * @return the element with the given id or null if no such element exists
     */
    public Component getToolBarItem(String id);

    /**
     * Returns the id of the template. This id must be unique for toolbars in
     * the viptool.
     * 
     * @return the id of the template
     */
    public String getId();

    // <><><><><><><><><><><><><><><><><><><><><>
    // toolbar creation methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a concrete instance of the template. The returned toolbar is a
     * clone of the template with fully cloned actions, that are registered in
     * the action handler with the same instance id. This enables access to the
     * same actions as they are in the toolbar (good for using the actions
     * somewhere else, e.g. in menus).
     * 
     * 
     * @param instance
     *            an unique instance id for further access to the same toolbar
     * @return a toolbar based on the template
     */
    public IExtendedToolBar createToolBar(int instance);

    public IExtendedToolBar createToolBar(int instance, IActionFilter filter);

    public IExtendedToolBar createToolBar(int instance, IActionFilter filter,
            IActionPostProcessor postProcessor);
}
