package viptool.base.io;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import viptool.base.editor.IEditor;
import viptool.base.io.event.IFileIOEvent;
import viptool.base.io.event.IFileInterpreterListener;
import viptool.project.IAbstractFile;

/**
 * This is an implementation of the IFileInterpreter interface and implements
 * some of the interface methods. It also implements the listener methods and
 * provides two fire methods for firing the load and save event.
 * 
 * @see viptool.base.io.IFileInterpreter
 */
public abstract class FileInterpreter implements IFileInterpreter
{
    // the abstract file that is parsed by this file interpreter
    private IAbstractFile            abstractFile_;

    // the provider that has created this interpreter
    private IFileInterpreterProvider provider_;

    // a set of listeners
    private Set                      fileInterpreterListeners_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs the file interpreter based on the given abstract file. The
     * provider is also stored to know which provider has created this
     * interpreter.
     * 
     * @param provider
     *            the provider which created the interpreter
     * @param file
     *            the file to which this interpreter should be connected
     */
    public FileInterpreter(IFileInterpreterProvider provider, IAbstractFile file)
    {
        provider_ = provider;
        abstractFile_ = file;

        fileInterpreterListeners_ = new HashSet();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // helper methods for the file interpreter listeners
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This method notifies all file interpreter listeners that the interpreter
     * has loaded a file.
     * 
     * @param editor
     *            the editor that has loaded the file
     */
    protected void fireLoadEvent(IEditor editor)
    {
        Iterator it;
        IFileInterpreterListener listener;

        it = fileInterpreterListeners_.iterator();
        while(it.hasNext())
        {
            listener = (IFileInterpreterListener)it.next();
            listener.fileLoaded(new IFileIOEvent(abstractFile_, editor));
        }
    }

    /**
     * This method notifies all file interpreter listeners that the interpreter
     * has saved changes to the file.
     * 
     * @param editor
     *            the editor that has saved the file
     */
    protected void fireSavedEvent(IEditor editor)
    {
        Iterator it;
        IFileInterpreterListener listener;

        it = fileInterpreterListeners_.iterator();
        while(it.hasNext())
        {
            listener = (IFileInterpreterListener)it.next();
            listener.fileLoaded(new IFileIOEvent(abstractFile_, editor));
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the FileInterpreter interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.base.io.FileInterpreter */
    public IAbstractFile getAbstractFile()
    {
        return abstractFile_;
    }

    /* @see viptool.base.io.FileInterpreter */
    public IFileInterpreterProvider getProvider()
    {
        return provider_;
    }

    /* @see viptool.base.io.FileInterpreter */
    public void addFileInterpreterListener(IFileInterpreterListener listener)
    {
        fileInterpreterListeners_.add(listener);
    }

    /* @see viptool.base.io.FileInterpreter */
    public void removeFileInterpreterListener(IFileInterpreterListener listener)
    {
        fileInterpreterListeners_.remove(listener);
    }
}
