package viptool.base.io;

import viptool.base.error.FileInterpreterException;
import viptool.base.io.event.IFileInterpreterListener;
import viptool.project.IAbstractFile;

/**
 * This interface provides methods for an file interpreter. A file interpreter
 * reads a file a unique way. That means two file interpreters can read the same
 * file, but will do that in a unique way. For example the text file interpreter
 * will read lines of text and the petri net file interpreter will generate a
 * petri net from the content.
 */
public interface IFileInterpreter
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // general
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the provider that is responsible for this file interpreter. Must
     * always return a valid reference.
     * 
     * @return a reference to the managing file interpreter provider
     */
    public IFileInterpreterProvider getProvider();

    /**
     * Returns the file that is associated with this file interpreter. Must
     * always return a valid file.<br>
     * The returned reference is an abstract file, that is a file that exists
     * but the content is not yet determined and not yet interpreted.
     * 
     * @return a reference to an existing file
     */
    public IAbstractFile getAbstractFile();

    // <><><><><><><><><><><><><><><><><><><><><>
    // input and output operations
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Interpretes the assigned abstract file. After interpreting, an editor can
     * display the file. This method is called load because load is more
     * self-explanatory than interprete.<br>
     * 
     * @throws FileInterpreterException -
     *             if the file cannot be loaded
     */
    public void load() throws FileInterpreterException;

    /**
     * Saves the interpreted file to the real existing file. This is the
     * opposite way of interpreting, e.g. a graph is coded in xml code again.
     * <br>
     * 
     * @throws FileInterpreterException -
     *             if the file cannot be saved
     */
    public void save() throws FileInterpreterException;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Methods for event handling
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds the given file interpreter listener to the file interpreter. These
     * listeners are notified when the interpreter saves or loads the file.
     * 
     * @param listener
     *            the listener to notify
     */
    public void addFileInterpreterListener(IFileInterpreterListener listener);

    /**
     * Removes the given file interpreter listener from the file interpreter.
     * The listener will no longer be notified when the a file is loaded or
     * saved by the interpreter.
     * 
     * @param listener
     *            the listener that should be removed
     */
    public void removeFileInterpreterListener(IFileInterpreterListener listener);

}
