package viptool.base.io;

import viptool.project.IAbstractFile;

/**
 * This interface is for the file interpreter manager that manages all file
 * interpreter providers.
 */
public interface IFileInterpreterManager
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // adding and removing file interpreter providers
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds a new file type to the file interpreter manager.
     * file interpreter providers are used to determine the file type of a file
     * respecting the content of a file.
     * 
     * @param provider
     *            the new file type provider
     */
    public void addFileInterpreterProvider(IFileInterpreterProvider provider);

    /**
     * Removes the given file interpreter provider from the file interpreter
     * manager. The file interpreter can no longer be used in viptool now.
     * 
     * @param provider
     *            the file interpreter provider that should be removed from the
     *            manager
     */
    public void removeFileInterpreterProvider(IFileInterpreterProvider provider);

    // <><><><><><><><><><><><><><><><><><><><><>
    // get file interpreter providers
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the default file interpreter for the given abstract file. With
     * the help of the file interpreter provider information like the icon can
     * be retireved.
     * 
     * @param file
     *            the file for which the file interpreter provider is needed
     * @return the default file interpreter provider for the given file
     */
    public IFileInterpreterProvider getDefaultFileInterpreterProvider(
            IAbstractFile file);

    /**
     * Returns all file interpreter providers that would accept the given file.
     * If the given file isn't accepted by any provider, an empty array will be
     * returned. The array is sorted by the priority of each interpreter
     * provider. This means that the default provider will be at the first index
     * zero.
     * 
     * @param file
     *            the file for which file interpreter providers are searched
     * @return an array containing all file interpreter providers that would
     *         accept the given file
     */
    public IFileInterpreterProvider[] getMatchingFileInterpreterProviders(
            IAbstractFile file);

    /**
     * Returns an array that contains all file type providers.
     * 
     * @return an array of all file type providers
     */
    public IFileInterpreterProvider[] getFileInterpreterProviders();

    /**
     * Returns the file interpreter provider instance with the given class.
     * 
     * @param clazz
     *            the class of the file interpreter provider to return
     * @return the file interpreter provider matching the given class or null
     */
    public IFileInterpreterProvider getFileInterpreterProvider(Class clazz);
}
