package viptool.base.io;

import javax.swing.Icon;

import viptool.base.IWorkbench;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.wizard.IWizard;
import viptool.project.IAbstractFile;

/**
 * This is the interface that all file type providers must implement. The
 * interface contains methods for creating abstract files of the handled file
 * types as well as for testing existing files, if they match the own file
 * format.<br>
 * It also has some methods for gui purpose e.g. for returning an icon or a
 * description.<br>
 * File type providers can also implement the getNewFileWizard method to provide
 * gui wizards for creating new files.
 */
public interface IFileInterpreterProvider
{
    /**
     * Tests whether the content of the file matches the file type of the
     * provider. If it matches the method should return true, false otherwise.
     * The test should be done quickly but collision free.
     * 
     * @param file
     *            the file to test
     * @return true if the file matches the file type, false otherwise
     * @throws FileInterpreterProviderException -
     *             if the file cannot be tested
     */
    public boolean matchFormat(IAbstractFile file)
            throws FileInterpreterProviderException;

    /**
     * Creates a abstract file based on the given folder and file. It is not
     * necessary to test the file again, because matchFormat should have be
     * called before createFromFile is called.
     * 
     * @param file
     *            the file to which the project element should be assigned
     * @return the created project element (should never return null)
     * @throws FileInterpreterProviderException -
     *             if the file cannot be read
     */
    public IFileInterpreter createFromFile(IAbstractFile file)
            throws FileInterpreterProviderException;

    /**
     * Should create a new file that matches the file type.
     * 
     * @param file
     *            the file that should be created
     * @return the created abstract file
     * @throws FileInterpreterProviderException -
     *             if the file cannot be created
     */
    public IFileInterpreter createNewFile(IAbstractFile file)
            throws FileInterpreterProviderException;

    /**
     * Returns the file type as a string. e.g. "petri net file".
     * 
     * @return file type
     */
    public String getFileType();

    /**
     * Returns a description about the content of the file.
     * 
     * @return file description
     */
    public String getDescription();

    /**
     * Returns the icon of the file
     * 
     * @return icon for the file
     */
    public Icon getIcon();

    /**
     * Returns a wizard that can in several steps create a new file of the
     * providers file type.
     * 
     * @return a wizard for creating a new file
     */
    public IWizard getNewFileWizard();

    /**
     * Returns whether a wizard exists for this file type. If this method
     * returns true, getNewFileWizard MUST NOT return null in a further call.
     * 
     * @return true if a wizard exists for this filetype, false otherwise
     */
    public boolean hasNewFileWizard();

    /**
     * Tests whether a new file can be created at the current workbench state.
     * Will return false if this type of file cannot be created in the current
     * workbench. Returns true otherwise.
     * 
     * @param workbench
     *            the current workbench
     * @return true if a file can be created, false otherwise
     */
    public boolean canCreateNewFile(IWorkbench workbench);
}
