package viptool.base.wizard;

import java.awt.event.WindowListener;

import viptool.base.gui.dialog.IStandardModalDialog;

/**
 * The wizard interface is the basic interface for creating dialog based
 * wizards. Wizards a returned e.g. from FileTypeProviders to create new files.
 * A Wizard has several methods for navigating through a process like
 * getFirstPage, getNextPage and getPrevoiusPage.<br>
 * The wizard is responsible for all logical changes. Only visual things are
 * automated.
 */
public interface IWizard extends WindowListener
{
    /**
     * Should return the first page of the wizard. This method should also reset
     * the wizard into its initial state. getFirstPage() is only called once per
     * wizard process. MUST NOT return null.<br>
     * On the first page the following button configuration is initially set:
     * <ul>
     * <li>previous: disabled</li>
     * <li>next: disabled</li>
     * <li>finish: disabled</li>
     * <li>cancel: enabled</li>
     * </ul>
     * The button state is set before the call to the method, so the method can
     * change the button configuration (enabling, disabling).
     * 
     * @param dialog
     *            the dialog in which the wizard is loaded
     * @return the first wizard page
     */
    public IWizardPage getFirstPage(IStandardModalDialog dialog);

    /**
     * Should return the next page of the wizard. This method can return null if
     * there is no next page. <br>
     * On the next page the following button configuration is initially set:
     * <ul>
     * <li>previous: return of canGoToPrevious() from the wizard page</li>
     * <li>next: disabled</li>
     * <li>finish: disabled</li>
     * <li>cancel: enabled</li>
     * </ul>
     * The button state is set before the call to the method, so the method can
     * change the button configuration (enabling, disabling).
     * 
     * @return the next wizard page
     */
    public IWizardPage getNextPage();

    /**
     * Should return the previous page of the wizard. This method can return
     * null if there is no previous page. <br>
     * On the previous page the following button configuration is initially set:
     * <ul>
     * <li>previous: return of canGoToPrevious() from the wizard page</li>
     * <li>next: enabled</li>
     * <li>finish: disabled</li>
     * <li>cancel: enabled</li>
     * </ul>
     * The button state is set before the call to the method, so the method can
     * change the button configuration (enabling, disabling).
     * 
     * @return
     */
    public IWizardPage getPreviousPage();

    /**
     * Is called when the wizard should finish the process. The dialog is closed
     * after that call returns.
     */
    public void finish();

    /**
     * This method is called when the user canceled the operation. The wizard
     * can clean up now. The cancel process can not be stopped. After the call
     * the dialog is automatically closed.
     */
    public void cancel();

    /**
     * Returns the title for the wizard. The title is displayed in the title bar
     * of the dialog in which the wizard is loaded.
     * 
     * @return the title of the wizard
     */
    public String getTitle();

}
