package viptool.base.wizard;

import java.awt.Image;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.JComponent;

import viptool.base.IWorkbench;
import viptool.base.error.CannotOpenFileException;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.gui.component.FileNamePanel;
import viptool.base.gui.component.FolderTreePanel.FolderTreeSelection;
import viptool.base.gui.component.event.IFileNamePanelListener;
import viptool.base.gui.dialog.DialogConstants;
import viptool.base.io.IFileInterpreter;
import viptool.base.io.IFileInterpreterProvider;
import viptool.project.FileSystem;
import viptool.project.IAbstractFolder;
import viptool.project.IProject;

/**
 * The new file wizard allows the user to specify details about the creation of a new file.
 */
public class NewFileWizard extends SimpleWizard
{
    private boolean                  finished_;
    private boolean                  canFinishAfterFileCreation_;
    private boolean                  forceExtension_;
    private boolean                  openAfterCreation_;
    private String                   title_;
    private String                   description_;
    private String                   extension_;
    private String                   defaultFolder_;
    private NewFilePage              filePage_;
    private Image                    image_;
    private IWorkbench               workbench_;
    private IFileInterpreter         createdFile_;
    private IFileInterpreterProvider fileProvider_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>

    public class NewFilePage extends SimpleWizardPage implements
            IFileNamePanelListener
    {
        private boolean       isValid_;
        private FileNamePanel panel_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructor
        // <><><><><><><><><><><><><><><><><><><><><>
        public NewFilePage()
        {
            isValid_ = false;
            panel_ = new FileNamePanel(workbench_);
            panel_.addFileNamePanelListener(this);
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // supporting methods
        // <><><><><><><><><><><><><><><><><><><><><>

        public FolderTreeSelection getSelectedPath()
        {
            return panel_.getSelectedPath();
        }

        public String getEnteredName()
        {
            return panel_.getEnteredName();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implmented methods from SimpleWizardPage
        // <><><><><><><><><><><><><><><><><><><><><>
        public boolean canFinish()
        {
            return canFinishAfterFileCreation_;
        }

        public boolean isPageValid()
        {
            return isValid_;
        }

        public boolean canGoToPrevoius()
        {
            return false;
        }

        public JComponent getContent()
        {
            return panel_;
        }

        public Image getImage()
        {
            return image_;
        }

        public String getMainDescription()
        {
            return description_;
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implemented methods from IFileNamePanelListener
        // <><><><><><><><><><><><><><><><><><><><><>

        public void focusGainedOnName()
        {
            getWizardDialog().setDetailedDescription(
                    "Please enter the name " + "for the new file");
        }

        public void focusGainedOnPathTree()
        {
            getWizardDialog().setDetailedDescription(
                    "Please select the folder for " + "the new file");
        }

        public void focusLost()
        {
            getWizardDialog().setDetailedDescription(null);
        }

        public void nameNotValid()
        {
            getWizardDialog().setNotice(
                    "Please enter a name for the new file!",
                    DialogConstants.TYPE_ERROR);

            isValid_ = false;
            firePageInvalidEvent();
        }

        public void validated()
        {
            String fileName;
            IAbstractFolder folder;

            // check if file exists
            if (!panel_.getSelectedPath().isVirtualFolder())
            {
                folder = panel_.getSelectedPath().getFolder();

                if (folder != null)
                {
                    fileName = folder.getFullPath() + File.separator
                            + getEnteredName();
                }
                else
                {
                    fileName = panel_.getSelectedPath().getProject().getAbstractFile().getPathFile().getAbsolutePath()
                            + File.separator + getEnteredName();
                }

                if (!fileName.endsWith("." + extension_) && forceExtension_)
                {
                    fileName += "." + extension_;
                }

                if ((new File(fileName)).exists())
                {
                    getWizardDialog().setNotice("This file already exists!",
                            DialogConstants.TYPE_ERROR);
                    isValid_ = false;
                    firePageInvalidEvent();
                    return;
                }
            }

            isValid_ = true;
            getWizardDialog().removeNotice();
            firePageValidEvent();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructor
    // <><><><><><><><><><><><><><><><><><><><><>

    public NewFileWizard(IWorkbench workbench,
            IFileInterpreterProvider provider, String defaultFolder,
            String extension, String title, String description, Image image)
    {
        this(workbench, provider, defaultFolder, extension, title, description, image, true);
    }

    public NewFileWizard(IWorkbench workbench,
            IFileInterpreterProvider provider, String defaultFolder,
            String extension, String title, String description, Image image,
            boolean openAfterCreation)
    {
        title_ = title;
        description_ = description;
        extension_ = extension;
        defaultFolder_ = defaultFolder;
        image_ = image;
        createdFile_ = null;
        workbench_ = workbench;
        fileProvider_ = provider;
        finished_ = false;
        forceExtension_ = true;
        canFinishAfterFileCreation_ = true;
        openAfterCreation_ = openAfterCreation;

        filePage_ = new NewFilePage();
        addPage(filePage_);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // settings
    // <><><><><><><><><><><><><><><><><><><><><>

    public void setCanFinishAfterFileCreation(boolean canFinish)
    {
        canFinishAfterFileCreation_ = canFinish;
    }

    public void setOpenAfterCreation(boolean openAfterCreation)
    {
        openAfterCreation_ = openAfterCreation;
    }

    public void setForceExtension(boolean forceExtension)
    {
        forceExtension_ = forceExtension;
    }

    public IFileInterpreter getCreatedFile()
    {
        return createdFile_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implmented methods from IWizard
    // <><><><><><><><><><><><><><><><><><><><><>

    public void cancel()
    {
        if (createdFile_ != null)
        {
            createdFile_.getAbstractFile().getParentFolder().remove(
                    createdFile_.getAbstractFile());
            createdFile_.getAbstractFile().getParentFolder().update(false);
        }
    }

    public void finish()
    {
        String fileName;
        FolderTreeSelection selection = filePage_.getSelectedPath();
        IFileInterpreterProvider provider;
        IAbstractFolder folder = null;

        try
        {
            // create the folder first, if necessary
            if (selection.isVirtualFolder())
            {
                provider = workbench_.getAbstractFolderProvider();

                if (selection.getParent() == null)
                {
                    if (selection.getFolder() != null)
                    {
                        folder = FileSystem.getInstance().createAbstractFolder(
                                selection.getProject(),
                                selection.getFolder().getFullPath());
                    }
                }
                else
                {
                    folder = FileSystem.getInstance().createAbstractFolder(
                            selection.getProject(), selection.getParent(),
                            selection.getFolder().getFullPath());
                }

                provider.createNewFile(folder);
            }
            else
            {
                folder = selection.getFolder();
            }

            fileName = filePage_.getEnteredName();
            if (!fileName.endsWith("." + extension_) && forceExtension_)
            {
                fileName += "." + extension_;
            }

            if (folder == null)
            {
                // create the new petri net file
                fileName = selection.getProject().getAbstractFile().getFile().getParentFile().getAbsolutePath()
                        + File.separator + fileName;

                createdFile_ = fileProvider_.createNewFile(FileSystem.getInstance().createAbstractFile(
                        selection.getProject(), fileName));
            }
            else
            {
                // create the new petri net file
                fileName = folder.getFullPath() + File.separator + fileName;
                createdFile_ = fileProvider_.createNewFile(FileSystem.getInstance().createAbstractFile(
                        selection.getProject(), folder, fileName));
            }

            selection.getProject().updateFiles(true);

            // show it in the tree
            workbench_.selectFile(createdFile_.getAbstractFile());

            if (createdFile_ != null && openAfterCreation_)
            {
                workbench_.openFile(createdFile_);
            }
        }
        catch (CannotOpenFileException exception)
        {
            workbench_.showExceptionInDialog(exception);
        }
        catch (FileInterpreterProviderException exception)
        {
            workbench_.showExceptionInDialog(exception);
        }

        finished_ = true;
    }

    public String getTitle()
    {
        return title_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the WindowListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see java.awt.event.WindowListener */
    public void windowActivated(WindowEvent event)
    {
        IProject project;
        IAbstractFolder folder;

        if (getCurrentPage() == filePage_)
        {
            project = workbench_.getSelectedProject();
            folder = workbench_.getSelectedFolder();

            if (folder != null)
            {
                filePage_.panel_.initialize(folder, null);
            }
            else if (project != null)
            {
                filePage_.panel_.initialize(project, defaultFolder_);
            }
            else
            {
                filePage_.panel_.initialize(
                        workbench_.getCurrentWorkspace().getProjects()[0],
                        defaultFolder_);
            }
        }
    }

    /* @see java.awt.event.WindowListener */
    public void windowClosed(WindowEvent event)
    {
        // if finished was not called the user has cancelled the wizard
        if (!finished_)
        {
            cancel();
        }
    }
}
