package viptool.extension;

import sun.launcher.resources.launcher;
import viptool.base.IWorkbench;
import viptool.base.action.IActionHandler;
import viptool.base.error.XMLException;
import viptool.base.gui.menu.IMenuTemplate;
import viptool.logger.ILogger;
import viptool.util.resources.IResourceLoader;

/**
 * This class has to be extended by all VipTool Extenisons. It is used to
 * initialize the Extension.
 */

public abstract class Extension
{
    private IWorkbench      workbench_      = null;

    private IResourceLoader resourceLoader_ = null;

    /**
     * This method is called if the Viptool Extension is loaded from the
     * ExtensionManager. It should register the providers of this Extension and
     * its actions, menu and toolbar.
     * 
     * @param workbench
     *            workbench which loads the extension
     */
    public void initialize(IWorkbench workbench)
    {
        workbench_ = workbench;
        resourceLoader_ = new ExtensionResourceLoader(this, getResourceBase());
    }

    public void toolInitialized()
    {
    }

    /**
     * This method returns the workspace into which this instance of the
     * extension runs.
     * 
     * @return workbench as IWorkbench
     */
    public final IWorkbench getWorkbench()
    {
        return workbench_;
    }

    /**
     * This method returns the Logger which is used from the current workspace.
     * 
     * @return logger as ILogger
     */
    public ILogger getLogger()
    {
        return workbench_.getLogger();
    }

    /**
     * This method returns the ActionHandler which is used from the current
     * workspace.
     * 
     * @return actionhandler as IActionHandler
     */
    public IActionHandler getActionHandler()
    {
        return workbench_.getActionHandler();
    }

    /**
     * This method should be overwritten by all classes which extends Extension.
     * It is called if the extension should be stoped by user.
     */
    public void terminate()
    {
    }

    /**
     * This abstract method has to be overwritten by all implementations of
     * Extension. It should return the name of the extension.
     * 
     * @return name of the extension as String
     */
    public abstract String getName();

    /**
     * This method overwrites the toString-method from Object. It returns the
     * name of the extension which is given by getName as String for the
     * extension.
     * 
     */
    public String toString()
    {
        return getName();
    }

    /**
     * This method registers the menubar for this Extension.
     * 
     * @param template
     * 			{@link IMenuTemplate}
     */
    public void registerMenuBar(IMenuTemplate template)
    {
    	getWorkbench().getMenuBarManager().addTemplate(template);
    }

//    /**
//     * This method registers the contextmenu for this Extension. The contextmenu
//     * has to be given as XML-file with the format specified at
//     * http://viptool.von-klenze.de/schema/contextmenu.xsd.
//     * 
//     * @param relativePath
//     *            relative path to the xml-file
//     */
//    public void registerContextMenu(String relativePath)
//    {
//        try
//        {
//            getWorkbench().getContextMenuManager().addContextMenu(
//                    getResourceLoader().getResource(relativePath));
//        }
//        catch (XMLException exception)
//        {
//            getWorkbench().showExceptionInDialog(exception);
//        }
//    }

    /**
     * This method returns the ResourceLoader of the running workbench.
     * 
     * @return resourceloader as IResourceLoader
     */
    public IResourceLoader getResourceLoader()
    {
        return resourceLoader_;
    }

    /**
     * This abstract method has to be overwritten by all implementations. It
     * should return the relative path from the Extenison main file (the file
     * which extends Extension) to the directory which contains resource files.
     * 
     * @return relative path as String
     */
    protected abstract String getResourceBase();
}
