package viptool.extension;

import java.util.HashSet;
import java.util.Vector;
import java.util.jar.JarFile;

import org.w3c.dom.Document;

import viptool.base.error.VipToolException;
import viptool.base.error.VipToolExtensionException;
import viptool.util.ClassFile;

public interface IExtensionInfo {

	/**
	 * This method returns true if the Extension is packed into an JarFile.
	 * 
	 * @return
	 * 		true if JarFile
	 */
	public boolean isJarPlugin();
	
	/**
	 * This method returns true if the Extension is contained into an directory.
	 * 
	 * @return
	 * 		true if no JarFile
	 */
	public boolean isFilePlugin();
	
	/**
	 * This method returns an Vector of ClassFiles if the extension is contained into
	 * an directory, else it returns null.
	 * 
	 * @return
	 * 		Vector of ClassFiles
	 */	
	public Vector getClassFiles();
	
	/**
	 * This method returns the main class of the extension.
	 * 
	 * @return
	 * 		Class Object of the extension
	 */
	public Class getExtensionClass();
	
	/**
	 * This method removes the given ClassFile form the list of ClassFiles which
	 * belong to this extension
	 * 
	 * @param classfile
	 * 		ClassFile to remove
	 */
	public void removeClassFile(ClassFile classfile);
	
	/**
	 * This method adds the given ClassFile to the list of ClassFiles which belongs to
	 * this extension.
	 * 
	 * @param classfile
	 * 		ClassFile to add
	 */
	public void addClassFile(ClassFile classfile)  throws VipToolException ;
	
	/**
	 * This method returns the main class file as ClassFile from the extension if the 
	 * extension is contained into an directory else it returns null.
	 * 
	 * @return
	 * 		ClassFile of the Extension
	 */
	public ClassFile getExtenisonClassFile();
	
	/**
	 * This method returns the jarfile of the extension it the extension is packed into an
	 * jarfile else it returns null.
	 * 
	 * @return
	 * 		JarFile of the extension
	 */
	public JarFile getJarFile();
	
	/**
	 * This method adds the class files of the extension to the URL-list of the extenisonloader.
	 */
	public void registerClasses();
	
	/**
	 * This method returns the name of the extension as string.
	 * 
	 * @return
	 * 		name of the extension
	 */	
	public String getName();
	
	/**
	 * This method sets the name of the Extension to the given name.
	 * 
	 * @param name
	 * 		new name of the extension
	 */
	public void setName(String name);
	
	/**
	 * This method returns an description for this extension.
	 * 
	 * @return
	 * 		description as String
	 */
	public String getDescription();
	
	/**
	 * This method sets the description of the extension to the given string.
	 * 
	 * @param description
	 * 		new description
	 */
	public void setDescription(String description);
	
	/**
	 * This method returns an XML-DOC-Node which all information about this extension. This
	 * is used to save the extionsion into the list of known extenison.
	 * 
	 * @return
	 * 		Node for extenison
	 */
	public void getNode(Document doc) throws VipToolException;
	
	/**
	 * This method returns an Array of extensions which are needed to run this extension.
	 * 
	 * @return
	 * 		Array of Extensions
	 */
	public IExtensionInfo[] getRequiredExtensions();
	
	/**
	 * This method adds the given extension to the list of required extensions.
	 */
	public void addRequiredExtenison(IExtensionInfo extension);
	
	/**
	 * This method removes the given extension to the list of required extensions.
	 */
	public void removeRequiredExtenison(IExtensionInfo extension);
	
	/**
	 * This method returns the full qualified class name of the extension class.
	 * 
	 * @return
	 * 		name of the extension class
	 */
	public String getClassName();
	
	/**
	 * If autoload is true the extension will be automatically started
	 * when VipTool starts.
	 * 
	 * @param autoload
	 * 		new autoload state
	 */
	public void setAutoload(boolean autoload);
	
	/**
	 * returns the autoload state
	 * 
	 * @return
	 * 		autoload state
	 */
	public boolean isAutoload();
	
	/**
	 * This method returns a Vector which contains all fqcn of required extensions. 
	 */
	public HashSet<String> getRequiredNames();
	
	/**
	 * This method test if all required Extensions are avaiable.
	 */
	public void testForRequired(IExtensionManager extManager) throws VipToolExtensionException;
	
	/**
	 * This methode returns a String which is a valid manifest for this extensions.
	 * This means the String contains all requred extensions the main class the name
	 * and the description.
	 */
	public String getManifest();
	
	/**
	 * This method returns the current version of the Extension. In case of a jar-Extension
	 * the version info from the jar-manifest is used, in case of a file-extension last
	 * modification date is used as extension version.
	 * 
	 * @return
	 * 		version as long
	 */
	public long getVersion();
	
}
