package viptool.filesystem;

import java.util.Iterator;

import viptool.filesystem.error.FileExistsException;
import viptool.filesystem.error.InvalidContentException;
import viptool.filesystem.error.InvalidNameException;
import viptool.filesystem.error.NoSuchFileException;

/**
 * A file system is a hierarchical structure with folder an files. This is a
 * very primitive file system, that can contain folder and files but does not
 * support locking or something like this.<br>
 * You can use a file system session to get easier access to the file system (
 * the sessions supports working with relative paths).
 */
public interface IFileSystem
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // Folder manipulation
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates the given folder in the file system. If the folder already
     * exists, an exception is thrown. The folder name must me absolute (start
     * with /).
     * 
     * @param folder
     *            the folder to create
     * @throws FileExistsException -
     *             if the folder already exists
     * @throws InvalidNameException -
     *             if folder is no absolute name
     */
    public void createFolder(String folder) throws FileExistsException,
            InvalidNameException;

    /**
     * Removes the given folder from the file system. The folder name must be
     * absolute (start with /). If the folder does not exists, an exception is
     * thrown. The root folder / cannot be removed.
     * 
     * @param folder
     *            the folder to remove from the file system
     * @throws NoSuchFileException -
     *             if the folder does not exist
     * @throws InvalidNameException -
     *             if folder name is not absolute or the folder is / or
     *             otherwise invalid
     */
    public void removeFolder(String folder) throws NoSuchFileException,
            InvalidNameException;

    /**
     * Tests if the given folder exists in the file system. Returns true if the
     * given folder is an existing folder and false otherwise. The folder name
     * must be absolute.
     * 
     * @param folder
     *            the folder to search and test
     * @return true if folder is an existing folder, false otherwise
     */
    public boolean isFolder(String folder);

    /**
     * Returns the content of a folder as an array of strings. This does not
     * indicate you if the entries are folders or files. If you need this
     * purpose use the folder iterator instead.<br>
     * The folder name must be absolute.
     * 
     * @param folder
     *            the name of the folder
     * @return the content of the given folder as string array
     */
    public String[] getFolderContent(String folder) throws NoSuchFileException,
            InvalidNameException;

    /**
     * Returns an iterator for walking through the folders content. The iterator
     * will return instances of the class FileSystemEntry. The folder name must
     * be absolute.
     * 
     * @param folder
     *            the name of the folder
     * @return an iterator that iterates over the folders content and will
     *         return file system entries
     */
    public Iterator getFolderIterator(String folder)
            throws NoSuchFileException, InvalidNameException;

    // <><><><><><><><><><><><><><><><><><><><><>
    // File manipulation
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates the given file with the given content. The file name must be
     * absolute (start with /). The content cannot be null. If the file already
     * exists, an exception is thrown.
     * 
     * @param file
     *            the file to create
     * @param content
     *            the file content
     * @throws FileExistsException -
     *             if the file already exists
     * @throws InvalidContentException -
     *             if the file content is null
     * @throws NoSuchFileException -
     *             if the folder in which the file should be created does not
     *             exist
     * @throws InvalidNameException -
     *             if the file name is not absolute or not valid, e.g. ends with /
     */
    public void createFile(String file, Object content)
            throws FileExistsException, InvalidNameException,
            NoSuchFileException, InvalidContentException;

    /**
     * Removes the given file from the file system. The file name must be
     * absolute (start with /). If the file was not found, an exception is
     * thrown.
     * 
     * @param file
     *            the file to removed
     * @throws NoSuchFileException -
     *             if the file cannot be found
     * @throws InvalidNameException -
     *             if file is no absolute name
     */
    public void removeFile(String file) throws NoSuchFileException,
            InvalidNameException;

    /**
     * Returns the content of the file with the given name. The file name must
     * be absolute (start with /). If no such file exists, an exception is
     * thrown.
     * 
     * @param file
     *            the file from which to return the content
     * @return the content of the file
     * @throws NoSuchFileException -
     *             if the file was not found
     * @throws InvalidNameException -
     *             if file is no absolute name
     */
    public Object getFile(String file) throws NoSuchFileException,
            InvalidNameException;

    /**
     * Moves the given file to the given destination. The file is removed at the
     * old position. The file names (source and destination) must be absolute
     * (start with /).
     * 
     * @param source
     *            the source file
     * @param destination
     *            the destination file
     * @throws NoSuchFileException -
     *             if the source file does not exists
     * @throws FileExistsException -
     *             if the destination file already exists
     * @throws InvalidNameException -
     *             if source or destination is no absolute name
     */
    public void moveFile(String source, String destination)
            throws NoSuchFileException, FileExistsException,
            InvalidNameException;
}
