package viptool.filesystem;

import java.util.Iterator;

import viptool.filesystem.error.FileExistsException;
import viptool.filesystem.error.InvalidContentException;
import viptool.filesystem.error.InvalidNameException;
import viptool.filesystem.error.NoSuchFileException;

/**
 * A file system session is an easy interface for manipulating the file system.
 * Unlike the file system itself, the file system session can work with relative
 * paths and working folders.
 */
public interface IFileSystemSession
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // Folder operations
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Creates the folder with the given name. If the name does not start with
     * an / the folder is created under the current working folder. If the
     * folder already exists, an exception is thrown.
     * 
     * @param name
     *            the name for the new folder
     * @throws FileExistsException -
     *             if the folder already exists
     * @throws InvalidNameException -
     *             if the name is not a valid folder name
     */
    public void createFolder(String name) throws FileExistsException,
            InvalidNameException;

    /**
     * Removes the folder with the given name. If the name does not start with
     * an / the folder relative to the current working folder is removed. If the
     * folder does not exists, an exception is thrown.
     * 
     * @param name
     *            the name of the folder to be removed
     * @throws NoSuchFileException -
     *             if the folder does not exists
     * @throws InvalidNameException -
     *             if folder is /
     */
    public void removeFolder(String name) throws NoSuchFileException,
            InvalidNameException;

    /**
     * Tests if the given folder exists in the file system. Returns true if the
     * given folder is an existing folder and false otherwise.
     * 
     * @param folder
     *            the folder to search and test
     * @return true if folder is an existing folder, false otherwise
     */
    public boolean isFolder(String folder);

    /**
     * Changes the current working folder to the given folder. If the folder
     * does not exist, an exception is thrown.
     * 
     * @param path
     *            the path for the new working folder
     * @throws NoSuchFileException -
     *             if the given folder does not exist
     * @throws InvalidNameException -
     *             if the given file name is not valid
     */
    public void changeFolder(String path) throws NoSuchFileException,
            InvalidNameException;

    /**
     * Returns the current working folder.
     * 
     * @return the current working folder
     */
    public String getCurrentFolder();

    /**
     * Returns the content of a folder as an array of strings. This does not
     * indicate you if the entries are folders or files. If you need this
     * purpose use the folder iterator instead.
     * 
     * @param folder
     *            the name of the folder
     * @return the content of the given folder as string array
     */
    public String[] getFolderContent(String folder) throws NoSuchFileException,
            InvalidNameException;

    /**
     * Returns an iterator for walking through the folders content. The iterator
     * will return instances of the class FileSystemEntry.
     * 
     * @param folder
     *            the name of the folder
     * @return an iterator that iterates over the folders content and will
     *         return file system entries
     */
    public Iterator getFolderIterator(String folder)
            throws NoSuchFileException, InvalidNameException;

    // <><><><><><><><><><><><><><><><><><><><><>
    // File operations
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Creates a file with the given name and the given content. The content
     * cannot be null. If the file name does not start with / the file is
     * created under the current working folder. If the name is already in use,
     * an exception is thrown.
     * 
     * @param name
     *            the name for the new file
     * @param content
     *            the file content, can be null
     * @throws FileExistsException -
     *             if the file name is already in use
     * @throws InvalidContentException -
     *             if the file content is null
     * @throws NoSuchFolderException -
     *             if the folder does not exist
     * @throws InvalidNameException -
     *             if the file name is not valid, e.g. ends with /
     */
    public void createFile(String name, Object content)
            throws FileExistsException, InvalidContentException,
            NoSuchFileException, InvalidNameException;

    /**
     * Removes the file with the name relative to the current working folder, if
     * the file does not start with /. If the file does not exist, an exception
     * is thrown.
     * 
     * @param name
     *            the file name that has to be removed
     * @throws NoSuchFileException -
     *             if the given file was not found
     * @throws InvalidNameException -
     *             if the given file name is not valid
     */
    public void removeFile(String name) throws NoSuchFileException,
            InvalidNameException;

    /**
     * Returns the content of the file with the given name. If no such file
     * exists, an exception is thrown.
     * 
     * @param name
     *            the name of the file
     * @return the content of the file
     * @throws NoSuchFileException -
     *             if the file was not found
     * @throws InvalidNameException -
     *             if the given file name is not valid
     */
    public Object getFile(String name) throws NoSuchFileException,
            InvalidNameException;

    /**
     * Moves the given file to the given destination. The file is removed at the
     * old position.
     * 
     * @param source
     *            the source file
     * @param destination
     *            the destination file
     * @throws NoSuchFileException -
     *             if the source file does not exists
     * @throws FileExistsException -
     *             if the destination file already exists
     * @throws InvalidNameException -
     *             if the given file name is not valid
     */
    public void moveFile(String source, String destination)
            throws NoSuchFileException, FileExistsException,
            InvalidNameException;
}
