package viptool.impl;

import java.awt.Color;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URI;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.KeyStroke;
import javax.swing.UIManager;

import viptool.Constants;
import viptool.base.IWorkbench;
import viptool.base.action.ContainerAction;
import viptool.base.action.ExtendedAction;
import viptool.base.action.SeparatorAction;
import viptool.base.action.WindowAction;
import viptool.base.editor.IEditor;
import viptool.base.error.FileInterpreterException;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.error.VipToolException;

import viptool.base.gui.menu.IMenuTemplate;

import viptool.base.gui.toolbar.IToolBarTemplate;

import viptool.base.io.IFileInterpreterManager;
import viptool.extension.Extension;
import viptool.extension.IExtensionInfo;
import viptool.impl.base.MainFrame;
import viptool.impl.base.Workbench;
import viptool.impl.base.action.ActionHandler;
import viptool.impl.base.action.extensions.ExtensionManagerAction;
import viptool.impl.base.action.extensions.LoadExtensionAction;
import viptool.impl.base.action.file.CloseChildWindowAction;
import viptool.impl.base.action.file.ExitViptoolAction;
import viptool.impl.base.action.file.LoadProjectAction;
import viptool.impl.base.action.file.LoadWorkspaceAction;
import viptool.impl.base.action.file.NewDialogAction;
import viptool.impl.base.action.file.SaveChildWindowAction;
import viptool.impl.base.action.help.AboutAction;
import viptool.impl.base.editor.EditorManager;
import viptool.impl.base.gui.SplashScreen;
import viptool.impl.base.gui.menu.ExtendedMenuBar;
import viptool.impl.base.gui.menu.MenuBarManager;
import viptool.impl.base.gui.menu.MenuTemplate;
import viptool.impl.base.gui.toolbar.ToolBarManager;
import viptool.impl.base.gui.toolbar.ToolBarTemplate;
import viptool.impl.base.io.FileInterpreterManager;
import viptool.impl.extension.ExtensionManager;
import viptool.impl.logger.Logger;
import viptool.impl.preferences.PreferenceStore;
import viptool.impl.project.AbstractFolderProvider;
import viptool.impl.project.ProjectGuiLinker;
import viptool.impl.project.io.ProjectFileProvider;
import viptool.impl.project.io.WorkspaceFileProvider;
import viptool.impl.text.editor.TextEditorProvider;
import viptool.impl.text.io.TextFileProvider;
import viptool.impl.util.resources.ResourceConstants;
import viptool.impl.util.resources.ResourceLoader;
import viptool.project.FileSystem;
import viptool.project.IAbstractFile;
import viptool.project.IWorkspace;
import viptool.util.CommandLineParser;
import viptool.util.GeneralUtils;

/**
 * Main class of the application. Implements the entry point and creates the
 * main frame. Hosts all managers. Is implemented with the singleton pattern.
 */
public class Viptool
{
    // the one and only instance to the viptool
    private static Viptool instance_  = null;

    // the workspace that is currently loaded
    private IWorkbench     workbench_ = null;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new instance of the viptool object. The viptool object is the
     * main class. it hosts the main frame, and all managers.
     */
    private Viptool()
    {
        instance_ = this;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Entry Point
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Starts the program. A mainframe is created and displayed. The program
     * doesn't take any arguments at the moment.
     * 
     * @param args
     *            program arguments. Ignored at the moment
     */
    public static void main(String[] args)
    {
        String tmpDir = System.getProperty("java.io.tmpdir") + File.separator;

        SplashScreen.getInstance();

        // first create the log file
        Logger.getInstance().setFile(new File(tmpDir + "log.txt"));

        Logger.getInstance().log("------------------------------");
        Logger.getInstance().log("|  VipTool session started.  |");
        Logger.getInstance().log("------------------------------");
        Logger.getInstance().log(
                "Version: " + ResourceConstants.VIPTOOL_VERSION + " / "
                        + System.getProperty("os.name"));

        // set look and feel
        try
        {
            // use the systems default look and feel
            if (System.getProperty("os.name").startsWith("Windows"))
            {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            }
        }
        catch (Exception e)
        {
            // just print the stack trace. We don't have to exit the program,
            // the user is just getting the default look
            e.printStackTrace();
        }

        // use dynamic resizing in dialogs. looks much cooler!
        Toolkit.getDefaultToolkit().setDynamicLayout(true);

        UIManager.put("TabbedPane.tabAreaBackground", Color.RED);

        // create a viptool class and initialize
        getInstance().initialize(args);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the only instance of the viptool class. If there is no instance
     * yet, a new instance will be created.
     * 
     * @return the instance of the viptool
     */
    public synchronized static Viptool getInstance()
    {
        if (instance_ == null)
        {
            instance_ = new Viptool();
        }

        return instance_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Initialization
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Initialize the viptool. Creates all managers and the mainframe. Registers
     * the default petrinet nodes place and transition.
     */
    public void initialize(String[] args)
    {
        String path;
        Extension extension;
        IAbstractFile file;
        IWorkspace workspace = null;
        Iterator<IExtensionInfo> it;
        final MainFrame mainFrame;
        CommandLineParser commandLine = new CommandLineParser(args);

        try
        {
            if (commandLine.isOptionSet("--preferenceFile"))
            {
                ((PreferenceStore) PreferenceStore.getInstance()).setDefaultPath(commandLine.getArgument("--preferenceFile"));
            }

            // load the preferences
            PreferenceStore.getInstance().loadPreferences(null);

            // create the workbench for this session
            workbench_ = Workbench.getInstance();
            ((Workbench) workbench_).setCommandLineOptions(args);

            // standard file interpreters
            getFileInterpreterManager().addFileInterpreterProvider(
                    new TextFileProvider());

            getFileInterpreterManager().addFileInterpreterProvider(
                    new AbstractFolderProvider());

            getFileInterpreterManager().addFileInterpreterProvider(
                    new WorkspaceFileProvider());

            getFileInterpreterManager().addFileInterpreterProvider(
                    new ProjectFileProvider());

            // register the standard text editors
            EditorManager.getInstance().addEditorProvider(
                    new TextEditorProvider());

            // load the basic actions and menus
            this.addDefaultActions();
            
            // menu bars
            this.addDefaultMenuTemplate();
            this.addDefaultMenuOpenedTemplate();       

            // toolbars
            this.addDefaultToolbarTemplate();
            this.addDefaultOpenedToolbarTemplate();

            // create our main frame
            ProjectGuiLinker.getInstance();
            mainFrame = MainFrame.getInstance();

            // get the close event to autosave the workspace
            mainFrame.addWindowListener(new WindowAdapter()
            {
                public void windowClosing(WindowEvent event)
                {
                    getInstance().exit();
                }
            });

            // creates the managers.
            // initialize all viptool stuff first, so the extension can access
            // this stuff
            if (commandLine.isOptionSet("--extensionFile"))
            {
                getExtensionManager().setExtensionFile(
                        commandLine.getArgument("--extensionFile"));
            }
            getExtensionManager().loadAutoLoadExtensions();

            if (commandLine.getArgument("--loglevel") != null)
            {
                Logger.getInstance().setLogLevel(
                        Integer.valueOf(commandLine.getArgument("--loglevel")));
            }

            // load a workspace if given
            try
            {
                if (commandLine.isOptionSet("--workspace"))
                {
                    file = FileSystem.getInstance().createAbstractFile(
                            commandLine.getArgument("--workspace"));
                }
                else
                {
                    path = PreferenceStore.getInstance().getPreferenceChildNode(
                            ResourceConstants.PREFERENCE_NODE_WORKSPACE).readString(
                            ResourceConstants.PREFERENCE_LAST_WORKSPACE, null);

                    if (path == null)
                    {
                        path = Workbench.getInstance().getPropertyPath()
                                + "default" + File.separator;
                        (new File(path)).mkdirs();
                        path += ResourceConstants.WORKSPACE_DEFAULT_NAME;
                    }

                    file = FileSystem.getInstance().createAbstractFile(path);
                    if (!file.getFile().exists())
                    {
                        FileInterpreterManager.getInstance().getFileInterpreterProvider(
                                WorkspaceFileProvider.class).createNewFile(file);
                    }
                }

                workspace = (IWorkspace) FileInterpreterManager.getInstance().getDefaultFileInterpreterProvider(
                        file).createFromFile(file);
            }
            catch (FileInterpreterProviderException exception)
            {
                Workbench.getInstance().showExceptionInDialog(exception);
            }

            Logger.getInstance().info("VipTool initialized!");

            // initalize mainframe
            MainFrame.getInstance().initialize();

            if (workspace != null)
            {
                workbench_.setCurrentWorkspace(workspace);
            }

            // make it visible
            mainFrame.setVisible(true);
            SplashScreen.getInstance().dispose();

            // notify the extensions
            it = ExtensionManager.getInstance().getOrderedExtensions().iterator();
            while (it.hasNext())
            {
                extension = ExtensionManager.getInstance().getRunningExtension(
                        it.next());
                if (extension != null)
                {
                    extension.toolInitialized();
                }
            }
        }
        catch (Exception exception)
        {
            SplashScreen.getInstance().dispose();

            IWorkbench workbench = Workbench.getInstance();
            if (workbench != null)
            {
                workbench.showExceptionInDialog(new VipToolException(
                        exception.getMessage(), exception));
            }
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // access methods to the managers and
    // mainframe
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the mainframe that is hosted by the viptool.
     * 
     * @return a reference to the main frame
     */
    public MainFrame getMainFrame()
    {
        return MainFrame.getInstance();
    }

    /**
     * Returns the extension manager for the viptool. It manages all extensions
     * loaded into the viptool.
     * 
     * @return a reference to the extension manager
     */
    public ExtensionManager getExtensionManager()
    {
        return ExtensionManager.getInstance();
    }

    /**
     * Returns the file type manager for the viptool. The file type manager is
     * responsible for detecting the right file type respecting the content of a
     * file.
     * 
     * @return a reference to the file type manager
     */
    public IFileInterpreterManager getFileInterpreterManager()
    {
        return FileInterpreterManager.getInstance();
    }

    /**
     * Returns the workbench. The workbench is the main interface for all
     * modules in viptool to communicate with the viptool itself.
     * 
     * @return the workbench of this session
     */
    public IWorkbench getWorkbench()
    {
        return workbench_;
    }

    public void exit()
    {
        IEditor[] editors;

        try
        {
            if (Workbench.getInstance().getCurrentWorkspace() != null)
            {
                Workbench.getInstance().getCurrentWorkspace().save();
            }
        }
        catch (FileInterpreterException exception)
        {
            Workbench.getInstance().showExceptionInDialog(exception);

            // TODO: Add "Close anyway"-dialog
        }

        // save dirty editors
        editors = Workbench.getInstance().getOpenEditors();
        for (int i = 0; i < editors.length; i++)
        {
            Workbench.getInstance().selectWindow(editors[i]);
            editors[i].canClose(true);
        }

        MainFrame.getInstance().setVisible(false);
        MainFrame.getInstance().dispose();

        // save the preferences (must be the last operation)
        PreferenceStore.getInstance().savePreferences();

        System.exit(0);
    }
    
    /**
     * Adds the default actions, that are necessary for each extension.
     */
    private void addDefaultActions() {
    	URI resource;
    	SeparatorAction separatorAction;
    	ContainerAction fileAction;
    	ContainerAction loadAction;
    	ContainerAction recentWorkspaceAction;
    	ContainerAction extensionsAction;
    	ContainerAction helpAction;
    	NewDialogAction newDialogAction;
    	LoadWorkspaceAction loadWorkspaceAction;
    	LoadProjectAction loadProjectAction;
    	SaveChildWindowAction saveChildWindowAction;
    	CloseChildWindowAction closeChildWindowAction;
    	ExitViptoolAction exitViptoolAction;
    	ExtensionManagerAction extensionManagerAction;
    	LoadExtensionAction loadExtensionAction;
    	AboutAction aboutAction;
    	
    	separatorAction = new SeparatorAction();
    	separatorAction.putValue("Id", "separator");
    	ActionHandler.getInstance().addAction(separatorAction);
    	
    	fileAction = new ContainerAction();
    	fileAction.putValue("Id", "file");
    	fileAction.putValue("Name", "File");
    	fileAction.putValue("MnemonicKey", new Integer('F'));
    	ActionHandler.getInstance().addAction(fileAction);
    	
    	loadAction = new ContainerAction();
    	loadAction.putValue("Id", "file.load");
    	loadAction.putValue("Name", "Load...");
    	loadAction.putValue("MnemonicKey", new Integer('L'));
    	ActionHandler.getInstance().addAction(loadAction);
    	
    	recentWorkspaceAction = new ContainerAction();
    	recentWorkspaceAction.putValue("Id", "file.recentworkspace");
    	recentWorkspaceAction.putValue("Name", "Recent workspaces...");
    	recentWorkspaceAction.putValue("MnemonicKey", new Integer('R'));
    	ActionHandler.getInstance().addAction(recentWorkspaceAction);
    	
    	extensionsAction = new ContainerAction();
    	extensionsAction.putValue("Id", "extensions");
    	extensionsAction.putValue("Name", "Extensions");
    	extensionsAction.putValue("MnemonicKey", new Integer('E'));
    	ActionHandler.getInstance().addAction(extensionsAction);
    	
    	helpAction = new ContainerAction();
    	helpAction.putValue("Id", "help");
    	helpAction.putValue("Name", "Help");
    	helpAction.putValue("MnemonicKey", new Integer('H'));
    	ActionHandler.getInstance().addAction(helpAction); 	
    	
        newDialogAction = new NewDialogAction();
    	newDialogAction.putValue("Id", "file.new");
    	newDialogAction.putValue("Name", "New...");
    	newDialogAction.putValue("AcceleratorKey", KeyStroke.getKeyStroke("ctrl N"));
    	newDialogAction.putValue("MnemonicKey", new Integer('N'));
    	newDialogAction.putValue("ShortDescription", "Opens the new dialog");
    	newDialogAction.putValue("LongDescription", "Opens the new dialog to create new folders, files, projects, etc.");
    	resource = ResourceLoader.getInstance().getResource(
                GeneralUtils.convertIdToPath("viptool.util.resources.icon.newFile16.gif"));
		try {
			newDialogAction.putValue("SmallIcon", new ImageIcon(
			        resource.toURL()));
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		ActionHandler.getInstance().addAction(newDialogAction); 	
		
		loadWorkspaceAction = new LoadWorkspaceAction();
		loadWorkspaceAction.putValue("Id", "file.load.workspace");
		loadWorkspaceAction.putValue("Name", "Workspace");
		loadWorkspaceAction.putValue("MnemonicKey", new Integer('W'));
		loadWorkspaceAction.putValue("ShortDescription", "loads an existing workspace");
		loadWorkspaceAction.putValue("LongDescription", "loads an existing workspace and closes the current one");
    	resource = ResourceLoader.getInstance().getResource(
                GeneralUtils.convertIdToPath("viptool.util.resources.icon.viptool16.gif"));
		try {
			loadWorkspaceAction.putValue("SmallIcon", new ImageIcon(
			        resource.toURL()));
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		ActionHandler.getInstance().addAction(loadWorkspaceAction); 	
		
		loadProjectAction = new LoadProjectAction();
		loadProjectAction.putValue("Id", "file.load.project");
		loadProjectAction.putValue("Name", "Project");
		loadProjectAction.putValue("MnemonicKey", new Integer('P'));
		loadProjectAction.putValue("ShortDescription", "loads an existing project");
		loadProjectAction.putValue("LongDescription", "loads an existing project into the current workspace");
    	resource = ResourceLoader.getInstance().getResource(
                GeneralUtils.convertIdToPath("viptool.util.resources.icon.project16.gif"));
		try {
			loadProjectAction.putValue("SmallIcon", new ImageIcon(
			        resource.toURL()));
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		ActionHandler.getInstance().addAction(loadProjectAction); 	
		
		saveChildWindowAction = new SaveChildWindowAction();
		saveChildWindowAction.putValue("Id", "file.save");
		saveChildWindowAction.putValue("Name", "Save");
		saveChildWindowAction.putValue("MnemonicKey", new Integer('S'));
		saveChildWindowAction.putValue("AcceleratorKey", KeyStroke.getKeyStroke("ctrl S"));
		saveChildWindowAction.putValue("ShortDescription", "saves the current file");
		saveChildWindowAction.putValue("LongDescription", "saves the file that is currently active (in an editor)");
    	resource = ResourceLoader.getInstance().getResource(
                GeneralUtils.convertIdToPath("viptool.util.resources.icon.saveFile16.gif"));
		try {
			saveChildWindowAction.putValue("SmallIcon", new ImageIcon(
			        resource.toURL()));
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		ActionHandler.getInstance().addAction(saveChildWindowAction); 	
		
		closeChildWindowAction = new CloseChildWindowAction();
		closeChildWindowAction.putValue("Id", "file.close");
		closeChildWindowAction.putValue("Name", "Close");
		closeChildWindowAction.putValue("MnemonicKey", new Integer('C'));
		closeChildWindowAction.putValue("AcceleratorKey", KeyStroke.getKeyStroke("ctrl W"));
		closeChildWindowAction.putValue("ShortDescription", "Closes the current window");
		closeChildWindowAction.putValue("LongDescription", "Closes the current editor window");
    	resource = ResourceLoader.getInstance().getResource(
                GeneralUtils.convertIdToPath("viptool.util.resources.icon.closeFile16.gif"));
		try {
			closeChildWindowAction.putValue("SmallIcon", new ImageIcon(
			        resource.toURL()));
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		ActionHandler.getInstance().addAction(closeChildWindowAction); 	
		
		exitViptoolAction = new ExitViptoolAction();
		exitViptoolAction.putValue("Id", "app.exit");
		exitViptoolAction.putValue("Name", "Exit");
		exitViptoolAction.putValue("MnemonicKey", new Integer('x'));
		exitViptoolAction.putValue("ShortDescription", "Exit the viptool");
		exitViptoolAction.putValue("LongDescription", "Exit the viptool");
		ActionHandler.getInstance().addAction(exitViptoolAction); 	
		
		extensionManagerAction = new ExtensionManagerAction();
		extensionManagerAction.putValue("Id", "extension.manager");
		extensionManagerAction.putValue("Name", "Extension-Manager");
		extensionManagerAction.putValue("MnemonicKey", new Integer('m'));
		extensionManagerAction.putValue("ShortDescription", "Opens the interface of the extension-manager.");
		extensionManagerAction.putValue("LongDescription", "Opens the interface of the extension-manager.");
		ActionHandler.getInstance().addAction(extensionManagerAction); 	
		
		loadExtensionAction = new LoadExtensionAction();
		loadExtensionAction.putValue("Id", "extension.load");
		loadExtensionAction.putValue("Name", "Load Extension");
		loadExtensionAction.putValue("MnemonicKey", new Integer('l'));
		loadExtensionAction.putValue("ShortDescription", "Loads an extension.");
		loadExtensionAction.putValue("LongDescription", "Loads an extension form an folder or a jar-file.");
		ActionHandler.getInstance().addAction(loadExtensionAction); 	
		
		aboutAction = new AboutAction();
		aboutAction.putValue("Id", "help.about");
		aboutAction.putValue("Name", "About...");
		aboutAction.putValue("MnemonicKey", new Integer('A'));
		aboutAction.putValue("ShortDescription", "Shows information about the viptool");
		aboutAction.putValue("LongDescription", "Shows a dialog with detailed information about the viptool.");
		ActionHandler.getInstance().addAction(aboutAction); 
 
    }

    /**
     * Adds a toolbar template for situations where no editor is opened.
     */
    private void addDefaultMenuTemplate(){
    	
    	IMenuTemplate template;
    	
    	template = new MenuTemplate(ExtendedMenuBar.class, Constants.ID_MENU_DEFAULT);
    	template.addMenuItem("file", "", "", 1);
    	template.addMenuItem("file.new", "file", "", 1);
    	template.addMenuItem("file.load", "file", "", 1);
    	template.addMenuItem("file.load.workspace", "file.load", "", 1);
    	template.addMenuItem("file.load.project", "file.load", "file.load.workspace", 1);
    	template.addMenuItem("file.load", "file", "file.load", 1);
    	template.addMenuItem("separator", "file", "file.load", 1);
    	template.addMenuItem("app.exit", "file", "file.load",2);
    	
    	template.addMenuItem("extensions", "", "", 1);
    	template.addMenuItem("extension.manager", "extensions", "", 1);
    	template.addMenuItem("extension.load", "extensions", "extension.manager", 1);
    	
    	template.addMenuItem("help", "", "", 1);
    	template.addMenuItem("help.about", "help", "", 1);
    	
    	MenuBarManager.getInstance().addTemplate(template);
       
    }
    
    /**
     * Adds a toolbar template for situations where at least one editor is opened.
     */
    private void addDefaultMenuOpenedTemplate(){
    	
    	IMenuTemplate template;
    	
    	template = new MenuTemplate(ExtendedMenuBar.class, Constants.ID_MENU_DEFAULT_OPENED);
    	template.setParent(MenuBarManager.getInstance().getTemplate(Constants.ID_MENU_DEFAULT));
    	
    	template.addMenuItem("file", "", "", 1);
    	template.addMenuItem("file.save", "file", "file.load", 1);
    	template.addMenuItem("file.close", "file", "file.save", 1);
    	
    	MenuBarManager.getInstance().addTemplate(template);
       
    }
    
    /**
     * Adds a toolbar template that is activated when no editor is opened.
     */
    private void addDefaultToolbarTemplate() {
    	IToolBarTemplate toolBarTemplate;
    	
    	toolBarTemplate = new ToolBarTemplate(Constants.ID_TOOLBAR_DEFAULT);
    	toolBarTemplate.addToolBarItem("file.new", "", "", 0);
    	toolBarTemplate.addToolBarItem("file.load.workspace", "", "", 1);
    	toolBarTemplate.addToolBarItem("file.load.project", "", "", 2);
    	toolBarTemplate.addToolBarItem("file.load.project", "", "", 3);
    	
    	ToolBarManager.getInstance().addTemplate(toolBarTemplate);
    }
    
    /**
     * Adds a toolbar template that is activated when an editor is opened. Extends the defaultm toolbar template.
     */
    private void addDefaultOpenedToolbarTemplate() {
    	IToolBarTemplate toolBarTemplate;
    	
    	toolBarTemplate = new ToolBarTemplate(Constants.ID_TOOLBAR_DEFAULT_OPENED);
    	toolBarTemplate.setParent(ToolBarManager.getInstance().getTemplate(Constants.ID_TOOLBAR_DEFAULT));
    	toolBarTemplate.addToolBarItem("file.load.project", "", "file.load.project", 1);
    	toolBarTemplate.addToolBarItem("file.save", "", "file.load.project", 2);
    	toolBarTemplate.addToolBarItem("file.close", "", "file.load.project", 3);
    	
    	ToolBarManager.getInstance().addTemplate(toolBarTemplate);
    }
}
