package viptool.impl.base;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.Toolkit;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JToolBar;

import org.w3c.dom.Element;

import static viptool.Constants.*;
import viptool.base.editor.IEditor;
import viptool.base.gui.IView;
import viptool.base.gui.IWindow;
import viptool.impl.base.extendedPane.ExtendedPane;
import viptool.impl.base.extendedPane.event.ActiveWindowChangeEvent;
import viptool.impl.base.extendedPane.event.IExtendedPaneListener;
import viptool.impl.base.extendedPane.event.WindowCloseEvent;
import viptool.impl.base.gui.menu.MenuBarManager;
import viptool.impl.base.gui.toolbar.ToolBarManager;
import viptool.impl.logger.LoggerConsole;
import viptool.impl.project.gui.ProjectTreePane;
import viptool.impl.util.resources.ResourceConstants;
import viptool.impl.util.resources.ResourceLoader;

/**
 * Main frame of the tool. Manages toolbar and menu and hosts the extended pane
 * that manages the child windows. <br>
 * Designed in the singleton pattern to allow only one main frame per instance.
 */
public class MainFrame extends JFrame implements IExtendedPaneListener, IWindow
{
    // the only instance
    private static MainFrame instance_       = null;

    // the extended pane that manages the child windows
    private ExtendedPane     extendedPane_   = null;

    // the current toolbar
    private JToolBar         toolbar_        = null;

    // our layout manager
    private BorderLayout     layoutManager_  = null;

    // in case of no open window, this toolbar will be displayed
    private JToolBar         defaultToolBar_ = null;

    // in case of no open window, this menubar will be displayed
    private JMenuBar         defaultMenuBar_ = null;

    // counter for counting the petriNetWindows
    private int              netCounter_     = 0;

    // the console for our log events
    private LoggerConsole    logConsole_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new main frame. There can exist only one main frame per program
     * instance. The main frame manages menu and toolbar and contains an
     * extended pane for managaging child windows
     */
    private MainFrame()
    {
        super(ResourceConstants.MAINFRAME_TITLE);

        Rectangle rc;

        instance_ = this;

        // give us a chance to save the workspace
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        // set border layout to fill up the whole window with the extended pane
        layoutManager_ = new BorderLayout();
        getContentPane().setLayout(layoutManager_);

        // create an extended pane which manage our window layout
        extendedPane_ = new ExtendedPane(ProjectTreePane.GROUP_NAME,
                ProjectTreePane.class, false);

        // register by the extendedpane so we will get active window changed
        // messages
        extendedPane_.addExtendedPaneListener(this);

        // create the editor group
        extendedPane_.createGroup(IEditor.GROUP_NAME, IEditor.class, false);
        extendedPane_.openGroup(IEditor.GROUP_NAME,
                ExtendedPane.POSITION_FRAME_RIGHT);

        // create and open a window for the group of views
        extendedPane_.createGroup(IView.GROUP_NAME, IView.class, true);
        extendedPane_.openGroup(IView.GROUP_NAME,
                ExtendedPane.POSITION_FRAME_BOTTOM);

        // create the default tool- and menubar for the case, that no window is
        // open
        defaultToolBar_ = ToolBarManager.getInstance().createToolBar(
                ID_TOOLBAR_DEFAULT, this);
        defaultMenuBar_ = MenuBarManager.getInstance().createMenuBar(
                ID_MENU_DEFAULT, this);

        // set these bars so that actions can be performed without having a
        // window
        setToolBar(defaultToolBar_);
        setMenu(defaultMenuBar_);

        // add the extended pane
        getContentPane().add(extendedPane_, BorderLayout.CENTER);

        // set our icon
        setIconImage(ResourceLoader.getInstance().getImage(
                ResourceConstants.ICON_VIPTOOL));

        // default size and location
        rc = new Rectangle();
        rc.x = (Toolkit.getDefaultToolkit().getScreenSize().width - 1000) / 2;
        rc.y = (Toolkit.getDefaultToolkit().getScreenSize().height - 535) / 2;
        rc.width = 900;
        rc.height = 535;
        setBounds(rc);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    public synchronized static MainFrame getInstance()
    {
        if (instance_ == null)
        {
            instance_ = new MainFrame();
        }

        return instance_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>

    public void initialize()
    {
        // add the project pane
        extendedPane_.addChildWindow(ProjectTreePane.getInstance());

        // add the console with the log file
        logConsole_ = new LoggerConsole();
        Workbench.getInstance().openView(logConsole_);

        extendedPane_.setDividerLocation(logConsole_,
                ResourceConstants.CONSOLE_DIVIDER_LOCATION);

        extendedPane_.setDividerLocation(ProjectTreePane.getInstance(),
                ResourceConstants.PROJECTTREE_DIVIDER_LOCATION);

        extendedPane_.addExtendedPaneListener((Workbench) Workbench.getInstance());
    }

    /**
     * returns the number of the yet created petrinets
     * 
     * @return an integer representing the number of the yet created petrinets
     */
    public int getNetCounter()
    {
        return netCounter_;
    }

    /**
     * method has to be called BEFORE a new petrinet is created. It increases
     * the number of nets.
     */
    public void incrementNetCounter()
    {
        netCounter_++;
    }

    /**
     * Gets the exteneded pane that manages the child windows.
     * 
     * @return the extended pane
     */
    public ExtendedPane getExtendedPane()
    {
        return extendedPane_;
    }

    public JFrame getWindow()
    {
        return this;
    }

    /**
     * Gets the default ToolBar of the frame. Method can be used for getting the
     * toolbar so that the buttons can be received outside this class, e.g. to
     * set some properties of the buttons.
     * 
     * @return the default toolbar of the frame
     */
    public JToolBar getDefaultToolBar()
    {
        return defaultToolBar_;
    }

    /**
     * Called by the method activeWindowChanged, so that the menubar belonging
     * to the new active window will be displayed. If there isn't any window
     * open, a default menubar will be displayed.
     * 
     * @param menu
     *            the menubar belonging to the new active window
     */
    private void setMenu(JMenuBar menu)
    {
        if (getJMenuBar() == menu)
            return;

        // set the default menubar if no one belongs to the active window,
        // especially if no window is open
        if (menu == null)
            menu = defaultMenuBar_;

        // set the new menu and update the main frame
        setJMenuBar(menu);

        menu.invalidate();
        validate();
    }

    /**
     * Called by the method activeWindowChanged, so that the toolbar belonging
     * to the new active window will be displayed. If there isn't any window
     * open, a default toolbar will be displayed.
     * 
     * @param toolbar
     *            the toolbar belonging to the new active window
     */
    private void setToolBar(JToolBar toolbar)
    {
        // standard toolbar position and orientation
        int orientation = JToolBar.HORIZONTAL;
        Object constraints = BorderLayout.NORTH;

        if (toolbar_ == toolbar)
            return;

        // remove the old one
        if (toolbar_ != null)
        {
            // store the current toolbar position to give the new toolbar the
            // same position (not used yet)
            /*
             * constraints = layoutManager_.getConstraints(toolbar_);
             * orientation = toolbar_.getOrientation();
             */
            remove(toolbar_);
        }

        // set the default toolbar if no one belongs to the active window,
        // especially if no window is open
        if ((toolbar_ = toolbar) == null)
            toolbar_ = defaultToolBar_;

        // disallow floating at the moment, because we cannt restore the
        // position of an undocked toolbar yet
        toolbar_.setFloatable(false);
        // add the new toolbar at the same position the old one was
        toolbar_.setOrientation(orientation);
        getContentPane().add(toolbar_, constraints);

        toolbar_.invalidate();
        validate();

        repaint();
    }

    /**
     * With this method it is possible to align all dividers of the different
     * splitpanes. So you can avoid, that windows are too huge or too small.
     */
    public void layoutMainFrame(Element root)
    {
        extendedPane_.loadLayoutFromXML(root);
    }

    public void setVisible(boolean visible)
    {
        super.setVisible(visible);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the ActiveWindowChangeListener
    // interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Called by the extended pane, when the active window changed. The method
     * getActiveWindow() from the event object returns the new activated window.
     * 
     * @param event
     *            an event object containing information about the new active
     *            window
     */
    public void activeWindowChanged(ActiveWindowChangeEvent event)
    {
        // if there is no active window restore the default toolbar and
        // menubar
        if (event.getActiveWindow() == null)
        {
            // set default toolbar and menu
            setMenu(defaultMenuBar_);
            setToolBar(defaultToolBar_);
            setTitle(ResourceConstants.MAINFRAME_TITLE);
            return;
        }

        // set the new menubar
        setMenu(event.getActiveWindow().getMenuBar());

        // set the mew toolbar
        setToolBar(event.getActiveWindow().getToolBar());

        // set the new title
        setTitle(ResourceConstants.MAINFRAME_TITLE + " - "
                + event.getActiveWindow().getTitle());
    }

    public void windowClose(WindowCloseEvent event)
    {
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IWindow interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.gui.IWindow */
    public Component getComponent()
    {
        return this;
    }

    /* @see viptool.base.gui.IWindow */
    public Icon getIcon()
    {
        return new ImageIcon(getIconImage());
    }
}
