package viptool.impl.base;

import java.awt.Image;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.JFrame;

import viptool.base.IWorkbench;
import viptool.base.action.IActionHandler;
import viptool.base.editor.IEditor;
import viptool.base.editor.IEditorManager;
import viptool.base.editor.IEditorProvider;
import viptool.base.error.CannotOpenFileException;
import viptool.base.error.EditorException;
import viptool.base.error.EditorNotOpenedException;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.error.VipToolException;
import viptool.base.error.VipToolRuntimeException;
import viptool.base.event.IWorkbenchListener;
import viptool.base.event.WorkspaceChangeEvent;
import viptool.base.gui.IChildWindow;
import viptool.base.gui.IView;
import viptool.base.gui.dialog.event.IStandardDialogListener;
import viptool.base.gui.menu.IContextMenuManager;
import viptool.base.gui.menu.IMenuBarManager;
import viptool.base.gui.toolbar.IToolBarManager;
import viptool.base.io.IFileInterpreter;
import viptool.base.io.IFileInterpreterManager;
import viptool.base.io.IFileInterpreterProvider;
import viptool.base.io.IFileLocker;
import viptool.base.wizard.IWizard;
import viptool.impl.base.action.ActionHandler;
import viptool.impl.base.editor.EditorManager;
import viptool.impl.base.extendedPane.event.ActiveWindowChangeEvent;
import viptool.impl.base.extendedPane.event.IExtendedPaneListener;
import viptool.impl.base.extendedPane.event.WindowCloseEvent;
import viptool.impl.base.gui.dialog.ExceptionDialog;
import viptool.impl.base.gui.dialog.StandardModalDialog;
import viptool.impl.base.gui.dialog.WizardDialog;
import viptool.impl.base.gui.menu.ContextMenuManager;
import viptool.impl.base.gui.menu.MenuBarManager;
import viptool.impl.base.gui.toolbar.ToolBarManager;
import viptool.impl.base.io.FileInterpreterManager;
import viptool.impl.job.JobScheduler;
import viptool.impl.job.gui.JobWindow;
import viptool.impl.logger.Logger;
import viptool.impl.preferences.PreferenceStore;
import viptool.impl.project.AbstractFolderProvider;
import viptool.impl.project.ProjectGuiLinker;
import viptool.impl.project.gui.ProjectTreePane;
import viptool.impl.util.resources.ResourceLoader;
import viptool.job.IJobScheduler;
import viptool.job.event.IJobSchedulerListener;
import viptool.job.event.JobEvent;
import viptool.logger.ILogger;
import viptool.project.IAbstractFile;
import viptool.project.IAbstractFolder;
import viptool.project.IAbstractFolderProvider;
import viptool.project.IProject;
import viptool.project.IWorkspace;
import viptool.util.GeneralUtils;
import viptool.util.resources.IResourceLoader;
import viptool.util.resources.ResourceConstants;

public class Workbench implements IWorkbench, IJobSchedulerListener,
        IExtendedPaneListener
{
    // the and only instance
    private static IWorkbench  instance_        = null;

    // the current workspace
    private IWorkspace         workspace_       = null;

    // open views
    private HashSet            openViews_       = null;

    // open editors
    // key: IAbstractFile
    // value: HashSet with IEditor entries
    private HashMap            openEditors_     = null;

    private ArrayList<IEditor> openEditorsList_ = null;

    // listeners
    private HashSet            listeners_       = null;

    private String[]           commandLine_     = null;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    private Workbench()
    {
        commandLine_ = null;
        openViews_ = new HashSet();
        openEditors_ = new HashMap();
        listeners_ = new HashSet();
        openEditorsList_ = new ArrayList<IEditor>();

        JobScheduler.getInstance().addJobSchedulerListener(this);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    public static synchronized IWorkbench getInstance()
    {
        if (instance_ == null)
        {
            instance_ = new Workbench();
        }

        return instance_;
    }

    public void setCommandLineOptions(String[] args)
    {
        if (commandLine_ == null)
        {
            commandLine_ = args.clone();
        }
    }

    public String[] getCommandLineOptions()
    {
        return commandLine_.clone();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>
    // TODO: replace this method when a adaquate solution is found
    public IAbstractFolderProvider getAbstractFolderProvider()
    {
        return (IAbstractFolderProvider) getFileInterpreterManager().getFileInterpreterProvider(
                AbstractFolderProvider.class);
    }

    /**
     * Opens the given child window in the main frame. The child window is
     * focused too.
     * 
     * @param childWindow
     *            the child window to open
     * @return true if the child window was opened, false otherwise
     */
    private boolean openChildWindow(IChildWindow childWindow)
    {
        if (childWindow != null)
        {
            MainFrame.getInstance().getExtendedPane().addChildWindow(
                    childWindow);

            return true;
        }

        return false;
    }

    /**
     * Closes the given child window. If force is set to true, the child window
     * will closed even if canClose returns false.
     * 
     * @param childWindow
     *            the child window to close
     * @param force
     *            a flag indication whether the window should be closed even if
     *            canClose returns false
     * @return true if the child window was closed, false otherwise
     */
    private boolean closeChildWindow(IChildWindow childWindow, boolean force)
    {
        return MainFrame.getInstance().getExtendedPane().removeChildWindow(
                childWindow, force);
    }

    /**
     * Notifies all workbench listeners that the workspace has changed.
     * 
     * @param oldWorkspace
     *            the workspace that was previously loaded
     */
    private void fireWorkspaceChangeEvent(IWorkspace oldWorkspace)
    {
        Iterator it;
        WorkspaceChangeEvent event = new WorkspaceChangeEvent(this,
                oldWorkspace, getCurrentWorkspace());

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IWorkbenchListener) it.next()).workspaceChanged(event);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IWorkbench interface
    // <><><><><><><><><><><><><><><><><><><><><>

    // <><><><><><><><><><><><><><><><><><><><><>
    // General methods
    // <><><><><><><><><><><><><><><><><><><><><>

    public void showDialog(IStandardDialogListener listener, String title,
            String mainDescription, JComponent content, Image image,
            int buttons, int defaultButton)
    {
        StandardModalDialog dialog = new StandardModalDialog(title,
                mainDescription, content, image, buttons, defaultButton);
        dialog.addStandardDialogListener(listener);
        dialog.setVisible(true);
    }

    /* @see viptool.base.IWorkbench */
    public JFrame getMainFrame()
    {
        return MainFrame.getInstance();
    }

    /* @see viptool.base.IWorkbench */
    public void showExceptionInDialog(VipToolException exception)
    {
        new ExceptionDialog(exception);
    }

    /* @see viptool.base.IWorkbench */
    public void showExceptionInDialog(VipToolRuntimeException exception)
    {
        new ExceptionDialog(exception);
    }

    /* @see viptool.base.IWorkbench */
    public void closeWindow(IChildWindow window, boolean force)
    {
        if (window instanceof IEditor)
        {
            closeEditor((IEditor) window, force);
        }
        else if (window instanceof IView)
        {
            closeView((IView) window, force);
        }
        else
        {
            closeChildWindow(window, force);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Workspace support
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.IWorkbench */
    public void setCurrentWorkspace(IWorkspace workspace)
    {
        IWorkspace oldWorkspace = workspace_;

        workspace_ = workspace;

        // save the workspace in the preferences
        PreferenceStore.getInstance().getPreferenceChildNode(
                ResourceConstants.PREFERENCE_NODE_WORKSPACE).writeString(
                ResourceConstants.PREFERENCE_LAST_WORKSPACE,
                workspace_.getAbstractFile().getFullPath());
        PreferenceStore.getInstance().savePreferences();

        fireWorkspaceChangeEvent(oldWorkspace);
    }

    /* @see viptool.base.IWorkbench */
    public IWorkspace getCurrentWorkspace()
    {
        return workspace_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Project support
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.IWorkbench */
    public IProject getSelectedProject()
    {
        return ProjectGuiLinker.getInstance().getSelectedProject();
    }

    /* @see viptool.base.IWorkbench */
    public IAbstractFolder getSelectedFolder()
    {
        return ProjectGuiLinker.getInstance().getSelectedFolder();
    }

    /* @see viptool.base.IWorkbench */
    public IAbstractFile getSelectedFile()
    {
        IAbstractFile element = ProjectGuiLinker.getInstance().getSelectedElement();

        if (element instanceof IAbstractFolder)
        {
            return element;
        }

        return null;
    }

    /* @see viptool.base.IWorkbench */
    public void selectFile(IAbstractFile file)
    {
        ProjectGuiLinker.getInstance().makeVisible(file);
    }

    /* @see viptool.base.IWorkbench */
    public void selectWindow(IChildWindow window)
    {
        MainFrame.getInstance().getExtendedPane().selectWindow(window);
    }
    
    /* @see viptool.base.IWorkbench */
    public boolean windowIsSelected(IChildWindow window)
    {
        return (MainFrame.getInstance().getExtendedPane().getActiveWindow() == window);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // View support
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.IWorkbench */
    public void openView(IView view)
    {
        if (!openViews_.contains(view))
        {
            if (openChildWindow(view))
            {
                openViews_.add(view);
            }
        }
    }

    /* @see viptool.base.IWorkbench */
    public void closeView(IView view, boolean force)
    {
        if (openViews_.contains(view))
        {
            if (closeChildWindow(view, force))
            {
                openViews_.remove(view);
            }
        }
    }

    /* @see viptool.base.IWorkbench */
    public void closeAllViews(boolean force)
    {
        IView[] views = getOpenViews();

        for (int i = 0; i < views.length; i++)
        {
            if (!(views[i] instanceof ProjectTreePane))
            {
                closeView(views[i], force);
            }
        }
    }

    /* @see viptool.base.IWorkbench */
    public IView[] getOpenViews()
    {
        return (IView[]) openViews_.toArray(new IView[openViews_.size()]);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Editor support
    // <><><><><><><><><><><><><><><><><><><><><>

    public void closeAllEditors(boolean force)
    {
        IEditor[] editors = getOpenEditors();

        for (int i = 0; i < editors.length; i++)
        {
            closeEditor(editors[i], force);
        }
    }

    public void closeAllEditors(IAbstractFile file, boolean force)
    {
        IEditor[] editors = findEditors(file);

        for (int i = 0; i < editors.length; i++)
        {
            closeEditor(editors[i], force);
        }
    }

    public void closeEditor(IEditor editor, boolean force)
    {
        Set editors;

        if (openEditorsList_.contains(editor))
        {
            if (closeChildWindow(editor, force))
            {
                openEditorsList_.remove(editor);
                try
                {
                    editor.getAbstractFile().releaseLock(editor);

                    editors = (Set) openEditors_.get(editor.getAbstractFile());
                    editors.remove(editor);

                    if (editors.size() == 0)
                    {
                        openEditors_.remove(editor.getAbstractFile());
                    }
                }
                catch (EditorNotOpenedException exception)
                {
                    // should not be
                }
            }
        }
    }

    public IEditor[] findEditors(IAbstractFile file)
    {
        HashSet set = (HashSet) openEditors_.get(file);

        if (set == null)
        {
            return new IEditor[0];
        }

        return (IEditor[]) set.toArray(new IEditor[set.size()]);
    }

    public IEditor findLockingEditor(IAbstractFile file)
    {
        IFileLocker locker = file.getFileLocker();

        if (locker instanceof IEditor)
        {
            return (IEditor) locker;
        }

        return null;
    }

    public IEditor[] getOpenEditors()
    {
        return (IEditor[]) openEditorsList_.toArray(new IEditor[openEditorsList_.size()]);
    }

    public void openEditor(IEditor editor)
    {
        IAbstractFile file;
        HashSet set;

        editor.initialize();

        try
        {
            for (IEditor openEditor : openEditorsList_)
            {
                if (openEditor.getClass().equals(editor.getClass())
                        && openEditor.getAbstractFile().equals(
                                editor.getAbstractFile()))
                {
                    MainFrame.getInstance().getExtendedPane().selectWindow(
                            openEditor);
                    return;
                }
            }
        }
        catch (EditorNotOpenedException exception)
        {
            showExceptionInDialog(exception);
        }

        if (openChildWindow(editor))
        {
            try
            {
                file = editor.getAbstractFile();
                openEditorsList_.add(editor);

                set = (HashSet) openEditors_.get(file);
                if (set == null)
                {
                    set = new HashSet();
                }

                set.add(editor);
                openEditors_.put(file, set);
            }
            catch (EditorNotOpenedException exception)
            {
                // can't be theoretically
            }
        }
    }

    /* @see viptool.base.IWorkbench */
    public void openFile(IAbstractFile file) throws CannotOpenFileException
    {
        // test if the file is already opened
        IFileInterpreterProvider fileInterpreterProvider = FileInterpreterManager.getInstance().getDefaultFileInterpreterProvider(
                file);

        if (fileInterpreterProvider == null)
        {
            throw new CannotOpenFileException(file,
                    "No default file interpreter found."
                            + "This means there is no file interpreter to "
                            + "open the specified file!");
        }

        try
        {
            IFileInterpreter fileInterpreter = fileInterpreterProvider.createFromFile(file);

            openFile(fileInterpreter);
        }
        catch (FileInterpreterProviderException exception)
        {
            throw new CannotOpenFileException(file, exception);
        }
    }

    /* @see viptool.base.IWorkbench */
    public void openFile(IFileInterpreter fileInterpreter)
            throws CannotOpenFileException
    {
        IEditorProvider editorProvider = EditorManager.getInstance().getDefaultEditorProvider(
                fileInterpreter);

        if (editorProvider == null)
        {
            throw new CannotOpenFileException(
                    fileInterpreter.getAbstractFile(),
                    "No default editor found."
                            + "This means there is no editor to "
                            + "open the specified file!");
        }

        try
        {
            IEditor editor = editorProvider.createEditor(fileInterpreter);

            // opens the editor
            openEditor(editor);
        }
        catch (EditorException exception)
        {
            throw new CannotOpenFileException(
                    fileInterpreter.getAbstractFile(), exception);
        }

    }

    /* @see viptool.base.IWorkbench */
    public void openFile(IAbstractFile file,
            IFileInterpreterProvider fileInterpreterProvider)
            throws CannotOpenFileException
    {
        try
        {
            if (!fileInterpreterProvider.matchFormat(file))
            {
                throw new CannotOpenFileException(file,
                        "The given file interpreter provider cannot read the "
                                + "given file format!");
            }

            IFileInterpreter fileInterpreter = fileInterpreterProvider.createFromFile(file);
            openFile(fileInterpreter);
        }
        catch (FileInterpreterProviderException exception)
        {
            throw new CannotOpenFileException(file, exception);
        }
    }

    /* @see viptool.base.IWorkbench */
    public void openFile(IAbstractFile file,
            IFileInterpreterProvider fileInterpreterProvider,
            IEditorProvider editorProvider) throws CannotOpenFileException
    {
        if (!editorProvider.accept(fileInterpreterProvider))
        {
            throw new CannotOpenFileException(file,
                    "The given editor provider cannot open the specified "
                            + "file interpreter provider!");
        }

        try
        {
            if (!fileInterpreterProvider.matchFormat(file))
            {
                throw new CannotOpenFileException(file,
                        "The given file interpreter provider cannot read the "
                                + "given file format!");
            }

            IFileInterpreter fileInterpreter = fileInterpreterProvider.createFromFile(file);
            IEditor editor = editorProvider.createEditor(fileInterpreter);

            // opens the editor
            openEditor(editor);
        }
        catch (FileInterpreterProviderException exception)
        {
            throw new CannotOpenFileException(file, exception);
        }
        catch (EditorException exception)
        {
            throw new CannotOpenFileException(file, exception);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // listener support
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.base.IWorkbench */
    public void addWorkbenchListener(IWorkbenchListener listener)
    {
        if (listener != null)
        {
            listeners_.add(listener);
        }
    }

    /* @see viptool.base.IWorkbench */
    public void removeWorkbenchListener(IWorkbenchListener listener)
    {
        listeners_.remove(listener);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // provider support
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.base.IWorkbench */
    public IFileInterpreterManager getFileInterpreterManager()
    {
        return FileInterpreterManager.getInstance();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // logger support
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.base.IWorkbench */
    public ILogger getLogger()
    {
        return Logger.getInstance();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IJobSchedulerListener
    // interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.job.event.IJobSchedulerListener */
    public void jobCancelled(JobEvent jobEvent)
    {
        JobWindow jobWindow;
        jobWindow = JobWindow.getInstance();
        jobWindow.removeJobPanel(jobEvent);
    }

    /* @see viptool.job.event.IJobSchedulerListener */
    public void jobFinished(JobEvent jobEvent)
    {
        JobWindow jobWindow;

        jobWindow = JobWindow.getInstance();
        jobWindow.removeJobPanel(jobEvent);
    }

    /* @see viptool.job.event.IJobSchedulerListener */
    public void jobStarted(JobEvent jobEvent)
    {
        JobWindow jobWindow;

        jobWindow = JobWindow.getInstance();
        jobWindow.addJobPanel(jobEvent);
        openView(jobWindow);
    }

    /* @see viptool.job.event.IJobSchedulerListener */
    public void newJobWaiting(int mode)
    {
    }

    public IActionHandler getActionHandler()
    {
        return ActionHandler.getInstance();
    }

    public IContextMenuManager getContextMenuManager()
    {
        return ContextMenuManager.getInstance();
    }

    public IEditorManager getEditorManager()
    {
        return EditorManager.getInstance();
    }

    public IMenuBarManager getMenuBarManager()
    {
        return MenuBarManager.getInstance();
    }

    public IToolBarManager getToolBarManager()
    {
        return ToolBarManager.getInstance();
    }

    public IJobScheduler getJobScheduler()
    {
        return JobScheduler.getInstance();
    }

    public void startWizard(IWizard wizard)
    {
        new WizardDialog(wizard);
    }

    public IResourceLoader getResourceLoader()
    {
        return ResourceLoader.getInstance();
    }

    public String getPropertyPath()
    {
        File file;
        String path = GeneralUtils.getUsersHomeDirectory();

        if (System.getProperty("os.name").startsWith("Windows"))
        {
            path += "Viptool";
        }
        else
        {
            path += ".viptool";
        }

        file = new File(path);
        if (!file.exists())
        {
            if (!file.mkdirs())
            {
                Logger.getInstance().error(
                        "Cannot create settings path " + path);
            }
        }

        return path + File.separator;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implemented methods from ExtendedPaneListener
    // <><><><><><><><><><><><><><><><><><><><><>

    public void activeWindowChanged(ActiveWindowChangeEvent event)
    {
    }

    public void windowClose(WindowCloseEvent event)
    {
        closeWindow(event.getWindowToClose(), false);
    }
}
