package viptool.impl.base.action;

import java.net.URI;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import viptool.base.action.ContainerAction;
import viptool.base.action.ExtendedAction;
import viptool.base.action.IActionHandler;
import viptool.base.action.IActionPostProcessor;
import viptool.base.action.SeparatorAction;
import viptool.base.error.VipToolException;
import viptool.base.error.XMLException;
import viptool.impl.base.Workbench;
import viptool.impl.extension.extensionloader.ExtensionLoader;
import viptool.impl.logger.Logger;
import viptool.impl.util.resources.ResourceLoader;
import viptool.util.GeneralUtils;
import viptool.util.resources.ResourceConstants;

public class ActionHandler implements IActionHandler {
	private static ActionHandler instance_;

	private HashMap templateActions_;

	private HashMap instanceActions_;

	private HashMap postProcessors_;

	private ActionHandler() {
		templateActions_ = new HashMap();
		instanceActions_ = new HashMap();
		postProcessors_ = new HashMap();

		// add the separator action. All separators will have the same id
		addAction(new SeparatorAction());
	}

	public static synchronized IActionHandler getInstance() {
		if (instance_ == null) {
			instance_ = new ActionHandler();
		}

		return instance_;
	}

	private ExtendedAction createActionFromElement(Element actionElement) {
		Object[] parentKeys;
		Object parentValue;
		String name;
		String value;
		String className;
		ExtendedAction action;
		ExtendedAction parentAction;
		Node attribute;
		NamedNodeMap attributes;
		URI resource;

		action = null;

		try {
			// first, the action has to be created, if no class is given,
			// create a simple AbstractAction

			if (actionElement.getNodeName().equals(
					IActionHandler.ELEMENT_CONTAINER)) {
				action = new ContainerAction();
			} else if (actionElement.getNodeName().equals(
					IActionHandler.ELEMENT_SEPARATOR)) {
				action = new SeparatorAction();
			} else {
				className = actionElement
						.getAttribute(ExtendedAction.ATTR_CLASS);

				action = (ExtendedAction) ExtensionLoader.getInstance()
						.loadClass(className).newInstance();
			}

			if (actionElement.hasAttribute(ExtendedAction.ATTR_INHERITFROM)
					&& actionElement.getAttribute(
							ExtendedAction.ATTR_INHERITFROM).length() > 0) {
				parentAction = ActionHandler.getInstance().getAction(
						actionElement
								.getAttribute(ExtendedAction.ATTR_INHERITFROM));

				if (parentAction != null) {
					parentKeys = parentAction.getKeys();

					for (int i = 0; i < parentKeys.length; i++) {
						parentValue = parentAction
								.getValue((String) parentKeys[i]);
						action.putValue((String) parentKeys[i], parentValue);
					}
				} else {
					Logger.getInstance()
							.warning(
									"An action shoud inherit from "
											+ actionElement
													.getAttribute(ExtendedAction.ATTR_INHERITFROM)
											+ " but the parent action cannot be found!");
				}
			}

			// read out all attributes
			if ((attributes = actionElement.getAttributes()) != null) {
				for (int i = 0; i < attributes.getLength(); i++) {
					attribute = attributes.item(i);

					name = attribute.getNodeName();
					value = attribute.getNodeValue();

					// some attributes have to be converted like
					// - accelerator key
					// - mnemonic key
					// - smallIcon
					if (name.equals(Action.ACCELERATOR_KEY)) {
						action.putValue(Action.ACCELERATOR_KEY,
								KeyStroke.getKeyStroke(value));
					} else if (name.equals(Action.MNEMONIC_KEY)) {
						action.putValue(Action.MNEMONIC_KEY,
								new Integer(value.charAt(0)));
					} else if (name.equals(Action.SMALL_ICON)) {
						resource = ResourceLoader.getInstance().getResource(
								GeneralUtils.convertIdToPath(value));

						if (resource != null) {
							action.putValue(Action.SMALL_ICON, new ImageIcon(
									resource.toURL()));
						} else {
							Logger.getInstance().warning(
									"Icon not found: " + value + "(action: "
											+ action.getIdKey() + ")");
						}
					} else if (name.equals(ExtendedAction.ATTR_SEEK)) {
						action.putValue(ExtendedAction.ATTR_SEEK, new Integer(
								value));
					} else if (name.equals(ExtendedAction.ATTR_TYPE)) {
						action.putValue(name, value);

						if (value.equals(ExtendedAction.TYPE_RADIO)
								|| value.equals(ExtendedAction.TYPE_CHECKBOX)) {
							action.putValue(AbstractAction.SELECTED_KEY,
									new Boolean(false));
						}
					} else {
						action.putValue(name, value);
					}

				}
			}

		} catch (Exception exception) {
			Workbench.getInstance().showExceptionInDialog(
					new VipToolException(exception));
		}

		// create a file in the file system for the action given in the
		// xml file
		return action;
	}

	private void processAction(int instance, ExtendedAction action) {
		Iterator it;
		IActionPostProcessor postProcessor;
		HashSet postProcessors = (HashSet) postProcessors_.get(new Integer(
				instance));

		Logger.getInstance().debug(
				"ActionHandler: post process action " + action.getIdKey()
						+ " (instance: " + instance + ")");

		if (postProcessors != null) {
			it = postProcessors.iterator();
			while (it.hasNext()) {
				postProcessor = (IActionPostProcessor) it.next();

				Logger.getInstance().debug(
						"\tusing post processor: " + postProcessor);
				postProcessor.processAction(action);
			}
		}
	}

	/**
	 * Registers the action and copies values of parent action.
	 */
	public void addAction(ExtendedAction action) {
		ExtendedAction parentAction;
		Object[] parentKeys;
		Object parentValue;
		
		Logger.getInstance().debug(
				"ActionHandler: Adding Action " + action.getIdKey());

		if (action.getValue(ExtendedAction.ATTR_INHERITFROM) != null) {
			parentAction = getAction((String)action.getValue(ExtendedAction.ATTR_INHERITFROM));

			if (parentAction != null) {
				parentKeys = parentAction.getKeys();

				for (int i = 0; i < parentKeys.length; i++) {
					if (action.getValue((String) parentKeys[i]) == null) {
						parentValue = parentAction.getValue((String) parentKeys[i]);
						action.putValue((String) parentKeys[i], parentValue);
					}
				}
			} else {
				Logger.getInstance()
						.warning(
								"An action shoud inherit from "
										+ (String)action.getValue(ExtendedAction.ATTR_INHERITFROM)
										+ " but the parent action cannot be found!");
			}
		}
		
		
		templateActions_.put(action.getIdKey(), action);

	}

	public ExtendedAction getAction(String id) {
		return (ExtendedAction) templateActions_.get(id);
	}

	public ExtendedAction getAction(int instance, String id) {
		HashMap actions;
		ExtendedAction action;

		if (instance == 0) {
			Logger.getInstance().debug(
					"ActionHandler: Returning template action " + id);
			return getAction(id);
		}

		try {
			// first search the instance map, perhaps the action is already
			// there
			actions = (HashMap) instanceActions_.get(new Integer(instance));
			if (actions == null) {
				// now instance map, so create a new one
				actions = new HashMap();
				instanceActions_.put(new Integer(instance), actions);
			}

			// search the action in the insatnce map now
			action = (ExtendedAction) actions.get(id);
			if (action == null) {
				// the action was not found. It has to be cloned and stored
				action = (ExtendedAction) ((ExtendedAction) templateActions_
						.get(id)).clone();

				// tell the action its instance (for debug purpose)
				action.putValue(ExtendedAction.INSTANCE_KEY, new Integer(
						instance));

				// post process this action
				processAction(instance, action);

				// store the action in the instance map
				actions.put(id, action);

				// initialize the action
				action.initialize();
			}

			Logger.getInstance().debug(
					"Action Handler: Returning action " + id + "(instance "
							+ instance + ")");
			return action;
		} catch (CloneNotSupportedException exception) {
			return null;
		}
	}

	public void removeActions(int instance) {
		instanceActions_.remove(new Integer(instance));
	}

	public void registerActionPostProcessor(int instance,
			IActionPostProcessor postProcessor) {
		Integer instanceObject = new Integer(instance);
		HashSet postProcessors;

		if ((postProcessors = (HashSet) postProcessors_.get(instanceObject)) == null) {
			postProcessors = new HashSet();
			postProcessors_.put(instanceObject, postProcessors);
		}

		postProcessors.add(postProcessor);
	}
}
