package viptool.impl.base.editor;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

import viptool.base.editor.IEditorManager;
import viptool.base.editor.IEditorProvider;
import viptool.base.io.IFileInterpreter;
import viptool.base.io.IFileInterpreterProvider;
import viptool.impl.base.Workbench;
import viptool.impl.base.action.contextmenu.OpenWithAction;
import viptool.impl.logger.Logger;
import viptool.logger.ILogger;

/**
 * The editor manager manages all known editors and is responsible for selecting
 * the right editor depending on the a file interpreter. The class is written
 * respecting the singleton interface.
 */
public class EditorManager implements IEditorManager
{
    // the one and only instance
    private static EditorManager instance_     = null;

    // a counter storing the last priority, incremented if a new editor provider
    // is added
    private int                  lastPriority_ = 0;

    // the set with all registered editor providers
    private SortedSet            editorProviders_;

    private class EditorProviderInfo
    {
        private IEditorProvider provider_;

        private int             priority_;

        public EditorProviderInfo(IEditorProvider provider, int priority)
        {
            provider_ = provider;
            priority_ = priority;
        }

        public IEditorProvider getEditorProvider()
        {
            return provider_;
        }

        public int getPriority()
        {
            return priority_;
        }
    }

    private class EditorComparator implements Comparator
    {
        public int compare(Object obj1, Object obj2)
        {
            EditorProviderInfo info1 = (EditorProviderInfo) obj1;
            EditorProviderInfo info2 = (EditorProviderInfo) obj2;

            return info2.getPriority() - info1.getPriority();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new instance of the file type manager.
     */
    public EditorManager()
    {
        editorProviders_ = new TreeSet(new EditorComparator());
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the instance of the file type manager. If there isn't already one
     * a new one will be created.
     * 
     * @return the only instance of the file type manager
     */
    public static synchronized EditorManager getInstance()
    {
        if (instance_ == null)
        {
            instance_ = new EditorManager();
        }

        return instance_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IEditorManager
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.editor.IEditorManager */
    public void addEditorProvider(IEditorProvider editorProvider)
    {
        Iterator it;
        EditorProviderInfo info;

        it = editorProviders_.iterator();
        while (it.hasNext())
        {
            info = (EditorProviderInfo) it.next();
            if (info.getEditorProvider() == editorProvider)
            {
                // do not add, if already exists
                return;
            }
        }

        editorProviders_.add(new EditorProviderInfo(editorProvider,
                lastPriority_));

        // add an action for the editor
        Workbench.getInstance().getActionHandler().addAction(
                new OpenWithAction(editorProvider));

        lastPriority_++;
    }

    /* @see viptool.base.editor.IEditorManager */
    public void removeEditorProvider(IEditorProvider editorProvider)
    {
        Iterator it;
        EditorProviderInfo info;

        it = editorProviders_.iterator();
        while (it.hasNext())
        {
            info = (EditorProviderInfo) it.next();
            if (info.getEditorProvider() == editorProvider)
            {
                // only remove if exists
                editorProviders_.remove(info);
                return;
            }
        }

        return;
    }

    /* @see viptool.base.editor.IEditorManager */
    public IEditorProvider getDefaultEditorProvider(
            IFileInterpreter fileInterpreter)
    {
        Iterator it;
        EditorProviderInfo info;

        it = editorProviders_.iterator();
        while (it.hasNext())
        {
            info = (EditorProviderInfo) it.next();

            if (info.getEditorProvider().accept(fileInterpreter.getProvider()))
            {
                return info.getEditorProvider();
            }
        }

        Logger.getInstance().log(
                ILogger.WARRNING_LEVEL,
                "No default provider found for the file interpreter'"
                        + fileInterpreter.getProvider().toString() + "'");
        return null;
    }

    /* @see viptool.base.editor.IEditorManager */
    public IEditorProvider[] getMatchingEditorProviders(
            IFileInterpreter fileInterpreter)
    {
        Iterator it;
        EditorProviderInfo info;
        ArrayList providers = new ArrayList();

        it = editorProviders_.iterator();
        while (it.hasNext())
        {
            info = (EditorProviderInfo) it.next();

            if (fileInterpreter == null
                    || info.getEditorProvider().accept(
                            fileInterpreter.getProvider()))
            {
                providers.add(info.getEditorProvider());
            }
        }

        return (IEditorProvider[]) providers.toArray(new IEditorProvider[providers.size()]);
    }

    public ArrayList<IEditorProvider> getMatchingEditorProviders(
            IFileInterpreterProvider[] fileInterpreterProviders)
    {
        Iterator it;
        EditorProviderInfo info;
        ArrayList<IEditorProvider> editorProviders = new ArrayList<IEditorProvider>();

        it = editorProviders_.iterator();
        while (it.hasNext())
        {
            info = (EditorProviderInfo) it.next();
            for (int i = 0; i < fileInterpreterProviders.length; i++)
            {
                if (info.getEditorProvider().accept(fileInterpreterProviders[i]))
                {
                    editorProviders.add(info.getEditorProvider());
                }
            }
        }

        return editorProviders;
    }

    /* @see viptool.base.editor.IEditorManager */
    public IEditorProvider[] getEditorProviders()
    {
        return getMatchingEditorProviders((IFileInterpreter) null);
    }

    /* @see viptool.base.editor.IEditorManager */
    public IEditorProvider getEditorProvider(Class clazz)
    {
        Iterator it;
        EditorProviderInfo info;

        it = editorProviders_.iterator();
        while (it.hasNext())
        {
            info = (EditorProviderInfo) it.next();
            if (info.getEditorProvider().getClass().equals(clazz))
            {
                return info.getEditorProvider();
            }
        }

        return null;
    }
}
