package viptool.impl.base.extendedPane;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ComponentEvent;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map.Entry;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import viptool.base.gui.IChildWindow;
import viptool.base.gui.INonCloseableChildWindow;
import viptool.base.gui.INonFocusableChildWindow;
import viptool.impl.base.extendedPane.event.ActiveWindowChangeEvent;
import viptool.impl.base.extendedPane.event.IExtendedPaneListener;
import viptool.impl.base.extendedPane.event.WindowCloseEvent;
import viptool.impl.logger.Logger;
import viptool.impl.util.resources.ResourceLoader;
import viptool.util.resources.ResourceConstants;

/**
 * Class is needed to manage all possible child windows. It constructs a very
 * flexible window to do so. Felixible splitting is implemented as well as
 * moving tabs.
 */
public class ExtendedPane extends JPanel implements ChangeListener,
        MouseListener, MouseMotionListener
{
    // don't change the integer values!! They are used in create group!
    /**
     * position for using with openGroup. Indicates that the group should be
     * opened on the top of the frame. This is the default.
     */
    public static final int                  POSITION_FRAME_TOP     = 0;

    /**
     * position for using with openGroup. Indicates that the group should be
     * opened on the bottom of the frame
     */
    public static final int                  POSITION_FRAME_BOTTOM  = 1;

    /**
     * position for using with openGroup. Indicates that the group should be
     * opened on the left of the frame
     */
    public static final int                  POSITION_FRAME_LEFT    = 2;

    /**
     * position for using with openGroup. Indicates that the group should be
     * opened on the right of the frame
     */
    public static final int                  POSITION_FRAME_RIGHT   = 3;

    /**
     * position for using with openGroup. Indicates that the group should be
     * opened on the top of the active child window
     */
    public static final int                  POSITION_ACTIVE_TOP    = 4;

    /**
     * position for using with openGroup. Indicates that the group should be
     * opened on the bottom of the active child window
     */
    public static final int                  POSITION_ACTIVE_BOTTOM = 5;

    /**
     * position for using with openGroup. Indicates that the group should be
     * opened on the left of the active child window
     */
    public static final int                  POSITION_ACTIVE_LEFT   = 6;

    /**
     * position for using with openGroup. Indicates that the group should be
     * opened on the right of the active child window
     */
    public static final int                  POSITION_ACTIVE_RIGHT  = 7;

    // stores the component that has currently the focus
    private IChildWindow                     activeWindow_          = null;

    // stores all child windows that are managed by this pane
    // key: java.awt.Component
    // value: viptool.gui.extendedPane.ChildWindow
    private HashMap<Component, IChildWindow> childWindows_          = new HashMap<Component, IChildWindow>();

    // stores all groups of the extended pane
    // key: java.awt.String name of the group
    // value: viptool.gui.extendedPane.ExtendedPane.Group group
    private HashMap<String, Group>           groups_                = new HashMap<String, Group>();

    // stores the last active pane. Necessary if a new window is added and no
    // pane is active. Then the new window will be added to the last active
    // pane.
    private ExtendedTabbedPane               lastActiveTabbedPane_  = null;

    // list of all activeWindowChanged listeners
    private ArrayList                        activeWindowListeners_ = new ArrayList();

    // the drag window for enabling the move of tabs inside our pane
    private DragWindow                       dragWindow_            = new DragWindow();

    // the default group name
    private String                           defaultGroup_          = "";

    // stores all tabs for child windows, ensures that the tab keeps the same
    // after changing the tabbed pane
    // key: viptool.gui.extendedPane.ChildWindow the child window
    // value: value: viptool.gui.extendedPane.ExtendedPane.Tab the tab
    private HashMap<IChildWindow, Tab>       tabs_                  = new HashMap<IChildWindow, Tab>();

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classe
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * This class is a helper class for the loading layout algorithm. If a
     * window is not found it is replaced by the empty window. The last step of
     * the algorithm is to remove all empty windows. This ensures right
     * layouting of groups, respects its closeability, etc.
     */
    private class EmptyChildWindow implements IChildWindow
    {
        private JComponent compontent_ = new JPanel();

        public boolean canClose(boolean force)
        {
            return true;
        }

        public JMenuBar getMenuBar()
        {
            return null;
        }

        public JToolBar getToolBar()
        {
            return null;
        }

        public String getToolTipText()
        {
            return "I does not exist!";
        }

        public void windowActivated()
        {
        }

        public Component getComponent()
        {
            return compontent_;
        }

        public Icon getIcon()
        {
            return null;
        }

        public String getTitle()
        {
            return "empty window";
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner class for grouping
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This class is a class for grouping functionality. An object of this class
     * represents a group. With grouping different child windows can be grouped.
     */
    private class Group
    {
        // the filter that determines which child windows can be in this group
        private Class   filter_;

        // name of the group
        private String  name_;

        // indicates if the group is closeable. If not, at least one area would
        // be left opened even if all child windows were closed
        private boolean closeable_;

        // a list of all open tabbed panes
        private HashSet tabbedPanes_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Copy constructor. Creates a new group and copies the properties of
         * the given group.
         * 
         * @param group
         *            the source group
         */
        public Group(Group group)
        {
            this(group.name_, group.filter_, group.closeable_);
        }

        /**
         * Creates a group with the given name and the given filter.
         * 
         * @param name
         *            name of the group
         * @param filter
         *            filter class for the group
         */
        public Group(String name, Class filter)
        {
            this(name, filter, true);
        }

        /**
         * Creates a group with the given name and the given filter. The third
         * parameter can be used to mark the group as non closeable.
         * 
         * @param name
         *            name of the group
         * @param filter
         *            filter class for the group
         * @param closeable
         *            indicates wheter the group can be closed or not
         */
        public Group(String name, Class filter, boolean closeable)
        {
            super();
            setName(name);

            name_ = name;
            filter_ = filter;
            closeable_ = closeable;

            tabbedPanes_ = new HashSet();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // property methods
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Returns the name of the group.
         * 
         * @return name of the group
         */
        public String getName()
        {
            return name_;
        }

        public Class getFilter()
        {
            return filter_;
        }

        /**
         * Indicates if the group can be closed or not.
         * 
         * @return true if the group can be closed, false otherwise.
         */
        public boolean isCloseable()
        {
            return closeable_;
        }

        /**
         * Tests whether the given child window can be added to that group (that
         * is if the child window matches the filer) or not.
         * 
         * @param childWindow
         *            the child window that wants to be added to the group
         * @return true if the child window can be added, false otherwise
         */
        public boolean canAddChildWindow(IChildWindow childWindow)
        {
            return filter_.isAssignableFrom(childWindow.getClass());
        }

        /**
         * Adds an open tabbed pane to the group. Only visible and open tabbed
         * panes must be added to the group. If a tabbed pane is close, use
         * removeTabbedPane to remove the tabbed pane from the group.
         * 
         * @param tabbedPane
         *            the tabbed pane that is opened and should be added to the
         *            group
         */
        public void addTabbedPane(ExtendedTabbedPane tabbedPane)
        {
            tabbedPanes_.add(tabbedPane);
        }

        /**
         * Removes a tabbed pane from the group that was closed and is no longer
         * visible. Only remove closed tabbed panes from the group.
         * 
         * @param tabbedPane
         *            the closed tabbed pane that should be removed from the
         *            group
         */
        public void removeTabbedPane(ExtendedTabbedPane tabbedPane)
        {
            tabbedPanes_.remove(tabbedPane);
        }

        /**
         * Returns the count of tabbed panes assigned to this group. This number
         * is equal to the count of open tabbed panes, that are currently open.
         * 
         * @return the count of tabbed panes in the group
         */
        public int getTabbedPaneCount()
        {
            return tabbedPanes_.size();
        }

        /**
         * Returns an open tabbed pane of the group or null if no tabbed pane is
         * open. If more than one tabbed pane is open, the returned tabbed pane
         * is random.
         * 
         * @return an open tabbed pane that is in this group
         */
        public ExtendedTabbedPane getOpenTabbedPane()
        {
            if (getTabbedPaneCount() > 0)
            {
                return (ExtendedTabbedPane) tabbedPanes_.iterator().next();
            }

            return null;
        }

        public void clear()
        {
            tabbedPanes_.clear();
        }

        public boolean containsTabbedPane(ExtendedTabbedPane tabbedPane)
        {
            return tabbedPanes_.contains(tabbedPane);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner class for our own TabbedPane
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This class extends the JTabbedPane for use with grouping. It has only two
     * additional flags to remember the assigned group and two methods to open
     * or close them.
     */
    private class ExtendedTabbedPane extends JTabbedPane
    {
        // the group this tabbed pane is in
        private Group group_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Creates a new tabbed pane and assigns it to the given group.
         * 
         * @param group
         *            the group to which this tabbed should belong to
         */
        public ExtendedTabbedPane(Group group)
        {
            super();
            setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
            group_ = group;
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // property methods
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Returns the assigned group.
         * 
         * @return assigned group
         */
        public Group getGroup()
        {
            return group_;
        }

        /**
         * Marks the tabbed pane as opened. That means that the tabbed pane is
         * added to its assigned group.
         */
        public void open()
        {
            group_.addTabbedPane(this);
        }

        /**
         * Marks the tabbed panse as closed. That means that the tabbed pane is
         * removed from its assigned group.
         */
        public void close()
        {
            group_.removeTabbedPane(this);
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // inserting tabs
        // <><><><><><><><><><><><><><><><><><><><><>

        public void addTab(IChildWindow childWindow)
        {
            super.addTab(childWindow.getTitle(), childWindow.getComponent());

            setTabComponentAt(indexOfComponent(childWindow.getComponent()),
                    getTab(childWindow));
        }

        public void insertTab(IChildWindow childWindow, int index)
        {
            super.insertTab(childWindow.getTitle(), childWindow.getIcon(),
                    childWindow.getComponent(), childWindow.getToolTipText(),
                    index);
            setTabComponentAt(index, getTab(childWindow));
        }

        private Tab getTab(IChildWindow childWindow)
        {
            Tab tab;

            tab = tabs_.get(childWindow);
            if (tab == null)
            {
                tab = new Tab(childWindow);
            }

            return tab;
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner class for the dragging window
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This class is the drag window that is created when the user starts a drag
     * operation.
     */
    private class DragWindow extends JComponent
    {
        // the index of the tab in the tabbed pane where the operation started
        private int                startIndex_;

        // the tabbed pane that contains the dragged tab
        private ExtendedTabbedPane startPane_;

        // the original glass pane. Is restored after the drag operation
        private Component          glassPane_;

        // the rectangle that is displayed to give the user feedback about
        // his operastion
        private DragRectangle      dragRectangle_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // inner class that manages the rectangle that
        // is displayed on the glass pane to give the
        // user a feedback
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * This class is responsible for calculating the rectangle while the
         * user is performing a drag operation.
         */
        private class DragRectangle
        {
            // the rectangle that is actually displayed on the glass pane
            private Rectangle          rectangle_;

            // the original rectangle of the tab that is dragged
            private Rectangle          tabRectangle_;

            // true if a split operation has to be done
            private boolean            splitting_;

            // true if the frame has to be split
            private boolean            splitFrame_;

            // true if the user has made a valid drag operation, that is when
            // moving and splitting has to be done
            private boolean            validDrag_;

            // the side where the tab should be placed after splitting
            // Values are taken from JTabbedPane: TOP, BOTTOM, LEFT, RIGHT
            private int                splitSide_;

            // the pane where the tab should move to
            private ExtendedTabbedPane insertPane_;

            // index where the tab should be inserted in the pane. Set to -1
            // to indicate that the tab should be inserted at the end
            private int                insertIndex_;

            // the distance between the mouse coordiniates and the left upper
            // corner of the tab rectangle. This should avoid jumping of the
            // rectangle around the cursor
            private int                deltaX_;

            private int                deltaY_;

            // size of the area where the program assume that the user want to
            // perform a frame splitting operation
            private int                frameSplitSize_ = 10;

            // <><><><><><><><><><><><><><><><><><><><><>
            // constructors
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * Constructs a new drag rectangle.
             * 
             * @param tabRectangle
             *            The original rectangle of the tab that has to be
             *            dragged. The coordinates have to be relative to the
             *            drag window.
             * @param mousePoint
             *            The actual position of the mouse cursor. Need to
             *            calculate the distance between the cursor and the
             *            upper-left corner of the rectangle. Must be given
             *            relative to the drag window
             */
            public DragRectangle(Rectangle tabRectangle, Point mousePoint)
            {
                splitting_ = false;
                splitFrame_ = false;
                validDrag_ = false;
                splitSide_ = 0;
                insertPane_ = null;
                insertIndex_ = -1;
                tabRectangle_ = tabRectangle;
                rectangle_ = (Rectangle) tabRectangle.clone();

                deltaX_ = mousePoint.x - tabRectangle.x;
                deltaY_ = mousePoint.y - tabRectangle.y;
            }

            // <><><><><><><><><><><><><><><><><><><><><>
            // state methods
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * Returns the state of the current drag operation.
             * 
             * @return true if current drag operation is valid, false otherwise
             */
            public boolean isValidDrag()
            {
                return validDrag_;
            }

            /**
             * Returns the tabbed pane where the tab should be inserted. Returns
             * null if the mouse was not released over a tabbed pane
             * 
             * @return the tabbed pane where to insert the tab
             */
            public ExtendedTabbedPane getInsertPane()
            {
                return insertPane_;
            }

            /**
             * Returns the position for the dragged tab. Returns -1 if the tab
             * should be inserted at the end of the tab list.
             * 
             * @return index where to insert the tab in the new tabbed pane
             */
            public int getInsertIndex()
            {
                return insertIndex_;
            }

            /**
             * Returns true if the user want to do a splitting operation.
             * 
             * @return true, if the user want to split a window, false otherwise
             */
            public boolean isSplitting()
            {
                return splitting_;
            }

            /**
             * Returns true if the user want to do a frame split. Attention:
             * Although this flag is true, isSplitting() can return false. This
             * flag is only valid if isSplitting() returns true.
             * 
             * @return true, if to split the frame, false otherwise
             */
            public boolean splitFrame()
            {
                return splitFrame_;
            }

            /**
             * Returns the side where to add the dragged window. Uses the
             * constants of JTabbedPane: TOP, BOTTOM, LEFT, RIGHT
             * 
             * @return the side where to add the dragged tab
             */
            public int getSplitSide()
            {
                return splitSide_;
            }

            // <><><><><><><><><><><><><><><><><><><><><>
            // renew rectangle
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * Paints the current rectangle with use of the specified Graphics
             * object
             * 
             * @param gc
             *            the graphics object to use for painting
             */
            public void paintRectangle(Graphics gc)
            {
                if (rectangle_ != null)
                {
                    gc.drawRect(rectangle_.x, rectangle_.y, rectangle_.width,
                            rectangle_.height);
                }
            }

            /**
             * restores the rectangle of the dragged tab and moves it to the
             * specified position
             * 
             * @param toPoint
             *            the new position relative to the drag window
             */
            private void moveRectangle(Point toPoint)
            {
                validDrag_ = false;
                splitting_ = false;

                // use the stored distance to avoid jumping of the rectangle
                rectangle_.x = toPoint.x - deltaX_;
                rectangle_.y = toPoint.y - deltaY_;

                rectangle_.width = tabRectangle_.width;
                rectangle_.height = tabRectangle_.height;

                // update the cursor
                DragWindow.this.setCursor(ResourceLoader.getInstance().getCursor(
                        ResourceConstants.CURSOR_DRAG_CHILDWINDOW));
            }

            // <><><><><><><><><><><><><><><><><><><><><>
            // detecting mouse movement and position
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * Called if the mouse moved. This method calculates the rectangle
             * to display and tests wheter the user want to move a tab or split.
             * 
             * @param mousePoint
             *            position of the cursor relative to to the drag window
             */
            public void mouseMoved(Point mousePoint)
            {
                Point pointInPane;
                Component component;
                IChildWindow child;
                Rectangle rectangle;
                ExtendedTabbedPane pane;

                if (mousePoint == null)
                    return;

                // When the mouse moved we have to do several test to determine
                // if the user want to move a tab to a new pane or to split
                // some window.
                //
                // General: When the user is in the near of our border
                // assume that he wants a frame split. Do this test
                // first.
                // JTabbedPane: This case is quite easy. If the user is over a
                // tab assume that he wants to insert the dragged
                // tab before the tab under the mouse. If the mouse
                // is over a rectangle behind the last tab assume
                // that he wants to add the tab at the end of the
                // list. We have to respect one special case: If
                // the user near to the frame border, assume a
                // frame splitting
                // ChildWindow: In this case we have to test if the user want to
                // do a splitting operation. So we have to test
                // where the mouse in the child window points to.
                // If the cursor matches specific regions within
                // the child window assume a splitting operation

                // To test if the mouse is near the border of our pane we need
                // the rectangle of the pane. We have to convert the coordinates
                // so that they are relative to glass pane
                rectangle = SwingUtilities.convertRectangle(
                        ExtendedPane.this.getParent(),
                        ExtendedPane.this.getBounds(), DragWindow.this);

                // convert the new position into our own coordinates. Since the
                // focus stays by the pane where the drag operation has started
                // all mouse messages was sent from this pane and all
                // coordinates are relative to that pane. So we have to use the
                // start pane to convert the coordinates
                mousePoint = SwingUtilities.convertPoint(startPane_,
                        mousePoint, DragWindow.this);

                // test every side and create a rectangle to show the user he is
                // near the border of our pane and would perform an splitting
                // operation
                // upper side
                if (mousePoint.y < rectangle.y + frameSplitSize_)
                {
                    rectangle_.x = rectangle.x;
                    rectangle_.y = rectangle.y;
                    rectangle_.width = rectangle.width - 1;
                    rectangle_.height = frameSplitSize_;

                    activateSplitting(JTabbedPane.TOP, true);
                    return;
                }
                // lower side
                else if (mousePoint.y > rectangle.getMaxY() - frameSplitSize_)
                {
                    rectangle_.x = rectangle.x;
                    rectangle_.y = (int) rectangle.getMaxY() - frameSplitSize_;
                    rectangle_.width = rectangle.width - 1;
                    rectangle_.height = frameSplitSize_ - 1;

                    activateSplitting(JTabbedPane.BOTTOM, true);
                    return;
                }
                // right side
                else if (mousePoint.x > rectangle.getMaxX() - frameSplitSize_)
                {
                    rectangle_.x = (int) rectangle.getMaxX() - frameSplitSize_;
                    rectangle_.y = rectangle.y;
                    rectangle_.width = frameSplitSize_ - 1;
                    rectangle_.height = rectangle.height - 1;

                    activateSplitting(JTabbedPane.RIGHT, true);
                    return;
                }
                // left side
                else if (mousePoint.x < rectangle.x + frameSplitSize_)
                {
                    rectangle_.x = rectangle.x;
                    rectangle_.y = rectangle.y;
                    rectangle_.width = frameSplitSize_ - 1;
                    rectangle_.height = rectangle.height - 1;

                    activateSplitting(JTabbedPane.LEFT, true);
                    return;
                }

                // if the user is not in the near of a border he might want to
                // move the tab to another tabbed pane or to split a tabbed pane
                // We have to determine the component under the mouse pointer.
                // We use our extended pane to find it out. To be successful by
                // calling findComponentAt we have to use coordinates that are
                // relative to the extended pane. So first do convert the
                // coordinates
                pointInPane = SwingUtilities.convertPoint(DragWindow.this,
                        mousePoint, ExtendedPane.this);
                // use the coordinates to find the visible component that is
                // under the cursor
                component = ExtendedPane.this.findComponentAt(pointInPane);
                if (component instanceof ExtendedTabbedPane)
                {
                    mouseOverTabbedPane(mousePoint,
                            (ExtendedTabbedPane) component);
                }
                else if (component.getParent() instanceof ExtendedTabbedPane)
                {
                    mouseOverTabbedPane(mousePoint,
                            (ExtendedTabbedPane) component.getParent());
                }
                // since the visible component can be a child of a child window
                // we must recursivly search the parents of the found component
                // until we have found th child window. If the return is null,
                // the cursor is not over a child window
                else if ((child = ExtendedPane.this.findChildWindow(component)) != null)
                {
                    dragRectangle_.mouseOverComponent(mousePoint,
                            child.getComponent());
                }
                else if ((pane = findTabbedPane(component)) != null)
                {
                    mouseOverTabbedPane(mousePoint, pane);
                }
                else
                {
                    moveRectangle(mousePoint);
                }
            }

            /**
             * Calculates the rectangle if the mouse is over a tabbed pane.
             * 
             * @param mousePoint
             *            position of the cursor relative to the drag window
             * @param pane
             *            the tabbed pane under the mouse cursor
             */
            private void mouseOverTabbedPane(Point mousePoint,
                    ExtendedTabbedPane pane)
            {
                int index;
                Point pointOnPane;
                Rectangle pseudoRect;
                Component dragComponent;

                dragComponent = startPane_.getComponentAt(startIndex_);
                // test whether the mouse is over a tab. convert the point into
                // the coordinates relative to the pane at get the index of the
                // tab at that point. If the index is -1 the mouse is not over
                // a tab.
                pointOnPane = SwingUtilities.convertPoint(DragWindow.this,
                        mousePoint, pane);
                index = pane.indexAtLocation(pointOnPane.x, pointOnPane.y);

                // if the mouse is over a tabbed pane, the user don't want to
                // split
                splitting_ = false;

                // update the cursor, otherwise the splitcursor will be
                // displayed
                // if you move the mouse from upper splitframe position directly
                // onto the highest tabbedPane
                DragWindow.this.setCursor(ResourceLoader.getInstance().getCursor(
                        ResourceConstants.CURSOR_DRAG_CHILDWINDOW));

                if (index != -1)
                {
                    // we are over a tab. Assume the user want to insert the
                    // dragged tab at this index and draw the rectangle over the
                    // the current tab to show the "result"
                    // Be carefull, the coordinates of the rectangle are
                    // relative to the pane and must be converted
                    rectangle_ = SwingUtilities.convertRectangle(pane,
                            pane.getBoundsAt(index), DragWindow.this);

                    insertIndex_ = index;
                    insertPane_ = pane;
                    validDrag_ = true;
                }
                else
                {
                    // perhaps the user want to add the tab and the end of the
                    // list. Calculate the rectangle of a additional tab. Get
                    // the rectangle of the last tab and move it to the end.
                    // After that test whether the mouse is in the pseudo tab.
                    pseudoRect = pane.getBoundsAt(pane.getTabCount() - 1);

                    // Be carefull about the orientation of the tabs
                    switch (pane.getTabPlacement())
                    {
                    case JTabbedPane.LEFT:
                    case JTabbedPane.RIGHT:
                        pseudoRect.y += pseudoRect.height;
                        break;

                    case JTabbedPane.TOP:
                    case JTabbedPane.BOTTOM:
                        pseudoRect.x += pseudoRect.width;
                        break;
                    }

                    // if the mouse is inside the rectangle asssume the user
                    // want to add the tab at the end of the list
                    if (pseudoRect.contains(pointOnPane))
                    {
                        rectangle_ = SwingUtilities.convertRectangle(pane,
                                pseudoRect, DragWindow.this);

                        insertIndex_ = -1;
                        insertPane_ = pane;
                        validDrag_ = true;
                    }
                    else
                    {
                        // the user is elswhere on the pane. Just move it
                        moveRectangle(mousePoint);

                        insertIndex_ = -1;
                        insertPane_ = null;
                        validDrag_ = false;
                    }
                }

                // test whether the insert pane can get the new window
                if (insertPane_ != null)
                {
                    if (!insertPane_.getGroup().canAddChildWindow(
                            findChildWindow(dragComponent)))
                    {
                        insertIndex_ = -1;
                        insertPane_ = null;
                        validDrag_ = false;
                    }
                }
            }

            /**
             * Calculates the rectangle if the mouse is over a component
             * 
             * @param mousePoint
             *            position of the cursor relative to the drag window
             * @param component
             *            the component under the mouse cursor
             */
            private void mouseOverComponent(Point mousePoint,
                    Component component)
            {
                Point pointInComponent;
                Rectangle componentRect;

                // first convert the point into component coordinates. So we can
                // easily test if the point lies in a specific region
                pointInComponent = SwingUtilities.convertPoint(DragWindow.this,
                        mousePoint, component);

                // lower third
                if (pointInComponent.y > component.getHeight() * 2 / 3)
                {
                    // create a rectangle that is displayed in the lower third
                    // of the component to show the user that he can release the
                    // mouse button to perform a splitting operation
                    componentRect = SwingUtilities.convertRectangle(
                            component.getParent(), component.getBounds(),
                            DragWindow.this);

                    rectangle_.x = componentRect.x;
                    rectangle_.y = componentRect.y + componentRect.height * 2
                            / 3;
                    rectangle_.width = componentRect.width;
                    rectangle_.height = componentRect.height / 3;

                    // Assume the user wants to split vertical and move the
                    // tab to the lower split child.
                    activateSplitting(JTabbedPane.BOTTOM, false);
                    insertPane_ = (ExtendedTabbedPane) component.getParent();
                }
                // upper third
                else if (pointInComponent.y < component.getHeight() / 3)
                {
                    componentRect = SwingUtilities.convertRectangle(
                            component.getParent(), component.getBounds(),
                            DragWindow.this);

                    rectangle_.x = componentRect.x;
                    rectangle_.y = componentRect.y;
                    rectangle_.width = componentRect.width;
                    rectangle_.height = componentRect.height / 3;

                    activateSplitting(JTabbedPane.TOP, false);
                    insertPane_ = (ExtendedTabbedPane) component.getParent();
                }
                // rigth third
                else if (pointInComponent.x > component.getWidth() * 2 / 3)
                {
                    componentRect = SwingUtilities.convertRectangle(
                            component.getParent(), component.getBounds(),
                            DragWindow.this);

                    rectangle_.x = componentRect.x + componentRect.width * 2
                            / 3;
                    rectangle_.y = componentRect.y;
                    rectangle_.width = componentRect.width / 3;
                    rectangle_.height = componentRect.height;

                    activateSplitting(JTabbedPane.RIGHT, false);
                    insertPane_ = (ExtendedTabbedPane) component.getParent();
                }
                // left third
                else if (pointInComponent.x < component.getWidth() / 3)
                {
                    componentRect = SwingUtilities.convertRectangle(
                            component.getParent(), component.getBounds(),
                            DragWindow.this);

                    rectangle_.x = componentRect.x;
                    rectangle_.y = componentRect.y;
                    rectangle_.width = componentRect.width / 3;
                    rectangle_.height = componentRect.height;

                    activateSplitting(JTabbedPane.LEFT, false);
                    insertPane_ = (ExtendedTabbedPane) component.getParent();
                }
                else
                {
                    moveRectangle(mousePoint);

                    insertIndex_ = -1;
                    insertPane_ = null;
                }
            }

            // <><><><><><><><><><><><><><><><><><><><><>
            // supporting methods
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * activates the splitting mode. Just sets some flags and updates
             * the cursor in an arrow style.
             * 
             * @param splitSide
             *            the side to split. Uses JTabbedPane constants TOP,
             *            BOTTOM, LEFT and RIGHT
             * @param splitFrame
             *            true if to split the frame, false otherwise
             */
            private void activateSplitting(int splitSide, boolean splitFrame)
            {
                splitting_ = true;
                splitFrame_ = splitFrame;
                splitSide_ = splitSide;
                validDrag_ = true;

                // Tests which sort of arrow is needed, so to speak in which
                // direction the split has to be done
                if (splitSide_ == JTabbedPane.BOTTOM)
                { // update the cursor
                    DragWindow.this.setCursor(ResourceLoader.getInstance().getCursor(
                            ResourceConstants.CURSOR_SPLITBOTTOM));
                    return;
                }
                else if (splitSide_ == JTabbedPane.TOP)
                { // update the cursor
                    DragWindow.this.setCursor(ResourceLoader.getInstance().getCursor(
                            ResourceConstants.CURSOR_SPLITTOP));
                    return;
                }
                else if (splitSide_ == JTabbedPane.RIGHT)
                { // update the cursor
                    DragWindow.this.setCursor(ResourceLoader.getInstance().getCursor(
                            ResourceConstants.CURSOR_SPLITRIGHT));
                    return;
                }
                else if (splitSide_ == JTabbedPane.LEFT)
                { // update the cursor
                    DragWindow.this.setCursor(ResourceLoader.getInstance().getCursor(
                            ResourceConstants.CURSOR_SPLITLEFT));
                }
            }
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Creates a new drag window and sets it visibility to false.
         */
        public DragWindow()
        {
            // the drag pane is visible after initialisation so we have to hide
            // it after construction. Otherwise isStarted() returns true.
            setVisible(false);
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // methods for dragging
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * The BeginDrag method is called to start a drag operation. Only tabs
         * from JTabbedPanes can be dragged and moved to other positions. The
         * method BeginDrag ensures not the start of the operation. If the
         * coordinates specified by mousePoint does not point on a tab specified
         * by pane, the dragging operation will not start.<br>
         * isStarted() can be called to determine whether the operation has
         * started or not.<br>
         * The coordinates must be relative to the specified pane.
         * 
         * @param mousePoint
         *            the coordinates of the current cursor position. The
         *            coordinates have to be relative to the pane. If the point
         *            is not contained by any tab, the operation is canceled by
         *            the method.
         * @param pane
         *            The pane where the user has clicked on. Must not be null
         *            otherwise the operation is canceled.
         */
        public void BeginDrag(Point mousePoint, ExtendedTabbedPane pane)
        {
            if (mousePoint == null || pane == null)
                return;

            // Set this component as glass pane first. The glass pane is used
            // to draw the rectangle that is moved by the user. The glass pane
            // is transparent and only displays things drawed in paintComponent.
            // We must call setGlassPane before all SwingUtilities.convert...
            // methods to be sure that coordinates are transformed correctly
            glassPane_ = ExtendedPane.this.getRootPane().getGlassPane();
            ExtendedPane.this.getRootPane().setGlassPane(this);

            // get the index of the tab that the user wishes to drag. If the
            // user has not clicked on a tab we can cancel the operation
            startIndex_ = pane.indexAtLocation(mousePoint.x, mousePoint.y);
            if (startIndex_ < 0)
                return;

            // store the pane that hosts the dragged tab and create a new drag
            // rectangle that is displayed on the glass pane and should show
            // the user where he moved the tab. The new drag rectangle must be
            // created with coordinates relative to the glass pane
            startPane_ = pane;
            dragRectangle_ = new DragRectangle(SwingUtilities.convertRectangle(
                    pane, pane.getBoundsAt(startIndex_), this),
                    SwingUtilities.convertPoint(pane, mousePoint, this));

            // show our glass pane to make the rectangle visible
            setVisible(true);
        }

        /**
         * This method ends the current dragging operation. If no dragging has
         * started the method just returns. If the user has moved the rectangle
         * to a valid destination the methods do the move and splitting if
         * necessary. After the panes has been relayouted the original glass
         * pane will be restored and our window will be hide.
         */
        public void EndDrag()
        {
            IChildWindow child;
            Component component;

            if (!isStarted())
                return;

            // if the user made a valid drag operation we have to move or split
            // a tab
            if (dragRectangle_.isValidDrag())
            {
                component = startPane_.getComponentAt(startIndex_);
                child = (IChildWindow) childWindows_.get(component);

                if (dragRectangle_.isSplitting())
                {
                    // split
                    switch (dragRectangle_.getSplitSide())
                    {
                    case JTabbedPane.BOTTOM:
                        split(
                                dragRectangle_.splitFrame() ? ExtendedPane.this.getComponent(0)
                                        : dragRectangle_.getInsertPane(),
                                child, false, true);
                        break;

                    case JTabbedPane.TOP:
                        split(
                                dragRectangle_.splitFrame() ? ExtendedPane.this.getComponent(0)
                                        : dragRectangle_.getInsertPane(),
                                child, false, false);
                        break;

                    case JTabbedPane.LEFT:
                        split(
                                dragRectangle_.splitFrame() ? ExtendedPane.this.getComponent(0)
                                        : dragRectangle_.getInsertPane(),
                                child, true, false);
                        break;

                    case JTabbedPane.RIGHT:
                        split(
                                dragRectangle_.splitFrame() ? ExtendedPane.this.getComponent(0)
                                        : dragRectangle_.getInsertPane(),
                                child, true, true);
                        break;
                    }
                }
                else
                {
                    // just move
                    moveChildWindow(child, dragRectangle_.getInsertPane(),
                            dragRectangle_.getInsertIndex());
                }

                // select the dragged tab
                selectWindow(child);
            }

            setVisible(false);
            // restore the original glass pane
            ExtendedPane.this.getRootPane().setGlassPane(glassPane_);
        }

        /**
         * Determines if a dragging operation has started. Since this is
         * 
         * @return
         */
        public boolean isStarted()
        {
            // the dragging operations has started when we are visible
            return isVisible();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // overriden methods from javax.swing.JComponent
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Paints the rectangle. Set the line style and color and calls the
         * paint method of our drag rectangle.
         * 
         * @param gc
         *            the graphics object that is used to pain the component
         */
        protected void paintComponent(Graphics gc)
        {
            // show a nice red rectangle
            if (dragRectangle_.isValidDrag())
            {
                gc.setColor(Color.GREEN);
            }
            else
            {
                gc.setColor(Color.RED);
            }

            if (dragRectangle_ != null)
                dragRectangle_.paintRectangle(gc);
        }

        /**
         * Drags the rectangle to the given point and repaint it. The call is
         * ignored if no drag operation was started.
         * 
         * @param dragTo
         *            the point to which the drag rectangle should be moved
         */
        public void drag(Point dragTo)
        {
            // if the drag operation was not started, ignore the call
            if (!isStarted())
                return;

            // move the rectangle
            dragRectangle_.mouseMoved(dragTo);
            repaint();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner class for tab rendering
    // <><><><><><><><><><><><><><><><><><><><><>
    private class Tab extends JComponent implements MouseMotionListener,
            MouseListener, IExtendedPaneListener
    {
        private Font         defaultFont_;
        private Font         selectedFont_;
        private IChildWindow childWindow_;
        private JLabel       label_;
        private JLabel       closeLabel_;

        public Tab(IChildWindow childWindow)
        {
            childWindow_ = childWindow;

            label_ = new JLabel(childWindow.getTitle(), childWindow.getIcon(),
                    SwingConstants.LEFT);
            label_.setToolTipText(childWindow.getToolTipText());
            label_.addMouseListener(this);
            label_.addMouseMotionListener(this);

            closeLabel_ = new JLabel(ResourceLoader.getInstance().getIcon(
                    ResourceConstants.ICON_CLOSEINACTIVE));
            closeLabel_.addMouseMotionListener(this);
            closeLabel_.addMouseListener(this);

            setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
            add(label_);

            if (!(childWindow instanceof INonCloseableChildWindow))
            {
                add(Box.createHorizontalStrut(5));
                add(closeLabel_);
            }

            selectedFont_ = label_.getFont().deriveFont(Font.BOLD);
            defaultFont_ = label_.getFont().deriveFont(Font.PLAIN);
            setFont(defaultFont_);

            tabs_.put(childWindow, this);
            addExtendedPaneListener(this);
        }

        private void redispatchMouseEvent(MouseEvent event)
        {
            ExtendedTabbedPane pane;

            if (event.getComponent() == closeLabel_)
            {
                // don't redispatch close label events
                return;
            }

            pane = findTabbedPane(event.getComponent());
            pane.dispatchEvent(SwingUtilities.convertMouseEvent(
                    event.getComponent(), event, pane));
        }

        public void setForeground(Color color)
        {
            label_.setForeground(color);
        }

        public Dimension getPreferredSize()
        {
            Dimension dim = super.getPreferredSize();
            return new Dimension(dim.width, 18);
        }

        public Dimension getMinimumSize()
        {
            return getPreferredSize();
        }

        public void mouseClicked(MouseEvent event)
        {
            if (event.getSource() == closeLabel_)
            {
                fireWindowCloseEvent(childWindow_);
            }

            redispatchMouseEvent(event);
        }

        public void mouseEntered(MouseEvent event)
        {
            if (event.getSource() == closeLabel_)
            {
                closeLabel_.setIcon(ResourceLoader.getInstance().getIcon(
                        ResourceConstants.ICON_CLOSE));
            }

            redispatchMouseEvent(event);
        }

        public void mouseExited(MouseEvent event)
        {
            if (event.getSource() == closeLabel_)
            {
                closeLabel_.setIcon(ResourceLoader.getInstance().getIcon(
                        ResourceConstants.ICON_CLOSEINACTIVE));
            }

            redispatchMouseEvent(event);
        }

        public void mouseDragged(MouseEvent event)
        {
            redispatchMouseEvent(event);
        }

        public void mouseMoved(MouseEvent event)
        {
            redispatchMouseEvent(event);
        }

        public void mousePressed(MouseEvent event)
        {
            redispatchMouseEvent(event);
        }

        public void mouseReleased(MouseEvent event)
        {
            redispatchMouseEvent(event);
        }

        public void activeWindowChanged(ActiveWindowChangeEvent event)
        {
            if (event.getActiveWindow() == childWindow_)
            {
                label_.setFont(selectedFont_);
            }
            else
            {
                label_.setFont(defaultFont_);
            }
        }

        public void windowClose(WindowCloseEvent event)
        {
            tabs_.remove(event.getWindowToClose());
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner class for ChildWindow selection listener
    // <><><><><><><><><><><><><><><><><><><><><>
    private class ExtendedComponentListener implements FocusListener,
            MouseListener, ContainerListener
    {
        public ExtendedComponentListener(Component component)
        {
            if (component instanceof ExtendedTabbedPane)
            {
                // the tabbed pane needs only the mouse listener
                component.addFocusListener(this);
            }
            else if (component instanceof Container)
            {
                // add all listeners to a container
                addListenersToComponents((Container) component);
            }
            else
            {
                // a normal component needs the focus and mouse listener
                component.addFocusListener(this);
                component.addMouseListener(this);
            }
        }

        /**
         * This method enables the user to add Listeners to all Components
         * inside the frame, at the moment only the FocusListener is important
         * 
         * @param component
         *            the component, to which the Listener should be added or if
         *            it is a Container, to whose Components the Listener should
         *            be added
         */
        private void addListenersToComponents(Container container)
        {
            // add the Listener
            container.addFocusListener(this);
            container.addMouseListener(this);
            container.addContainerListener(this);

            // If there may be Components, find them all and recall this method
            // for everyone of them
            for (int i = 0; i < container.getComponentCount(); i++)
            {
                new ExtendedComponentListener(container.getComponent(i));
            }
        }

        private void selectChildWindow(ComponentEvent event)
        {
            ExtendedTabbedPane pane;
            IChildWindow child;

            // Case that the Component is superior (has to be a
            // ExtendedTabbedPane)
            if (event.getComponent() instanceof ExtendedTabbedPane)
            {
                pane = (ExtendedTabbedPane) event.getComponent();
                child = (IChildWindow) findChildWindow(pane.getSelectedComponent());
            }
            // Case that the Component is subordinate to a ChildWindow
            else
            {
                child = findChildWindow(event.getComponent());
            }

            // notify our listeners that the active tab has changed
            activeComponentChanged(child);
        }

        public void mouseClicked(MouseEvent event)
        {
            selectChildWindow(event);
        }

        public void mouseEntered(MouseEvent event)
        {
        }

        public void mouseExited(MouseEvent event)
        {
        }

        public void mousePressed(MouseEvent event)
        {
        }

        public void mouseReleased(MouseEvent event)
        {
        }

        public void componentAdded(ContainerEvent event)
        {
            new ExtendedComponentListener(event.getChild());
        }

        public void componentRemoved(ContainerEvent event)
        {
        }

        public void focusGained(FocusEvent event)
        {
            selectChildWindow(event);
        }

        public void focusLost(FocusEvent event)
        {
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new extended pane. This pane is a very flexible window
     * layout panel that manages child windows. Every child window is displayed
     * in a tab. Felixible splitting is implemented as well as moving tabs.
     */
    public ExtendedPane()
    {
        this("", Object.class, true);
    }

    /**
     * Creates a new extended pane with the posibility to name and filter the
     * default group.
     * 
     * @param defaultGroup
     *            name of the default group
     * @param defaultFilter
     *            filter for the default group
     * @param closeable
     *            indicates wheter the new group should be closable or not
     */
    public ExtendedPane(String defaultGroup, Class defaultFilter,
            boolean closeable)
    {
        // create with the border layout, so that the child component (tabbed
        // pane or split pane) is resized to the size of the JPanel)
        super(new BorderLayout());

        defaultGroup_ = defaultGroup;

        // create the first tabbed pane that will fill the whole window
        // and add it
        lastActiveTabbedPane_ = createTabbedPane(createInternalGroup(
                defaultGroup, defaultFilter, closeable));
        add(lastActiveTabbedPane_, BorderLayout.CENTER);

        lastActiveTabbedPane_.open();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // methods to update displayed windows
    // <><><><><><><><><><><><><><><><><><><><><>
    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#updateToolTipText(viptool
     * .base.extendedPane.IChildWindow)
     */
    public void updateToolTipText(IChildWindow childToBeUpdated)
    {
        String newToolTipText;
        ExtendedTabbedPane pane;

        newToolTipText = childToBeUpdated.getToolTipText();
        pane = (ExtendedTabbedPane) childToBeUpdated.getComponent().getParent();

        pane.setToolTipTextAt(
                pane.indexOfComponent(childToBeUpdated.getComponent()),
                newToolTipText);
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#updateTitle(viptool.base
     * .extendedPane.IChildWindow)
     */
    public void updateTitle(IChildWindow childToBeUpdated)
    {
        String newTitle;
        ExtendedTabbedPane pane;

        newTitle = childToBeUpdated.getTitle();
        pane = (ExtendedTabbedPane) childToBeUpdated.getComponent().getParent();

        pane.setTitleAt(pane.indexOfComponent(childToBeUpdated.getComponent()),
                newTitle);

        validate();
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#updateIcon(viptool.base.
     * extendedPane.IChildWindow)
     */
    public void updateIcon(IChildWindow childToBeUpdated)
    {
        Icon newIcon;
        ExtendedTabbedPane pane;

        newIcon = childToBeUpdated.getIcon();
        pane = (ExtendedTabbedPane) childToBeUpdated.getComponent().getParent();

        pane.setIconAt(pane.indexOfComponent(childToBeUpdated.getComponent()),
                newIcon);

        validate();
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#updateWindow(viptool.base
     * .extendedPane.IChildWindow)
     */
    public void updateWindow(IChildWindow childToBeUpdated)
    {
        updateToolTipText(childToBeUpdated);
        updateTitle(childToBeUpdated);
        updateIcon(childToBeUpdated);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // grouping
    // <><><><><><><><><><><><><><><><><><><><><>
    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#createGroup(java.lang.String
     * , java.lang.Class, boolean)
     */
    public void createGroup(String name, Class filter, boolean closeable)
    {
        createInternalGroup(name, filter, closeable);
    }

    /**
     * * Creates a new group with the given name and the given filter. The third
     * parameter can be used to mark the group as non closeable. A non closeable
     * group has always an open tabbed pane.
     * 
     * @param name
     *            name of the group to create
     * @param filter
     *            filter for the group
     * @param closeable
     *            set to true to mark an group as non-closeable, false otherwise
     * @return the created group
     */
    private Group createInternalGroup(String name, Class filter,
            boolean closeable)
    {
        Group group;

        if ((group = groups_.get(name)) != null)
        {
            return group;
        }

        group = new Group(name, filter, closeable);
        groups_.put(name, group);

        return group;
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#openGroup(java.lang.String,
     * int)
     */
    public void openGroup(String name, int position)
    {
        Group group;
        ExtendedTabbedPane tabbedPane;

        group = (Group) groups_.get(name);
        if (group == null)
        {
            return;
        }
        tabbedPane = createTabbedPane(group);

        if (activeWindow_ == null && position > 3)
        {
            position -= 4;
        }

        switch (position)
        {
        case POSITION_FRAME_BOTTOM:
            split(getComponent(0), tabbedPane, false, true);
            break;

        case POSITION_FRAME_LEFT:
            split(getComponent(0), tabbedPane, true, false);
            break;

        case POSITION_FRAME_RIGHT:
            split(getComponent(0), tabbedPane, true, true);
            break;

        case POSITION_ACTIVE_TOP:
            split(activeWindow_.getComponent().getParent(), tabbedPane, false,
                    false);
            break;

        case POSITION_ACTIVE_BOTTOM:
            split(activeWindow_.getComponent().getParent(), tabbedPane, false,
                    true);
            break;

        case POSITION_ACTIVE_LEFT:
            split(activeWindow_.getComponent().getParent(), tabbedPane, true,
                    false);
            break;

        case POSITION_ACTIVE_RIGHT:
            split(activeWindow_.getComponent().getParent(), tabbedPane, true,
                    true);
            break;

        case POSITION_FRAME_TOP:
        default:
            split(getComponent(0), tabbedPane, false, false);
            break;
        }

        tabbedPane.open();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Manipulating the members (child windows)
    // add, remove, replace, move
    // <><><><><><><><><><><><><><><><><><><><><>
    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#addChildWindow(viptool.base
     * .extendedPane.IChildWindow)
     */
    public void addChildWindow(IChildWindow childWindow)
            throws IllegalArgumentException
    {
        String group;

        group = findMatchingGroup(childWindow);
        addChildWindow(group, childWindow);
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#addChildWindow(java.lang
     * .String, viptool.base.extendedPane.IChildWindow)
     */
    public void addChildWindow(String group, IChildWindow childWindow)
            throws IllegalArgumentException
    {
        Group insertGroup;
        ExtendedTabbedPane pane;

        if (childWindow.getComponent() == null)
        {
            throw new IllegalArgumentException("The child windows component "
                    + "must not be null!");
        }
        if (childWindow.getTitle() == null)
        {
            throw new IllegalArgumentException("The child windows title must "
                    + "not be null!");
        }

        // first try to add the new window beside the active one
        if (activeWindow_ != null)
        {
            pane = findTabbedPane(activeWindow_.getComponent());
        }
        else
        {
            pane = lastActiveTabbedPane_;
        }

        insertGroup = pane.getGroup();
        // test whether this group can take the childwindow
        if (!insertGroup.canAddChildWindow(childWindow))
        {
            if (!group.equals(defaultGroup_))
            {
                if (!(insertGroup.getName().equals(group)))
                {
                    insertGroup = (Group) groups_.get(group);
                }

                // again do the filter test
                if (!insertGroup.canAddChildWindow(childWindow))
                {
                    throw new IllegalArgumentException(
                            "The child windows doesn't "
                                    + "matches the given group!");
                }
            }
            else
            {
                insertGroup = (Group) groups_.get(defaultGroup_);
                if (!insertGroup.canAddChildWindow(childWindow))
                {
                    throw new IllegalArgumentException(
                            "The child windows doesn't "
                                    + "matches the default filter!");
                }
            }
        }
        pane = insertGroup.getOpenTabbedPane();

        if (pane == null)
        {
            // the group has to be reopened
            openGroup(insertGroup.getName(), POSITION_ACTIVE_TOP);
            pane = insertGroup.getOpenTabbedPane();
        }

        pane.addTab(childWindow);

        // register us as focus listener so we can determine the active window
        // for setting menu and toolbar
        new ExtendedComponentListener(childWindow.getComponent());

        // add the window to the map
        childWindows_.put(childWindow.getComponent(), childWindow);

        // focus the new window and activate it
        pane.setSelectedComponent(childWindow.getComponent());
        activeComponentChanged(childWindow);
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#removeChildWindow(viptool
     * .base.extendedPane.IChildWindow, boolean)
     */
    public boolean removeChildWindow(IChildWindow childWindow, boolean force)
    {
        Iterator<IChildWindow> it;

        // first of all, ignore all invalid child windows
        if (childWindow == null)
            return false;
        if (childWindow.getComponent() == null)
            return false;

        // ask the window if it is ready to close and cancel the operation
        // if forse is set to false
        if (!childWindow.canClose(force) && !force)
        {
            return false;
        }

        // remove the window from the parent tabbed pane
        removeTab(childWindow.getComponent());

        // remove the window from our maps
        childWindows_.remove(childWindow.getComponent());

        // find a focusable window
        it = childWindows_.values().iterator();
        childWindow = null;
        while (it.hasNext())
        {
            childWindow = it.next();
            if (!(childWindow instanceof INonFocusableChildWindow))
            {
                childWindow.getComponent().requestFocus();
                break;
            }
            childWindow = null;
        }

        fireActiveWindowChangeEvent(childWindow);
        activeWindow_ = childWindow;
        return true;
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#removeChildWindow(viptool
     * .base.extendedPane.IChildWindow)
     */
    public void removeChildWindow(IChildWindow childWindow)
    {
        removeChildWindow(childWindow, false);
    }

    /*
     * @seeviptool.impl.base.extendedPane.IExtendedPane#removeTab(viptool.base.
     * extendedPane.IChildWindow)
     */
    public void removeTab(IChildWindow childWindow)
    {
        removeTab(childWindow.getComponent());
    }

    /**
     * Removes a component from its current tabbed pane. If the tab to remove is
     * the last one, the tabbed pane is also removed and the window is
     * relayoutet.
     * 
     * @param component
     *            the component to remove from a tab. Must be assigned to a
     *            child window
     */
    private void removeTab(Component component)
    {
        int dividerLocation;
        JSplitPane splitPane;
        JSplitPane splitPaneContainer;
        ExtendedTabbedPane tabbedPane;
        Component child;
        Container container;

        // get the hosting tabbed pane first
        tabbedPane = (ExtendedTabbedPane) component.getParent();
        // if the tabbed pane has more than one tab we can just remove the
        // component from the tab.
        // if the parent of the tabbed pane is our panel it is also easy.
        // the tabbed pane must stay so that new windows can be added. So
        // we also have only to remove the tab from the tabbed pane
        if (tabbedPane.getTabCount() >= 2 || tabbedPane.getParent() == this)
        {
            tabbedPane.remove(component);
            return;
        }

        // test wether we can remove the tabbed pane because perhaps it is the
        // last tab of a non closeable group
        if (tabbedPane.getGroup().getTabbedPaneCount() == 1
                && !tabbedPane.getGroup().isCloseable())
        {
            tabbedPane.remove(component);
            return;
        }

        // Since the parent of the tabbed pane is not our panel it must be a
        // split pane. Since the the tab is the last one the tabbed pane must
        // be removed and the window relayouted
        splitPane = (JSplitPane) tabbedPane.getParent();
        tabbedPane.close();

        // we have to save the other side of the split pane. Because the split
        // pane is removed, the other side becomes a child of the parent of the
        // split pane.
        if (splitPane.getLeftComponent() == tabbedPane)
            child = splitPane.getRightComponent();
        else
            child = splitPane.getLeftComponent();

        // get the parent of the split pane. This parent must get the child that
        // we have saved
        container = splitPane.getParent();
        // if our panel is the container, its easy because the child becomes
        // a child of our panel. Otherwise the container is a split pane and
        // we have to use setLeftComponent and setRightComponent to add the
        // child
        if (container == this)
        {
            container.remove(splitPane);
            container.add(child);
        }
        else
        {
            // get the container
            splitPaneContainer = (JSplitPane) container;
            // save the divider location to restore the layout
            dividerLocation = splitPaneContainer.getDividerLocation();
            // determine if to insert the child on the left or right side
            if (splitPaneContainer.getLeftComponent() == splitPane)
                splitPaneContainer.setLeftComponent(child);
            else
                splitPaneContainer.setRightComponent(child);
            // restore the divider location to keep the layout
            splitPaneContainer.setDividerLocation(dividerLocation);
        }

        // show the changes
        container.validate();
    }

    /**
     * Replaces the component toRemove from its container with the specified
     * toAdd component.
     * 
     * @param container
     *            the container in which to make the replacement
     * @param toRemove
     *            the component to replace
     * @param toAdd
     *            the component that should replace toRemove
     */
    private void replaceComponent(Container container, Component toRemove,
            Component toAdd)
    {
        int dividerLocation;
        JSplitPane pane;

        // if the container is a split pane we have to use setLeftComponent and
        // setRightComponent instead of add
        if (container instanceof JSplitPane)
        {
            pane = (JSplitPane) container;
            dividerLocation = pane.getDividerLocation();
            if (pane.getLeftComponent() == toRemove)
                pane.setLeftComponent(toAdd);
            else
                pane.setRightComponent(toAdd);
            pane.setDividerLocation(dividerLocation);
        }
        else if (container != null)
        {
            container.remove(toRemove);
            container.add(toAdd);
        }
    }

    /**
     * Moves a child window from one tab to another. If one of the arguments is
     * null no change will be made.
     * 
     * @param childWindow
     *            the child window to move
     * @param toPane
     *            the tabbed pane where the child window should be moved to
     * @param toIndex
     *            insert position for the moved tab. Set to -1 to add the tab at
     *            the end
     */
    private void moveChildWindow(IChildWindow childWindow,
            ExtendedTabbedPane to, int toIndex)
    {
        Container from;

        if (childWindow == null || to == null)
            return;

        // get current hosting tabbed pane
        from = childWindow.getComponent().getParent();

        // if the destination tabbed pane is euqal to the source tabbed pane
        // and there is only one tab in it, a moving operation is senseless
        if (from == to && to.getTabCount() == 1)
            return;

        // remove the tab from its current tabbed pane
        removeTab(childWindow.getComponent());

        // if the index is lower than zero, the tab should be appended as last
        // tab, otherwise the index specifes the insert position
        if (toIndex < 0)
        {
            to.addTab(childWindow);
        }
        else
        {
            to.insertTab(childWindow, toIndex);
        }

        // refresh the panes to display the changes
        to.validate();
        from.validate();
    }

    /**
     * Moves a component from one tab to another. If one of the arguments is
     * null no change will be made.
     * 
     * @param component
     *            the component to move. Must be assigned to a child window.
     * @param toPane
     *            the tabbed pane where the child window should be moved to
     * @param toIndex
     *            insert position for the moved tab. Set to -1 to add the tab at
     *            the end
     */
    /*
     * not used at the moment private void moveChildWindow(Component component,
     * JTabbedPane to, int toIndex) {
     * moveChildWindow((ChildWindow)childWindows_.get(component), to, toIndex);
     * }
     */

    // <><><><><><><><><><><><><><><><><><><><><>
    // methods required for splitting
    // <><><><><><><><><><><><><><><><><><><><><>
    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#splitActiveHorizontal(boolean
     * )
     */
    public void splitActiveHorizontal(boolean toRight)
    {
        split(activeWindow_.getComponent().getParent(), activeWindow_, true,
                toRight);
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#splitActiveVertical(boolean)
     */
    public void splitActiveVertical(boolean toLower)
    {
        split(activeWindow_.getComponent().getParent(), activeWindow_, false,
                toLower);
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#splitFrameHorizontal(boolean
     * )
     */
    public void splitFrameHorizontal(boolean toRight)
    {
        split(getComponent(0), activeWindow_, true, toRight);
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#splitFrameVertical(boolean)
     */
    public void splitFrameVertical(boolean toLower)
    {
        split(getComponent(0), activeWindow_, false, toLower);
    }

    /**
     * Replaces the component componentToReplace with a split pane and inserts
     * the specified child window as well as the componentToReplace component
     * into the split pane. The flags configure the orientation of the split
     * pane and the the position of the child window
     * 
     * @param componentToReplace
     *            the component that should be replaced with the new split pane
     * @param childWindow
     *            the component that should be inserted into the new side of the
     *            split pane. Can be a ExtendedTabbedPane or a childwindow
     * @param horizontal
     *            set to true if the orientation of the split pane should be
     *            horizontal, false otherwise
     * @param toRight
     *            specifies if the new tabbed pane should be on the right (down)
     *            side (set to true) or on the left (upper) sider (set to false)
     */
    private void split(Component componentToReplace, Object insertComponent,
            boolean horizontal, boolean toRight)
    {
        // the old group
        Group oldGroup = null;
        // the new pane for the child window
        ExtendedTabbedPane newPane;
        // the new split window
        JSplitPane splitPane;
        // the child window if insertComponent is one
        IChildWindow childWindow = null;
        // the group if insertComponent is one
        ExtendedTabbedPane tabbedPane = null;
        // Container in which the componentToReplace lies
        Container container;

        // if no component is specified we cannot perform the
        // operation. The insertComponent must be also a group or child window
        if (componentToReplace == null)
            return;
        container = componentToReplace.getParent();

        if (insertComponent instanceof IChildWindow)
        {
            childWindow = (IChildWindow) insertComponent;
        }
        else if (insertComponent instanceof ExtendedTabbedPane)
        {
            tabbedPane = (ExtendedTabbedPane) insertComponent;
        }
        else
        {
            // error: insertComponent is neither a childwindow nor a group
            return;
        }

        // if the tabbed pane is the host of the child window and the child
        // windows is the only tab of the tabbed pane, the operation effects
        // no change. It can be cancelled now
        if (componentToReplace instanceof ExtendedTabbedPane
                && childWindow != null)
        {
            if (childWindow.getComponent().getParent() == componentToReplace
                    && ((ExtendedTabbedPane) componentToReplace).getTabCount() == 1)
                return;
        }

        // create a new split pane that replaces the component that has to
        // be split
        if (horizontal)
            splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        else
            splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

        // replace the component with the split pane
        replaceComponent(container, componentToReplace, splitPane);

        if (childWindow != null)
        {
            oldGroup = ((ExtendedTabbedPane) childWindow.getComponent().getParent()).getGroup();
            newPane = createTabbedPane(oldGroup);
        }
        else
        {
            // we have a group to insert. Its TabbedPane already exists
            newPane = tabbedPane;
        }
        // add the components to the split pane
        splitPane.setLeftComponent(toRight ? componentToReplace : newPane);
        splitPane.setRightComponent(toRight ? newPane : componentToReplace);
        newPane.open();

        // remove the child window from its current tabbed pane
        if (childWindow != null)
        {
            removeTab(childWindow);
            newPane.addTab(childWindow);
        }

        // register by the new tabbed pane, so we get a message if the
        // current tab changes or the tabbed pane gains the focus
        newPane.addChangeListener(this);
        newPane.addMouseListener(this);
        newPane.addMouseMotionListener(this);
        new ExtendedComponentListener(newPane);

        // set the divider at half
        if (splitPane.getOrientation() == JSplitPane.HORIZONTAL_SPLIT)
        {
            splitPane.setDividerLocation(-1);
        }
        else
        {
            splitPane.setDividerLocation(-1);
        }

        // relayout
        validate();

        // try to set the focus to the moved child
        if (childWindow != null)
        {
            newPane.setSelectedComponent(childWindow.getComponent());
            newPane.requestFocus();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // all about active window changes
    // <><><><><><><><><><><><><><><><><><><><><>
    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#addActiveWindowChangeListener
     * (viptool.base.extendedPane.event.ActiveWindowChangeListener)
     */
    public void addExtendedPaneListener(IExtendedPaneListener listener)
    {
        activeWindowListeners_.add(listener);
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#removeActiveWindowChangeListener
     * (viptool.base.extendedPane.event.ActiveWindowChangeListener)
     */
    public void removeExtendedPaneListener(IExtendedPaneListener listener)
    {
        activeWindowListeners_.remove(listener);
    }

    /**
     * This method notifies all listeners that the active window has changed.
     * 
     * @param newActive
     *            the window that is now active
     */
    protected void fireActiveWindowChangeEvent(IChildWindow newActive)
    {
        Iterator it;
        ActiveWindowChangeEvent event = new ActiveWindowChangeEvent(this,
                newActive);

        // notify every listener about the changed window
        it = activeWindowListeners_.iterator();
        while (it.hasNext())
        {
            ((IExtendedPaneListener) it.next()).activeWindowChanged(event);
        }

        // notify the window itself
        if (newActive != null)
        {
            newActive.windowActivated();
        }
    }

    protected void fireWindowCloseEvent(IChildWindow window)
    {
        Iterator it;
        WindowCloseEvent event = new WindowCloseEvent(this, window);

        // notify every listener about the changed window
        it = activeWindowListeners_.iterator();
        while (it.hasNext())
        {
            ((IExtendedPaneListener) it.next()).windowClose(event);
        }
    }

    /**
     * This method is called by the class if the active window has changed. It
     * is called by the focusGained or stateChanged method. The method set the
     * activeWindow_ field and notifies their listeners (e.g. the main frame)
     * that the active windows has changed (so the main frame can display the
     * assigned menu or toolbar)
     * 
     * @param newActive
     *            the ChildWindow that is now active
     */
    private void activeComponentChanged(IChildWindow newActive)
    {
        if (newActive == null
                || (activeWindow_ != null && activeWindow_ == newActive)
                || newActive instanceof INonFocusableChildWindow)
        {
            return;
        }

        Container container = newActive.getComponent().getParent();
        // store the last active pane
        if (container instanceof ExtendedTabbedPane)
            lastActiveTabbedPane_ = (ExtendedTabbedPane) container;

        // search the hosting child window
        activeWindow_ = (IChildWindow) childWindows_.get(newActive.getComponent());
        fireActiveWindowChangeEvent(activeWindow_);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new tabbed pane and assigns it to the given group. The tabbed
     * pane is not marked as open. This must be done manually.
     * 
     * @param group
     *            the group to which the tabbed pane should belong to
     */
    private ExtendedTabbedPane createTabbedPane(Group group)
    {
        ExtendedTabbedPane tabbedPane = new ExtendedTabbedPane(group);

        // We have also to add a listener that watches tab changes. So we can
        // determine the active component if a tab change occurs in the active
        // tabbed pane (the focus event will not be fired in such case)
        tabbedPane.addChangeListener(this);

        // register us as focus listener by the tabbed pane. By doing this, we
        // don't have to set a listener to every component just to every
        // tabbed pane. If a tabbed pane gains the focus we will ask that pane
        // which
        // tab is selected
        new ExtendedComponentListener(tabbedPane);

        // register as MouseListener for dragging tabs between tabbed panes or
        // splitting.
        tabbedPane.addMouseListener(this);
        tabbedPane.addMouseMotionListener(this);

        return tabbedPane;
    }

    /**
     * Tries to find a group that can host the given child window.
     * 
     * @param childWindow
     *            the child window that should be hosted
     * @return the name of a group that can host the child window or the name of
     *         the default group
     */
    private String findMatchingGroup(IChildWindow childWindow)
    {
        Iterator it;
        String name;
        Group group;

        it = groups_.keySet().iterator();
        while (it.hasNext())
        {
            name = (String) it.next();
            group = (Group) groups_.get(name);

            if (group.canAddChildWindow(childWindow))
            {
                return name;
            }
        }

        return defaultGroup_;
    }

    /**
     * Returns the tabbed pane that contains the specified component.
     * 
     * @param component
     *            the component which container is searched
     * @return the tabbed pane that contains the component
     */
    private ExtendedTabbedPane findTabbedPane(Component component)
    {
        while (!(component instanceof ExtendedTabbedPane) && component != null)
            component = component.getParent();

        return (ExtendedTabbedPane) component;
    }

    /**
     * Returns the child window that is parent of the specified component.
     * 
     * @param component
     *            the component which child window is searched
     * @return the child window that is parent the component
     */
    private IChildWindow findChildWindow(Component component)
    {
        IChildWindow childWindow = null;

        while (!(childWindow instanceof IChildWindow) && component != null)
        {
            childWindow = (IChildWindow) childWindows_.get(component);
            if (childWindow != null)
            {
                break;
            }

            component = component.getParent();
        }
        return childWindow;
    }

    /*
     * @see
     * viptool.impl.base.extendedPane.IExtendedPane#selectWindow(viptool.base
     * .extendedPane.IChildWindow)
     */
    public void selectWindow(IChildWindow childWindow)
    {
        // search the tabbed pane and select the right tab
        ExtendedTabbedPane pane = findTabbedPane(childWindow.getComponent());
        if (pane != null)
        {
            // dirty hack
            int index = pane.indexOfComponent(childWindow.getComponent());
            pane.setSelectedIndex(-1);
            pane.setSelectedIndex(index);

            // request focus for the window
            childWindow.getComponent().requestFocus();
        }
    }

    public void setForegroundForTab(IChildWindow childWindow, Color color)
    {
        int index;
        ExtendedTabbedPane pane = findTabbedPane(childWindow.getComponent());

        if (pane != null)
        {
            index = pane.indexOfComponent(childWindow.getComponent());
            if (index >= 0)
            {
                pane.getTabComponentAt(index).setForeground(color);
            }
        }
    }

    public IChildWindow getActiveWindow()
    {
        return activeWindow_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // layout methods for the extended pane
    // <><><><><><><><><><><><><><><><><><><><><>

    public void setDividerLocation(IChildWindow hostedWindow, int location)
    {
        ExtendedTabbedPane tabbedPane;
        JSplitPane splitPane;

        tabbedPane = findTabbedPane(hostedWindow.getComponent());
        if (tabbedPane == null)
        {
            return;
        }

        if (tabbedPane.getParent() instanceof JSplitPane)
        {
            splitPane = (JSplitPane) tabbedPane.getParent();
            splitPane.setDividerLocation(location);
        }
    }

    public int getDividerLocation(IChildWindow hostedWindow)
    {
        ExtendedTabbedPane tabbedPane;
        JSplitPane splitPane;

        tabbedPane = findTabbedPane(hostedWindow.getComponent());
        if (tabbedPane == null)
        {
            return -1;
        }

        if (tabbedPane.getParent() instanceof JSplitPane)
        {
            splitPane = (JSplitPane) tabbedPane.getParent();
            return splitPane.getDividerLocation();
        }

        return -1;
    }

    public void loadLayoutFromXML(Element root)
    {
        String title;
        String clazz;
        JComponent topComponent;
        IChildWindow childWindowToSelect = null;
        ArrayList<IChildWindow> windowsToRemove;

        // clear the groups first, because the tabbed panes of the group will
        // be removed
        for (Group group : groups_.values())
        {
            group.clear();
        }

        // set the new extended pane component
        topComponent = loadComponentFromXML(root);
        remove(0);
        add(topComponent, BorderLayout.CENTER);

        title = root.getAttribute("activeTitle");
        clazz = root.getAttribute("activeClass");

        // remove the empty windows
        windowsToRemove = new ArrayList<IChildWindow>();
        for (IChildWindow childWindow : childWindows_.values())
        {
            if (childWindow instanceof EmptyChildWindow)
            {
                windowsToRemove.add(childWindow);
            }

            if (childWindow.getTitle().equals(title)
                    && childWindow.getClass().getCanonicalName().equals(clazz))
            {
                childWindowToSelect = childWindow;
            }
        }

        for (IChildWindow childWindow : windowsToRemove)
        {
            removeChildWindow(childWindow);
        }

        if (childWindowToSelect != null)
        {
            selectWindow(childWindowToSelect);
        }
    }

    private JComponent loadComponentFromXML(Node root)
    {
        Node node;

        node = root.getFirstChild();
        while (!(node instanceof Element))
        {
            node = node.getNextSibling();
        }

        if (node instanceof Element)
        {
            if (node.getNodeName().equals("splitPane"))
            {
                return loadSplitPaneFromXML((Element) node);
            }
            else if (node.getNodeName().equals("tabbedPane"))
            {
                return loadTabbedPaneFromXML((Element) node);
            }
        }

        return null;
    }

    private JSplitPane loadSplitPaneFromXML(Element root)
    {
        int orientation = JSplitPane.HORIZONTAL_SPLIT;
        int dividerLocation = -1;
        Node node;
        JComponent left = null;
        JComponent right = null;
        JSplitPane splitPane = new JSplitPane();

        try
        {
            orientation = Integer.parseInt(root.getAttribute("orientation"));
            dividerLocation = Integer.parseInt(root.getAttribute("divider"));
        }
        catch (NumberFormatException exception)
        {
            // ignore
        }

        node = root.getFirstChild();
        while (node != null)
        {
            if (node.getNodeName().equals("left"))
            {
                left = loadComponentFromXML(node);
            }
            else if (node.getNodeName().equals("right"))
            {
                right = loadComponentFromXML(node);
            }

            node = node.getNextSibling();
        }

        splitPane.setLeftComponent(left);
        splitPane.setRightComponent(right);
        splitPane.setDividerLocation(dividerLocation);
        splitPane.setOrientation(orientation);

        return splitPane;
    }

    private ExtendedTabbedPane loadTabbedPaneFromXML(Element root)
    {
        int selectedIndex = 0;
        boolean groupCloseable = true;
        String groupName = "<unknown>";
        String groupFilter;
        String windowTitle;
        String windowClass;
        IChildWindow childWindow;
        Class groupFilterClass;
        Group group;
        Element window;
        NodeList nodes;
        ExtendedTabbedPane tabbedPane;

        groupFilterClass = Object.class;

        try
        {
            groupName = root.getAttribute("groupName");
            groupFilter = root.getAttribute("groupFilter");
            groupFilterClass = Class.forName(groupFilter);
            groupCloseable = Boolean.parseBoolean(root.getAttribute("groupCloseable"));
            selectedIndex = Integer.parseInt(root.getAttribute("selectedIndex"));
        }
        catch (NumberFormatException exception)
        {
            selectedIndex = 0;
        }
        catch (ClassNotFoundException exception)
        {
            Logger.getInstance().warning(
                    "Filter class for group '" + groupName
                            + "' not found! Using object class as filter.");
        }

        group = createInternalGroup(groupName, groupFilterClass, groupCloseable);
        tabbedPane = createTabbedPane(group);
        tabbedPane.open();

        // add child windows to tapped pane
        nodes = root.getElementsByTagName("window");
        for (int i = 0; i < nodes.getLength(); i++)
        {
            window = (Element) nodes.item(i);

            // find the the window
            windowTitle = window.getAttribute("title");
            windowClass = window.getAttribute("class");

            for (Entry<Component, IChildWindow> entry : childWindows_.entrySet())
            {
                childWindow = entry.getValue();

                if (childWindow.getClass().getCanonicalName().equals(
                        windowClass)
                        && windowTitle.equals(childWindow.getTitle()))
                {
                    // window found, add it to our tab pane
                    tabbedPane.addTab(childWindow);

                    break;
                }
            }
        }

        if (tabbedPane.getTabCount() == 0)
        {
            childWindow = new EmptyChildWindow();
            tabbedPane.addTab(childWindow);
            childWindows_.put(childWindow.getComponent(), childWindow);
        }

        if (selectedIndex >= 0 && selectedIndex < tabbedPane.getTabCount())
        {
            tabbedPane.setSelectedIndex(selectedIndex);
        }

        return tabbedPane;
    }

    public void storeLayoutAsXML(Document document, Element root)
    {
        Element element;

        element = document.createElement("extendedPane");
        root.appendChild(element);
        if (getActiveWindow() != null)
        {
            element.setAttribute("activeTitle", getActiveWindow().getTitle());
            element.setAttribute("activeClass",
                    getActiveWindow().getClass().getCanonicalName());
        }

        element.appendChild(componentLayoutAsXML(document, getComponent(0)));
    }

    private Element componentLayoutAsXML(Document document, Component component)
    {
        if (component instanceof JSplitPane)
        {
            return splitPaneLayoutAsXML(document, (JSplitPane) component);
        }
        else if (component instanceof ExtendedTabbedPane)
        {
            return tabbedPaneLayoutAsXML(document,
                    (ExtendedTabbedPane) component);
        }

        return null;
    }

    private Element splitPaneLayoutAsXML(Document document, JSplitPane splitPane)
    {
        Element root;
        Element left;
        Element right;
        Element element;

        root = document.createElement("splitPane");
        left = document.createElement("left");
        right = document.createElement("right");
        root.appendChild(left);
        root.appendChild(right);

        // store attributes
        root.setAttribute("divider",
                String.valueOf(splitPane.getDividerLocation()));
        root.setAttribute("orientation",
                String.valueOf(splitPane.getOrientation()));

        // store subcomponents
        element = componentLayoutAsXML(document, splitPane.getLeftComponent());
        if (element != null)
        {
            left.appendChild(element);
        }

        element = componentLayoutAsXML(document, splitPane.getRightComponent());
        if (element != null)
        {
            right.appendChild(element);
        }

        return root;
    }

    private Element tabbedPaneLayoutAsXML(Document document,
            ExtendedTabbedPane tabbedPane)
    {
        Group group;
        Element root;
        Element child;
        IChildWindow childWindow;

        root = document.createElement("tabbedPane");

        // set Attributes
        group = tabbedPane.getGroup();
        root.setAttribute("groupName", group.getName());
        root.setAttribute("groupFilter", group.getFilter().getCanonicalName());
        root.setAttribute("groupCloseable", String.valueOf(group.isCloseable()));
        root.setAttribute("selectedIndex",
                String.valueOf(tabbedPane.getSelectedIndex()));

        // add childs
        for (int i = 0; i < tabbedPane.getTabCount(); i++)
        {
            child = document.createElement("window");

            childWindow = findChildWindow(tabbedPane.getComponentAt(i));
            if (childWindow != null)
            {
                child.setAttribute("title", childWindow.getTitle());
                child.setAttribute("class",
                        childWindow.getClass().getCanonicalName());

                root.appendChild(child);
            }
            else
            {
                System.err.println("Childwindow not found!");
            }
        }

        return root;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the ChangeListener interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Called by the framework if the selected tab of a tabbed pane changed.
     * Used to determine the active window and to notify the main frame about
     * such changes to update menu and/or toolbar
     * 
     * @param changeEvent
     *            a change event object containing information about the change
     * @see javax.swing.event.ChangeListener.stateChanged(ChangeEvent arg0)
     */
    public void stateChanged(ChangeEvent changeEvent)
    {
        ExtendedTabbedPane pane = (ExtendedTabbedPane) changeEvent.getSource();
        activeComponentChanged((IChildWindow) findChildWindow(pane.getSelectedComponent()));

        // gain the focus otherwise the following behaviour is observed:
        // if there are two tabbed panes named left and right. Say the right
        // one has two tabs, tab1 and tab2. tab1 is selected, the left pane
        // has the focus.
        // If the user now clicks on tab2 in the right pane, this method will
        // be called but tab2 doesn't gain the focus. So the user cklicks on
        // tab2 but the menu and toolbar wouldn't change.
        // Preserve this by requesting the focus for the last clicked tabbed
        // pane
        pane.requestFocus();

        // dragWindow_.BeginDrag(pane.getBoundsAt(pane.getSelectedIndex()),
        // pane);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the MouseListener interface
    // <><><><><><><><><><><><><><><><><><><><><>
    public void mouseClicked(MouseEvent mouseEvent)
    {
        // index of the double clicked childWindow in the tabbedPane
        int index = -1;
        // point of the double click relative to the whole window
        Point point;
        // the rectangle of the double clicked childWindow in the tabbedPane
        // Rectangle rectangle;
        // the childWindow double clicked on
        IChildWindow child;
        // the "father" of the childWindow child
        ExtendedTabbedPane pane;

        // get the point to recognize which Component is at that place
        point = mouseEvent.getPoint();

        // determine the source of the event
        if (mouseEvent.getComponent() instanceof ExtendedTabbedPane)
        {

            // proceed only if a doubleclick is performed
            if (mouseEvent.getClickCount() == 2)
            {
                // get the tabbedPane at the double click location
                pane = (ExtendedTabbedPane) mouseEvent.getSource();

                // get the index of the childWindow at the double click location
                index = pane.indexAtLocation(point.x, point.y);

                // end method if no childWindow head is selected
                if (index < 0)
                {
                    return;
                }

                // get the childWindow at the double click location
                child = (IChildWindow) childWindows_.get(pane.getComponentAt(index));

                // set the childWindow as an active one, otherwise there might
                // arise
                // problems when there are so many tabs that they have to be
                // displayed in two or more rows
                pane.setSelectedComponent(child.getComponent());

                // make sure that the found child implements the interface
                // Renameable
                // so that special methods are available
                // if(child instanceof Renameable)
                // {
                // // the rectangle (the childWindow head) is needed to know
                // where
                // // the rename textfield has to be set and what size it should
                // // have
                // rectangle = pane.getBoundsAt(index);
                //
                // // any other rename action will be performed by the class
                // // Renamer
                // if(((Renameable) child).isMultiLine())
                // {
                // new MultiLineRenamer(Workbench.getInstance(), rectangle,
                // (Renameable) child, pane, null);
                // }
                // else
                // {
                // new SingleLineRenamer(Workbench.getInstance(), rectangle,
                // (Renameable) child, pane, null);
                // }
                // }
            }
        }
        else
        {
            child = findChildWindow(mouseEvent.getComponent());
            activeComponentChanged(child);
        }
    }

    public void mouseEntered(MouseEvent mouseEvent)
    {
    }

    public void mouseExited(MouseEvent mouseEvent)
    {
    }

    public void mousePressed(MouseEvent mouseEvent)
    {
    }

    public void mouseReleased(MouseEvent mouseEvent)
    {
        if (mouseEvent.getButton() == MouseEvent.BUTTON1
                && dragWindow_.isStarted())
            dragWindow_.EndDrag();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the MouseMotionListener interface
    // <><><><><><><><><><><><><><><><><><><><><>
    public void mouseDragged(MouseEvent mouseEvent)
    {
        // if the drag operations has not started yet, start it now.
        // otherwise notify the drag window about the mouse move.
        // Since the focus stays by the window that is dragged, our drag window
        // will not receive any mouse message and we must forward mouse messages
        // here
        if (!dragWindow_.isStarted())
        {
            // just makes sense to drag tabs
            if (mouseEvent.getComponent() instanceof ExtendedTabbedPane)
            {
                dragWindow_.BeginDrag(mouseEvent.getPoint(),
                        (ExtendedTabbedPane) mouseEvent.getComponent());
            }
        }
        else
        {
            dragWindow_.drag(mouseEvent.getPoint());
        }
    }

    public void mouseMoved(MouseEvent mouseEvent)
    {
    }

}
