package viptool.impl.base.gui.dialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.BadLocationException;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import viptool.base.error.VipToolException;
import viptool.base.error.VipToolRuntimeException;
import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.event.ButtonClickedEvent;
import viptool.base.gui.dialog.event.IStandardDialogListener;
import viptool.impl.util.resources.ResourceLoader;
import viptool.util.resources.ResourceConstants;

public class ExceptionDialog implements IStandardDialogListener
{
    private static final String LABEL_STACKTRACE        = "Stacktrace";

    private static final String LABEL_EXCEPTION_HISTORY = "Exception history";

    private StandardModalDialog dialog_                 = null;

    // <><><><><><><><><><><><><><><><><><><><><>
    // subclasses
    // <><><><><><><><><><><><><><><><><><><><><>
    public class ExceptionPanel extends JPanel implements TreeSelectionListener
    {
        private JLabel      stacktraceLabel_  = null;

        private JLabel      historyLabel_     = null;

        private JTextPane   stacktrace_       = null;

        private JTree       exceptionHistory_ = null;

        private Exception   exception_        = null;

        private JScrollPane treeScrollPane_   = null;

        private JScrollPane stackScrollPane_  = null;

        private JSplitPane  splitPanel_       = null;

        private JPanel      topPanel_         = null;

        private JPanel      bottomPanel_      = null;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * This constructor creates the exception panel and initializes it with
         * the given exception.
         * 
         * @param exception
         *            the exception that should be shown in the panel
         */
        public ExceptionPanel(VipToolException exception)
        {
            super();
            exception_ = exception;

            initialize();
        }

        public ExceptionPanel(VipToolRuntimeException exception)
        {
            super();
            exception_ = exception;

            initialize();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // supporting methods
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * This method initializes the panel and will fill the elements with the
         * information given in the exception.
         */
        private void initialize()
        {
            ImageIcon icon = new ImageIcon(ResourceLoader.getInstance()
                    .getImage(ResourceConstants.ICON_ERROR_SMALL));
            DefaultMutableTreeNode root;
            DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();

            // layout constraints
            GridBagConstraints historyConstraints = new GridBagConstraints();
            historyConstraints.fill = GridBagConstraints.BOTH;
            historyConstraints.gridy = 1;
            historyConstraints.weightx = 1.0;
            historyConstraints.weighty = 1.0;
            historyConstraints.anchor = GridBagConstraints.NORTH;
            historyConstraints.gridx = 0;

            GridBagConstraints stacktraceConstraints = new GridBagConstraints();
            stacktraceConstraints.fill = GridBagConstraints.BOTH;
            stacktraceConstraints.gridy = 3;
            stacktraceConstraints.weightx = 1.0;
            stacktraceConstraints.weighty = 1.0;
            stacktraceConstraints.gridx = 0;

            GridBagConstraints stacktraceLabelConstraints = new GridBagConstraints();
            stacktraceLabelConstraints.fill = GridBagConstraints.HORIZONTAL;
            stacktraceLabelConstraints.gridy = 2;
            stacktraceLabelConstraints.weightx = 1.0;
            stacktraceLabelConstraints.weighty = 0;
            stacktraceLabelConstraints.gridx = 0;

            GridBagConstraints historyLabelConstraints = new GridBagConstraints();
            historyLabelConstraints.fill = GridBagConstraints.HORIZONTAL;
            historyLabelConstraints.gridy = 0;
            historyLabelConstraints.weightx = 1.0;
            historyLabelConstraints.weighty = 0;
            historyLabelConstraints.gridx = 0;

            // the controls
            splitPanel_ = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

            topPanel_ = new JPanel();
            topPanel_.setLayout(new GridBagLayout());

            bottomPanel_ = new JPanel();
            bottomPanel_.setLayout(new GridBagLayout());

            stacktraceLabel_ = new JLabel();
            stacktraceLabel_.setText(LABEL_STACKTRACE);

            stacktrace_ = new JTextPane();
            stacktrace_.setEditable(false);
            stacktrace_.setForeground(Color.BLUE);
            stacktrace_.setMinimumSize(new Dimension(10, 50));
            stackScrollPane_ = new JScrollPane(stacktrace_);
            fillStackTrace(exception_);

            historyLabel_ = new JLabel();
            historyLabel_.setText(LABEL_EXCEPTION_HISTORY);

            // create the root node
            root = new DefaultMutableTreeNode(exception_.toString());
            root.setUserObject(exception_);

            // create the tree
            // the renderer for own icons
            renderer.setLeafIcon(icon);
            renderer.setClosedIcon(icon);
            renderer.setOpenIcon(icon);

            // the tree itself
            exceptionHistory_ = new JTree(root);
            exceptionHistory_.setVisibleRowCount(5);
            exceptionHistory_
                    .setToolTipText("shows the exception history (which exception was thrown first)");
            exceptionHistory_.putClientProperty("JTree.lineStyle", "None");
            exceptionHistory_.getSelectionModel().setSelectionMode(
                    TreeSelectionModel.SINGLE_TREE_SELECTION);
            exceptionHistory_.addTreeSelectionListener(this);
            exceptionHistory_.setCellRenderer(renderer);

            // create a scroll pane, so the tree is scrollable
            treeScrollPane_ = new JScrollPane(exceptionHistory_);
            // fill the tree with the history
            fillHistory(root);
            exceptionHistory_.expandPath(new TreePath(root.getPath()));
            exceptionHistory_.setSelectionPath(new TreePath(root.getPath()));

            setLayout(new BorderLayout());
            add(splitPanel_, BorderLayout.CENTER);
            splitPanel_.setLeftComponent(topPanel_);
            splitPanel_.setRightComponent(bottomPanel_);

            topPanel_.add(historyLabel_, historyLabelConstraints);
            topPanel_.add(treeScrollPane_, historyConstraints);
            bottomPanel_.add(stacktraceLabel_, stacktraceLabelConstraints);
            bottomPanel_.add(stackScrollPane_, stacktraceConstraints);
        }

        private void fillStackTrace(Throwable throwable)
        {
            StackTraceElement[] elements = throwable.getStackTrace();

            stacktrace_.setText("");
            for (int i = 0; i < elements.length; i++)
            {
                try
                {
                    stacktrace_.getStyledDocument().insertString(
                            stacktrace_.getStyledDocument().getLength(),
                            elements[i].toString() + "\n", null);
                }
                catch (BadLocationException exception)
                {
                    stacktrace_.setText("Can't display stacktrace: "
                            + exception.getMessage());
                }
            }
        }

        private void fillHistory(DefaultMutableTreeNode root)
        {
            Throwable cause = exception_.getCause();

            if(cause == null)
            {
                exceptionHistory_.setVisible(false);
                historyLabel_.setVisible(false);
                return;
            }

            fillNodes(root, cause);
        }

        private void fillNodes(DefaultMutableTreeNode parent,
                Throwable throwable)
        {
            Throwable cause = throwable.getCause();
            DefaultMutableTreeNode child;

            child = new DefaultMutableTreeNode(throwable.toString());
            child.setUserObject(throwable);

            parent.add(child);
            exceptionHistory_.expandPath(new TreePath(child.getPath()));

            if(cause != null)
            {
                fillNodes(child, cause);
            }
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the TreeSelectionListener
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see javax.swing.event.TreeSelectionListener */
        public void valueChanged(TreeSelectionEvent event)
        {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) exceptionHistory_
                    .getLastSelectedPathComponent();
            String mainDescription = "No message available!";
            Throwable throwable;

            if(node == null)
            {
                throwable = exception_;
            }
            else
            {
                throwable = (Throwable) node.getUserObject();

            }

            if(throwable.getMessage() != null)
            {
                mainDescription = throwable.getMessage();
            }

            // when the root element is selected the first time, the dialog
            // variable is not set yet, because this panel is created before
            // the dialog (the panel must be given in the constructor of the
            // dialog
            if(dialog_ != null)
            {
                dialog_.setMainDescription(mainDescription);
            }

            fillStackTrace(throwable);
        }

    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    public ExceptionDialog(VipToolException exception)
    {
        JPanel panel = new ExceptionPanel(exception);
        String mainDescription = exception.getMessage() != null ? exception
                .getMessage() : "No message available!";

        dialog_ = new StandardModalDialog("Error", mainDescription, panel,
                ResourceLoader.getInstance().getImage(
                        ResourceConstants.IMAGE_ERROR),
                DialogConstants.BUTTON_OK, DialogConstants.BUTTON_OK);
        dialog_.addStandardDialogListener(this);
        dialog_.setDetailedDescription("See exception history or "
                + "stacktrace for more details!");

        if(exception.getCause() != null)
        {
            dialog_.setNotice("Select exception to show stacktrace!",
                    DialogConstants.TYPE_WARNING);
        }

        dialog_.pack();
        dialog_.setVisible(true);
    }

    public ExceptionDialog(VipToolRuntimeException exception)
    {
        JPanel panel = new ExceptionPanel(exception);
        String mainDescription = exception.getMessage() != null ? exception
                .getMessage() : "No message available!";

        dialog_ = new StandardModalDialog("Error", mainDescription, panel,
                ResourceLoader.getInstance().getImage(
                        ResourceConstants.IMAGE_ERROR),
                DialogConstants.BUTTON_OK, DialogConstants.BUTTON_OK);
        dialog_.addStandardDialogListener(this);
        dialog_.setDetailedDescription("See exception history or "
                + "stacktrace for more details!");

        if(exception.getCause() != null)
        {
            dialog_.setNotice("Select exception to show stacktrace!",
                    DialogConstants.TYPE_WARNING);
        }

        dialog_.pack();
        dialog_.setVisible(true);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IStandardDialogListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.gui.dialog.event.IStandardDialogListener */
    public void buttonClicked(ButtonClickedEvent event)
    {
        if(event.getType() == DialogConstants.BUTTON_OK)
        {
            dialog_.setVisible(false);
            dialog_.dispose();
        }
    }

}
