package viptool.impl.base.gui.dialog;

import java.awt.BorderLayout;
import java.awt.Image;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import viptool.base.error.FileInterpreterProviderException;
import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.event.ButtonClickedEvent;
import viptool.base.gui.dialog.event.IStandardDialogListener;
import viptool.base.io.IFileInterpreterProvider;
import viptool.impl.base.Workbench;
import viptool.impl.base.gui.dialog.filefilter.Filter;
import viptool.project.FileSystem;
import viptool.project.IWorkspace;

/**
 * This class is a FileChooser based on the StandardModalDialog. It supports the
 * Viptool-Filetypes and checks selected file with providers if it is from the
 * wanted type.
 */
public class FileChooserDialog implements IStandardDialogListener
{

    // the filechooser which is shown into the dialogpanel
    private JFileChooser               chooser_;

    // the dialog which is represented by this object
    private StandardModalDialog        dialog_;

    // selected file
    private File                       file_;

    // providers for the expected filetypes
    private IFileInterpreterProvider[] providers_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>
    class FileChangedListener implements PropertyChangeListener
    {
        public void propertyChange(PropertyChangeEvent event)
        {
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_OK, false);
            dialog_.setNotice("You have to select a valid file!",
                    DialogConstants.TYPE_ERROR);

            // check if the file is valid
            for (int i = 0; i < providers_.length; i++)
            {
                try
                {
                    if (event.getNewValue() != null)
                    {
                        if (providers_[i].matchFormat(FileSystem.getInstance().createAbstractFile(
                                event.getNewValue().toString())))
                        {
                            dialog_.setButtonsEnabled(
                                    DialogConstants.BUTTON_OK, true);
                            dialog_.removeNotice();
                        }
                    }
                }
                catch (FileInterpreterProviderException exception)
                {

                }
            }
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates and shows a FileChooserDialog.
     * 
     * @param providers
     *            providers for choosable filetyes
     */
    public FileChooserDialog(IFileInterpreterProvider[] providers)
    {

        providers_ = providers;

        // create a FileChooser
        chooser_ = new JFileChooser();

        // hide JFileChooser-Buttons
        chooser_.setControlButtonsAreShown(false);

        // set path to the path of the current workspace
        // if there is one
        IWorkspace workspace = Workbench.getInstance().getCurrentWorkspace();
        if (workspace != null)
        {
            // get workspace-folder
            File path = workspace.getAbstractFile().getFile();
            path = path.getParentFile();
            chooser_.setCurrentDirectory(path);
        }

        // title and description of the dialog
        String title = "Load", description = null;
        // icon which is shown into the dialog
        Image icon = null;

        FileFilter defaultFilter = null;

        // set description (type of first provider)
        // and add all given providers as choosable filetypes
        for (int i = 0; i < providers_.length; i++)
        {
            FileFilter filter = new Filter(providers_[i]);
            chooser_.addChoosableFileFilter(filter);
            if (i == 0)
            {
                defaultFilter = filter;
                description = "Select a " + providers_[i].getFileType();
            }
        }

        // set first provider as default
        if (defaultFilter != null)
        {
            chooser_.setFileFilter(defaultFilter);
        }

        // create panel and add the chooser
        // to get a border around the chooser
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        panel.add(chooser_, BorderLayout.CENTER);

        // create the dialog
        dialog_ = new StandardModalDialog(title, description, panel, icon,
                DialogConstants.BUTTON_SET_OKCANCEL, DialogConstants.BUTTON_OK);

        // registers this object at the dialog as listener
        dialog_.addStandardDialogListener(this);

        // add a listener to update the ok button
        chooser_.addPropertyChangeListener(
                JFileChooser.SELECTED_FILE_CHANGED_PROPERTY,
                new FileChangedListener());
        chooser_.setSelectedFile(null);

        // show the dialog
        dialog_.pack();
        dialog_.setVisible(true);
    }

    /**
     * This methode returns the selected file if there is one.
     * 
     * @return selected file
     */
    public File getSelectedFile()
    {
        return file_;
    }

    // ********************************************************
    // Implementation of the IStandardDialogListener-Interface
    // ********************************************************

    /**
     * This methode closes the dialog if the OK- or Cancel-Button was clicked.
     * 
     * @param event
     */
    public void buttonClicked(ButtonClickedEvent event)
    {
        if (event.getType() == DialogConstants.BUTTON_OK)
        {
            // the OK-Button was pressed
            // get selected file
            file_ = chooser_.getSelectedFile();
            dialog_.setVisible(false);
        }
        if (event.getType() == DialogConstants.BUTTON_CANCEL)
        {
            // the Cancel-Button was pressed
            // reset file
            file_ = null;
            // and close dialog
            dialog_.setVisible(false);
        }
    }
}
