package viptool.impl.base.gui.dialog;

import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.event.ButtonClickedEvent;
import viptool.base.gui.dialog.event.IStandardDialogListener;
import viptool.base.io.IFileInterpreterProvider;
import viptool.impl.base.Workbench;
import viptool.impl.base.io.FileInterpreterManager;
import viptool.impl.util.resources.ResourceLoader;
import viptool.util.resources.ResourceConstants;

/**
 * This class provides a dialog for creating new files. The dialog shows a list
 * of all valid file type providers. A valid file type provider is a provider
 * that has a wizard for creating a file of its file type. The new dialog is
 * just for selecting one file type. <br>
 * After selection the wizard of the selected provider is called, and the new
 * dialog is closed.
 */
public class NewDialog implements IStandardDialogListener
{
    // the standard dialog using for displaying the content
    private StandardModalDialog      dialog_;

    // the selected provider
    private IFileInterpreterProvider selectedProvider_ = null;

    // the panel that display the provider
    private TreePanel                treePanel_        = null;

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This class creates a list with valid file type providers, that are
     * providers with a wizard. It creates a JList that displays all providers
     * with their icons.
     */
    public class TreePanel extends JScrollPane implements ListSelectionListener
    {
        // the list that displays the providers
        private JList  list_;

        // a vector with valid file type providers
        private Vector fileTypeProviders_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // inner classes
        // <><><><><><><><><><><><><><><><><><><><><>

        private class ProviderComparator implements Comparator
        {

            public int compare(Object obj1, Object obj2)
            {
                if(!(obj1 instanceof IFileInterpreterProvider)
                        || !(obj2 instanceof IFileInterpreterProvider))
                {
                    return 0;
                }

                return ((IFileInterpreterProvider) obj1)
                        .getFileType()
                        .compareTo(
                                ((IFileInterpreterProvider) obj2).getFileType());
            }

        }

        /**
         * The file type provider cell renderer represents one cell in the list.
         * He is responsible for displaying the right icon and text in the list.
         * In fact the list is nothing more than many single labels.
         */
        private class FileTypeProviderCellRenderer extends JLabel implements
                ListCellRenderer
        {
            // <><><><><><><><><><><><><><><><><><><><><>
            // Implementation of the ListCellRenderer interface
            // <><><><><><><><><><><><><><><><><><><><><>
            /* @see javax.swing.ListCellRenderer */
            public Component getListCellRendererComponent(JList list,
                    Object value, int index, boolean isSelected,
                    boolean cellHasFocus)
            {
                IFileInterpreterProvider provider;

                if(value instanceof IFileInterpreterProvider)
                {
                    provider = (IFileInterpreterProvider) value;
                    setText(provider.getFileType());
                    setIcon(provider.getIcon());
                }
                else
                {
                    setText(value.toString());
                }

                if(isSelected)
                {
                    setBackground(list.getSelectionBackground());
                    setForeground(list.getSelectionForeground());
                }
                else
                {
                    setBackground(list.getBackground());
                    setForeground(list.getForeground());
                }

                setEnabled(list.isEnabled());
                setFont(list.getFont());
                setOpaque(true);
                return this;
            }
        }

        /**
         * This class provides a method that reacts on double clicks on the
         * jlist. If a double click is performed, the selected index is
         * retrieved, and the next method is called.
         */
        private class ListMouseListener extends MouseAdapter
        {
            // <><><><><><><><><><><><><><><><><><><><><>
            // Overidden methods from java.awt.event.MouseAdapter
            // <><><><><><><><><><><><><><><><><><><><><>
            /* @see java.awt.event.MouseAdapter */
            public void mouseClicked(MouseEvent event)
            {
                int index;

                index = list_.locationToIndex(event.getPoint());
                if(index >= 0)
                {
                    setCurrentProvider(index);
                    if(event.getClickCount() == 2)
                    {
                        next();
                    }
                }

                super.mouseClicked(event);
            }
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Constructs a new tree panel that displays all valid file type
         * providers in a list.
         */
        public TreePanel()
        {
            // assign memory for the vector of valid file type providers
            fileTypeProviders_ = new Vector();

            // create the list that displays the providers
            list_ = new JList();
            list_.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            list_.addListSelectionListener(this);
            list_.addMouseListener(new ListMouseListener());
            getViewport().setView(list_);

            // add the cell renderer to the list (creates labels from the
            // vector that is created in fillList)
            list_.setCellRenderer(new FileTypeProviderCellRenderer());
            fillList();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // supporting methods
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Returns true if the list contains at least one valid file type
         * provider and false if no valid file type provider was found.
         * 
         * @return true if there are any valid file type providers
         */
        public boolean hasProviders()
        {
            return (fileTypeProviders_.size() > 0);
        }

        /**
         * Creates a list with valid file type providers and assigns the created
         * list to the JList to display the elements.
         */
        private void fillList()
        {
            FileInterpreterManager fileTypeManager = FileInterpreterManager
                    .getInstance();
            IFileInterpreterProvider providers[];

            providers = fileTypeManager.getFileInterpreterProviders();
            Arrays.sort(providers, new ProviderComparator());
            for (int i = 0; i < providers.length; i++)
            {
                if(providers[i].hasNewFileWizard()
                        && providers[i].canCreateNewFile(Workbench
                                .getInstance()))
                {
                    fileTypeProviders_.add(providers[i]);
                }
            }

            list_.setListData(fileTypeProviders_);
        }

        /**
         * Stets the selectedProvider_ variable based on the current index. If
         * the index is invalid the variable is set to null. The index must
         * represent the index in the jlist. This index is zero-based.
         * 
         * @param index
         *            zero-based index according to the jlist
         */
        private void setCurrentProvider(int index)
        {
            if(index < 0 || index >= fileTypeProviders_.size())
            {
                selectedProvider_ = null;
                return;
            }

            selectedProvider_ = (IFileInterpreterProvider) fileTypeProviders_
                    .get(index);

            dialog_.setDetailedDescription(selectedProvider_.getDescription());
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_NEXT, true);
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the ListSelectionListener interface
        // <><><><><><><><><><><><><><><><><><><><><>
        /* @see javax.swing.event.ListSelectionListener */
        public void valueChanged(ListSelectionEvent event)
        {
            int index = event.getFirstIndex();

            setCurrentProvider(index);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new dialog for creating new files. This dialog displays all
     * valid file type providers and initializes a new wizard process for
     * creating the selected file type.
     */
    public NewDialog()
    {
        treePanel_ = new TreePanel();
        dialog_ = new StandardModalDialog("New...", "Select a template",
                treePanel_, ResourceLoader.getInstance().getImage(
                        ResourceConstants.IMAGE_DIALOG_NEW),
                DialogConstants.BUTTON_SET_WIZARD, DialogConstants.BUTTON_NEXT);

        dialog_.setButtonsEnabled(DialogConstants.BUTTON_PREVIOUS
                | DialogConstants.BUTTON_NEXT | DialogConstants.BUTTON_FINISH,
                false);
        dialog_.addStandardDialogListener(this);

        if(!treePanel_.hasProviders())
        {
            dialog_.setNotice("No file type providers found!",
                    DialogConstants.TYPE_ERROR);
        }
    }

    public void show()
    {
        dialog_.setVisible(true);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Performs the action when the next button is clicked. If a valid provider
     * is selected, the method closes the dialog and opens the according wizard.<br>
     * Otherwise the method brings an error and returns.
     */
    private void next()
    {
        if(selectedProvider_ == null)
        {
            dialog_.setNotice("Please select a file type first!",
                    DialogConstants.TYPE_ERROR);
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_NEXT, false);
            return;
        }

        // hide the current dialog to show the new one
        dialog_.setVisible(false);
        new WizardDialog(selectedProvider_.getNewFileWizard());

        dialog_.dispose();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the StandardDialogListener
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.gui.common.dialogs.event.StandardDialogListener */
    public void buttonClicked(ButtonClickedEvent event)
    {
        switch (event.getType())
        {
        case DialogConstants.BUTTON_CANCEL:
            dialog_.dispose();
            return;

        case DialogConstants.BUTTON_NEXT:
            next();
            return;
        }
    }
}
