package viptool.impl.base.gui.dialog;

import javax.swing.JDialog;
import javax.swing.JPanel;

import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.event.ButtonClickedEvent;
import viptool.base.gui.dialog.event.IStandardDialogListener;
import viptool.base.wizard.IWizard;
import viptool.base.wizard.IWizardPage;

/**
 * The wizard dialog is a standard dialog for processing wizard steps. It takes
 * a wizard as argument on creation time and initialize a standard dialog. The
 * used button set contains the buttons previous, next, finish and cancel. The
 * cancel buttons is always enabled by default except a wizard page explicit
 * disables it.<br>
 * A click to finish or cancel closes the window after calling methods from the
 * wizard.
 */
public class WizardDialog implements IStandardDialogListener
{
    // the standard dialog that displays the wizard pages
    private StandardModalDialog dialog_;

    // the wizard that is processed in this dialog
    private IWizard             wizard_;

    // the current displayed wizard page
    private IWizardPage         currentPage_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new wizard dialog with the given wizard. The given wizard must
     * not be null. The dialog first shows the page returned by getFirstPage().
     * 
     * @param wizard
     *            the wizard to process in this dialog instance
     */
    public WizardDialog(IWizard wizard)
    {
        wizard_ = wizard;

        dialog_ = new StandardModalDialog(wizard.getTitle(), "", new JPanel(),
                null, DialogConstants.BUTTON_SET_WIZARD,
                DialogConstants.BUTTON_NEXT);

        dialog_.setButtonsEnabled(DialogConstants.BUTTON_PREVIOUS
                | DialogConstants.BUTTON_NEXT | DialogConstants.BUTTON_FINISH,
                false);
        dialog_.setButtonsEnabled(DialogConstants.BUTTON_CANCEL, true);

        // be sure the window is disposed after closing so we will receive
        // the window closed event when the user pressed the x in the title bar
        dialog_.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

        dialog_.addStandardDialogListener(this);
        dialog_.addWindowListener(wizard);

        currentPage_ = wizard_.getFirstPage(dialog_);
        dialog_.setMainDescription(currentPage_.getMainDescription());
        dialog_.setDetailedDescription(null);
        dialog_.setImage(currentPage_.getImage());
        dialog_.removeNotice();
        dialog_.setContent(currentPage_.getContent());
        
        dialog_.pack();
        dialog_.setVisible(true);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the StandardDialogListener interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.gui.common.dialogs.even.StandardDialogListener */
    public void buttonClicked(ButtonClickedEvent event)
    {
        IWizardPage nextPage = null;

        switch (event.getType())
        {
        case DialogConstants.BUTTON_NEXT:
            // set the default button configuration
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_NEXT
                    | DialogConstants.BUTTON_FINISH, false);
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_CANCEL, true);

            // get the new page
            nextPage = wizard_.getNextPage();
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_PREVIOUS,
                    nextPage.canGoToPrevoius());
            break;

        case DialogConstants.BUTTON_PREVIOUS:
            // set the default button configuration
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, false);
            
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_CANCEL
                    | DialogConstants.BUTTON_NEXT, true);

            // get the new page
            nextPage = wizard_.getPreviousPage();
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_PREVIOUS,
                    nextPage.canGoToPrevoius());
            break;

        case DialogConstants.BUTTON_FINISH:
            nextPage = null;
            wizard_.finish();
            dialog_.setVisible(false);
            dialog_.dispose();
            return;

        case DialogConstants.BUTTON_CANCEL:
            nextPage = null;
            wizard_.cancel();
            dialog_.setVisible(false);
            dialog_.dispose();
            return;
        }

        // if we have a next page, display the page know
        if(nextPage != null)
        {
            currentPage_ = nextPage;

            // initialize the top
            dialog_.setMainDescription(currentPage_.getMainDescription());
            dialog_.setDetailedDescription(null);
            dialog_.setImage(currentPage_.getImage());

            // reset notice
            dialog_.removeNotice();

            // set the new content
            dialog_.setContent(currentPage_.getContent());
        }
        else
        {
            // show a warning
            dialog_.setNotice("The page cannot be displayed. This is not "
                    + "necessarily an error!", DialogConstants.TYPE_WARNING);
            dialog_.setButtonsEnabled(event.getType(), false);
        }
    }
}
