package viptool.impl.base.gui.layoutmanager.listlayout;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager2;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Vector;


/**
 * This layout manager supports a GUI like the JList. The advantage of
 * using the ListLayout instead of JList is the normal behavior of 
 * list components, especially the reaction on mouse events.
 */
public class ListLayout implements LayoutManager2{

	/**
	 * This class is a container for all information about a component.
	 */
	class ComponentInfo{

		public Component cmp_;
		public ListLayoutConstraints constraints_;
		public String name_;
		
		public ComponentInfo(Component cmp, Object constraints, String name){
			cmp_ = cmp;
			if(constraints instanceof ListLayoutConstraints){
				constraints_ = (ListLayoutConstraints)constraints;
			}else{
				constraints_ = null;
			}
			name_ = name;
		}
	}
	
	private Vector components_;
	private HashMap comps_;
	
	private int heightPref_;
	private int widthPref_;
	
	private int heightMin_;
	private int widthMin_;
	
	private int heightMax_;
	private int widthMax_;
	
	private Container parent_;
	
	private boolean removeLock_ = false;
	
	/**
	 * This method creates a new ListLayout. The parameter container is necessary
	 * for some layout operations like removing a list component.
	 */
	public ListLayout(Container parent){
		components_ = new Vector();
		comps_ = new HashMap();
		parent_ = parent;
	}
	
	/**
	 * method of LayoutManager2 interface 
	 * This method adds a new component to this layout manager
	 */
	public void addLayoutComponent(Component comp, Object constraints) {
		if(comps_.get(comp) == null){
			ComponentInfo compInfo = new ComponentInfo(comp, constraints, comp.toString());
			if(compInfo.constraints_ != null){
				int pos = compInfo.constraints_.getPos();
				if(pos >= 0 && pos <= components_.size()){
					components_.insertElementAt(compInfo, pos);
				}else{
					components_.add(compInfo);
					compInfo.constraints_ = new ListLayoutConstraints(components_.indexOf(compInfo), 0);
				}
			}else{
				components_.add(compInfo);
				compInfo.constraints_ = new ListLayoutConstraints(components_.indexOf(compInfo), 0);
			}
			comps_.put(comp, compInfo);
			calcSize();
		}
	}

	/**
	 * method of LayoutManager2 interface
	 * This method returns 0 (float), this means the layout is left-aligned
	 */
	public float getLayoutAlignmentX(Container target) {
		return 0f;
	}

	/**
	 * method of LayoutManager2 interface
	 * This method returns 0 (float), this means the layout is top-aligned
	 */
	public float getLayoutAlignmentY(Container target) {
		return 0f;
	}

	/**
	 * method of LayoutManager2 interface
	 * This method recalculate the size of the layout.
	 */
	public void invalidateLayout(Container target) {
		calcSize();
	}

	/**
	 * method of LayoutManager2 interface
	 * This method returns the maximum layout size.
	 */
	public Dimension maximumLayoutSize(Container target) {
		if(widthMax_ == 0 || heightMax_ == 0){
			calcSize();
		}
		return new Dimension(widthMax_, heightMax_);
	}

	/**
	 * method of LayoutManager2 interface
	 * This method adds a new layout component with the given name. 
	 */
	public void addLayoutComponent(String name, Component comp) {
		if(comps_.get(comp) == null){
			ComponentInfo compInfo = new ComponentInfo(comp, null, comp.toString());
			components_.add(compInfo);
			compInfo.constraints_ = new ListLayoutConstraints(components_.indexOf(compInfo), 0);
			comps_.put(comp, compInfo);
			calcSize();
		}
	}

	/**
	 * method of LayoutManager2 interface
	 * This method layout the container.
	 */
	public void layoutContainer(Container target) {
		synchronized (target.getTreeLock()) {
	        Insets insets = target.getInsets();
	        int maxwidth = target.getWidth() - (insets.left + insets.right);
	        int maxheight = target.getHeight() - (insets.left + insets.right); 
	        int y = insets.top;
	        int x = insets.left;
	        
	        Enumeration comps = components_.elements();
			
	        while(comps.hasMoreElements()){
				ComponentInfo compInfo = (ComponentInfo)comps.nextElement();
				Component c = compInfo.cmp_;
								
				y += compInfo.constraints_.getSpaceTop();
				int top = y; 
				int left = compInfo.constraints_.getSpaceLeft() + x;
				int height = compInfo.cmp_.getPreferredSize().height;
				
				y += height + compInfo.constraints_.getSpaceBottom();
				
				if(top > maxheight){
					height = 0;
				}else if(top + height > maxheight){
					height = maxheight-top;
				}
				
				c.setBounds(left, top, maxwidth, height);
			}
		}
	}

	/**
	 * method of LayoutManager2 interface
	 * This method returns the minimum layout size.
	 */
	public Dimension minimumLayoutSize(Container parent) {
		return new Dimension(widthMin_, heightMin_);
	}

	/**
	 * method of LayoutManager2 interface
	 * This method returns the preferred layout size.
	 */
	public Dimension preferredLayoutSize(Container parent) {
		if(widthPref_ == 0 || heightPref_ == 0){
			calcSize();
		}
		int width = widthPref_, height = heightPref_;
		
		return new Dimension(width, height);
	}

	/**
	 * method of LayoutManager2 interface
	 * This method returns the given layout component.
	 * 
	 * @param comp
	 * 		component to remove
	 */
	public void removeLayoutComponent(Component comp) {
		if(!removeLock_){
			removeComponent(comp);
		}
	}
	
	/**
	 * This method removes the given component.
	 * 
	 * @param comp
	 * 		component to remove
	 */
	public void removeComponent(Component comp) {
		ComponentInfo compInfo = (ComponentInfo)comps_.get(comp);
		if(compInfo != null){
			components_.remove(compInfo);
			comps_.remove(comp);
			parent_.remove(compInfo.cmp_);
		}
	}
	
	/**
	 * This method remove the component with the given index.
	 * 
	 * @param index
	 * 		index of the component to remove
	 */
	synchronized public void removeComponent(int index){
		if(index >= 0 && index < components_.size()){
			ComponentInfo compInfo = (ComponentInfo)components_.get(index);
			if(compInfo != null){
				comps_.remove(compInfo.cmp_);
				components_.remove(compInfo);
				
				removeLock_ = true;
				parent_.removeAll();
				removeLock_ = false;
				
				Enumeration comps = components_.elements();
				while(comps.hasMoreElements()){
					parent_.add(((ComponentInfo)comps.nextElement()).cmp_);
				}
				
				parent_.repaint();
			}
		}
	}
	
	/**
	 * This method moves the component with the given index up.
	 * 
	 * @param index
	 * 		index of the component to move up
	 */
	public void moveComponentUp(int index){
		if(index > 0 && index < components_.size()){
			ComponentInfo compInfo = (ComponentInfo)components_.get(index);
			components_.remove(index);
			components_.insertElementAt(compInfo, index-1);
			layoutContainer(parent_);
			parent_.invalidate();
		}
	}
	
	/**
	 * This method moves the component with the given index down.
	 * 
	 * @param index
	 * 		index of the component to move down
	 */
	public void moveComponentDown(int index){
		if(index >= 0 && index < components_.size()-1){
			ComponentInfo compInfo = (ComponentInfo)components_.get(index);
			components_.remove(index);
			components_.insertElementAt(compInfo, index+1);
			layoutContainer(parent_);
			parent_.invalidate();
		}
	}
	
	/**
	 * This method moves the given component up.
	 * 
	 * @param comp
	 * 		component to move up
	 */
	public void moveComponentUp(Component comp){
		moveComponentUp(components_.indexOf(comps_.get(comp)));
	}
	
	/**
	 * This method moves the given component up.
	 * 
	 * @param comp
	 * 		component to move up
	 */
	public void moveComponentDown(Component comp){
		moveComponentDown(components_.indexOf(comps_.get(comp)));
	}
	
	/**
	 * This method moves the selected components up. To use this feature it is necessary
	 * that the components which are shown into the list implement the interface
	 * ListLayoutSelectionSupport. The selection mechanism and the markup of selected
	 * components is a job of the components itself.
	 */
	public void moveSelectedUp(){
		Enumeration comps = components_.elements();
		while(comps.hasMoreElements()){
			ComponentInfo comp = (ComponentInfo)comps.nextElement();
			if(comp.cmp_ instanceof ListLayoutSelectionSupport){
				if(((ListLayoutSelectionSupport)(comp.cmp_)).isSelected()){
					moveComponentUp(comp.cmp_);
				}
			}
		}
	}
	
	/**
	 * This method moves the selected components down. To use this feature it is necessary
	 * that the components which are shown into the list implement the interface
	 * ListLayoutSelectionSupport. The selection mechanism and the markup of selected
	 * components is a job of the components itself.
	 */
	public void moveSelectedDown(){
		Vector sel = new Vector();
		Enumeration comps = components_.elements();
		while(comps.hasMoreElements()){
			ComponentInfo comp = (ComponentInfo)comps.nextElement();
			if(comp.cmp_ instanceof ListLayoutSelectionSupport){
				if(((ListLayoutSelectionSupport)(comp.cmp_)).isSelected()){
					sel.add(comp.cmp_);
				}
			}
		}
		for(int i=sel.size()-1; i>=0; i--){
			moveComponentDown((Component)sel.get(i));
		}
	}
	
	/**
	 * This method removes the selected components. To use this feature it is necessary
	 * that the components which are shown into the list implement the interface
	 * ListLayoutSelectionSupport. The selection mechanism and the markup of selected
	 * components is a job of the components itself.
	 */
	public void removeSelected(){
		Vector sel = new Vector();
		Enumeration comps = components_.elements();
		while(comps.hasMoreElements()){
			ComponentInfo comp = (ComponentInfo)comps.nextElement();
			if(comp.cmp_ instanceof ListLayoutSelectionSupport){
				if(((ListLayoutSelectionSupport)(comp.cmp_)).isSelected()){
					sel.add(comp.cmp_);
				}
			}
		}
		comps = sel.elements();
		while(comps.hasMoreElements()){
			removeComponent((Component)comps.nextElement());
		}
	}
	
	/**
	 * This method calculates the different sizes of the layout.
	 */
	private void calcSize(){
		widthMax_ = widthMin_ = widthPref_ = heightMax_ = heightMin_ = heightPref_ = 0;
		
		Enumeration components = components_.elements();
		while(components.hasMoreElements()){
			ComponentInfo compInfo = (ComponentInfo)components.nextElement();
			int width = compInfo.cmp_.getMaximumSize().width 
						+ compInfo.constraints_.getSpaceLeft() 
						+ compInfo.constraints_.getSpaceRight();
			if(width >= widthMax_){
				widthMax_ = width;
			}
			
			width = compInfo.cmp_.getMinimumSize().width
						+ compInfo.constraints_.getSpaceLeft() 
						+ compInfo.constraints_.getSpaceRight();
			if(width >= widthMin_){
				widthMin_ = width;
			}
			
			width = compInfo.cmp_.getPreferredSize().width 
						+ compInfo.constraints_.getSpaceLeft() 
						+ compInfo.constraints_.getSpaceRight();
			if(width >= widthPref_){
				widthPref_ = width;
			}
			
			heightMax_ += compInfo.cmp_.getMaximumSize().height 
						+ compInfo.constraints_.getSpaceTop() 
						+ compInfo.constraints_.getSpaceBottom();
			heightMin_ += compInfo.cmp_.getMinimumSize().height 
						+ compInfo.constraints_.getSpaceTop()
						+ compInfo.constraints_.getSpaceBottom();
			heightPref_ += compInfo.cmp_.getPreferredSize().height 
						+ compInfo.constraints_.getSpaceTop() 
						+ compInfo.constraints_.getSpaceBottom();
		}
	}
	
	/**
	 * This method returns a vector which contains the selected elements.
	 * 
	 * @return
	 * 		Vector of Components
	 */
	public Vector getSelected(){
		Vector selected = new Vector();
		
		Enumeration comps = components_.elements();
		while(comps.hasMoreElements()){
			ComponentInfo comp = (ComponentInfo)comps.nextElement();
			if(comp.cmp_ instanceof ListLayoutSelectionSupport){
				if(((ListLayoutSelectionSupport)(comp.cmp_)).isSelected()){
					selected.add(comp.cmp_);
				}
			}
		}
		
		return selected;
	}
}
