package viptool.impl.base.gui.menu;

import java.awt.Component;
import java.awt.Container;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSeparator;
import javax.swing.MenuElement;

import viptool.base.action.ContainerAction;
import viptool.base.action.ExtendedAction;
import viptool.base.action.IActionFilter;
import viptool.base.action.IActionHandler;
import viptool.base.action.IActionPostProcessor;
import viptool.base.action.PlaceHolderAction;
import viptool.base.action.SeparatorAction;
import viptool.base.gui.menu.IExtendedMenu;
import viptool.base.gui.menu.event.ExtendedMenuEvent;
import viptool.base.gui.menu.event.IExtendedMenuListener;
import viptool.impl.base.action.ActionHandler;
import viptool.impl.base.action.StandardActionFilter;
import viptool.impl.base.action.StandardActionPostProcessor;
import viptool.impl.logger.Logger;

/**
 * The extended menu bar is a menu bar that can listen to a template and changes
 * itself if the template is changed.
 */
public class ExtendedMenu implements IExtendedMenu, IExtendedMenuListener
{
    private int                  instance_;

    private HashSet              listeners_;

    private HashMap              elements_;

    private HashMap              buttonGroups_;

    private MenuElement          root_;

    private IActionFilter        filter_;

    private IActionPostProcessor postProcessor_;

    public ExtendedMenu(MenuElement root, int instance)
    {
        instance_ = instance;
        filter_ = new StandardActionFilter();
        postProcessor_ = new StandardActionPostProcessor();

        buttonGroups_ = new HashMap();
        listeners_ = new HashSet();

        elements_ = new HashMap();

        root_ = root;
        elements_.put("", root);
    }

    protected MenuElement getRootElement()
    {
        return root_;
    }

    public Component getMenuItem(String id)
    {
        return (Component) elements_.get(id);
    }

    public ExtendedAction addMenuItem(String id, String group, String insertAt,
            int seek)
    {
        String buttonGroupId;
        String type;
        ExtendedAction action;
        Component menuItem;
        ButtonGroup buttonGroup;

        // get the action and read important information
        action = ActionHandler.getInstance().getAction(instance_, id);
        if (action == null)
        {
            Logger.getInstance().error(
                    "ExtendedMenu.addMenuItem: The action '" + id
                            + "' was not found. Be sure you load the "
                            + "appropriate action xml file before "
                            + "constructing the menu.");
            return null;
        }

        if (!filter_.matchesFilter(action))
        {
            return null;
        }

        // check if the menu item is already, separators are ignored
        // automatically becaue they are not in the list
        if (getMenuItem(id) != null)
        {
            // the menu item is already there, don't add it twice
            return action;
        }

        if (action instanceof SeparatorAction)
        {
            menuItem = new JSeparator();
        }
        else if (action instanceof PlaceHolderAction)
        {
            menuItem = new JMenuItem(action);
            menuItem.setVisible(false);
        }
        else if (action instanceof ContainerAction)
        {
            menuItem = new JMenu(action);
        }
        else
        {
            type = (String) action.getValue(ExtendedAction.ATTR_TYPE);

            // first create the new menu item, that has to be inserted
            if (ExtendedAction.TYPE_RADIO.equals(type))
            {
                menuItem = new JRadioButtonMenuItem(action);

                buttonGroupId = (String) action.getValue(ExtendedAction.ATTR_GROUP);

                if (buttonGroupId != null && buttonGroupId.length() > 0)
                {
                    buttonGroup = (ButtonGroup) buttonGroups_.get(buttonGroupId);

                    if (buttonGroup == null)
                    {
                        buttonGroup = new ButtonGroup();
                        buttonGroups_.put(buttonGroupId, buttonGroup);
                    }

                    buttonGroup.add((JRadioButtonMenuItem) menuItem);
                }
            }
            else if (ExtendedAction.TYPE_CHECKBOX.equals(type))
            {
                menuItem = new JCheckBoxMenuItem(action);
            }
            else
            {
                menuItem = new JMenuItem(action);
            }
        }

        postProcessor_.processAction(action);

        // add the component to our template bar
        addComponent(menuItem, id, group, insertAt, seek);
        return action;
    }

    private void addComponent(Component component, String id,
            String containerId, String insertAt, int seek)
    {
        int index = -1;
        Container menuContainer = (Container) elements_.get(containerId);
        Component[] menuElements;

        if (seek != 0 && insertAt.length() > 0)
        {
            if (menuContainer instanceof JMenu)
            {
                menuElements = ((JMenu) menuContainer).getMenuComponents();
            }
            else
            {
                menuElements = menuContainer.getComponents();
            }

            index = menuElements.length;
            for (int i = 0; i < menuElements.length; i++)
            {
                if (menuElements[i] instanceof JMenuItem)
                {
                    if (((ExtendedAction) ((JMenuItem) menuElements[i]).getAction()).getIdKey().equals(
                            insertAt))
                    {
                        index = i + seek;
                        break;
                    }
                }
            }

            if (index < 0)
            {
                // add at beginning
                index = 0;
            }

            if (index > menuElements.length)
            {
                // add to end
                index = -1;
            }
        }

        menuContainer.add(component, index);
        if (id != null && !id.equals(IActionHandler.ELEMENT_SEPARATOR))
        {
            elements_.put(id, component);
        }

        fireMenuItemAddedEvent(id, containerId, insertAt, seek);
    }

    public ExtendedAction removeMenuItem(String id)
    {
        Component component = (Component) elements_.get(id);

        if (component != null)
        {
            component.getParent().remove(component);
            elements_.remove(id);

            fireMenuItemRemovedEvent(id);
        }

        return ActionHandler.getInstance().getAction(instance_, id);
    }

    public void setFilter(IActionFilter filter)
    {
        filter_ = filter;
    }

    public void setPostProcessor(IActionPostProcessor postProcessor)
    {
        postProcessor_ = postProcessor;
    }

    public IActionFilter getFilter()
    {
        return filter_;
    }

    public IActionPostProcessor getPostProcessor()
    {
        return postProcessor_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // listener support
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Adds the given listener to the menu template. Listeners are notified
     * about changes in the template.
     * 
     * @param listener
     *            the listener to add
     */
    public void addExtendedMenuListener(IExtendedMenuListener listener)
    {
        if (listener != null)
        {
            listeners_.add(listener);
        }
        else
        {
            Logger.getInstance().warning(
                    "null was given as new listener to a menu template!");
        }
    }

    /**
     * Removes the given listener from the menu template. This listener will no
     * longer be notified about changes in the template.
     * 
     * @param listener
     *            the listener to remove
     */
    public void removeExtendedMenuListener(IExtendedMenuListener listener)
    {
        if (listener != null)
        {
            listeners_.remove(listener);
        }
        else
        {
            Logger.getInstance().warning(
                    "null was given as listener for removing "
                            + "to a menu template!");
        }
    }

    /**
     * This method will notify all the template listeners, that an element was
     * added to the template.
     * 
     * @param id
     *            the id of the element that was added
     * @param container
     *            the enclosing container of the element
     * @param insertAt
     *            the positional string parameter where to add the component
     * @param seek
     *            the parameter for determining the exact position relative to
     *            insertAt
     */
    private void fireMenuItemAddedEvent(String id, String container,
            String insertAt, int seek)
    {
        Iterator it;
        IExtendedMenuListener listener;
        ExtendedMenuEvent event;

        event = new ExtendedMenuEvent(this, id, container, insertAt, seek);

        it = listeners_.iterator();
        while (it.hasNext())
        {
            listener = (IExtendedMenuListener) it.next();
            listener.menuElementAdded(event);
        }
    }

    /**
     * Notifies the listeners that the element with the specified id was removed
     * from the template.
     * 
     * @param id
     *            the id that was removed from the template
     */
    private void fireMenuItemRemovedEvent(String id)
    {
        Iterator it;
        IExtendedMenuListener listener;
        ExtendedMenuEvent event;

        event = new ExtendedMenuEvent(this, id);

        it = listeners_.iterator();
        while (it.hasNext())
        {
            listener = (IExtendedMenuListener) it.next();
            listener.menuElementRemoved(event);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IExtendedMenuListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    public void menuElementAdded(ExtendedMenuEvent event)
    {
        addMenuItem(event.getId(), event.getContainer(), event.getInsertAt(),
                event.getSeek());
    }

    public void menuElementRemoved(ExtendedMenuEvent event)
    {
        removeMenuItem(event.getId());
    }
}
