package viptool.impl.base.gui.menu;

import java.awt.Component;
import java.awt.Container;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;

import viptool.base.action.ExtendedAction;
import viptool.base.action.IActionFilter;
import viptool.base.action.IActionHandler;
import viptool.base.action.IActionPostProcessor;
import viptool.base.error.VipToolRuntimeException;
import viptool.base.gui.menu.IExtendedMenu;
import viptool.base.gui.menu.IMenuTemplate;
import viptool.base.gui.menu.event.ExtendedMenuEvent;
import viptool.base.gui.menu.event.IExtendedMenuListener;
import viptool.impl.base.action.StandardActionFilter;
import viptool.impl.base.action.StandardActionPostProcessor;
import viptool.impl.logger.Logger;

/* @see viptool.base.gui.menu.IMenuTemplate */
public class MenuTemplate implements IMenuTemplate, IExtendedMenuListener
{
    // the id of the template
    private String        id_;

    // the menu bar of the template
    private IExtendedMenu extendedMenu_;

    // 
    private Class         menuClass_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new menu template with the specified id. The menu template uses
     * an ExtendedMenuBar to store the template. If a menu is requested the
     * template menu is cloned.
     * 
     * @param id
     *            the id for the new template
     */
    public MenuTemplate(Class menuClass, String id)
    {
        id_ = id;
        menuClass_ = menuClass;

        extendedMenu_ = createStorage(0);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>

    private IExtendedMenu createStorage(int instance)
    {
        Object menuClass = null;

        try
        {
            menuClass = menuClass_.getConstructor(new Class[]
            { int.class }).newInstance(new Object[]
            { new Integer(instance) });
        }
        catch (InstantiationException exception)
        {
            throw new VipToolRuntimeException(exception);
        }
        catch (IllegalAccessException exception)
        {
            throw new VipToolRuntimeException(exception);
        }
        catch (InvocationTargetException exception)
        {
            throw new VipToolRuntimeException(exception);
        }
        catch (NoSuchMethodException exception)
        {
            throw new VipToolRuntimeException(exception);
        }

        return (IExtendedMenu) menuClass;
    }

    private void removeUnnecessarySeparatorsContainer(Container menu)
    {
        boolean removeSeparator;
        Component[] components;

        removeSeparator = true;
        components = menu.getComponents();
        for (int i = 0; i < components.length; i++)
        {
            if (components[i] instanceof JMenuItem)
            {
                removeSeparator = false;
                if (components[i] instanceof JMenu)
                {
                    removeUnnecessarySeparatorsContainer((Container) components[i]);
                }
            }
            else if (components[i] instanceof JSeparator)
            {
                if (removeSeparator || i == (components.length - 1))
                {
                    menu.remove(components[i]);
                }

                removeSeparator = true;
            }
        }
    }

    /**
     * Clones the submenu with the given id. Only the components given as
     * argument are cloned. The clones are added to the given menuBar at the
     * same position they are in the original.
     * 
     * @param destination
     *            the cloned menu elements are added to this menu bar
     * @param menuId
     *            the id of the submenu (container) that has to be cloned
     * @param components
     *            the elements of the container that has to be cloned
     */
    private void cloneMenu(IExtendedMenu destination, String menuId,
            Component[] components)
    {
        String id;
        JMenuItem item;
        ExtendedAction action;

        // iterate over all childrens and clone every one
        for (int i = 0; i < components.length; i++)
        {
            if (components[i] instanceof JMenuItem)
            {
                // it is a menu item, let us get the information from the
                // / corresponding action
                item = (JMenuItem) components[i];
                action = (ExtendedAction) item.getAction();

                Logger.getInstance().debug(
                        "MenuTemplate: Clone action " + action.getIdKey());

                id = action.getIdKey();
                // add the item to the menu bar
                Logger.getInstance().debug(
                        "MenuTemplate: Add action " + action.getIdKey());

                if (destination.addMenuItem(id, menuId, "", 0) != null)
                {
                    // if the item is a submenu, clone these children to
                    if (item instanceof JMenu)
                    {
                        // call the method recursibley to the children
                        cloneMenu(destination, id,
                                ((JMenu) item).getMenuComponents());
                    }
                }
            }
            else if (components[i] instanceof JSeparator)
            {
                // Its a separator, add it to the end of the current container
                // separators have all the same id
                destination.addMenuItem(IActionHandler.ELEMENT_SEPARATOR,
                        menuId, "", 0);
            }
        }
    }

    private void cloneContainer(IExtendedMenu destination, Container menu)
    {
        ExtendedAction action;
        Component[] components;

        components = menu.getComponents();
        for (int i = 0; i < components.length; i++)
        {
            if (components[i] instanceof JMenuItem)
            {
                action = (ExtendedAction) ((JMenuItem) components[i]).getAction();

                if (destination.addMenuItem(action.getIdKey(), "", "", 0) != null)
                {
                    if (components[i] instanceof JMenu)
                    {
                        cloneMenu(destination, action.getIdKey(),
                                ((JMenu) components[i]).getMenuComponents());
                    }
                }
            }
            else if (components[i] instanceof JSeparator)
            {
                destination.addMenuItem(IActionHandler.ELEMENT_SEPARATOR, "",
                        "", 0);
            }

        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IMenuTemplate interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.gui.menu.IMenuTemplate */
    public String getId()
    {
        return id_;
    }

    /* @see viptool.base.gui.menu.IMenuTemplate */
    public void addMenuItem(String id, String path, String insertAt, int seek)
    {
        extendedMenu_.addMenuItem(id, path, insertAt, seek);
    }

    /* @see viptool.base.gui.menu.IMenuTemplate */
    public void removeMenuItem(String id)
    {
        extendedMenu_.removeMenuItem(id);
    }

    /* @see viptool.base.gui.menu.IMenuTemplate */
    public Component getMenuItem(String id)
    {
        return extendedMenu_.getMenuItem(id);
    }

    /* @see viptool.base.gui.menu.IMenuTemplate */
    public void setParent(IMenuTemplate parent)
    {
        IExtendedMenu parentBar;

        // create a new menu bar as root to clear old entries
        extendedMenu_ = createStorage(0);

        // clone the parent bar as base for the new template
        parentBar = parent.createMenu(0);

        // add us as listener to the parent, so we can changes us, if the parent
        // changes
        parentBar.addExtendedMenuListener((ExtendedMenuBar) extendedMenu_);

        cloneContainer(extendedMenu_, (Container) parentBar.getMenuItem(""));
    }

    /* @see viptool.base.gui.menu.IMenuTemplate */
    public IExtendedMenu createMenu(int instance)
    {
        return createMenu(instance, new StandardActionFilter());
    }

    public IExtendedMenu createMenu(int instance, IActionFilter filter)
    {
        return createMenu(instance, filter, new StandardActionPostProcessor());

    }

    /* @see viptool.base.gui.menu.IMenuTemplate */
    public IExtendedMenu createMenu(int instance, IActionFilter filter,
            IActionPostProcessor postProcessor)
    {
        IExtendedMenu menuBar;

        // if the template is requested we can just return our menu bar
        if (instance == 0)
        {
            return extendedMenu_;
        }

        // create a new menubar that will be a clone of the template but with
        // an instance id that is not equal to zero
        menuBar = createStorage(instance);

        if (IExtendedMenuListener.class.isAssignableFrom(menuBar.getClass()))
        {
            // let the new menu bar be notified about changes in the template
            extendedMenu_.addExtendedMenuListener((IExtendedMenuListener) menuBar);
        }
        else
        {
            Logger.getInstance().warning(
                    "The template creates a menu, that does"
                            + "not implement the IExtendedMenuListener"
                            + " interface, so that changes at the "
                            + "template will not affect the created menu!");
        }

        // clone our template
        menuBar.setFilter(filter);
        menuBar.setPostProcessor(postProcessor);
        cloneContainer(menuBar, (Container) extendedMenu_.getMenuItem(""));

        if (instance > 0)
        {
            removeUnnecessarySeparatorsContainer((Container) menuBar.getMenuItem(""));
        }

        return menuBar;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IMenuTemplateListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    public void menuElementAdded(ExtendedMenuEvent event)
    {
        extendedMenu_.addMenuItem(event.getId(), event.getContainer(),
                event.getInsertAt(), event.getSeek());
    }

    public void menuElementRemoved(ExtendedMenuEvent event)
    {
        extendedMenu_.removeMenuItem(event.getId());
    }
}
