package viptool.impl.base.gui.toolbar;

import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.event.ChangeListener;

import viptool.base.action.ExtendedAction;
import viptool.base.action.ExtendedToolbarAction;
import viptool.base.action.IActionHandler;
import viptool.base.action.PlaceHolderAction;
import viptool.base.action.SeparatorAction;
import viptool.base.gui.toolbar.IExtendedToolBar;
import viptool.base.gui.toolbar.event.ExtendedToolBarEvent;
import viptool.base.gui.toolbar.event.IExtendedToolBarListener;
import viptool.impl.base.action.ActionHandler;
import viptool.impl.logger.Logger;
import viptool.impl.util.resources.ResourceLoader;
import viptool.util.resources.ResourceConstants;

/**
 * The extended toolbar is a toolbar that can listen to a template and changes
 * itself if the template is changed.
 */
public class ExtendedToolBar extends JToolBar implements IExtendedToolBar,
        IExtendedToolBarListener
{
    // the id of this extended toolbar, has to be unique in viptool
    private int     instance_;

    // set containing all registered listeners of this toolbar
    private HashSet listeners_;

    // map containing all elements of this toolbar, component referenced by id
    // key: java.lang.String
    // value: java.awt.Component
    private HashMap elements_;

    // map containing all button groups of this toolbar, groups referenced by id
    // key: java.lang.String
    // value: javax.swing.ButtonGroup
    private HashMap buttonGroups_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new ExtendedToolBar with the given int as id. All maps and
     * sets will be initialized and the toolbar itself will be added to the
     * elements map so that it can be get as parent by the direct childs.
     * 
     * @param instance
     *            the id of this concrete instance
     */
    public ExtendedToolBar(int instance)
    {
        instance_ = instance;

        buttonGroups_ = new HashMap();
        listeners_ = new HashSet();

        elements_ = new HashMap();
        elements_.put("", ExtendedToolBar.this);

        // setting the layout
        setLayout((new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 0, 5)));
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * 
     */
    private void addComponent(Component component, String id,
            String containerId, String insertAt, int seek)
    {
        int index = -1;
        Container container = (Container) elements_.get(containerId);
        Component[] components;

        if (seek != 0 && insertAt.length() > 0)
        {
            if (container instanceof JButton)
            {
                // will be implemented soon; for popup menus on buttons
                components = new Component[0];
            }
            else
            {
                components = container.getComponents();
            }

            index = components.length;
            for (int i = 0; i < components.length; i++)
            {
                if (components[i] instanceof AbstractButton)
                {
                    if (((ExtendedAction) ((AbstractButton) components[i]).getAction()).getIdKey().equals(
                            insertAt))
                    {
                        index = i + seek;
                        break;
                    }
                }
            }

            if (index < 0)
            {
                // add at beginning
                index = 0;
            }

            if (index > components.length)
            {
                // add to end
                index = -1;
            }
        }

        container.add(component, index);
        if (id != null && !id.equals(IActionHandler.ELEMENT_SEPARATOR))
        {
            elements_.put(id, component);
        }

        fireToolBarItemAddedEvent(id, containerId, insertAt, seek);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // event firering
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This method will notify all the template listeners, that an element was
     * added to the template.
     * 
     * @param id
     *            the id of the element that was added
     * @param container
     *            the enclosing container of the element
     * @param insertAt
     *            the positional string parameter where to add the component
     * @param seek
     *            the parameter for determining the exact position relative to
     *            insertAt
     */
    private void fireToolBarItemAddedEvent(String id, String container,
            String insertAt, int seek)
    {
        Iterator it;
        IExtendedToolBarListener listener;
        ExtendedToolBarEvent event;

        event = new ExtendedToolBarEvent(this, id, container, insertAt, seek);

        it = listeners_.iterator();
        while (it.hasNext())
        {
            listener = (IExtendedToolBarListener) it.next();
            listener.toolBarElementAdded(event);
        }
    }

    /**
     * Notifies the listeners that the element with the specified id was removed
     * from the template.
     * 
     * @param id
     *            the id that was removed from the template
     */
    private void fireToolBarItemRemovedEvent(String id)
    {
        Iterator it;
        IExtendedToolBarListener listener;
        ExtendedToolBarEvent event;

        event = new ExtendedToolBarEvent(this, id);

        it = listeners_.iterator();
        while (it.hasNext())
        {
            listener = (IExtendedToolBarListener) it.next();
            listener.toolBarElementRemoved(event);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the interface
    // viptool.base.gui.toolbar.IExtendedToolBar
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see:viptool.base.gui.toolbar.IExtendedToolBar */
    public void addExtendedToolBarListener(IExtendedToolBarListener listener)
    {
        if (listener != null)
        {
            listeners_.add(listener);
        }
        else
        {
            Logger.getInstance().warning(
                    "null was given as new listener to a toolbar template!");
        }
    }

    /* @see:viptool.base.gui.toolbar.IExtendedToolBar */
    public void removeExtendedToolBarListener(IExtendedToolBarListener listener)
    {
        if (listener != null)
        {
            listeners_.remove(listener);
        }
        else
        {
            Logger.getInstance().warning(
                    "null was given as listener for removing "
                            + "to a toolbar template!");
        }
    }

    /* @see:viptool.base.gui.toolbar.IExtendedToolBar */
    public ExtendedAction addToolBarItem(String id, String group,
            String insertAt, int seek)
    {
        String buttonGroupId;
        String type;
        ExtendedAction action;
        Component toolBarItem;
        ButtonGroup buttonGroup;

        // get the action and read important information
        action = ActionHandler.getInstance().getAction(instance_, id);

        // check if the toolbar item is already, separators are ignored
        // automatically becaue they are not in the list
        if (getToolBarItem(id) != null)
        {
            // the toolbar item is already there, don't add it twice
            return action;
        }

        if (action instanceof SeparatorAction)
        {
            toolBarItem = new JToolBar.Separator();
        }
        else if (action instanceof PlaceHolderAction)
        {
            toolBarItem = new JButton(action);
            toolBarItem.setVisible(false);
        }
        // else if (action instanceof GroupAction)
        // {
        // // will be implemented in near future...
        // }
        else
        {
            type = (String) action.getValue(ExtendedAction.ATTR_TYPE);

            // first create the new toolbar item, that has to be inserted
            if (ExtendedAction.TYPE_RADIO.equals(type))
            {
                toolBarItem = new JToggleButton(action);

                buttonGroupId = (String) action.getValue(ExtendedAction.ATTR_GROUP);

                if (buttonGroupId != null && buttonGroupId.length() > 0)
                {
                    buttonGroup = (ButtonGroup) buttonGroups_.get(buttonGroupId);

                    if (buttonGroup == null)
                    {
                        buttonGroup = new ButtonGroup();
                        buttonGroups_.put(buttonGroupId, buttonGroup);
                    }

                    buttonGroup.add((JToggleButton) toolBarItem);
                }
            }
            else if (ExtendedAction.TYPE_CHECKBOX.equals(type))
            {
                toolBarItem = new JToggleButton(action);
            }
            else if (ExtendedAction.TYPE_SLIDER.equals(type))
            {
                toolBarItem = new JButton(action);
                JSlider slider = new JSlider();
                ((JButton) toolBarItem).add(slider);
                if (action instanceof ExtendedToolbarAction)
                {
                    slider.addChangeListener(((ChangeListener) action));
                    ((ExtendedToolbarAction) action).initSlider(slider);
                }
            }
            else if (ExtendedAction.TYPE_TEXT.equals(type))
            {
                toolBarItem = new JButton(action);
                JTextField field = new JTextField(30);
                ((JButton) toolBarItem).add(field);
                if (action instanceof ExtendedToolbarAction)
                {
                    field.addActionListener(((ActionListener) action));
                    ((ExtendedToolbarAction) action).initTextfield(field);
                }
            }
            else if (ExtendedAction.TYPE_COMBOBOX.equals(type))
            {
                toolBarItem = new JButton(action);
                JComboBox box = new JComboBox();
                ((JButton) toolBarItem).add(box);
                if (action instanceof ExtendedToolbarAction)
                {
                    box.addActionListener(((ActionListener) action));
                    ((ExtendedToolbarAction) action).initCombobox(box);
                }
            }
            else
            {
                toolBarItem = new JButton(action);
            }
        }

        if (toolBarItem instanceof AbstractButton)
        {
            if (action.getValue(ExtendedAction.SMALL_ICON) != null)
            {
                ((AbstractButton) toolBarItem).setText("");
            }
            else
            {
                // add an invisible icon so the button will have the same
                // size
                ((AbstractButton) toolBarItem).setIcon(ResourceLoader.getInstance().getIcon(
                        ResourceConstants.ICON_EMPTY));

                // add a space to the end so the text will be in the middle
                // of the button
                ((AbstractButton) toolBarItem).setText(((AbstractButton) toolBarItem).getText()
                        + " ");
            }
        }

        // add the component to our template bar
        addComponent(toolBarItem, id, group, insertAt, seek);
        return action;
    }

    /* @see:viptool.base.gui.toolbar.IExtendedToolBar */
    public ExtendedAction removeToolBarItem(String id)
    {
        Component component = (Component) elements_.get(id);

        if (component != null)
        {
            component.getParent().remove(component);
            elements_.remove(id);

            fireToolBarItemRemovedEvent(id);
        }

        return ActionHandler.getInstance().getAction(instance_, id);
    }

    /* @see:viptool.base.gui.toolbar.IExtendedToolBar */
    public JToolBar getToolBar()
    {
        return this;
    }

    /* @see:viptool.base.gui.toolbar.IExtendedToolBar */
    public Component getToolBarItem(String id)
    {
        return (Component) elements_.get(id);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the interface
    // viptool.base.gui.toolbar.event.IExtendedToolBarListener
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see:viptool.base.gui.toolbar.event.IExtendedToolBarListener */
    public void toolBarElementAdded(ExtendedToolBarEvent event)
    {
        addToolBarItem(event.getId(), event.getContainer(),
                event.getInsertAt(), event.getSeek());
    }

    /* @see:viptool.base.gui.toolbar.event.IExtendedToolBarListener */
    public void toolBarElementRemoved(ExtendedToolBarEvent event)
    {
        removeToolBarItem(event.getId());
    }

}
