package viptool.impl.base.gui.toolbar;

import java.net.URI;
import java.util.HashMap;

import javax.swing.JToolBar;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import viptool.base.action.IActionHandler;
import viptool.base.action.WindowAction;
import viptool.base.action.WindowActionPostProcessor;
import viptool.base.error.XMLException;
import viptool.base.gui.IWindow;
import viptool.base.gui.toolbar.IToolBarManager;
import viptool.base.gui.toolbar.IToolBarTemplate;
import viptool.impl.base.Workbench;
import viptool.impl.base.action.ActionHandler;
import viptool.impl.logger.Logger;
import viptool.impl.util.resources.ResourceConstants;
import viptool.impl.util.resources.ResourceLoader;

/* @see:viptool.base.gui.toolbar.IToolBarManager */
public class ToolBarManager implements IToolBarManager
{
    /** the element name for the the root element for xml toolbar files */
    public static final String     ELEMENT_TOOLBAR = "toolbar";

    // the one and only toolbar manager instance
    private static IToolBarManager instance_       = null;

    // the loaded templates
    // key: java.lang.String - the id of the template
    // value: viptool.base.gui.toolbar.IToolBarTemplate - the template itself
    private HashMap                templates_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs an instance of the toolbar template manager.
     */
    private ToolBarManager()
    {
        templates_ = new HashMap();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the one and only instance of the toolbar template manager. If no
     * instance exists, a new one is created. Null will never be returned.
     * 
     * @return an instance of the toolbar template manager
     */
    public static synchronized IToolBarManager getInstance()
    {
        if(instance_ == null)
        {
            instance_ = new ToolBarManager();
        }

        return instance_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Reads a toolbar group from an xml element into the given template. The
     * group element must represent a valid group as defined in the according
     * xml schema in viptool sdk.
     * 
     * @param groupElement
     *            the group element that contains the information
     * @param template
     *            the template that should receive the group
     * @param container
     *            the container of the container that should be added (the new
     *            container will be a child of the given container)
     */
    private void readToolBarContainer(Element groupElement,
            IToolBarTemplate template, String container)
    {
        Node child;
        NodeList childs;
        Element element = null;

        int seek = 0;
        int lastSeek = 1;
        String id = "";
        String insertAt = "";
        String lastId = "";

        // now read the toolbar
        childs = groupElement.getChildNodes();
        for (int i = 0; i < childs.getLength(); i++)
        {
            child = childs.item(i);

            // if we have an element and the element has attributes we have an
            // action or a group. Read the common attributes
            if(child instanceof Element)
            {
                element = (Element) child;

                id = element.getAttribute(WindowAction.ATTR_ID);
                insertAt = element.getAttribute(WindowAction.ATTR_INSERTAT);
                seek = Integer.valueOf(
                        element.getAttribute(WindowAction.ATTR_SEEK))
                        .intValue();

                // if the insertAt parameter was not set, use the last id as
                // insert at parameter
                if(insertAt.length() == 0)
                {
                    insertAt = lastId;
                    seek = lastSeek;
                }

                // add the item to the template
                template.addToolBarItem(id, container, insertAt, seek);

                // store the id as last id so we can set insertAt for the next
                // elementif not given
                if(!id.equals(IActionHandler.ELEMENT_SEPARATOR))
                {
                    lastId = id;
                    lastSeek = 1;
                }
                else
                {
                    // the separator has no own id, so we take the insertAt as
                    // id (must be set) and increment seek
                    lastId = insertAt;
                    lastSeek++;
                }
            }
        } // end for
    }
    
    public void addEmptyTemplate(String id, String inheritFrom)
    {
        IToolBarTemplate toolBarTemplate = null;

            // create a destination template
            toolBarTemplate = new ToolBarTemplate(id);

            // read if the toolbar should inherit from another toolbar
            if(inheritFrom != null) {

                toolBarTemplate.setParent(getTemplate(inheritFrom));
            }

            // add the new template to the template manager
            addTemplate(toolBarTemplate);
    }

    /* @see:viptool.base.gui.toolbar.IToolBarManager */
    public void addTemplate(IToolBarTemplate template)
    {
        templates_.put(template.getId(), template);
    }

    /* @see:viptool.base.gui.toolbar.IToolBarManager */
    public IToolBarTemplate getTemplate(String id)
    {
        return (IToolBarTemplate) templates_.get(id);
    }

    /* @see:viptool.base.gui.toolbar.IToolBarManager */
    public JToolBar createToolBar(String id, IWindow window)
    {
        int instance;

        if(templates_.containsKey(id))
        {
            instance = window.hashCode();

            ActionHandler.getInstance().registerActionPostProcessor(instance,
                    new WindowActionPostProcessor(window));
            return (JToolBar) getTemplate(id).createToolBar(instance)
                    .getToolBarItem("");
        }
        else
        {
            Logger.getInstance().warning(
                    "MenuTemplate '" + id
                            + "' not present in the MenuBarManager");

            return null;
        }
    }
}
