package viptool.impl.base.io;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

import viptool.base.error.FileInterpreterProviderException;
import viptool.base.io.IFileInterpreterManager;
import viptool.base.io.IFileInterpreterProvider;
import viptool.impl.logger.Logger;
import viptool.logger.ILogger;
import viptool.project.IAbstractFile;

/**
 * The FileTypeManager manages all known filetypes and is responsible for
 * selecting the right ProjectElement depending on the content of a file. The
 * class is written respecting the singleton interface.
 */
public class FileInterpreterManager implements IFileInterpreterManager
{
    // the one and only instance
    private static FileInterpreterManager instance_     = null;

    // a counter storing the last priority, incremented if a new file
    // interpreter provider is added
    private int                           lastPriority_ = 0;

    // the set with all registered file interpreter providers
    // the
    private SortedSet                     fileInterpreterProviders_;

    private class FileInterpreterProviderInfo
    {
        private IFileInterpreterProvider provider_;

        private int                      priority_;

        public FileInterpreterProviderInfo(IFileInterpreterProvider provider,
                int priority)
        {
            provider_ = provider;
            priority_ = priority;
        }

        public IFileInterpreterProvider getFileInterpreterProvider()
        {
            return provider_;
        }

        public int getPriority()
        {
            return priority_;
        }
    }

    private class FileInterpreterComparator implements Comparator
    {
        public int compare(Object obj1, Object obj2)
        {
            FileInterpreterProviderInfo info1 = (FileInterpreterProviderInfo)obj1;
            FileInterpreterProviderInfo info2 = (FileInterpreterProviderInfo)obj2;

            return info2.getPriority() - info1.getPriority();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new instance of the file type manager.
     */
    public FileInterpreterManager()
    {
        fileInterpreterProviders_ = new TreeSet(new FileInterpreterComparator());
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the instance of the file type manager. If there isn't already one
     * a new one will be created.
     * 
     * @return the only instance of the file type manager
     */
    public static synchronized FileInterpreterManager getInstance()
    {
        if(instance_ == null)
        {
            instance_ = new FileInterpreterManager();
        }

        return instance_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IFileInterpreterManager
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.io.IFileInterpreterManager */
    public void addFileInterpreterProvider(
            IFileInterpreterProvider fileInterpreterProvider)
    {
        Iterator it;
        FileInterpreterProviderInfo info;

        it = fileInterpreterProviders_.iterator();
        while(it.hasNext())
        {
            info = (FileInterpreterProviderInfo)it.next();
            if(info.getFileInterpreterProvider() == fileInterpreterProvider)
            {
                // do not add, if already exists
                return;
            }
        }

        fileInterpreterProviders_.add(new FileInterpreterProviderInfo(
                fileInterpreterProvider, lastPriority_));

        lastPriority_++;
    }

    /* @see viptool.base.io.IFileInterpreterManager */
    public void removeFileInterpreterProvider(
            IFileInterpreterProvider fileInterpreterProvider)
    {
        Iterator it;
        FileInterpreterProviderInfo info;

        it = fileInterpreterProviders_.iterator();
        while(it.hasNext())
        {
            info = (FileInterpreterProviderInfo)it.next();
            if(info.getFileInterpreterProvider() == fileInterpreterProvider)
            {
                // only remove if exists
                fileInterpreterProviders_.remove(info);
                return;
            }
        }

        return;
    }

    /* @see viptool.base.io.IFileInterpreterManager */
    public IFileInterpreterProvider getDefaultFileInterpreterProvider(
            IAbstractFile file)
    {
        Iterator it;
        FileInterpreterProviderInfo info;

        it = fileInterpreterProviders_.iterator();
        while(it.hasNext())
        {
            info = (FileInterpreterProviderInfo)it.next();

            try
            {
                if(info.getFileInterpreterProvider().matchFormat(file))
                {
                    return info.getFileInterpreterProvider();
                }
            }
            catch(FileInterpreterProviderException exception)
            {
                Logger.getInstance().log(ILogger.ERROR_LEVEL,
                        exception.getMessage() + "");
            }
        }

        Logger.getInstance().log(
                ILogger.WARRNING_LEVEL,
                "No default provider found for the file '" + file.getFullPath()
                        + "'");
        return null;
    }

    /* @see viptool.base.io.IFileInterpreterManager */
    public IFileInterpreterProvider[] getMatchingFileInterpreterProviders(
            IAbstractFile file)
    {
        Iterator it;
        FileInterpreterProviderInfo info;
        ArrayList providers = new ArrayList();

        it = fileInterpreterProviders_.iterator();
        while(it.hasNext())
        {
            info = (FileInterpreterProviderInfo)it.next();

            try
            {
                if(file == null
                        || info.getFileInterpreterProvider().matchFormat(file))
                {
                    providers.add(info.getFileInterpreterProvider());
                }
            }
            catch(FileInterpreterProviderException exception)
            {
                // TODO: log message
            }
        }

        return (IFileInterpreterProvider[])providers
                .toArray(new IFileInterpreterProvider[providers.size()]);
    }

    /* @see viptool.base.io.IFileInterpreterManager */
    public IFileInterpreterProvider[] getFileInterpreterProviders()
    {
        return getMatchingFileInterpreterProviders(null);
    }

    /* @see viptool.base.io.IFileInterpreterManager */
    public IFileInterpreterProvider getFileInterpreterProvider(Class clazz)
    {
        Iterator it;
        FileInterpreterProviderInfo info;

        it = fileInterpreterProviders_.iterator();
        while(it.hasNext())
        {
            info = (FileInterpreterProviderInfo)it.next();
            if(info.getFileInterpreterProvider().getClass().equals(clazz))
            {
                return info.getFileInterpreterProvider();
            }
        }

        return null;
    }
}
