package viptool.impl.console;

import java.awt.Color;
import java.awt.Component;

import javax.swing.Icon;
import javax.swing.JMenuBar;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import viptool.console.IConsole;
import viptool.impl.util.resources.ResourceConstants;
import viptool.logger.ILogger;

/**
 * The class console is needed to create a window, that displays text. This
 * class extends the class JTextPane. The content type of the text is HTML.
 */
public class Console extends JScrollPane implements IConsole
{

    // default color (black)
    private Color     defaultColor_ = Color.black;

    // info color (green)
    private Color     infoColor_    = ResourceConstants.COLOR_CONSOLE_INFO;

    // warning color (orange)
    private Color     warningColor_ = ResourceConstants.COLOR_CONSOLE_WARNING;

    // error color (red)
    private Color     errorColor_   = ResourceConstants.COLOR_CONSOLE_ERROR;

    // debug color (ligth grey)
    private Color     debugColor_   = ResourceConstants.COLOR_CONSOLE_DEBUG;

    // the number of characters, that are allready written in your Logger
    private int       offset_;

    // the style of the text pane
    private Style     style_;

    // the name of this console
    private String    name_;

    // the text pane in this console
    private JTextPane textPane_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Create a console object. The text in the console will not be editable and
     * the content type is text/html. The EditorKit knows this type by default
     */
    public Console(String name)
    {

        super();

        textPane_ = new JTextPane()
        {
            public boolean getScrollableTracksViewportWidth()
            {
                // this is necessary to disable line wrap
                return false;
            }
        };

        textPane_.setEditable(false);

        name_ = name;

        getViewport().setBackground(Color.WHITE);

        setViewportView(textPane_);

        StyledDocument styledDocument = textPane_.getStyledDocument();
        // add style to the document of the text pane
        style_ = StyleContext.getDefaultStyleContext().getStyle(
                StyleContext.DEFAULT_STYLE);
        style_ = styledDocument.addStyle("Style", style_);

        // set the font of the text in the console
        textPane_.setFont(ResourceConstants.FONT_DEFAULT_CONSOLE);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementing the IConsole Interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Set the color for default messages.
     * 
     * @param color
     *            the new Color
     */
    public void setDefaultColor(Color color)
    {

        defaultColor_ = color;
    }

    /**
     * Set the color for debug messages.
     * 
     * @param htmlColor
     *            the new Color
     */
    public void setDebugColor(Color color)
    {

        debugColor_ = color;
    }

    /**
     * Set the color for info messages.
     * 
     * @param htmlColor
     *            the new Color
     */
    public void setInfoColor(Color color)
    {

        infoColor_ = color;
    }

    /**
     * Set the color for warning messages.
     * 
     * @param htmlColor
     *            the new Color
     */
    public void setWarningColor(Color color)
    {

        warningColor_ = color;
    }

    /**
     * Set the color for error messages.
     * 
     * @param htmlColor
     *            the color must be given as an String and it must be a known
     *            html color. Otherwise the color will be default.
     */
    public void setErrorColor(Color color)
    {

        errorColor_ = color;
    }

    /**
     * Append the text to the text that is already diplayed in your console. The
     * color of the text depends on the level of the message.
     * 
     * @param text
     *            the text, that should br displayed
     * @param level
     *            the level of the text, that should be displayed
     */
    public void appendText(String text, int level)
    {

        if (level == ILogger.INFO_LEVEL)
        {

            setFontColor(infoColor_);
        }
        else if (level == ILogger.WARRNING_LEVEL)
        {

            setFontColor(warningColor_);
        }
        else if (level == ILogger.ERROR_LEVEL)
        {

            setFontColor(errorColor_);
        }
        else if (level == ILogger.DEBUG_LEVEL)
        {

            setFontColor(debugColor_);
        }
        else
        {

            setFontColor(defaultColor_);
        }

        StyledDocument styledDocument = textPane_.getStyledDocument();

        // count the charcters, that are allready written to the console
        offset_ = styledDocument.getLength();

        try
        {

            // append the new text with the corresponding style and jump to the
            // next line
            styledDocument.insertString(offset_, text + "\n",
                    styledDocument.getStyle("Style"));
        }
        catch (BadLocationException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Private Methods
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Set the color of the text, that will be displayed in the console.
     * 
     * @param color
     *            the collor of the next text, that will be displayed
     */
    private void setFontColor(Color color)
    {

        StyleConstants.setForeground(style_, color);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementing the ChildWindow interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.gui.extendedPane.ChildWindow */
    public boolean canClose(boolean force)
    {

        return true;
    }

    /* @see viptool.gui.extendedPane.ChildWindow */
    public Component getComponent()
    {

        return this;
    }

    /* @see viptool.gui.extendedPane.ChildWindow */
    public Icon getIcon()
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* @see viptool.gui.extendedPane.ChildWindow */
    public JMenuBar getMenuBar()
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* @see viptool.gui.extendedPane.ChildWindow */
    public String getTitle()
    {
        // TODO Auto-generated method stub
        return name_;
    }

    /* @see viptool.gui.extendedPane.ChildWindow */
    public JToolBar getToolBar()
    {
        // TODO Auto-generated method stub
        return null;
    }

    public void windowActivated()
    {
    }
}
