package viptool.impl.console;

import java.io.OutputStream;

import viptool.console.IConsole;
import viptool.logger.ILogger;

/**
 * This class is a subclass of OutputStream. All bytes, that would be written
 * on an output stream, will be redirected to a console. The bytes
 * are converted to a String, so that they can be displayed.
 */
public class ConsoleStream extends OutputStream {
	
	//the text you want to display
	String text_;
	
	//the console, where the text will be displayed
	IConsole console_;
	
	// <><><><><><><><><><><><><><><><><><><><><>
	//  Constructor
	// <><><><><><><><><><><><><><><><><><><><><>
	
	/**
	 * Create an OutputStream object. This object gets a console, where all
	 * information, that would be written on the stream, can be displayed.
	 * These information are written on the stream as bytes. So all the bytes,
	 * that should be displayed, must be converted into strings.
	 * 
	 * @param console
	 * 		the console to display strings
	 */
	public ConsoleStream(IConsole console)  {
		
		super();
		
		console_ = console;
	}
	
	// <><><><><><><><><><><><><><><><><><><><><>
	//  Supporting methods
	// <><><><><><><><><><><><><><><><><><><><><>
	
	/**
	 * Get the console belonging to the ConsoleStream
	 * 
	 * @return the console
	 */
	public IConsole getConsole()  {
		
		return console_;
	}
	
	// <><><><><><><><><><><><><><><><><><><><><>
	//  Overwritten methods from
	//  java.io.OutputStream
	// <><><><><><><><><><><><><><><><><><><><><>
	
	/**
	 * Instead of writing a byte array to an output stream, convert the whole
	 * array into a string. After that, the converted bytes can be displayed
	 * on the console.
	 * This call of this method has exactly the same effect as the call
	 * write(b, 0, b.length)
	 * If b is null, a NullPointerException is thrown.
	 * 
	 * @param b
	 * 		the data, that should be converted and displayed.
	 */
	public void write(byte[] b)  {
		
		//construct a new String by converting the specified subarray of bytes 
		//using the platform's default character encoding
		text_ = new String(b, 0, b.length);
		
		console_.appendText(text_, ILogger.DEFAULT_LEVEL);
	}
	
	/**
	 * Instead of writing a byte array to an output stream, convert a part of it 
	 * into a string. Convert 'len' bytes from the byte array starting at the 
	 * offset 'start'. After that, the converted bytes can be displayed at the 
	 * console. 
	 * If b is null, a NullPointerException is thrown.
	 * If start is negative, or len is negative, or start and len is greater than
	 * the length of the array b, then an IndexOutOfBoundsException is thrown.
	 * 
	 * @param b
	 * 		the data, that should be converted an displayed partally
	 * 
	 * @param start
	 * 		the start offset in the data
	 * 
	 * @param len
	 * 		the number of bytes, that should be converted and displayed
	 */
	public void write(byte[] b, int start, int len)  {
		
		//construct a new String by converting the specified subarray of bytes 
		//using the platform's default character encoding
		text_ = new String(b, start, b.length);
		
		console_.appendText(text_, ILogger.DEFAULT_LEVEL);
	}
	
	/**
	 * Instead of writing the specified byte to an output stream, convert this
	 * one byte into a string. After that, the byte can be displayed on the
	 * console.
	 * 
	 * @param b
	 * 		the byte, that should be converted an displayed
	 */
	public void write(int b)  {
		
		//you need a byte array to create a string
		byte[] byteArray = new byte[1];
		byteArray[0] = (byte)b;
		
		//construct a new String by converting the specified subarray of bytes 
		//using the platform's default character encoding
		text_ = new String(byteArray, 0, byteArray.length);
		
		console_.appendText(text_, ILogger.DEFAULT_LEVEL);
	}
		
		
	
}
