package viptool.impl.extension;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import viptool.base.error.VipToolException;
import viptool.base.error.VipToolExtensionException;
import viptool.extension.IExtensionInfo;
import viptool.extension.IExtensionLoader;
import viptool.extension.IExtensionManager;
import viptool.impl.base.Workbench;
import viptool.impl.extension.extensionloader.ExtensionLoader;
import viptool.util.ClassFile;

// for comments see: viptool.extension.IExtensionInfo

public class ExtensionInfo implements IExtensionInfo
{

    // contains class files which are required for this extension as ClassFile
    private Vector          classfiles_     = null;

    // contians all urls through which are all necessary classes can be found as
    // URL
    private HashMap         urls_           = null;

    // fqcn of the required extensions
    private HashSet<String> required_;

    // description of this extension
    private String          description_    = "";

    // name of this extension
    private String          name_;

    // version of the extension
    private long            version_;

    // full qualified name of the main class
    private String          fqcname_;

    // file of the main class as ClassFile
    private ClassFile       mainclass_;

    // Class for this extension or null
    private Class           extensionclass_ = null;

    // JarFile for this extension or null
    private JarFile         jarfile_        = null;

    // true if jarPlugin, false is filePlugin
    private boolean         isJarPlugin_;

    // this is true if the required urls are added to the extensionloader (can
    // be instanciated)
    private boolean         registered_     = false;

    // true if the extension is loaded by starting the VipTool
    private boolean         autoload_       = false;

    /**
     * This constructor creates an ExtensionInfo from a ClassFile, required
     * extensions an other class files if the are not contained at the url of
     * the main class have to be added after creating the ExtensionInfo.
     * 
     * @param mainclass
     *            main class file as ClassFile (VipTool SDK util)
     */
    public ExtensionInfo(ClassFile mainclass)
    {
        isJarPlugin_ = false;

        mainclass_ = mainclass;
        name_ = fqcname_ = mainclass.getClassName();

        urls_ = new HashMap();

        try
        {
            urls_.put(mainclass_.getClassURL().toString(),
                    mainclass_.getClassURL());
        }
        catch (VipToolException e)
        {
            Workbench.getInstance().showExceptionInDialog(e);
        }

        classfiles_ = new Vector();
        required_ = new HashSet<String>();

        jarfile_ = null;

        updateMainfestFile();
    }

    /**
     * This constructor creates a new ExtensionInfo from the given JarFile. The
     * full qualified class name of the main class has to be given as attribute
     * "Main-Class" into the manifest, the name as attribute "Name", the
     * description as attriubte "Description" and the required extensions as
     * comma separated list of full qualified classnames as attribute
     * "Required". If the JarFile isn't readable an IOException will be thrown,
     * if the required extensions not known a ViptoolException will be thrown.
     * 
     * @param jarfile
     *            JarFile which includes the extension
     * @throws IOException
     *             JarFile isn't readable
     * @throws VipToolException
     *             required extensions not known
     */
    public ExtensionInfo(JarFile jarfile) throws IOException, VipToolException
    {
        isJarPlugin_ = true;
        jarfile_ = jarfile;

        required_ = new HashSet<String>();

        Manifest m;

        m = jarfile.getManifest();
        fqcname_ = m.getMainAttributes().getValue("Extension-Class");
        try
        {
            String version = m.getMainAttributes().getValue("Implementation-Version");
            if (version != null)
            {
                version_ = Long.parseLong(version.trim());
            }
            else
            {
                version_ = 0;
            }
        }
        catch (NumberFormatException ex)
        {
            version_ = 0;
        }
        name_ = (m.getMainAttributes().getValue("Extension-Name") != null && m.getMainAttributes().getValue(
                "Extension-Name") != "") ? m.getMainAttributes().getValue("Extension-Name")
                : fqcname_;
        description_ = (m.getMainAttributes().getValue("Description") != null) ? m.getMainAttributes().getValue(
                "Description")
                : "no description defined in Manifest";

        classfiles_ = null;

        String required = m.getMainAttributes().getValue("Extension-List");
        if (required != null && required != "")
        {
            String[] req = required.split("[,\\s]");
            for (int i = 0; i < req.length; i++)
            {
                if (req[i] != null && req[i].length() > 1)
                {
                    required_.add(req[i]);
                }
            }
        }

        mainclass_ = null;
    }

    /**
     * This method creates an ExtensionInfo from the given node which have to be
     * format which is defined in the comment of getNode(Document). If the node
     * is in wrong format a VipTool-Exception will be thrown.
     * 
     * @param node
     *            node to create ExtensionInfo from
     * @return ExtensionInfo for the given node
     * @throws VipToolException
     *             is thrown if the node format isn't valid
     * @throws IOException
     */
    public static ExtensionInfo getExtensionFormNode(Node node,
            ExtensionManager extManager) throws VipToolException, IOException
    {
        ExtensionInfo extInfo = null;

        if (node.getNodeName().equals("jarExt"))
        {

            NodeList list = node.getChildNodes();

            for (int i = 0; i < list.getLength(); i++)
            {
                Node current = list.item(i);
                String nodename = current.getNodeName();
                if (nodename.equals("file"))
                {
                    String file = current.getTextContent();
                    extInfo = new ExtensionInfo(new JarFile(file));
                }
                else if (nodename.equals("required"))
                {
                    NodeList nl = current.getChildNodes();
                    int l = nl.getLength();
                    for (int j = 0; j < l; j++)
                    {
                        Node req = nl.item(j);
                        if (req.getNodeName().equals("otherExt"))
                        {
                            String reqName = req.getTextContent();
                            extInfo.required_.add(reqName);
                        }
                    }
                }
                else if (nodename.equals("autoload"))
                {
                    if (current.getTextContent().equals("true"))
                    {
                        extInfo.setAutoload(true);
                    }
                }
            }
        }

        else if (node.getNodeName().equals("fileExt"))
        {

            NodeList list = node.getChildNodes();

            for (int i = 0; i < list.getLength(); i++)
            {
                Node current = list.item(i);
                String nodename = current.getNodeName();
                if (nodename.equals("file"))
                {
                    String fileName = current.getTextContent();
                    File file = new File(fileName);
                    ClassFile cf = ClassFile.getClassFile(file);
                    extInfo = new ExtensionInfo(cf);
                }
                else if (nodename.equals("name"))
                {
                    extInfo.setName(current.getTextContent());
                }
                else if (nodename.equals("description"))
                {
                    extInfo.setDescription(current.getTextContent());
                }
                else if (nodename.equals("classes"))
                {
                    NodeList nl = current.getChildNodes();
                    int l = nl.getLength();
                    for (int j = 0; j < l; j++)
                    {
                        Node req = nl.item(j);
                        if (req.getNodeName().equals("classFile"))
                        {
                            String fileName = req.getTextContent();
                            File file = new File(fileName);
                            if (file != null && !file.isDirectory()
                                    && file.exists())
                            {
                                ClassFile cf = ClassFile.getClassFile(file);
                                extInfo.addClassFile(cf);
                            }
                        }
                    }
                }
                else if (nodename.equals("required"))
                {
                    NodeList nl = current.getChildNodes();
                    int l = nl.getLength();
                    for (int j = 0; j < l; j++)
                    {
                        Node req = nl.item(j);
                        if (req.getNodeName().equals("otherExt"))
                        {
                            String reqName = req.getTextContent();
                            extInfo.required_.add(reqName);
                        }
                    }
                }
                else if (nodename.equals("autoload"))
                {
                    if (current.getTextContent().equals("true"))
                    {
                        extInfo.setAutoload(true);
                    }
                }
            }
            extInfo.updateMainfestFile();
        }
        else
        {
            throw new VipToolException("ExtensionInfo: unknown node type");
        }
        return extInfo;
    }

    /**
     * This method adds the given classfile to the list of needed classfiles.
     * This means this classfile and all other classfiles which are avaiable
     * through this url can be instanciated after a call of registerClasses().
     * 
     * @param classfile
     *            classfile to add as ClassFile
     * @throws VipToolException
     */
    public void addClassFile(ClassFile classfile) throws VipToolException
    {
        classfiles_.add(classfile);
        urls_.put(classfile.getClassURL().toString(), classfile.getClassURL());
    }

    /**
     * This method adds the given IExtensionInfo to the list of needed
     * extensions. This means the given extension have to be known by the
     * ExtensionManager and will be loaded bevor this extension is loaded.
     * 
     * @param extension
     *            required extension as IExtensionInfo
     */
    public void addRequiredExtenison(IExtensionInfo extension)
    {
        required_.add(extension.getClassName());
    }

    /**
     * This method removes the given extension to the list of required
     * extensions.
     */
    public void removeRequiredExtenison(IExtensionInfo extension)
    {
        required_.remove(extension.getClassName());
    }

    /**
     * This method returns a Vector which contains all classfiles as ClassFile
     * which are needed by this extension.
     */
    public Vector getClassFiles()
    {
        return (Vector) classfiles_.clone();
    }

    /**
     * This method returns a description for this extension if there is one.
     */
    public String getDescription()
    {
        return description_;
    }

    /**
     * The method returns the main class of this extension as ClassFile if it is
     * a FilePlugin or null if it is a JarPlugin.
     */
    public ClassFile getExtenisonClassFile()
    {
        return mainclass_;
    }

    /**
     * This method returns a Class object of this extension.
     */
    public Class getExtensionClass()
    {
        registerClasses();
        if (extensionclass_ == null)
        {
            extensionclass_ = ExtensionLoader.getInstance().getClass(fqcname_);
        }
        return extensionclass_;
    }

    /**
     * This method returns the JarFile which contains this extension if it is a
     * JarPlugin or null if it is a FilePlugin.
     */
    public JarFile getJarFile()
    {
        return jarfile_;
    }

    /**
     * This method returns the name of this extension.
     */
    public String getName()
    {
        return name_;
    }

    /**
     * This method creates an DOM-Node which represents all information which
     * belong to this extension. This node is used to save the extension in the
     * list of known extensions. structur: jar-Extension: <jarExt> <file>[path
     * to jarFile]</file> <required> <otherExt>[full qualified name of the
     * extension class]</otherExt> </required> <autoload>[true or
     * false]</autoload> </jarExt>
     * 
     * file-Extension: <fileExt> <file>[path to main class]</file> <fqcn>[full
     * qualified class name]</fqn> <name>[name of the extension]</name>
     * <description>[description for the extension]</description> <classes>
     * <classFile>[path to the classfile]</classFile> ... <classFile>[path to
     * the classfile]</classFile> <classes> <required> <otherExt>[full qualified
     * name of the extension class]</otherExt> ... <otherExt>[full qualified
     * name of the extension class]</otherExt> </required> <autoload>[true or
     * false]</autoload> </fileExt>
     */
    public void getNode(Document doc) throws VipToolException
    {
        Node node = null;
        boolean error = false;
        if (isJarPlugin_)
        {
            node = doc.createElement("jarExt");
            Node file = doc.createElement("file");
            file.appendChild(doc.createTextNode(jarfile_.getName()));
            node.appendChild(file);
            Iterator<String> iter = required_.iterator();
            Node required = doc.createElement("required");
            while (iter.hasNext())
            {
                Node other = doc.createElement("otherExt");
                String cname = iter.next();
                if (cname != null && cname.length() > 0)
                {
                    other.appendChild(doc.createTextNode(cname));
                    required.appendChild(other);
                }
            }
            node.appendChild(required);
            Node autoload = doc.createElement("autoload");
            autoload.appendChild(doc.createTextNode((autoload_) ? "true"
                    : "false"));
            node.appendChild(autoload);
        }
        else
        {
            node = doc.createElement("fileExt");

            Node n = doc.createElement("file");
            n.appendChild(doc.createTextNode(mainclass_.getFile().getAbsolutePath()));
            node.appendChild(n);

            n = doc.createElement("fqcn");
            n.appendChild(doc.createTextNode(fqcname_));
            node.appendChild(n);

            n = doc.createElement("name");
            n.appendChild(doc.createTextNode(name_));
            node.appendChild(n);

            n = doc.createElement("description");
            n.appendChild(doc.createTextNode(description_));
            node.appendChild(n);

            {// add class files
                n = doc.createElement("classes");
                Node cl;
                Enumeration enumer = classfiles_.elements();
                while (enumer.hasMoreElements())
                {
                    ClassFile cf = (ClassFile) enumer.nextElement();
                    cl = doc.createElement("classFile");
                    cl.appendChild(doc.createTextNode(cf.getFile().getAbsolutePath()));
                    n.appendChild(cl);
                }
                node.appendChild(n);
            }

            {// add required extensions
                Iterator<String> iter = required_.iterator();
                Node required = doc.createElement("required");
                while (iter.hasNext())
                {
                    Node other = doc.createElement("otherExt");
                    String cname = iter.next();
                    if (cname != null && cname.length() > 0)
                    {
                        other.appendChild(doc.createTextNode(cname));
                        required.appendChild(other);
                    }
                }
                node.appendChild(required);
            }
            Node autoload = doc.createElement("autoload");
            autoload.appendChild(doc.createTextNode((autoload_) ? "true"
                    : "false"));
            node.appendChild(autoload);
        }
        if (node != null && error == false)
        {
            doc.getFirstChild().appendChild(node);
        }
        else
        {
            throw new VipToolException("Extension " + getName()
                    + " couldn't be saved");
        }
    }

    /**
     * This method returns an array of IExtensionInfo of all extensions which
     * are required by this one.
     */
    public IExtensionInfo[] getRequiredExtensions()
    {
        IExtensionInfo[] array = new IExtensionInfo[required_.size()];
        Iterator<String> iter = required_.iterator();
        for (int i = 0; iter.hasNext(); i++)
        {
            array[i] = ExtensionManager.getInstance().findExtenison(iter.next());
        }
        return array;
    }

    /**
     * retruns true if this is a FilePlugin
     */
    public boolean isFilePlugin()
    {
        return !isJarPlugin_;
    }

    /**
     * retruns true if this is a JarPlugin
     */
    public boolean isJarPlugin()
    {
        return isJarPlugin_;
    }

    /**
     * This method adds all for this extension necessary URLs to the
     * ExtensionLoader.
     */
    public void registerClasses()
    {
        if (!registered_)
        {
            if (isJarPlugin_)
            {
                ExtensionLoader.getInstance().addJarFile(jarfile_);
            }
            else
            {
                IExtensionLoader loader = ExtensionLoader.getInstance();
                loader.addClassFile(mainclass_);
                Iterator iter = urls_.values().iterator();
                while (iter.hasNext())
                {
                    loader.addClassURL((URL) iter.next());
                }
            }
        }
        registered_ = true;
    }

    /**
     * This method removes the given class file from the list of required
     * classes for this extension.
     */
    public void removeClassFile(ClassFile classfile)
    {
        if (!isJarPlugin_)
        {
            classfiles_.remove(classfile);
            if (registered_)
            {
                ExtensionLoader.getInstance().setClassAssertionStatus(
                        classfile.getClassName(), false);
            }
        }
    }

    /**
     * This method returns the full qualified class name of the main class of
     * this extension.
     */
    public String getClassName()
    {
        return fqcname_;
    }

    /**
     * This method sets the description of this extension to the given text.
     */
    public void setDescription(String description)
    {
        if (description == null)
        {
            description = "";
        }
        description_ = description;
    }

    /**
     * This method sets the name of this extension to the given text.
     */
    public void setName(String name)
    {
        if (name.length() > 0)
        {
            name_ = name;
        }
        else
        {
            name_ = getClassName();
        }
    }

    /**
     * This method returns true if the extension is loaded automatically after
     * starting the VipTool.
     */
    public boolean isAutoload()
    {
        return autoload_;
    }

    /**
     * This method sets the autoload-flag to the given value. If autoload is
     * true the extension will be loaded automatically after starting the
     * VipTool.
     */
    public void setAutoload(boolean autoload)
    {
        autoload_ = autoload;
    }

    public String toString()
    {
        return getName() + " " + ((isJarPlugin()) ? "(jar)" : "(file)");
    }

    /**
     * This method returns a Vector which contains all fqcn of required
     * extensions.
     */
    public HashSet<String> getRequiredNames()
    {
        return required_;
    }

    /**
     * This method test if all required Extensions are avaiable.
     */
    public void testForRequired(IExtensionManager extManager)
            throws VipToolExtensionException
    {
        Iterator<String> enumer = required_.iterator();
        while (enumer.hasNext())
        {
            String reqName = enumer.next();
            IExtensionInfo reqExt = extManager.findExtenison(reqName);
            if (reqExt == null)
            {
                throw new VipToolExtensionException(getName()
                        + ": required Extension " + reqName + " not found!",
                        reqName);
            }
        }
    }

   private void updateMainfestFile()
    {
        // update manifest for file plugin
        // TODO: Thomas: vorläufig deaktiviert
        if (!isJarPlugin_ && false)
        {
            // try to get path for mainfest
            if (mainclass_ != null)
            {
                File folder = mainclass_.getFile();

                boolean search = true;

                while (folder != null && search)
                {
                    folder = folder.getParentFile();
                    if (folder.isDirectory())
                    {
                        File[] files = folder.listFiles();
                        for (int i = 0; i < files.length; i++)
                        {
                            if (files[i].getName().equals(".project"))
                            {
                                search = false;
                                break;
                            }
                        }
                    }
                    else
                    {
                        search = false;
                    }
                }

                File manifest = new File(folder, "MANIFEST.MF");
                if (manifest.exists())
                {
                    if (!manifest.delete())
                    {
                        Workbench.getInstance().getLogger().warning(
                                getName() + ": manifest couldn't be updated!");
                        return;
                    }
                }

                FileOutputStream fos = null;
                BufferedOutputStream bos = null;
                OutputStreamWriter writer = null;
                try
                {
                    fos = new FileOutputStream(manifest);
                    bos = new BufferedOutputStream(fos);
                    writer = new OutputStreamWriter(bos);

                    writer.write(getManifest());

                    Workbench.getInstance().getLogger().info(
                            getName() + ": manifest successfully updated!");
                }
                catch (FileNotFoundException e)
                {
                    Workbench.getInstance().getLogger().warning(
                            getName()
                                    + ": file not found: manifest couldn't be updated!");
                }
                catch (IOException ex)
                {
                    Workbench.getInstance().getLogger().warning(
                            getName()
                                    + ": IOException: manifest couldn't be updated!");
                }
                finally
                {
                    if (writer != null)
                    {
                        try
                        {
                            writer.flush();
                            writer.close();
                        }
                        catch (IOException e)
                        {
                            // nothing to do
                        }
                    }
                    if (bos != null)
                    {
                        try
                        {
                            bos.flush();
                            bos.close();
                        }
                        catch (IOException e)
                        {
                            // nothing to do
                        }
                    }
                    if (fos != null)
                    {
                        try
                        {
                            fos.flush();
                            fos.close();
                        }
                        catch (IOException e)
                        {
                            // nothing to do
                        }
                    }
                }
            }
            else
            {
                Workbench.getInstance().getLogger().warning(
                        getName() + ": path to manifest not found!");
            }
        }
    }

    /**
     * This method returns a String which is a valid manifest for this
     * extensions. This means the String contains all required extensions the
     * main class the name and the description.
     */
    public String getManifest()
    {
        String manifest = "";

        long version = 0;

        if (mainclass_ != null)
        {
            version = mainclass_.getFile().lastModified();
        }

        manifest += "Manifest-Version: 1.0\n";
        // class
        manifest += "Extension-Class: " + getClassName() + "\n";
        // name
        manifest += "Extension-Name: " + getName() + "\n";
        // Version
        manifest += "Implementation-Version: " + version + "\n";
        // description
        manifest += "Description: " + getDescription() + "\n";
        // required
        manifest += "Extension-List: ";
        HashSet<String> required = getRequiredNames();
        Iterator<String> iter = required.iterator();
        while (iter.hasNext())
        {
            String extension = iter.next();
            manifest += extension;
            if (iter.hasNext())
            {
                manifest += " ";
            }
        }
        manifest += "\n";

        manifest += "Sealed: false\n";
        manifest += "Created-By: VipTool Manifest Generator";

        return manifest;
    }

    public long getVersion()
    {
        if (!isJarPlugin_)
        {
            if (mainclass_ == null)
            {
                return 0;
            }
            else
            {
                return mainclass_.getFile().lastModified();
            }
        }
        else
        {
            return version_;
        }
    }
}
