package viptool.impl.extension;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.URI;
import java.nio.charset.Charset;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.util.jar.JarFile;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import viptool.base.error.FileInterpreterProviderException;
import viptool.base.error.VipToolException;
import viptool.base.error.VipToolExtensionException;
import viptool.base.error.XMLException;
import viptool.extension.Extension;
import viptool.extension.IExtensionInfo;
import viptool.extension.IExtensionListener;
import viptool.extension.IExtensionManager;
import viptool.impl.base.Workbench;
import viptool.impl.logger.Logger;
import viptool.impl.util.XMLUtils;
import viptool.impl.util.resources.ResourceLoader;
import viptool.impl.util.resources.schema.SchemaConstants;
import viptool.project.FileSystem;

/**
 * The ExtensionManager handles all Viptool-Extensions. This means it loads the
 * Extensions by creating a new instance and registers it, so that extensions
 * can be disabled. This manager also manage a list of extensions which should
 * be loaded whenever the Viptool starts.
 */
public class ExtensionManager implements IExtensionManager
{
    // running instance of the extension manager
    private static ExtensionManager instance_      = null;

    // Map for all plugged extensions
    // Key: fqcname of the extension
    // Value: IExtensionInfo of the extension
    private HashMap                 extensions_;

    // instance of the autoload-list saver
    // this thread will be automatically run when the Viptool is closed
    private ShutdownSave            save_;

    // Vector of all ExtenisonListener
    private Vector                  listener_;

    private Vector                  loopDetection_ = new Vector();

    private IExtensionInfo          first_         = null;

    // data structure which stores the running instances of extensions
    // Key: IExtensionInfo for Extension
    // Value: instance of Extension
    private HashMap                 runningExt_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>

    private class FileComparator implements Comparator<File>
    {
        public int compare(File o1, File o2)
        {
            return o1.getName().compareTo(o2.getName());
        }

    }

    /**
     * This class saves a list of extensions which should be automatically
     * loaded by the next start of the Viptool. This thread is registered to
     * start if the Viptool will be closed.
     */
    class ShutdownSave extends Thread
    {
        // data structure which stores the known extensions
        private Vector extensions_;

        // the xml file to store extension info
        private String extensionFile_;

        /**
         * This method creates a new instance of the ShutdownSave-Thread for
         * extensions.
         */
        public ShutdownSave(String extensionFile)
        {
            extensions_ = new Vector();

            setExtensionFile(extensionFile);
        }

        /**
         * This method returns the path of the resource-file which saves the
         * known extensions.
         * 
         * @return file which saves the extensions
         */
        public String getExtensionFile()
        {
            return extensionFile_;
        }

        public URI getExtensionURI()
        {
            return (new File(getExtensionFile())).toURI();
        }

        public void setExtensionFile(String extensionFile)
        {
            extensionFile_ = extensionFile;
            if (extensionFile_ == null)
            {
                extensionFile_ = Workbench.getInstance().getPropertyPath()
                        + "extensions.xml";
            }
        }

        /**
         * run-method for thread
         */
        public void run()
        {
            saveExtensions();
        }

        /**
         * This method writes the known extensions to the resource-file.
         * Structure: root-node: extensions child-nodes: nodes from
         * ExtensionInfo.getNode()
         */
        public void saveExtensions()
        {
            Document doc = null;
            try
            {
                // initialize DocumentBuilder
                DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
                // create Document
                doc = docBuilder.newDocument();
            }
            catch (ParserConfigurationException e)
            {
                e.printStackTrace();
            }

            if (null != doc)
            {
                // create root
                Element root = doc.createElement("extensions");
                // add root to document
                doc.appendChild(root);

                // add extensions
                Enumeration enumer = extensions_.elements();
                while (enumer.hasMoreElements())
                {
                    IExtensionInfo extInfo = (IExtensionInfo) enumer.nextElement();
                    try
                    {
                        extInfo.getNode(doc);
                    }
                    catch (VipToolException ex)
                    {
                        ex.printStackTrace();
                    }
                }
            }

            // initialize TransformerFactory
            TransformerFactory tf = TransformerFactory.newInstance();
            Transformer t = null;
            try
            {
                // get transformer
                t = tf.newTransformer();
                // set use whitespaces
                t.setOutputProperty(OutputKeys.INDENT, "yes");
                // set output-type xml
                t.setOutputProperty(OutputKeys.METHOD, "xml");
                // Content-Type
                t.setOutputProperty(OutputKeys.MEDIA_TYPE, "text/xml");
                // set encoding
                t.setOutputProperty(OutputKeys.ENCODING, "utf-8");
                // create FileWriter
                OutputStreamWriter fw = new OutputStreamWriter(
                        new FileOutputStream(save_.getExtensionFile()),
                        Charset.forName("utf-8").newEncoder());
                // write to file
                t.transform(new DOMSource(doc), new StreamResult(fw));
            }
            catch (TransformerConfigurationException e)
            {
                e.printStackTrace();
            }
            catch (TransformerException e)
            {
                e.printStackTrace();
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }

        /**
         * This method adds an extension to the list of known extensions.
         * 
         * @param extInfo
         *            ExtensionInfo of the extension
         */
        public void addExtension(IExtensionInfo extInfo)
        {
            extensions_.add(extInfo);
        }

        /**
         * This metod removes the given extension form the vector of known
         * extensions.
         * 
         * @param extInfo
         */
        public void removeExtension(IExtensionInfo extInfo)
        {
            extensions_.remove(extInfo);
        }

        public void addExtension(IExtensionInfo extInfo, int pos)
        {
            int len = extensions_.size();
            if (pos > len || pos < 0)
            {
                extensions_.add(extInfo);
            }
            else
            {
                extensions_.insertElementAt(extInfo, pos);
            }
        }

        /**
         * This method returns a copy of the vector which contains all known
         * extensions.
         * 
         * @return vector with extension
         */
        public Vector getExtensions()
        {
            return (Vector) extensions_.clone();
        }

        /**
         * This method moves the given extension up into the list of known
         * extensions.
         * 
         * @param extInfo
         *            extension to move up
         */
        public void moveExtensionUp(IExtensionInfo extInfo)
        {
            int idx = extensions_.indexOf(extInfo);
            if (idx > 0 && idx < extensions_.size())
            {
                extensions_.remove(extInfo);
                extensions_.insertElementAt(extInfo, idx - 1);
            }
            else
            {
                Workbench.getInstance().getLogger().warning(
                        "Extension order couldn't be changed!");
            }
        }

        /**
         * This method moves the given extension down into the list of known
         * extensions.
         * 
         * @param extInfo
         *            extension to move down
         */
        public void moveExtensionDown(IExtensionInfo extInfo)
        {
            int idx = extensions_.indexOf(extInfo);
            if (idx >= 0 && idx < extensions_.size() - 1)
            {
                extensions_.remove(extInfo);
                extensions_.insertElementAt(extInfo, idx + 1);
            }
            else
            {
                Workbench.getInstance().getLogger().warning(
                        "Extension order couldn't be changed!");
            }
        }

    }

    /**
     * This method creates a new instance of the ExtensionManager.
     */
    private ExtensionManager()
    {
        extensions_ = new HashMap();
        runningExt_ = new HashMap();

        listener_ = new Vector();

        // register ShutdownSave for autorun as shutdown
        save_ = new ShutdownSave(null);
        Runtime rt = Runtime.getRuntime();
        rt.addShutdownHook(save_);
    }

    /**
     * This method adds the given extension to the list of known extensions.
     * 
     * @param extInfo
     *            extension to add
     */
    public boolean addExtension(IExtensionInfo extInfo)
    {
        if (extensions_.get(extInfo.getClassName()) == null)
        {
            extensions_.put(extInfo.getClassName(), extInfo);
            save_.addExtension(extInfo);
            // call listeners
            Enumeration enumer = listener_.elements();
            while (enumer.hasMoreElements())
            {
                ((IExtensionListener) enumer.nextElement()).extensionAdded(extInfo);
            }
            return true;
        }
        else
        {
            // test if extInfo is newer than known extension
            IExtensionInfo extInfoKnown = (IExtensionInfo) extensions_.get(extInfo.getClassName());
            if (extInfoKnown.getVersion() < extInfo.getVersion())
            {
                // remove old one
                extensions_.remove(extInfo.getClassName());
                save_.removeExtension(extInfoKnown);
                // add new one
                extensions_.put(extInfo.getClassName(), extInfo);
                save_.addExtension(extInfo);
                Workbench.getInstance().getLogger().info(
                        "The extension " + extInfo.getClassName()
                                + " was replaced by a new version!");
                return true;
            }
            else
            {
                Workbench.getInstance().getLogger().error(
                        "The extension " + extInfo.getClassName()
                                + " was already added!");
                return false;
            }
        }
    }

    /**
     * This method loads the extensions which are saved into the autoload-file.
     */
    public void loadAutoLoadExtensions()
    {
        Document document = null;
        Vector autoload = new Vector();

        // load extensions from xml file
        if ((new File(save_.getExtensionFile())).exists())
        {
            try
            {
                // create Parser
                document = XMLUtils.parseXMLFile(save_.getExtensionURI(),
                        ResourceLoader.getInstance().getResource(
                                SchemaConstants.SCHEMA_EXTENSIONS));

                Node root = document.getFirstChild();

                NodeList list = root.getChildNodes();
                int len = list.getLength();
                for (int i = 0; i < len; i++)
                {

                    Node extensionNode = list.item(i);
                    if (extensionNode.getNodeName().equals("jarExt")
                            || extensionNode.getNodeName().equals("fileExt"))
                    {

                        try
                        {
                            ExtensionInfo extInfo = ExtensionInfo.getExtensionFormNode(
                                    extensionNode, this);
                            if (addExtension(extInfo))
                            {
                                if (extInfo.isAutoload())
                                {
                                    autoload.add(extInfo);
                                }
                            }
                        }
                        catch (VipToolException ex)
                        {
                            Logger.getInstance().error(ex.getMessage());
                        }
                        catch (Exception e)
                        {
                            Logger.getInstance().error(
                                    "Extension couldn't be loaded!");
                        }
                    }
                }
            }
            catch (XMLException exception)
            {
                Logger.getInstance().error(exception.getMessage());
            }
        }

        // load extensions from extensions dir
        File extensionDir = new File(System.getProperty("user.dir")
                + File.separator + "extensions");
        if (extensionDir.exists() && extensionDir.isDirectory())
        {
            Logger.getInstance().info(
                    "Searching directory '" + extensionDir.getAbsolutePath()
                            + "'");

            ExtensionJarInterpreter interpreter = new ExtensionJarInterpreter();

            File[] files = extensionDir.listFiles();
            Arrays.sort(files, new FileComparator());

            for (int i = 0; i < files.length; i++)
            {
                if (files[i].getName().endsWith(".jar"))
                {
                    try
                    {
                        if (interpreter.matchFormat(FileSystem.getInstance().createAbstractFile(
                                files[i])))
                        {
                            try
                            {
                                IExtensionInfo extInfo = new ExtensionInfo(
                                        new JarFile(files[i]));
                                // we want to load the extension the next start
                                // as well
                                extInfo.setAutoload(true);
                                if (findExtenison(extInfo.getClassName()) == null)
                                {
                                    if (addExtension(extInfo))
                                    {
                                        autoload.add(extInfo);
                                    }
                                }
                            }
                            catch (Exception e)
                            {
                                Workbench.getInstance().showExceptionInDialog(
                                        new VipToolException(e));
                            }
                        }
                    }
                    catch (FileInterpreterProviderException e)
                    {
                        Logger.getInstance().warning(
                                "Error by testing " + files[i].getName()
                                        + " for VipTool extension jar.");
                    }
                }
            }
        }

        Iterator iter = extensions_.values().iterator();
        while (iter.hasNext())
        {
            IExtensionInfo extInfo = (IExtensionInfo) iter.next();
            try
            {
                extInfo.testForRequired(this);
            }
            catch (VipToolExtensionException e)
            {
                if (extInfo.isAutoload())
                {
                    extInfo.setAutoload(false);
                    autoload.remove(extInfo);
                }
                Workbench.getInstance().getLogger().log(Logger.ERROR_LEVEL,
                        e.getMessage());
            }
        }

        Enumeration enumer = autoload.elements();
        while (enumer.hasMoreElements())
        {
            try
            {
                IExtensionInfo extInfo = (IExtensionInfo) enumer.nextElement();
                loadExtension(extInfo);
            }
            catch (VipToolException e)
            {
                Workbench.getInstance().getLogger().log(Logger.ERROR_LEVEL,
                        e.getMessage());
            }
        }

    }

    /**
     * This method returns the running instance of the ExtenisonManager or
     * creates a new one if there is no running instance.
     * 
     * @return running instance of ExtensionManager
     */
    public static synchronized ExtensionManager getInstance()
    {
        if (instance_ == null)
        {
            instance_ = new ExtensionManager();
        }

        return instance_;
    }

    /**
     * This method stops the extension with the given name.
     * 
     * @param name
     *            full qualified name of the extension
     */
    public void stopExtension(String name)
    {
        IExtensionInfo info = (IExtensionInfo) extensions_.get(name);
        if (info != null)
        {
            stopExtension(info);
        }
        else
        {
            Logger.getInstance().error("Extension " + name + " isn't known!");
        }
    }

    /**
     * This method stops the given extension.
     * 
     * @param name
     *            extension to stop
     */
    public void stopExtension(IExtensionInfo extInfo)
    {
        Extension ext = (Extension) runningExt_.get(extInfo);
        ext.terminate();
        runningExt_.remove(extInfo);

        Logger.getInstance().info(
                "Extension " + extInfo.getName() + " stopped.");

        Enumeration enumer = listener_.elements();
        while (enumer.hasMoreElements())
        {
            ((IExtensionListener) enumer.nextElement()).extensionStopped(extInfo);
        }
    }

    /**
     * This method loads the extension.
     * 
     * @param extInfo
     *            extension to load
     */
    public Extension loadExtension(IExtensionInfo extInfo)
            throws VipToolException
    {
        extInfo.registerClasses();

        Extension instance = (Extension) runningExt_.get(extInfo);

        if (instance == null)
        {
            // test for required extensions
            IExtensionInfo[] required = extInfo.getRequiredExtensions();

            if (loopDetection_.contains(extInfo))
            {
                throw new VipToolException("Loop in required extensions path!");
            }

            if (required.length > 0)
            {
                if (first_ == null)
                {
                    first_ = extInfo;
                }
                loopDetection_.add(extInfo);
            }

            for (int i = 0; i < required.length; i++)
            {
                if (!isActivated(required[i]))
                {
                    if (required[i] != null)
                    {
                        loadExtension(required[i]);
                    }
                }
            }

            if (first_ != null && first_ == extInfo)
            {
                first_ = null;
                loopDetection_.clear();
            }

            // load extension
            Class clazz = extInfo.getExtensionClass();

            // Class clazz = ExtensionLoader.getInstance().getClass(fqcname_);
            try
            {
                instance = (Extension) clazz.newInstance();
                instance.initialize(Workbench.getInstance());
                runningExt_.put(extInfo, instance);
                
                Logger.getInstance().info(
                        "Extension " + extInfo.getName() + " was loaded.");
            }
            catch (InstantiationException e)
            {
                Logger.getInstance().error(
                        "Extension " + extInfo.getName()
                                + " couldn't be instantiated!");
            }
            catch (IllegalAccessException e)
            {
                Logger.getInstance().error(
                        "Extension " + extInfo.getName()
                                + ": security exception!");
            }
            catch (NoClassDefFoundError e)
            {
                Logger.getInstance().error(
                        "Class not found: " + extInfo.getClassName());
                return null;
            }
            catch (Exception e)
            {
            	e.printStackTrace();
                Logger.getInstance().error(e.getMessage());
                return null;
            }
        }
//        else
//        {
//            Logger.getInstance().info(
//                    "Extension " + extInfo.getName() + " was already loaded!");
//        }

        

        // call listener
        Enumeration enumer = listener_.elements();
        while (enumer.hasMoreElements())
        {
            ((IExtensionListener) enumer.nextElement()).extensionStarted(extInfo);
        }

        return instance;
    }

    /**
     * This method loads the extension with the given name.
     * 
     * @param name
     *            full qualified class name of the extension to load
     */
    public Extension loadExtension(String name) throws VipToolException
    {
        IExtensionInfo info = (IExtensionInfo) extensions_.get(name);
        Extension ext = null;
        if (info != null)
        {
            ext = loadExtension(info);
        }
        else
        {
            Logger.getInstance().error("Extension " + name + " isn't known!");
        }

        return ext;
    }

    /**
     * This method returns a set of all known extensions.
     * 
     * @return set of known extensions
     */
    public Collection getKnownExtensions()
    {
        return extensions_.values();
    }

    public void setExtensionFile(String extensionFile)
    {
        save_.setExtensionFile(extensionFile);
    }

    /**
     * This methode returns an ordered vector of all known extensions.
     * 
     * @return vector of extensions
     */
    public Vector getOrderedExtensions()
    {
        return save_.getExtensions();
    }

    /**
     * This method remove the given extension from the list of known extensions.
     * 
     * @param extInfo
     *            extension to remove
     */
    public void removeExtenison(IExtensionInfo extInfo)
    {
        extensions_.remove(extInfo.getClassName());
        save_.removeExtension(extInfo);
        Enumeration enumer = listener_.elements();
        while (enumer.hasMoreElements())
        {
            ((IExtensionListener) enumer.nextElement()).extensionRemoved(extInfo);
        }
    }

    /**
     * This method removes the extension with the given full qualified class
     * name from the list of known extensions.
     * 
     * @param name
     *            name of the extension to remove
     */
    public void removeExtenison(String name)
    {
        IExtensionInfo extInfo = (IExtensionInfo) extensions_.get(name);
        if (extInfo != null)
        {
            extensions_.remove(extInfo.getClassName());
            save_.removeExtension(extInfo);
            Enumeration enumer = listener_.elements();
            while (enumer.hasMoreElements())
            {
                ((IExtensionListener) enumer.nextElement()).extensionRemoved(extInfo);
            }
        }
    }

    /**
     * This method insert the given extension at the given position.
     * 
     * @param extInfo
     *            extension to insert
     * @param pos
     *            position to insert at
     */
    public void insertExtension(IExtensionInfo extInfo, int pos)
    {
        if (extensions_.get(extInfo.getClassName()) != null)
        {
            save_.removeExtension(extInfo);
            save_.addExtension(extInfo, pos);
        }
        else
        {
            extensions_.put(extInfo.getClassName(), extInfo);
            save_.addExtension(extInfo, pos);
        }
    }

    /**
     * This method returns the index of the given extension into the list of
     * known extensions.
     */
    public int getIndexOfExtension(IExtensionInfo extInfo)
    {
        return save_.extensions_.indexOf(extInfo);
    }

    public Extension getRunningExtension(IExtensionInfo info)
    {
        return (Extension) runningExt_.get(info);
    }

    /**
     * This method returns an ExtensionInfo for the given full qualified class
     * name if the extension is known (was registered).
     * 
     * @param fqcn
     *            full qualified name of the extension
     * @return extension with the given name or null
     */
    public IExtensionInfo findExtenison(String fqcn)
    {
        IExtensionInfo extInfo = (IExtensionInfo) extensions_.get(fqcn);
        if (extInfo != null)
        {
            return extInfo;
        }
        else
        {
            return null;
        }
    }

    /**
     * add ExtensionListener
     * 
     * @param listener
     *            listener to add
     */
    public void addExtensionListener(IExtensionListener listener)
    {
        listener_.add(listener);
    }

    /**
     * remove ExtensionListener
     * 
     * @param listener
     *            listener to remove
     */
    public void removeExtensionListener(IExtensionListener listener)
    {
        listener_.remove(listener);
    }

    /**
     * This method returns true if the extension is loaded.
     */
    public boolean isActivated(IExtensionInfo extInfo)
    {
        Extension ext = (Extension) runningExt_.get(extInfo);
        if (ext != null)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * This method moves the given extension up into the list of known
     * extensions.
     * 
     * @param extInfo
     *            extension to move up
     */
    public void moveExtensionUp(IExtensionInfo extInfo)
    {
        save_.moveExtensionUp(extInfo);
    }

    /**
     * This method moves the given extension down into the list of known
     * extensions.
     * 
     * @param extInfo
     *            extension to move down
     */
    public void moveExtensionDown(IExtensionInfo extInfo)
    {
        save_.moveExtensionDown(extInfo);
    }

}
