package viptool.impl.extension.extensionloader;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.jar.JarFile;

import viptool.base.error.VipToolException;
import viptool.extension.IExtensionLoader;
import viptool.impl.base.Workbench;
import viptool.impl.logger.Logger;
import viptool.util.ClassFile;

/**
 * The class ExtensionLoader extends URLClassLoader an is designed to load find
 * extensions in directories.
 */
public class ExtensionLoader extends URLClassLoader implements IExtensionLoader
{
    // instance of ExtensionLoader
    private static ExtensionLoader loader_       = null;

    // hashset of library paths
    private Set                    libraryPaths_ = null;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This method creates a new instance of ExensionLoader.
     */
    private ExtensionLoader()
    {
        super(new URL[] {}, ClassLoader.getSystemClassLoader());
        
        libraryPaths_ = new HashSet();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the Singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This method returns the running instance of ExtensionLoader or creates a
     * new one if there is no instance.
     * 
     * @return running instance of ExtensionLoader
     */
    public static ExtensionLoader getInstance()
    {
        if(loader_ == null)
        {
            loader_ = new ExtensionLoader();
        }
        return loader_;
    }
    
    // <><><><><><><><><><><><><><><><><><><><><>
    // Overridden methods from java.lang.ClassLoader
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see java.lang.ClassLoader */
    protected String findLibrary(String libname)
    {
        Iterator it;
        File file;
        String libraryPath;
        
        libname = System.mapLibraryName(libname);
        
        it = libraryPaths_.iterator();
        while(it.hasNext())
        {
            libraryPath = (String)it.next();
            file = new File(libraryPath + libname);
            
            if(file.exists())
            {
                return file.getAbsolutePath();
            }
        }
        
        return null;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IExtensionLoader interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This methode adds the given ClassFile to the URL-List. For details see
     * interface IExtensionLoader.
     */
    public void addClassFile(ClassFile classfile)
    {
        URL url;
        try
        {
            url = classfile.getClassURL();
            super.addURL(url);
        }
        catch (VipToolException e)
        {
            Workbench.getInstance().showExceptionInDialog(e);
        }
    }

    /**
     * This method adds the given JarFile to the URL-List. This means all
     * classes which are contained into the given directory can be loaded with
     * ExtensionLoader.getClass([full qualified classname]).
     */
    public void addJarFile(JarFile jarfile)
    {
        try
        {
            super.addURL(new File(jarfile.getName()).toURI().toURL());
        }
        catch (MalformedURLException e)
        {
            Logger.getInstance().warning(
                    "File " + jarfile.getName() + " couldn't be registered.");
        }
    }

    /**
     * This method try to load the class with the given name. If the class isn't
     * known an error-message will be written to the logger an null returned.
     * 
     * @param name
     *            full qualified class name
     * @return Class Object for the given name.
     */
    public Class getClass(String name)
    {
        try
        {
            return super.loadClass(name);
        }
        catch (ClassNotFoundException e)
        {
            Logger.getInstance().error("unknown class: " + name);
            return null;
        }
    }

    /**
     * This method adds the given URL to the URL-List of this ExtensionLoader.
     */
    public void addClassURL(URL url)
    {
        super.addURL(url);
    }
    
    /* @see viptool.extension.IExtensionLoader */
    public void addLibraryURL(URL url)
    {
        String path = url.getPath();
        
        if(!path.endsWith("/"))
        {
            path += "/";
        }
        
        libraryPaths_.add(path);
    }
}
