package viptool.impl.extension.gui.manager;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JPanel;

import viptool.base.error.VipToolException;
import viptool.extension.IExtensionInfo;
import viptool.extension.IExtensionListener;
import viptool.impl.base.Workbench;
import viptool.impl.base.gui.layoutmanager.listlayout.ListLayoutSelectionSupport;
import viptool.impl.extension.ExtensionManager;
import viptool.impl.util.resources.ResourceLoader;
import viptool.util.resources.ResourceConstants;

public class ListRowPanel extends JPanel implements MouseListener, ListLayoutSelectionSupport, IExtensionListener{

	//HTML format for button labels
	private static String BUTTON_START = "<html><span style='size:11; color:#0000FF; text-decoration:underline;'>";
	private static String BUTTON_END = "</span></html>";
		
	//body panel
	private JPanel body_;
	
	//label to show/hide body
	private JLabel button_;
	
	//icon expand
	private Icon expand_ = null;
	
	//icon minimize
	private Icon minimize_ = null;

	/**
	 * This is the background color for selected elements.
	 * WARNING: Changing this color affects all running instances of ListRowPanel
	 */
	public static Color SELECTED = new Color(184,207,229);
	
	/**
	 * This is the background color for deselected elements.
	 * WARNING: Changing this color affects all running instances of ListRowPanel
	 */
	public static Color UNSELECTED = Color.WHITE;
	
	//current background color
	private Color COLOR = null;
	
	//leading part of the HTML markup
	private static String HEADER_FOMAT_START = "<html><h3>";
	//following part of the HTML markup
	private static String HEADER_FOMAT_END = "</h3></html>";
	
	//part of the component which is always visible
	private JPanel header_;
	
	//label for running status
	private JLabel state_;
	
	//pointer to extensioninfo
	private IExtensionInfo extInfo_;
	
	//cursor for buttons
	Cursor cursor_ = new Cursor(Cursor.HAND_CURSOR);
	
	/**
	 * This method create a new ListRowPanel.
	 * 
	 * @param text
	 * 		text which is shown as description
	 * @param body
	 * 		this component is shown under the component in expanded view
	 * @param selected
	 * 		initial status for selection
	 */
	public ListRowPanel(IExtensionInfo extInfo , boolean selected){
		
		extInfo_ = extInfo;
		
		//initialize the background color
		if(selected){
			COLOR = SELECTED;
		}else{
			COLOR = UNSELECTED;
		}
		
		//add mouse listener for selection support
		addMouseListener(this);
		
		setBackground(COLOR);
		setLayout(new BorderLayout());
		
		//load image for expand icon
		expand_ = ResourceLoader.getInstance().getIcon(ResourceConstants.IMAGE_EXPAND);
		//load image for minimize icon
		minimize_ = ResourceLoader.getInstance().getIcon(ResourceConstants.IMAGE_MINIMIZE);
		
		//add header
		header_ = generateHeader(extInfo); 
		add(header_, BorderLayout.NORTH);
		
		//add body
		body_ = new ExtensionInfoPanel(extInfo);
		//hide body first
		body_.setVisible(false);
		add(body_, BorderLayout.CENTER);
		
		//register as extension listener for running status
		ExtensionManager.getInstance().addExtensionListener(this);
	}

	/**
	 * This method generates the short view of the extension
	 * 
	 * @param extInfo
	 * 		an IExtensionInfo
	 * 
	 * @return
	 * 		header of the entry
	 */
	private JPanel generateHeader(IExtensionInfo extInfo){
		JPanel panel = new JPanel();
		panel.setBackground(COLOR);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT));
	
		//generate expand/minimize button
		button_ = new JLabel(expand_);
		button_.addMouseListener(this);
		button_.setCursor(cursor_);
		panel.add(button_);
		
		//generate description label
		JLabel label = new JLabel(HEADER_FOMAT_START+extInfo.getName()+HEADER_FOMAT_END);
		panel.add(label);
		
		//generate label for running status
		state_ = new JLabel(BUTTON_START+((ExtensionManager.getInstance().isActivated(extInfo))?"running":"stopped")+BUTTON_END);
		state_.addMouseListener(this);
		state_.setCursor(cursor_);
		panel.add(state_);
		
		return panel;
	}
	
	
	/**
	 * implementation of the MouseListener interface
	 */
	public void mouseClicked(MouseEvent e) {
		if(e.getSource().equals(button_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			if(button_.getIcon().equals(expand_)){
				button_.setIcon(minimize_);
				body_.setVisible(true);
			}else{
				button_.setIcon(expand_);
				body_.setVisible(false);
			}
		}
		
		if(e.getSource().equals(this)  && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			if(COLOR.equals(SELECTED)){
				setSelected(false);
			}else{
				setSelected(true);
			}
		}
		
		if(e.getSource().equals(state_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			try {
				if(ExtensionManager.getInstance().isActivated(extInfo_)){
					ExtensionManager.getInstance().stopExtension(extInfo_);
				}else{
					ExtensionManager.getInstance().loadExtension(extInfo_);
				}
			} catch (VipToolException e1) {
				Workbench.getInstance().getLogger().error(e1.getMessage());
			}
		}
		
		revalidate();
	}

	public void mouseEntered(MouseEvent e) {
		//nothing to do
	}

	public void mouseExited(MouseEvent e) {
		//nothing to do
	}

	public void mousePressed(MouseEvent e) {
		//nothing to do	
	}

	public void mouseReleased(MouseEvent e) {
		//nothing to do
	}
	
	/**
	 * This method set the selection state for this row.
	 * 
	 * @param isselected
	 * 		true to select
	 */
	public void setSelected(boolean isselected){
		if(isselected){
			COLOR = SELECTED;
		}else{
			COLOR = UNSELECTED;
		}
		setBackground(COLOR);
		int cp = getComponentCount();
		for(int i=0; i<cp; i++){
			getComponent(i).setBackground(COLOR);
		}
	}
	
	/**
	 * This method returns the selection state.
	 * 
	 * @return
	 * 		true if selected, false else
	 */
	public boolean isSelected(){
		if(COLOR == SELECTED){
			return true;			
		}
		return false;
	}

	//not required
	public void extensionAdded(IExtensionInfo extInfo) {}

	//not required
	public void extensionRemoved(IExtensionInfo extInfo) {}

	/**
	 * implementation of extension listener
	 * This method is necessary to update the running label.
	 */
	public void extensionStarted(IExtensionInfo extInfo) {
		if(extInfo.equals(extInfo_)){
			state_.setText(BUTTON_START+((ExtensionManager.getInstance().isActivated(extInfo))?"running":"stopped")+BUTTON_END);
		}
	}

	/**
	 * implementation of extension listener
	 * This method is necessary to update the running label.
	 */
	public void extensionStopped(IExtensionInfo extInfo) {
		if(extInfo.equals(extInfo_)){
			state_.setText(BUTTON_START+((ExtensionManager.getInstance().isActivated(extInfo))?"running":"stopped")+BUTTON_END);
		}
	}
	
	/**
	 * This method returns the extension info which is represented by this component.
	 * 
	 * @return
	 * 		IExtensionInfo extension info
	 */
	public IExtensionInfo getExtensionInfo(){
		return extInfo_;
	}
}
