package viptool.impl.filesystem;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.NoSuchElementException;

import viptool.filesystem.FileSystemEntry;
import viptool.filesystem.IFileSystem;
import viptool.filesystem.error.FileExistsException;
import viptool.filesystem.error.FileSystemException;
import viptool.filesystem.error.InvalidContentException;
import viptool.filesystem.error.InvalidNameException;
import viptool.filesystem.error.NoSuchFileException;

public class SimpleFileSystem implements IFileSystem
{
    private IFolderMap root_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Subclasses
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Folder Map interface for folder maps. folder maps are used to store a
     * hierarchical system like a file system. A folder maps maps a string to
     * map if it is a folder, or to another object if the string denotes a file.
     */
    protected interface IFolderMap extends Map
    {
        /**
         * Returns the parent of this folder map or null if the map has no
         * parent.
         * 
         * @return the parent or null
         */
        public IFolderMap getParent();

        /**
         * Returns a new folder map instance. The new folder map has this map as
         * parent.
         * 
         * @return a new folder map with this map as parent
         */
        public IFolderMap newInstance();

        public String[] getKeys();

        public Iterator getStringIterator();
    }

    /**
     * The folder map is a hash map that can have a parent map. This parent map
     * is the enclosing hash map.
     */
    protected class FolderMap extends HashMap implements IFolderMap
    {
        /**
         * The folder iterator provides access to a folders content. The next()
         * method will return instances of the class FileSystemEntry.
         */
        private class FolderIterator implements Iterator
        {
            // iterator over the keys from the folder map
            private Iterator it_;

            // <><><><><><><><><><><><><><><><><><><><><>
            // Constructors
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * Construct a new folder iterator that walks through the elements
             * from the given folder.
             */
            public FolderIterator()
            {
                it_ = keySet().iterator();
            }

            // <><><><><><><><><><><><><><><><><><><><><>
            // Implementation of the Iterator interface
            // <><><><><><><><><><><><><><><><><><><><><>

            /* @see java.util.Iterator */
            public boolean hasNext()
            {
                return it_.hasNext();
            }

            /* @see java.util.Iterator */
            public Object next()
            {
                String name;
                Object object;

                name = (String) it_.next();
                object = get(name);

                return new FileSystemEntry(name, (object instanceof IFolderMap));
            }

            /* @see java.util.Iterator */
            public void remove()
            {
                throw new UnsupportedOperationException();
            }
        }

        // the parent hash map
        private IFolderMap parent_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Constructs a new folder map with the given parent. The parent can be
         * null.
         * 
         * @param parent
         *            the parent of the map or null
         */
        public FolderMap(IFolderMap parent)
        {
            parent_ = parent;
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the IFolderMap interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see IFolderMap */
        public IFolderMap getParent()
        {
            return parent_;
        }

        /* @see IFolderMap */
        public IFolderMap newInstance()
        {
            return new FolderMap(this);
        }

        /* @see IFolderMap */
        public String[] getKeys()
        {
            return (String[]) keySet().toArray(new String[keySet().size()]);
        }

        /* @see IFolderMap */
        public Iterator getStringIterator()
        {
            return new FolderIterator();
        }
    }

    /**
     * The folder map iterator iterates over all folder maps in which the a file
     * or folder lies. If the iterator is based on a folder, the map of the
     * folder itself will be the last element. The first element is always the
     * root map.
     */
    private class FolderMapIterator implements Iterator
    {
        // the current index
        private int        index_;

        // the last index in the string array parts_ that is a folder name
        private int        endIndex_;

        // the folders as string array
        private String[]   parts_;

        // the folder map that is returned on the next call to next()
        private IFolderMap map_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Constructs a new folder map iterator based on the given name. The
         * folder map iterator will iterate over all folders beginning at the
         * root.<br>
         * <b>Example:</b><br>
         * /abc/def will return the following maps:
         * <ol>
         * <li>folder map of /</li>
         * <li>folder map of abc</li>
         * <li>folder map of def (if def is a folder too)</li>
         * </ol>
         * 
         * @param name
         * @throws InvalidNameException
         */
        public FolderMapIterator(String name) throws InvalidNameException
        {
            map_ = root_;

            validateName(name);

            index_ = 0;

            if(name.length() > 1)
            {
                parts_ = name.split("/");
                endIndex_ = parts_.length - 1;

                /*
                 * // cut the last / if the folder ends with /. This is done by //
                 * decrementing the end index, because string operations are //
                 * very // expensive if(name.endsWith("/")) { endIndex_--; }
                 */
            }
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Getting state information
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Returns true if the iterator has handled all folders given by name.
         * If name was a file, this method will return false (because the last
         * item (the pure file name) was not processed). If the folder name does
         * not exist, this method will also return false.
         * 
         * @return true if and only if the name given to the iterator is an
         *         existing folder, false otherwise
         */
        public boolean hasFinished()
        {
            return (index_ > endIndex_);
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementing the Iterator interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see java.util.Iterator */
        public boolean hasNext()
        {
            return (index_ <= endIndex_ && map_ != null);
        }

        /* @see java.util.Iterator */
        public Object next()
        {
            Object object;
            IFolderMap map = map_;

            // Implement the specification of next
            if(!hasNext())
            {
                throw new NoSuchElementException();
            }

            /*
             * This method first returns the value of map_ in map. Then the map
             * for the next call is search and stored in map_. If no such map
             * exists, map_ will become null, so hasNext() will return false the
             * next time.
             */

            // as long as the current index is smaller than the end index, there
            // can be more elements that are folders
            if(index_ < endIndex_)
            {
                // because the root map should be returned by index = 0, and the
                // element at index 0 is empty (root), the next map can be
                // retrieved over the name at index 1
                object = map_.get(parts_[index_ + 1]);

                // if the object at this element is a folder map, the element
                // described a folder name. So store the object for the next
                // call to next()
                if(object instanceof IFolderMap)
                {
                    map_ = (IFolderMap) object;
                    index_++;
                }
                else
                {
                    // the element is a file name or does not exist, there will
                    // be no more element.
                    map_ = null;
                }
            }
            else
            {
                // the index is larger or equal to the end index. This means the
                // map that will now be returned is the last one.
                // Consider you have the folder /abc and iterate over it. The
                // end index will be 1. The first time next will return the root
                // map, set the index to 1 and store the folder mab of abc in
                // map_.
                // The next call to next() will set the map to null because
                // there is no more folder map
                map_ = null;
                index_++;
            }

            return map;
        }

        /* @see java.util.Iterator */
        public void remove()
        {
            throw new UnsupportedOperationException();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new empty file system structure that contains only the root
     * folder /.
     */
    public SimpleFileSystem()
    {
        root_ = new FolderMap(null);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Validation
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Validates the name and throws an exception of the given name is invalid.
     * A valid name must be absolute, and does not contain folders or files with
     * the name '..' or '.'.
     * 
     * @param name
     *            the name to validate
     * @throws InvalidNameException -
     *             if an invalid name was given
     */
    private void validateName(String name) throws InvalidNameException
    {
        // if the name doesn't start with /, we have no absolute name given
        if(!name.startsWith("/"))
        {
            throw new InvalidNameException();
        }

        // the name must not contain .. as folder or file name
        if(name.matches(".*/\\.\\./.*|\\.\\./.*|.*/\\.\\."))
        {
            throw new InvalidNameException();
        }

        // the name must not contain . as folder or file name
        if(name.matches(".*/\\./.*|\\./.*|.*/\\."))
        {
            throw new InvalidNameException();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Returns the folder map of the given name. If the name is a file,or the
     * name does not exists, an exception is thrown.
     * 
     * @param name
     *            the name of the folder to which the folder map should be
     *            retrieved
     * @throws NoSuchFileException -
     *             if such a name does not exists as file or folder
     */
    private IFolderMap getFolderMap(String name) throws NoSuchFileException,
            InvalidNameException
    {
        IFolderMap map = null;
        FolderMapIterator it = new FolderMapIterator(name);

        while (it.hasNext())
        {
            map = (IFolderMap) it.next();
        }

        if(!it.hasFinished())
        {
            throw new NoSuchFileException();
        }

        return map;
    }

    /**
     * Returns the last part of a given name. No check is done, if the name is
     * valid. A trailing slash (/) will be cut. <br>
     * <b>Example:</b> /abc/def/ will return def
     * 
     * @param name
     *            the name from which the last part is searched
     * @return the last part of the given name
     */
    private String getLastPart(String name)
    {
        int index;

        index = name.lastIndexOf('/', name.length() - 2);
        if(index != -1)
        {
            name = name.substring(index + 1);
        }

        if(name.endsWith("/"))
        {
            return name.substring(0, name.length() - 1);
        }

        return name;
    }

    /**
     * Splits a name in its folder and file components. There is no check done,
     * if the file and folders exist. If name ends with /, the file component
     * will be empty. The folder component will never be empty and will alwasy
     * end with /.
     * 
     * @param name
     *            the name to split in folder and file component
     * @return an array with the folder component at index 0, and the file
     *         component at index 1
     */
    private String[] splitName(String name)
    {
        int index;
        String[] result = { "", "" };

        // if name ends with /, the result component is the folder only
        if(name.endsWith("/"))
        {
            result[0] = name;
            return result;
        }

        index = name.lastIndexOf('/');

        // the first part is the folder component
        result[0] = name.substring(0, index + 1);
        // the last part is assumed to be the file component
        result[1] = name.substring(index + 1);

        return result;
    }

    /**
     * Resets the root folder map. The old folder map and file system is lost
     * after a call to this method.
     * 
     * @param root
     *            the root folder map
     */
    protected void setRootMap(IFolderMap root)
    {
        root_ = root;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Folder manipulation
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.filesystem.IFileSystem */
    public void createFolder(String folder) throws FileExistsException,
            InvalidNameException
    {
        int index;
        int endIndex;
        IFolderMap map;
        IFolderMap newMap;
        FolderMapIterator it;
        String[] parts;

        validateName(folder);

        /*
         * The method moves as long as possible through existing folders. After
         * walking, map contains the last existing folder map. If this folder
         * map contains already the name for the next folder, the
         * FileExistsException is thrown. Otherwise the missing folders are
         * created.
         */

        it = new FolderMapIterator(folder);
        parts = folder.split("/");
        endIndex = parts.length - 1;

        // cut the last / if the folder ends with /. This is done by
        // decrementing the end index, because string operations are very
        // expensive
        if(folder.endsWith("/"))
        {
            endIndex--;
        }

        // the index is the first part that has to be created after the loop
        map = null;
        index = 0;
        while (it.hasNext())
        {
            map = (IFolderMap) it.next();
            index++;
        }

        // test if the name is already in use
        if(map.get(parts[index]) != null)
        {
            throw new FileExistsException();
        }

        // create the missing folders
        for (int i = index; i <= endIndex; i++)
        {
            newMap = map.newInstance();
            map.put(parts[i], newMap);

            map = newMap;
        }
    }

    /* @see viptool.filesystem.IFileSystem */
    public void removeFolder(String folder) throws NoSuchFileException,
            InvalidNameException
    {
        IFolderMap map;
        IFolderMap parent;

        validateName(folder);

        // get the folder map
        map = getFolderMap(folder);

        // test if not the root folder was given (/)
        if((parent = map.getParent()) == null)
        {
            throw new InvalidNameException();
        }

        // remove the folder
        parent.remove(getLastPart(folder));
    }

    /* @see viptool.filesystem.IFileSystem */
    public boolean isFolder(String folder)
    {
        try
        {
            getFolderMap(folder);
        }
        catch (FileSystemException exception)
        {
            return false;
        }

        return true;
    }

    /* @see viptool.filesystem.IFileSystem */
    public String[] getFolderContent(String folder) throws NoSuchFileException,
            InvalidNameException
    {
        IFolderMap map = getFolderMap(folder);

        return (String[]) map.keySet().toArray(new String[map.keySet().size()]);
    }

    /* @see viptool.filesystem.IFileSystem */
    public Iterator getFolderIterator(String folder)
            throws NoSuchFileException, InvalidNameException
    {
        return getFolderMap(folder).getStringIterator();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // File manipulation
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.filesystem.IFileSystem */
    public void createFile(String file, Object content)
            throws FileExistsException, InvalidNameException,
            NoSuchFileException, InvalidContentException
    {
        String[] result = splitName(file);
        IFolderMap map = getFolderMap(result[0]);

        if(map.get(result[1]) != null)
        {
            throw new FileExistsException();
        }

        if(content == null)
        {
            throw new InvalidContentException();

        }

        map.put(result[1], content);
    }

    /* @see viptool.filesystem.IFileSystem */
    public void removeFile(String file) throws NoSuchFileException,
            InvalidNameException
    {
        Object object;
        String[] result = splitName(file);
        IFolderMap map = getFolderMap(result[0]);

        object = map.get(result[1]);
        if(object == null || object instanceof IFolderMap)
        {
            throw new NoSuchFileException();
        }

        map.remove(result[1]);
    }

    /* @see viptool.filesystem.IFileSystem */
    public Object getFile(String file) throws NoSuchFileException,
            InvalidNameException
    {
        Object object;
        String[] result = splitName(file);
        IFolderMap map = getFolderMap(result[0]);

        object = map.get(result[1]);
        if(object == null || object instanceof IFolderMap)
        {
            throw new NoSuchFileException();
        }

        return object;
    }

    /* @see viptool.filesystem.IFileSystem */
    public void moveFile(String source, String destination)
            throws NoSuchFileException, FileExistsException,
            InvalidNameException
    {
        try
        {
            createFile(destination, getFile(source));
        }
        catch (InvalidContentException exception)
        {
            // not possible
        }

        removeFile(source);
    }

}
