package viptool.impl.filesystem;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeMap;

import viptool.filesystem.FileSystemEntry;

public class SortedFileSystem extends SimpleFileSystem
{
    private Comparator comparator_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Subclasses
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * This class represents a folder entry in the sorted file system. The entry
     * has an index for sorting purpose.
     */
    private class FolderMapEntry extends FileSystemEntry
    {
        // the index of the entry
        private int index_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Constructs a new folder entry with name, folder information and an
         * index for sorting.
         * 
         * @param name
         *            the name of the entry
         * @param isFolder
         *            flag if the entry is a folder
         * @param index
         *            a sort index
         */
        public FolderMapEntry(String name, boolean isFolder, int index)
        {
            super(name, isFolder);

            index_ = index;
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // getter methods
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Returns the index of the entry.
         * 
         * @return the entry index
         */
        public int getIndex()
        {
            return index_;
        }

        public int hashCode()
        {
            return getName().hashCode();
        }

        public boolean equals(Object obj)
        {
            if(!(obj instanceof FolderMapEntry))
            {
                return false;
            }

            return ((FolderMapEntry) obj).getName().equals(getName());
        }
    }

    /**
     * The folder map is a hash map that can have a parent map. This parent map
     * is the enclosing hash map.
     */
    private class SortedFolderMap extends TreeMap implements IFolderMap
    {
        // the parent hash map
        private IFolderMap parent_;

        // a hashmap for the index saving
        private HashMap    indexMap_;

        // an internal index for sorting
        private int        index_;
        
        /**
         * The folder iterator provides access to a folders content. The next()
         * method will return instances of the class FileSystemEntry.
         */
        private class FolderIterator implements Iterator
        {
            // iterator over the keys from the folder map
            private Iterator it_;

            // <><><><><><><><><><><><><><><><><><><><><>
            // Constructors
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * Construct a new folder iterator that walks through the elements
             * from the given folder.
             */
            public FolderIterator()
            {
                it_ = keySet().iterator();
            }

            // <><><><><><><><><><><><><><><><><><><><><>
            // Implementation of the Iterator interface
            // <><><><><><><><><><><><><><><><><><><><><>

            /* @see java.util.Iterator */
            public boolean hasNext()
            {
                return it_.hasNext();
            }

            /* @see java.util.Iterator */
            public Object next()
            {
                String name;
                FolderMapEntry entry;

                entry = (FolderMapEntry)it_.next();
                name = entry.getName();
                return new FileSystemEntry(name, entry.isFolder());
            }

            /* @see java.util.Iterator */
            public void remove()
            {
                throw new UnsupportedOperationException();
            }
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Constructs a new folder map with the given parent. The parent can be
         * null.
         * 
         * @param parent
         *            the parent of the map or null
         */
        public SortedFolderMap(IFolderMap parent)
        {
            super(comparator_);

            index_ = 0;
            parent_ = parent;
            indexMap_ = new HashMap();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the IFolderMap interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see IFolderMap */
        public IFolderMap getParent()
        {
            return parent_;
        }

        /* @see IFolderMap */
        public IFolderMap newInstance()
        {
            return new SortedFolderMap(this);
        }

        /* @see IFolderMap */
        public String[] getKeys()
        {
            return (String[]) indexMap_.keySet().toArray(
                    new String[indexMap_.keySet().size()]);
        }
        
        /* @see IFolderMap */
        public Iterator getStringIterator()
        {
            return new FolderIterator();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Overridden methods from java.util.TreeMap
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see java.util.TreeMap */
        public Object put(Object key, Object value)
        {
            Object previous = get(key);

            // save the index for the entry. Necessary for getting objects
            indexMap_.put(key, new Integer(index_));
            super.put(new FolderMapEntry((String) key,
                    (value instanceof IFolderMap), index_), value);

            // increment the index for the next one
            index_++;

            return previous;
        }

        public Object get(Object key)
        {
            int index = -1;
            Integer integer;

            if((integer = (Integer) indexMap_.get(key)) != null)
            {
                index = integer.intValue();
            }

            return super.get(new FolderMapEntry((String) key, false, index));
        }
    }

    /**
     * Comparator that compares to folder map entries. One entry is smaller than
     * another one, if its index is smaller.
     */
    private class FolderMapEntryComparator implements Comparator
    {
        /* @see java.util.Comparator */
        public int compare(Object object1, Object object2)
        {
            if(!(object1 instanceof FolderMapEntry)
                    || !(object2 instanceof FolderMapEntry))
            {
                return 0;
            }

            return ((FolderMapEntry) object1).getIndex()
                    - ((FolderMapEntry) object2).getIndex();
        }

    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new empty file system structure that contains only the root
     * folder /.
     */
    public SortedFileSystem()
    {
        super();

        comparator_ = new FolderMapEntryComparator();
        setRootMap(new SortedFolderMap(null));
    }
}
