package viptool.impl.job;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import viptool.base.error.VipToolException;
import viptool.console.IConsole;
import viptool.impl.base.Workbench;
import viptool.impl.logger.Logger;
import viptool.impl.logger.LoggerConsole;
import viptool.impl.logger.StandardLogger;
import viptool.job.IJob;
import viptool.job.IJobScheduler;
import viptool.job.IProgressMonitor;
import viptool.job.error.JobRuntimeException;
import viptool.job.event.IJobSchedulerListener;
import viptool.job.event.JobEvent;
import viptool.logger.ILogger;
import viptool.util.resources.ResourceConstants;

/**
 * This class is the implementation of the job scheduler interface.
 * 
 * @see viptool.job.IJobScheduler
 */
public class JobScheduler implements IJobScheduler
{
    // for knowing the number of jobs, that can run in the background at the
    // same moment
    private int                 maxJobs_              = ResourceConstants.MAX_BACKGROUND_JOBS;

    // the only instance of the JobScheduler in the whole tool
    private static JobScheduler instance_             = null;

    // a map of jobs, that are running at the moment - equal if in the fore- or
    // background
    // key: viptool.job.IJob
    // value: java.lang.Thread
    private HashMap             backgroundRunningJobs_;

    private IJob                foregroundRunningJob_ = null;
    
    // the thread of teh currently foregrund running job
    private Thread				foregroundRunningThread_ = null;

    // a list of jobs, that are queued waiting for their running
    private ArrayList           backgroundWaitingJobs_;

    private ArrayList           foregroundWaitingJobs_;

    // a set of all listeners
    private HashSet             listeners_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This inner class is needed to get an iterator over all jobs known by the
     * job scheduler. An inner class is required because all jobs are stored in
     * different lists, so the iterators of these lists will be logical put
     * together in this class. Therefore, the iterator interface is implemented,
     * the method "remove" will throw an UnsupportedOperationException.
     */
    class JobIterator implements Iterator
    {
        // an iterator over the waiting jobs
        private Iterator waitingJobIterator_;

        // an iterator over the running jobs
        private Iterator runningJobIterator_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // constructor
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Constructs a new JobIterator. Only the iterators of the two
         * arraylists of the surrounding class will be get and saved in class
         * variables so that they are known for further operations.
         */
        private JobIterator()
        {
            // setting the iterators
            waitingJobIterator_ = getWaitingJobIterator();
            runningJobIterator_ = getRunningJobIterator();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // implementation of the iterator interface
        // <><><><><><><><><><><><><><><><><><><><><>
        /* @see java.util.iterator */
        public boolean hasNext()
        {
            // this iterator has further elements while the two iterators have
            // further elements
            if(waitingJobIterator_.hasNext() || runningJobIterator_.hasNext())
            {
                return true;
            }

            return false;
        }

        /* @see java.util.iterator */
        public Object next()
        {
            // if the first list has further elements, return the next
            if(waitingJobIterator_.hasNext())
            {
                return waitingJobIterator_.next();
            }
            // if the first has no further elements, but the second does, return
            // the next element of the second list
            else if(runningJobIterator_.hasNext())
            {
                return runningJobIterator_.next();
            }

            return null;
        }

        /* @see java.util.iterator */
        public void remove()
        {
            // operation has not to be supported at the moment
            throw new UnsupportedOperationException();
        }
    }

    /**
     * 
     */
    class JobThread extends Thread
    {
        //
        private IJob job_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // constructor
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * 
         */
        private JobThread(IJob job)
        {
            job_ = job;
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // overridden method from java.lang.Thread
        // <><><><><><><><><><><><><><><><><><><><><>
        /* @see java.lang.Thread */
        public void run()
        {
            IConsole console;
            IProgressMonitor progressMonitor;
            JobEvent jobEvent;
            int jobEndMode;
            ILogger logger = null;

            console = null;
            if(job_.needLoggerConsole())
            {
                logger = new StandardLogger();
                console = new LoggerConsole(job_.getName(), logger);
                // TODO: can't be closed so we dont open it
                //Workbench.getInstance().openView(console);
            }

            progressMonitor = new ProgressMonitor();
            jobEvent = new JobEvent(job_, console, progressMonitor);
            fireJobStarted(jobEvent);
            try
            {
                jobEndMode = job_.run(logger, progressMonitor);
            }
            catch (JobRuntimeException exception)
            {
                Workbench.getInstance().showExceptionInDialog(exception);
                jobEndMode = IJob.JOB_CANCELLED;
            }

            // testing wether a job has been done all its work or if it has been
            // cancelled. The difference is important for firering the events.
            if(jobEndMode == IJob.JOB_FINISHED)
            {
                fireJobFinished(jobEvent);
            }
            else if(jobEndMode == IJob.JOB_CANCELLED)
            {
                fireJobCancelled(jobEvent);
            }

            startNextJob(job_);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructor
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new JobScheduler. This constructor is private and must not
     * be used outside the "getInstance()" method of this class. This is, why
     * this constructor is private
     */
    private JobScheduler()
    {
        // initialising the class variables
        instance_ = this;
        backgroundRunningJobs_ = new HashMap();
        foregroundWaitingJobs_ = new ArrayList();
        backgroundWaitingJobs_ = new ArrayList();
        listeners_ = new HashSet();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Method has to be used for getting a new job scheduler instead of using
     * the constructor. This garantees, that only one instance of a job
     * scheduler will be active in one tool
     */
    public synchronized static JobScheduler getInstance()
    {
        if(instance_ == null)
        {
            instance_ = new JobScheduler();
        }

        return instance_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementing the IJobScheduler interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.job.IJobScheduler */
    public void addJob(IJob job)
    {
        if(job == null)
        {
            return;
        }

        int mode = job.getMode();

        // testing the job properties to know to which list it has to be added
        if(mode == IJob.MODE_FOREGROUND)
        {
            if(isForegroundUsed())
            {
                foregroundWaitingJobs_.add(job);
                fireNewJobWaiting(mode);
            }
            else
            {
                startJob(job);
            }
        }

        else if(mode == IJob.MODE_BACKGROUND)
        {
            if(getNumberBackgroundWorking() < maxJobs_)
            {
                startJob(job);
            }
            else
            {
                backgroundWaitingJobs_.add(job);
                fireNewJobWaiting(mode);
            }
        }
    }

    
    @SuppressWarnings({"deprecation"})
    /* @see viptool.job.IJobScheduler */
    public void cancelJob(IJob job)
    {
        Thread thread;
        JobEvent event = new JobEvent(job, null, null);

        try
        {
            Logger.getInstance()
                    .log(
                            "Trying to cancel job '" + job.getName()
                        + "' (first try)...");
            
            job.cancel();

            thread = (Thread) backgroundRunningJobs_.get(job);
            if(thread == null)
            {
            	thread = foregroundRunningThread_;
            }
            thread.join(2500);

            if(thread.isAlive())
            {
                Logger.getInstance().log(
                        "Trying to cancel job '" + job.getName()
                                + "' (second try)...");
                thread.interrupt();
                if(thread.isAlive())
                {
                    thread.join(2500);

                    Logger.getInstance().warning(
                            "The job '" + job.getName()
                                    + "' don't want to stop. I kill "
                                    + "the job now!");
                    thread.stop();
                }
            }

            Logger.getInstance().info("Job '" + job.getName() + "' terminated.");
            fireJobCancelled(event);
            startNextJob(job);
        }
        catch (InterruptedException exception)
        {
            Workbench.getInstance().showExceptionInDialog(
                    new VipToolException(exception));
        }
    }

    /* @see viptool.job.IJobScheduler */
    public Iterator getJobIterator()
    {
        return new JobIterator();
    }

    /* @see viptool.job.IJobScheduler */
    public IJob[] getJobs()
    {
        int foregroundRunning;

        if(foregroundRunningJob_ != null)
        {
            foregroundRunning = 1;
        }
        else
        {
            foregroundRunning = 0;
        }

        // initialize an array that will contain the two small arrays of the
        // surrounding class
        IJob[] support = new IJob[foregroundWaitingJobs_.size()
                + backgroundWaitingJobs_.size() + backgroundRunningJobs_.size()
                + foregroundRunning];

        // copying the arrays in the big one
        System
                .arraycopy(getWaitingJobs(), 0, support, 0,
                        (foregroundWaitingJobs_.size() + backgroundWaitingJobs_
                                .size()));
        System.arraycopy(getRunningJobs(), 0, support, (foregroundWaitingJobs_
                .size() + backgroundWaitingJobs_.size()),
                (backgroundRunningJobs_.size() + foregroundRunning));

        return support;
    }

    /* @see viptool.job.IJobScheduler */
    public Iterator getWaitingJobIterator()
    {

        ArrayList waitingJobs = backgroundWaitingJobs_;

        for (int i = 0; i < foregroundWaitingJobs_.size(); i++)
        {

            waitingJobs.add(foregroundWaitingJobs_.get(i));
        }
        return waitingJobs.iterator();
    }

    /* @see viptool.job.IJobScheduler */
    public IJob[] getWaitingJobs()
    {

        IJob[] foreground = new IJob[foregroundWaitingJobs_.size()];
        IJob[] background = new IJob[backgroundWaitingJobs_.size()];
        // initialize an array that will contain the two small arrays of the
        // surrounding class
        IJob[] support = new IJob[foregroundWaitingJobs_.size()
                + backgroundWaitingJobs_.size()];

        foreground = (IJob[]) foregroundWaitingJobs_.toArray(foreground);
        background = (IJob[]) backgroundWaitingJobs_.toArray(background);

        // copying the arrays in the big one
        System.arraycopy(foreground, 0, support, 0, foregroundWaitingJobs_
                .size());
        System.arraycopy(background, 0, support, foregroundWaitingJobs_.size(),
                backgroundWaitingJobs_.size());

        return support;
    }

    /* @see viptool.job.IJobScheduler */
    public Iterator getRunningJobIterator()
    {

        HashMap runningJobs = backgroundRunningJobs_;

        // add the job, that is running in the foreground
        if(foregroundRunningJob_ != null)
        {

            JobThread thread = new JobThread(foregroundRunningJob_);
            runningJobs.put(foregroundRunningJob_, thread);
        }

        return runningJobs.keySet().iterator();
    }

    /* @see viptool.job.IJobScheduler */
    public IJob[] getRunningJobs()
    {

        boolean test = false;
        int size;
        size = backgroundRunningJobs_.size();

        if(foregroundRunningJob_ != null)
        {
            size++;
            test = true;
        }

        IJob[] running = new IJob[size];
        // if running is larger than backgroundRunningJobs_, the last element
        // will
        // be null
        running = (IJob[]) backgroundRunningJobs_.keySet().toArray(running);

        if(test)
        {

            running[size - 1] = foregroundRunningJob_;
        }

        return running;
    }

    /* @see viptool.job.IJobScheduler */
    public int getNumberOfWaitingBackgroundJobs()
    {
        return backgroundWaitingJobs_.size();
    }

    /* @see viptool.job.IJobScheduler */
    public int getNumberOfWaitingForegroundJobs()
    {
        return foregroundWaitingJobs_.size();
    }

    /* @see viptool.job.IJobScheduler */
    public void addJobSchedulerListener(IJobSchedulerListener listener)
    {
        synchronized (listeners_)
        {
            // add listener to the list of listeners
            if(listener != null)
            {
                listeners_.add(listener);
            }
        }
    }

    /* @see viptool.job.IJobScheduler */
    public void removeJobSchedulerListener(IJobSchedulerListener listener)
    {
        synchronized (listeners_)
        {
            // remove listener from the list of listeners
            if(listener != null)
            {
                listeners_.remove(listener);
            }
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // starting jobs
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * 
     * 
     * @param job:
     */
    private void startJob(IJob job)
    {
        int mode = job.getMode();

        JobThread thread;
        thread = new JobThread(job);

        if(mode == IJob.MODE_BACKGROUND)
        {
            backgroundRunningJobs_.put(job, thread);
            backgroundWaitingJobs_.remove(job);
        }

        if(mode == IJob.MODE_FOREGROUND)
        {
            foregroundRunningJob_ = job;
            foregroundWaitingJobs_.remove(job);
            foregroundRunningThread_ = thread;
        }

        thread.start();
    }

    /**
     * 
     * @param job
     */
    private void startNextJob(IJob job)
    {
        IJob nextJob;
        int mode = job.getMode();

        if(mode == IJob.MODE_BACKGROUND)
        {
            backgroundRunningJobs_.remove(job);

            if(backgroundWaitingJobs_.size() > 0)
            {
                nextJob = (IJob) backgroundWaitingJobs_.get(0);
                startJob(nextJob);
            }
        }

        if(mode == IJob.MODE_FOREGROUND)
        {
            foregroundRunningJob_ = null;

            if(foregroundWaitingJobs_.size() > 0)
            {
                nextJob = (IJob) foregroundWaitingJobs_.get(0);
                startJob(nextJob);
            }
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>

    // TODO: we do not need this method anymore, because there is only one
    // foregroundRunningJob_

    /**
     * Tests whether any job is already running in the foreground or not
     * 
     * @return true if any job is running in the foreground, false otherwise
     */
    private boolean isForegroundUsed()
    {
        int mode;
        Iterator runningJobIterator;

        runningJobIterator = getRunningJobIterator();
        while (runningJobIterator.hasNext())
        {
            mode = ((IJob) (runningJobIterator.next())).getMode();

            // method can be excited, if the first job mode is foreground
            if(mode == IJob.MODE_FOREGROUND)
            {
                return true;
            }
        }

        return false;
    }

    // TODO: we do not need this method anymore, because there is only one list
    // of
    // jobs working in the background

    /**
     * Get the number of jobs, that are running in the background at the moment.
     * 
     * @return the number of jobs running in the background
     */
    private int getNumberBackgroundWorking()
    {
        int number = 0;
        int mode;
        Iterator runningJobIterator;

        runningJobIterator = getRunningJobIterator();
        while (runningJobIterator.hasNext())
        {
            mode = ((IJob) (runningJobIterator.next())).getMode();

            // counting the jobs, where the mode is background
            if(mode == IJob.MODE_BACKGROUND)
            {
                number++;
            }
        }

        return number;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementing the Listener pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Notifies the listener, that a job was started. The event object contains
     * additional information about the job.
     * 
     * @param jobEvent
     *            a job event with additional information
     */
    private void fireJobStarted(JobEvent jobEvent)
    {
        // necessary to avoid ConcurrentModificationExceptions
        IJobSchedulerListener[] listeners = (IJobSchedulerListener[]) listeners_
                .toArray(new IJobSchedulerListener[listeners_.size()]);

        for (int i = 0; i < listeners.length; i++)
        {
            listeners[i].jobStarted(jobEvent);
        }
    }

    /**
     * Notifies the listener, that a job was finished. That means, that the job
     * is ready with its work. The event object contains additional information
     * about the job.
     * 
     * @param jobEvent
     *            a job event with additional information
     */
    private void fireJobFinished(JobEvent jobEvent)
    {
        // necessary to avoid ConcurrentModificationExceptions
        IJobSchedulerListener[] listeners = (IJobSchedulerListener[]) listeners_
                .toArray(new IJobSchedulerListener[listeners_.size()]);

        for (int i = 0; i < listeners.length; i++)
        {
            listeners[i].jobFinished(jobEvent);

        }
    }

    /**
     * Notifies the listener, that a job was cancelled by a call to the cancel()
     * method. The event object contains additional information about the job.
     * 
     * @param jobEvent
     *            a job event with additional information
     */
    private void fireJobCancelled(JobEvent jobEvent)
    {
        // necessary to avoid ConcurrentModificationExceptions
        IJobSchedulerListener[] listeners = (IJobSchedulerListener[]) listeners_
                .toArray(new IJobSchedulerListener[listeners_.size()]);

        for (int i = 0; i < listeners.length; i++)
        {
            listeners[i].jobCancelled(jobEvent);
        }

    }

    /**
     * Notifies the listener that a new job has been added to the waiting jobs.
     * The method newJobWaiting() should return an integer representing the
     * job's mode so this integer must be given to this method.
     * 
     * @param mode
     *            the mode of the added job so that the listener knows wether a
     *            foreground or a background job is newly waiting.
     */
    private void fireNewJobWaiting(int mode)
    {
        // necessary to avoid ConcurrentModificationExceptions
        IJobSchedulerListener[] listeners = (IJobSchedulerListener[]) listeners_
                .toArray(new IJobSchedulerListener[listeners_.size()]);

        for (int i = 0; i < listeners.length; i++)
        {
            listeners[i].newJobWaiting(mode);
        }

    }
}
