package viptool.impl.preferences;

import java.awt.Color;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

import viptool.impl.logger.Logger;
import viptool.logger.ILogger;
import viptool.preferences.IPreferenceNode;
import viptool.util.GeneralUtils;

/**
 * This is a class for storing information about preferences. It is possible
 * to write all of them in a HashMap. All preferences that are stored in this 
 * HashMap can be saved in an XML file. It is alse possible to load such 
 * preferences from a correct XML file. The information in this file will be 
 * stored in your HashMap again.
 * A PreferenceNode should belong to the PreferenceStore, that builds the 
 * XML file, as a child. Of course, such a PreferenceNode can have childs, too.
 * So it is possible to create a hierarchical structur of preferences.
 */
public class PreferenceNode implements IPreferenceNode {

	//those strings are needed, when you load an XML file
	private String key_ = null;
	private String value_ = null;
	
	//the node, that belongs to this PreferenceNode in the XML file, that should
	//be loaded
	private Node testedNode_;
	
	//the name of this PreferenceNode
	private String name_;
	
	// Map for all preferences, that are stored
  // Key: name of the preference
  // Value: the value of the preference
	private HashMap preferences_;
	
	//Map for all preferences, that are stored
  // Key: name of the PreferenceNode
  // Value: the preferenceNode
	private HashMap childPreferences_;
	
	// <><><><><><><><><><><><><><><><><><><><><>
	//  Constructor
	// <><><><><><><><><><><><><><><><><><><><><>
	
	/**
    * Creates a new PreferenceNode. The three HashMaps, that will contain its 
    * childs, the belonging names and the preferences of this node, will be 
    * created.  Also its name will be stored. With this name you can identify
    * the PreferenceNode
    * 
    * @param name
    * 	the name of this PreferenceNode.
    */
	public PreferenceNode (String name)  {
		
		preferences_ = new HashMap();
		childPreferences_ = new HashMap();
		
		name_ = name;		
	}
   
   	// <><><><><><><><><><><><><><><><><><><><><>
		//  Private Methods
		// <><><><><><><><><><><><><><><><><><><><><>
   
   /**
    * This methode is needed, when you want to load preferences from an XML file.
    * All information from the XML file belonging to this PreferenceNode will be
    * checked. That means, that all prefernces will be stored in your HashMap 
    * with the correct keys and values. 
    * Also all the childs of this node and their names will be stored in the 
    * corresponding HashMap.
    * 
    * @param node
    * 	the node you want to analyze
    * 	this node should be marked with the preferenceNode tag, otherwise this
    * 	methode won�t work correctly
    */
    private void analyze(Node node)  {
   	
   	//this avoids, that you test too often
   	//otherwise you have to test at least twice as often
   	//"#text" is for example "\n"
   	if (node == null || node.getNodeName().equals("#text"))  {
   		
   		return;
   	}
   	
   	//if this is true, the last pair was not correct because you found
   	//no key, this value should belong to
   	if (node.getNodeName().equals("value") && value_ != null)  {
   		
   		//delete the last value
   		value_ = null;
   	}
   	
   	//test, whether it is a value tag
   	if (node.getNodeName().equals("value"))  {
   		
   		//the value of the preference is the childnode
   		value_ = node.getChildNodes().item(0).getNodeValue();
   	}
   	
   	//if this is true, the last pair was not correct because you found
   	//no value belonging to the last key
   	if (node.getNodeName().equals("preference") && key_ != null)  {
   		
   		//delete the last key
   		key_ = null;
   	}
   	
   	//test, whether it is a preference tag
   	if (node.getNodeName().equals("preference"))  {
   		
   		if (node.getAttributes().getLength() > 0)  {
   			
   			NamedNodeMap nodeMap = node.getAttributes();
   			
   			if (nodeMap != null)  {
   				
   				//you need the item of this node
   				Node id = nodeMap.getNamedItem("id");
   				
   				if (id != null)  {
   					
   					//the key of the preference
   					key_ = id.getNodeValue();
   				}
   			}
   		}
   	}
   	
   	//if you found the two entries, that belong together, you can write it to 
   	//the HashMap
   	if (key_ != null && value_ != null)  {
   		
   		writeString(key_, value_);
   		
   		//the strings must be set to null to find the next pair of preference
   		key_ = null;
   		value_ = null;
   	}
   	
   	//find the childs of this PreferenceNode
   	if (node.getNodeName().equals("preferenceNode") && !node.equals(testedNode_))  {
   		
   		if (node.getAttributes().getLength() > 0)  {
   			
   			NamedNodeMap nodeMap = node.getAttributes();
   			
   			if (nodeMap != null)  {
   				
   				//you need the item of this node
   				Node id = nodeMap.getNamedItem("id");
   				PreferenceNode preferenceNode;
   				String childName;
   				
   				if (id != null)  {
   					
   					//the name of the child
  					childName = id.getNodeValue();
  					System.out.println(""+childName);
  					
  					preferenceNode = new PreferenceNode(childName);
  					
  					addPreferenceChildNode(preferenceNode);
  					
  					preferenceNode.loadPreferences(node);
   				}
   			}
   		}
   	}
   	
   	//if it is a child node of this PreferenceNode you have to load the 
   	//preferences only from its own class but not from this one
   	//otherwise you have those information twice
   	if (node.getNodeName().equals("preferenceNode") && !node.equals(testedNode_))  {
			
			return;
		}
   	
   	//analyze all childnodes of the XML-document recursivly
		if(node.hasChildNodes()) 
		{
			//analyze all childnodes
			for(int i = 0; i < node.getChildNodes().getLength(); i++) {
				
				analyze(node.getChildNodes().item(i));
			}
		}	
   }
   
   /**
    * You will get one object out of the HashMap with all preferences. If there
    * is no object with this key, null will be returned and you will get an
    * information in the console. All objects will be returned as a String.
    * 
    * @param key
    * 	name of the object you want to get
    * @return
    * 	value belonging to this key as a String or null if this key is not 
    * 	correct
    */
   private String getPreference(String key)  {
   	
   	if (preferences_.containsKey(key))  {
   		
   		return (preferences_.get(key)).toString();
   	}
   	else {
   		
   		Logger.getInstance().log(ILogger.DEBUG_LEVEL, 
   				"There is no Object with the key: "+key);
   		return null;
   	}
   }
   
   // <><><><><><><><><><><><><><><><><><><><><>
	 //  Supporting Methods
	 // <><><><><><><><><><><><><><><><><><><><><>
   
   /**
    * You will get the HashMap with all stored references.
    * 
    * @return
    * 	HashMap with references
    */
   public HashMap getPreferencesAsHashMap()  {
   	
   	return preferences_;
   }
   
   /**
    * You will get a Set of all preferences. This Set is created out of the 
    * existing HashMap. The keys and the values of the included objects will be
    * connected with "=" ("key=value").
    * 
    * @return
    * 	a Set of preferences, where key and value are connected
    */
   public Set getPreferencesAsSet()  {
   	
   	return preferences_.entrySet();
   }
   
   /**
    * Delete one preference in your HashMap with the specified key.
    * 
    * @param key
    * 	the name of the object you want to delete.
    */
   public void deletePreference(String key)  {
   	
   	preferences_.remove(key);
   }
   
   /**
    * All entries of the HashMap, that includes the preferences are deleted.
    */
   public void deleteAllPreferences()  {
   	
   	preferences_.clear();
   }
   
   // <><><><><><><><><><><><><><><><><><><><><>
   //  Implementing the IPreferenceNode
   //  interface
   // <><><><><><><><><><><><><><><><><><><><><>
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public String getName()  {
  	 
  	 return name_;
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public void addPreferenceChildNode(IPreferenceNode node)  {
  	 
  	 childPreferences_.put(node.getName(), node);
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public void removePreferenceChildNode(String key)  {
  	 
  	 childPreferences_.remove(key);
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public HashMap getPreferenceChildNodes()  {
  	 
  	 return childPreferences_;
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public IPreferenceNode getPreferenceChildNode(String key)  {
  	 
  	 if (childPreferences_.containsKey(key))  {
    		
    		return (PreferenceNode)(childPreferences_.get(key));
    	}
    	else {
    		
    		//create a new child
    		PreferenceNode newNode = new PreferenceNode(key);
    		
    		addPreferenceChildNode(newNode);
    		
    		Logger.getInstance().log(ILogger.DEBUG_LEVEL, 
    				"There is no HashMap with the key: "+key+
    				", a new PreferenceNode was retuned with this key as name");
    		
    		return newNode;
    	}
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public void writeInt(String key, int value)  {
   	
   	preferences_.put(key, String.valueOf(value));
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public void writeDouble(String key, double value)  {
   	
   	preferences_.put(key, String.valueOf(value));
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public void writeString(String key, String value)  {
   	
   	preferences_.put(key, value);
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public void writeBoolean(String key, boolean value)  {
   	
   	preferences_.put(key, String.valueOf(value));
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public void writeColor(String key, Color value)  {
   	
   	preferences_.put(key, GeneralUtils.getRGBString(value));
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public int readInt(String key, int defaultValue)  {
  	 
  	 String valueString = getPreference(key);
  	 int valueInt;
  	 
  	 try  {
  		 
  		 valueInt = Integer.parseInt(valueString);
  		 return valueInt;
  	 }
  	 catch (NumberFormatException e)  {
  		 
  		 Logger.getInstance().log(ILogger.WARRNING_LEVEL,
				"The key is not belonging to an Integer object");
  		 
  		 return defaultValue;
  	 }
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public double readDouble(String key, double defaultValue)  {
  	 
  	 String valueString = getPreference(key);
  	 double valueDouble;
  	 
  	 try  {
  		 
  		 valueDouble = Double.parseDouble(valueString);
  		 return valueDouble;
  	 }
  	 catch (NumberFormatException e)  {
  		 
  		 Logger.getInstance().log(ILogger.WARRNING_LEVEL,
				"The key is not belonging to a Double object");
  		 
  		 return defaultValue;
  	 }
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public String readString(String key, String defaultValue)  {
  	 
  	 String valueString = getPreference(key);
  	 
  	 if (valueString != null)  {
  		 
  		 return valueString.trim();
  	 }
  	 else  {
  		 
  		 return defaultValue;
  	 }
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public boolean readBoolean(String key, boolean defaultValue)  {
  	 
  	 String valueString = getPreference(key);
  	 boolean valueBoolean;
  	 
  	 try  {
  		 
  		 valueBoolean = Boolean.parseBoolean(valueString);
  		 return valueBoolean;
  	 }
  	 catch (NumberFormatException e)  {
  		 
  		 Logger.getInstance().log(ILogger.WARRNING_LEVEL,
				"The key is not belonging to a Boolean object");
  		 
  		 return defaultValue;
  	 }
   }
   
   /* @see viptool.base.preferences.IPreferenceNode */
   public Color readColor(String key, Color defaultValue)  {
  	 
  	 String valueString = getPreference(key);
  	 Color valueColor;
  	 
  	 try  {
  		 
  		 valueColor = Color.getColor(valueString);
  		 return valueColor;
  	 }
  	 catch (NumberFormatException e)  {
  		 
  		 Logger.getInstance().log(ILogger.WARRNING_LEVEL,
				"The key is not belonging to a Color object");
  		 
  		 return defaultValue;
  	 }
   }
   
   /**
    * This method is basicly called from the PreferenceStore to save information
    * about the preferences in a XML file. All preferences from the 
    * corresponding HashMap will be written to the given document. If this 
    * PreferenceNode has childs, this methode is called for this nodes, too.
    * 
    * @see viptool.base.preferences.IPreferenceNode 
    * 
    * @param document
    * 	you will write all information in this document
    * @param root
    * 	the parent Element of the XML file
    * 	the new information will be appended to this Element
    * @return
    * 	the document, that contains the new information
    */
   public Document save(Document document, Node root)  {
  	 
  	 //the connected String from the HashMap, that should be divided
  	 String toDivide;
  	 //the key of the HashMap entry
  	 String key;
  	 //the value of the HashMap entry
  	 String value;
  	 
  	 Iterator it;
  	 
  	 int index;
			
  	 //key and value of one entry will be connected and listed in an iterator
  	 it = getPreferencesAsSet().iterator();
  	 
  	 //the child node tag
  	 Element childElement = null;
  	 
  	 //add the node element, that contains all preferences
     childElement = document.createElement("preferenceNode");
     childElement.setAttribute("id", name_);
     root.appendChild(childElement);
  	 
  	 while(it.hasNext())  {
  		 
  		 Object obj = it.next();
				
  		 toDivide = obj.toString();
				
  		 //The key and the value of the HashMap entry are connected with "=". 
  		 //So you have to find the index of it to divide the string.
  		 index = toDivide.indexOf("=");
				
  		 key = toDivide.substring(0, index);
				
  		 value = toDivide.substring(index+1);
  		 
  		 //add the preference element, that includes the key
  		 Element keyElement = document.createElement("preference");
  		 keyElement.setAttribute("id", key);
  		 childElement.appendChild(keyElement);
  		 
  		 //add the value element
  		 Element valueElement = document.createElement("value");
  		 keyElement.appendChild(valueElement);
				
  		 //add the value of the preference as text between the value tags
  		 Text valueText = document.createTextNode(value);
  		 valueElement.appendChild(valueText);
  	 }
  	 
  	 //the names of the childs
  	 it = childPreferences_.keySet().iterator();
  	 
  	 //save all information of the childs recursivly to create a hierarchical 
  	 //structure
  	 while(it.hasNext())  {
  		 
  		 //get the child of this node
  		 PreferenceNode child = (PreferenceNode)childPreferences_.get(it.next());
  		 
  		 child.save(document, childElement);
  	 }
  	 
  	 return document;
   }
   
   /**
    * This methode is basicly call from the PreferenceStore to load information 
    * about preferences from an existing XML file. The HashMaps of this
    * PreferenceStore objects will be filled with the information, that belong
    * to this node as childs.
    * 
    * @see viptool.base.preferences.IPreferenceNode 
    * 
    * @param node
    * 	the node, that owns all information
    */
   public void loadPreferences(Node node)   {
  	 
  	 //store the node, so that you get only the rigth information
  	 testedNode_ = node;
  	 
  	 analyze(node);
   }
}
