package viptool.impl.preferences;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import viptool.base.error.XMLException;
import viptool.impl.base.Workbench;
import viptool.impl.logger.Logger;
import viptool.impl.util.XMLUtils;
import viptool.logger.ILogger;
import viptool.preferences.IPreferenceNode;
import viptool.preferences.IPreferenceStore;
import viptool.util.GeneralUtils;

/**
 * This is the basic class for storing all information about preferences. Of
 * course it is possible to load preferences from XML files with the help of
 * this class. In this class you can store PreferenceNodes, that will contain
 * information about preferences. So you can create a hierachical structure,
 * because these nodes can als contain other nodes. This structure will be
 * visible in the saved XML file. In this class the basic information for the
 * XML file are set.
 */
public class PreferenceStore implements IPreferenceStore
{

    // the path of the file, that saves the preferences
    private String                 defaultPath_ = Workbench.getInstance().getPropertyPath()
                                                        + "preferences.xml";

    // the only instance
    private static PreferenceStore instance_    = null;

    // Map for all preferences, that are stored
    // Key: name of the PreferenceNode
    // Value: the PreferenceNode
    private HashMap                childPreferences_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructor
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Creates a new PreferenceStore. There can only exist one. The
     * PreferenceStore can store PreferenceNodes with its preferences in a
     * HashMap. This HashMap will now be created.
     */
    private PreferenceStore()
    {

        instance_ = this;

        childPreferences_ = new HashMap();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * If no PreferenceStore has been created, yet, create one. Otherwise return
     * the created PreferenceStore.
     */
    public synchronized static IPreferenceStore getInstance()
    {
        if (instance_ == null)
        {
            instance_ = new PreferenceStore();
        }

        return instance_;
    }

    /**
     * This methode is needed, when you want to load preferences from an XML
     * file. All the nodes of the XML file will be checked, whether they contain
     * information about childs of this store. In this case, those
     * PreferenceNodes will be created an stored in the HashMap. This methode
     * will be called recursivly to test all interesting nodes of the XML file.
     * 
     * @param node
     *            the node you want to analyze
     */
    private void analyze(Node node)
    {

        // this avoids, that you test too often
        // otherwise you have to test at least twice as often
        // "#text" is for example "\n"
        if (node == null || node.getNodeName().equals("#text"))
        {

            return;
        }

        // test, whether it is a preferenceNode tag
        if (node.getNodeName().equals("preferenceNode"))
        {

            if (node.getAttributes().getLength() > 0)
            {

                NamedNodeMap nodeMap = node.getAttributes();

                if (nodeMap != null)
                {

                    // you need the item of this node
                    Node id = nodeMap.getNamedItem("id");
                    PreferenceNode preferenceNode;
                    String childName;

                    if (id != null)
                    {

                        // the name of the child
                        childName = id.getNodeValue();

                        preferenceNode = new PreferenceNode(childName);
                        addPreferenceChildNode(preferenceNode);

                        preferenceNode.loadPreferences(node);
                    }
                }
            }
        }

        // you only have to save the childs of the preferences tag
        // the other information will be stored at PreferenceNode
        if (node.getNodeName().equals("#document")
                || node.getNodeName().equals("preferences"))
        {

            if (node.hasChildNodes())
            {

                // analyze all childnodes
                for (int i = 0; i < node.getChildNodes().getLength(); i++)
                {

                    analyze(node.getChildNodes().item(i));
                }
            }
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Supporting Methodes
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Set the default path to a new value. This path is used to save and load
     * information in the default case.
     * 
     * @param path
     *            the new default path
     */
    public void setDefaultPath(String path)
    {

        defaultPath_ = path;
    }

    /**
     * Get the default path where files can be save and from where can be
     * loaded.
     * 
     * @return the default path
     */
    public String getDefaultPath()
    {

        return defaultPath_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementing the IPreferenceStore
    // interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.preferences.IPreferenceStore */
    public void addPreferenceChildNode(IPreferenceNode node)
    {

        childPreferences_.put(node.getName(), node);
    }

    /* @see viptool.base.preferences.IPreferenceStore */
    public void removePreferenceChildNode(String key)
    {

        childPreferences_.remove(key);
    }

    /* @see viptool.base.preferences.IPreferenceStore */
    public HashMap getPreferenceChildNodes()
    {

        return childPreferences_;
    }

    /* @see viptool.base.preferences.IPreferenceStore */
    public IPreferenceNode getPreferenceChildNode(String key)
    {

        if (childPreferences_.containsKey(key))
        {

            return (PreferenceNode) (childPreferences_.get(key));
        }
        else
        {

            // create a new child
            PreferenceNode newNode = new PreferenceNode(key);

            addPreferenceChildNode(newNode);

            Logger.getInstance().log(
                    ILogger.DEBUG_LEVEL,
                    "There is no PreferenceNode with the key: "
                            + key
                            + ", a new PreferenceNode was retuned with this key as name");

            return newNode;
        }
    }

    /**
     * This method will store all PreferenceNodes with its information about
     * preferences in a XML file. They will be stored at the default path. The
     * XML file will have a hierachical structure.
     * 
     * @see PreferenceStore#savePreferences(String)
     */
    public void savePreferences()
    {
        savePreferences(null);
    }

    /**
     * This method will store all PreferenceNodes with its information about
     * preferences in a XML file. They will be stored at the given path or at a
     * default path, that also can be set. All those information must be stored
     * in the corresponding HashMaps. The XML file will have a hierachical
     * structure.
     * 
     * The XML file will look like this: <preferences> <preferenceNode
     * id="name of a node"> <preference id="key of the preference">
     * <value>"value"</value> </preference> </preferenceNode> <preferenceNode
     * id="name of a node"> <preference id="key of the preference">
     * <value>"value"</value> </preference> </preferenceNode> <preferenceNode
     * id="name of a node"> <preference id="key of the preference">
     * <value>"value"</value> </preference> </preferenceNode> </preferences>
     * 
     * @param path
     *            the path, where the file will be stored if path = null it will
     *            be stored at the default path, that can also be set
     */
    public void savePreferences(String path)
    {

        if (path == null)
        {

            path = defaultPath_;
        }

        Document document = null;

        try
        {

            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();

            // create a new document, which will contain the xml code
            document = builder.newDocument();
        }

        catch (ParserConfigurationException e)
        {

            // show error dialog
            GeneralUtils.showErrorDialog(e.getMessage(),
                    "Error creating DocumentBuilder!");
        }

        if (document != null)
        {

            // create root
            Element root = document.createElement("preferences");

            // add root to document
            document.appendChild(root);
        }

        Iterator it = childPreferences_.keySet().iterator();

        // save all information of the childs recursivly to create a
        // hierarchical
        // structure
        while (it.hasNext())
        {

            // get the child of this node
            PreferenceNode node = (PreferenceNode) childPreferences_.get(it.next());

            node.save(document, document.getLastChild());
        }

        try
        {
            XMLUtils.storeXMLFile(new File(path).toURI(), document);
        }
        catch (XMLException exception)
        {

            Workbench.getInstance().showExceptionInDialog(exception);
        }

    }

    /**
     * This methode will load preferences out of an existing XML file, that is
     * located at the given path. If the parameter path is set to null, the
     * methode will try to load this file from the default path. All the
     * information from the XML file will be stored in HashMaps of the
     * PrefereceStore and the corresponding PreferenceNodes. If there are keys,
     * that are allready written to this HashMap, this method will delete the
     * old values and set the new one.
     * 
     * @param path
     *            the path, from where the file will be loaded if path = null
     *            the methode will try to load from the default path, that also
     *            can be set
     */
    public void loadPreferences(String path)
    {

        if (path == null)
        {

            path = defaultPath_;
        }

        File resourceFile = new File(path);

        if (resourceFile.exists() && resourceFile.length() > 0)
        {

            // create Parser
            DocumentBuilder builder;
            try
            {

                builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
                Document document = builder.parse(resourceFile);

                // store the element of the XML file in your HashMap
                analyze(document);
            }
            catch (Exception e)
            {

                Logger.getInstance().log(ILogger.ERROR_LEVEL,
                        "Preferencefile not found (" + path + ")!");
            }
        }
        else
        {

            Logger.getInstance().log(
                    ILogger.WARRNING_LEVEL,
                    "Preferencefile not found ("
                            + path
                            + ")! If this is the first time you start VipTool, you can ignore this warning.");
        }
    }
}
