package viptool.impl.project;

import java.awt.Color;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.Icon;
import javax.swing.JOptionPane;
import javax.swing.tree.TreePath;

import viptool.base.action.ContainerAction;
import viptool.base.action.EmptyAction;
import viptool.base.editor.IEditor;
import viptool.base.error.CannotOpenFileException;
import viptool.base.error.EditorNotOpenedException;
import viptool.base.error.FileInterpreterException;
import viptool.base.error.VipToolException;
import viptool.base.event.IWorkbenchListener;
import viptool.base.event.WorkspaceChangeEvent;
import viptool.base.io.IFileInterpreterManager;
import viptool.base.io.IFileInterpreterProvider;
import viptool.base.io.IFileLocker;
import viptool.impl.Viptool;
import viptool.impl.base.MainFrame;
import viptool.impl.base.Workbench;
import viptool.impl.base.action.ActionHandler;
import viptool.impl.base.gui.menu.ContextMenuManager;
import viptool.impl.logger.Logger;
import viptool.impl.project.action.CloseElementAction;
import viptool.impl.project.action.CopyElementAction;
import viptool.impl.project.action.DeleteElementAction;
import viptool.impl.project.action.OpenElementAction;
import viptool.impl.project.action.PasteElementAction;
import viptool.impl.project.action.RenameElementAction;
import viptool.impl.project.action.SaveElementAction;
import viptool.impl.project.gui.ProjectTreeModel;
import viptool.impl.project.gui.ProjectTreeNode;
import viptool.impl.project.gui.ProjectTreePane;
import viptool.impl.project.io.ProjectFile;
import viptool.impl.project.io.ProjectFileProvider;
import viptool.impl.util.resources.ResourceConstants;
import viptool.impl.util.resources.ResourceLoader;
import viptool.project.IAbstractFile;
import viptool.project.IAbstractFolder;
import viptool.project.IProject;
import viptool.project.IProjectGuiLinker;
import viptool.project.IProjectTreeNode;
import viptool.project.event.IAbstractFileListener;
import viptool.project.event.IProjectListener;
import viptool.project.event.IWorkspaceListener;
import viptool.project.event.ProjectFileEvent;
import viptool.project.event.WorkspaceEvent;
import viptool.util.FileUtils;

/**
 * The class ProjectGuiLinker is used to translate the internal
 * project-structure to usable infomation for the tree. It also receives the
 * information from treemenuactions and drag and drop events and translates them
 * into usable information for the project- structure.
 */
public class ProjectGuiLinker implements IProjectListener, IWorkspaceListener,
        IWorkbenchListener, IProjectGuiLinker, IAbstractFileListener
{

    // the hashmap to translate nodes to projectitems
    private HashMap                                  nodeToProject_;

    // the hashmap to translate pojectitems to nodes
    private HashMap                                  projectToNode_;

    // hack
    private HashMap<IAbstractFile, IProjectTreeNode> projectFileToNode_;

    // reference to the fileinterpretermanager
    private IFileInterpreterManager                  fileInterpreterManager_ = null;

    // needed for singleton-pattern
    private static ProjectGuiLinker                  linker_                 = null;

    /**
     * This methode returns the current instance of the ProjectGuiLinker. If
     * there is no instance a new instance will be created and returned.
     * 
     * @return current instance of ProjectGuiLinker
     */
    public static ProjectGuiLinker getInstance()
    {
        if (linker_ == null)
        {
            linker_ = new ProjectGuiLinker();
        }
        return linker_;
    }

    /**
     * The constructor of the ProjectGuiLinker. This methode initialize the
     * necessary variables and registers the new instance as listener for the
     * current workspace.
     */
    private ProjectGuiLinker()
    {
        nodeToProject_ = new HashMap();
        projectToNode_ = new HashMap();
        projectFileToNode_ = new HashMap<IAbstractFile, IProjectTreeNode>();

        fileInterpreterManager_ = Viptool.getInstance().getFileInterpreterManager();

        Workbench.getInstance().addWorkbenchListener(this);

        if (Workbench.getInstance().getCurrentWorkspace() != null)
        {
            Workbench.getInstance().getCurrentWorkspace().addWorkspaceListener(
                    this);
        }

        // register contextmenu action
            this.addContextMenuActions();
//            ContextMenuManager.getInstance().addContextMenu(
//                    ResourceLoader.getInstance().getResource(
//                            ResourceConstants.XML_MENU_PROJECTTREE));
           ContextMenuManager.getInstance().insertContextMenuTemplate();
    }

    /**
     * This methode returns the projectitem which is represented by the given
     * node if it is an instance of the given Class c. If this node isn't an
     * instance of the given class or there is no item for that node null will
     * be returned.
     * 
     * @param node
     *            node which represents the item
     * @param clazz
     *            class of the item
     * @return the item, if there is one and it is from that class
     */
    private Object getItemForNode(IProjectTreeNode node, Class clazz)
    {
        Object obj = nodeToProject_.get(node);
        if (obj != null && clazz.isInstance(obj))
        {
            return obj;
        }
        else
        {
            return null;
        }
    }

    public boolean isOpen(IProjectTreeNode node)
    {
        Object obj = getItemForNode(node, IAbstractFile.class);
        if (obj != null)
        {
            IAbstractFile file = (IAbstractFile) obj;
            IEditor[] editors = Workbench.getInstance().findEditors(file);
            if (editors != null && editors.length > 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }

    public void makeVisible(IAbstractFile file)
    {
        ArrayList pathArray = new ArrayList();
        IProjectTreeNode[] pathNodes;
        IProjectTreeNode node = (IProjectTreeNode) projectToNode_.get(file);

        // create the path to the file
        pathArray.add(node);
        while ((node = (IProjectTreeNode) node.getParent()) != null)
        {
            pathArray.add(node);
        }

        pathNodes = (IProjectTreeNode[]) pathArray.toArray(new IProjectTreeNode[pathArray.size()]);
        // reverse array
        for (int left = 0, right = pathNodes.length - 1; left < right; left++, right--)
        {
            // exchange the first and last
            IProjectTreeNode tmp = pathNodes[left];
            pathNodes[left] = pathNodes[right];
            pathNodes[right] = tmp;
        }

        // expand the item
        TreePath path = new TreePath(pathNodes);
        ProjectTreePane.getInstance().getTree().makeVisible(path);
        ProjectTreePane.getInstance().getTree().setSelectionPath(path);
    }

    public void expandItem(IAbstractFile file)
    {
        ArrayList pathArray = new ArrayList();
        IProjectTreeNode[] pathNodes;
        IProjectTreeNode node = (IProjectTreeNode) projectToNode_.get(file);

        if (node == null)
        {
            // seems to be a project file
            node = projectFileToNode_.get(file);
            if (node == null)
            {
                // element not found
                return;
            }
        }

        // create the path to the file
        pathArray.add(node);
        while ((node = (IProjectTreeNode) node.getParent()) != null)
        {
            pathArray.add(node);
        }

        pathNodes = (IProjectTreeNode[]) pathArray.toArray(new IProjectTreeNode[pathArray.size()]);
        // reverse array
        for (int left = 0, right = pathNodes.length - 1; left < right; left++, right--)
        {
            // exchange the first and last
            IProjectTreeNode tmp = pathNodes[left];
            pathNodes[left] = pathNodes[right];
            pathNodes[right] = tmp;
        }

        // expand the item
        TreePath path = new TreePath(pathNodes);
        ProjectTreePane.getInstance().getTree().expandPath(path);
    }

    /**
     * This methode returns the parent folder of the diskfile this given node
     * represents.
     * 
     * @param node
     *            node from which the parent-folder is serached
     * @return File-Object which represents the parent-folder
     */
    private File getFolderforNode(IProjectTreeNode node)
    {
        if (node.isProject())
        {
            // get project-folder if the parent is the project
            IProject project = (IProject) getItemForNode(node, IProject.class);
            if (project != null)
            {
                return project.getAbstractFile().getFile().getParentFile();
            }
        }
        else if (node.isFolder())
        {
            IAbstractFolder folder = (IAbstractFolder) getItemForNode(node,
                    IAbstractFolder.class);
            if (folder != null)
            {
                return folder.getFile();
            }
        }
        else
        {
            node = (IProjectTreeNode) node.getParent();
            IAbstractFolder folder = (IAbstractFolder) getItemForNode(node,
                    IAbstractFolder.class);
            if (folder != null)
            {
                return folder.getFile();
            }
        }
        return null;
    }

    /**
     * This methode return element which is the parent of element which is
     * represented by the given node.
     * 
     * @param node
     *            node which parent is searchen
     * @return the folder (IAbstractFolder) or the project (IProject) which is
     *         parent of the given node
     */
    private Object getProjectFolderforNode(IProjectTreeNode node)
    {
        if (node.isProject())
        {
            IProject project = (IProject) getItemForNode(node, IProject.class);
            if (project != null)
            {
                return project;
            }
        }
        else if (node.isFolder())
        {
            IAbstractFolder folder = (IAbstractFolder) getItemForNode(node,
                    IAbstractFolder.class);
            if (folder != null)
            {
                return folder;
            }
        }
        else
        {
            node = (IProjectTreeNode) node.getParent();
            IAbstractFolder folder = (IAbstractFolder) getItemForNode(node,
                    IAbstractFolder.class);
            if (folder != null)
            {
                return folder;
            }
        }
        return null;
    }

    /**
     * This methode returns the project of the given node
     * 
     * @param node
     *            node from which the parent is searched
     * @return project to the node
     */
    private IProject getProjectForNode(IProjectTreeNode node)
    {
        while (!node.isProject())
        {
            node = (IProjectTreeNode) node.getParent();
        }

        return (IProject) getItemForNode(node, IProject.class);
    }

    /**
     * This methode returns the state of the element which is represented by the
     * given node. It returns true if the element wasn't changed, false else.
     * 
     * @param node
     * @return state of the node
     */
    public boolean isValid(IProjectTreeNode node)
    {
        IAbstractFile abstractFile = null;
        abstractFile = (IAbstractFile) getItemForNode(node, IAbstractFile.class);
        if (abstractFile != null)
        {
            return !abstractFile.isLocked();
        }
        return true;
    }

    /**
     * This methode returns the icon for the given node.
     * 
     * @param node
     * @return icon for the node
     */
    public Icon getIconForNode(IProjectTreeNode node)
    {
        IAbstractFile file = null;
        file = (IAbstractFile) getItemForNode(node, IAbstractFile.class);
        if (file != null)
        {
            IFileInterpreterProvider provider = fileInterpreterManager_.getDefaultFileInterpreterProvider(file);
            if (provider != null)
            {
                return provider.getIcon();
            }
        }
        return ResourceLoader.getInstance().getIcon(
                ResourceConstants.ICON_DEFAULT_FILE);
    }

    /**
     * This methode copies the diskfile which is represented by the given node
     * to the given target or the parent of the given target if the target isn't
     * a folder. The copied file will be registerd as new element by the
     * project.
     * 
     * @param node
     *            node which file shoud be copied
     * @param target
     *            target for the copy-operation
     * @return true is the file was copied, false else
     */
    public boolean copy(IProjectTreeNode node, IProjectTreeNode target)
    {
        // find target folder
        File targetFile = getFolderforNode(target);
        if (targetFile == null)
        {
            return false;
        }

        // get source item
        IAbstractFile item = null;
        item = (IAbstractFile) getItemForNode(target, IAbstractFile.class);

        // if item was changed and not saved
        if (item.isLocked())
        {
            return false;
        }

        File newFile = new File(targetFile, item.getFile().getName());
        try
        {
            FileUtils.copy(item.getFile(), newFile, false);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            return false;
        }

        Object parent = getProjectFolderforNode(target);
        if (parent == null)
        {
            return false;
        }

        if (parent instanceof IProject)
        {
            ((IProject) parent).updateFiles(false);
        }
        else
        {
            ((IAbstractFolder) parent).update(false);
        }
        return true;
    }

    /**
     * This methode creates a new folder with the given name as child of the
     * given parent. If the parent isn't a folder, the new folder will be
     * created as brother to the given parent.
     * 
     * @param parent
     *            parent of the new folder
     * @param name
     *            name of the new folder
     * @return true if the folder was created, false else
     */
    public boolean createFolder(IProjectTreeNode parent, String name)
    {
        File folder = getFolderforNode(parent);
        if (folder != null)
        {
            File newFolder = new File(folder, name);
            if (newFolder.mkdir())
            {
                parent.update();
                return true;
            }
        }
        return false;
    }

    /**
     * This methode deletes the diskfile of the given node and remove the
     * element form the project.
     * 
     * @param node
     *            node which should be deleted
     * @return true if the node was deleted, false else
     */
    public boolean delete(IProjectTreeNode node)
    {
        IAbstractFile item = (IAbstractFile) getItemForNode(node,
                IAbstractFile.class);

        if (node.isProject())
        {
            IProject project = getProjectToNode(node);
            project.delete();
        }
        else if (item != null)
        {
            if (!((IProjectTreeNode) node.getParent()).isProject())
            {
                item.getParentFolder().remove(item);
            }
            else
            {
                item.getProject().removeFile(item);
            }

            if (item instanceof IAbstractFolder)
            {
                // delete recursivly
                deleteFolder(item.getFile());
            }
            else
            {
                Workbench.getInstance().closeAllEditors(item, true);
                if (!item.getFile().delete())
                {
                    item.getFile().deleteOnExit();
                }
            }
            return true;
        }
        return false;
    }

    /**
     * This methode returns the diskfile as java.io.File which is represented by
     * the given node.
     * 
     * @param node
     * @return file which is represented by the given node
     */
    public File getFile(IProjectTreeNode node)
    {
        if (node.isProject())
        {
            ProjectFile prjfile = (ProjectFile) getItemForNode(node,
                    ProjectFile.class);
            return prjfile.getAbstractFile().getFile().getParentFile();
        }
        else
        {
            IAbstractFile item = (IAbstractFile) getItemForNode(node,
                    IAbstractFile.class);
            if (item != null)
            {
                return item.getFile();
            }
        }
        return null;
    }

    /**
     * This methode returns the abstract file which is represented by the given
     * node.
     * 
     * @param node
     * @return abstract file which is represented by the given node
     */
    public IAbstractFile getAbstractFile(IProjectTreeNode node)
    {

        Object object = nodeToProject_.get(node);

        if (object instanceof IAbstractFile)
        {
            return (IAbstractFile) object;
        }
        else if (object instanceof ProjectFile)
        {
            return ((ProjectFile) object).getAbstractFile();
        }

        return null;
    }

    /**
     * This methode imports the given file, that means the file will be copied
     * into through the parent given folder and the new file will be registerd
     * as element of the project of the given parent.
     * 
     * @param file
     *            diskfile which should be imported
     * @param parent
     *            parentfolder into which the file will be copied
     * @return true if the file was imported, false else
     */
    public boolean importFile(File file, IProjectTreeNode parent)
    {
        File targetFolder = getFolderforNode(parent);
        File targetFile = new File(targetFolder, file.getName());

        if (file == null)
        {
            Workbench.getInstance().showExceptionInDialog(
                    new VipToolException(
                            "File wasn't found or file is a directory!"));
            return false;
        }

        try
        {
            if (file.isDirectory())
            {
            	if(targetFile.getParentFile().getAbsolutePath().startsWith(file.getAbsolutePath())){
        			Logger.getInstance().warning("Recursive copy loop detected! Folder "+file.getName()+" not imported.");
        			return false;
        		}
                FileUtils.recursiveCopy(file, targetFile.getParentFile(),
                        new ProjectFileProvider());
                parent.update();
            }
            else
            {
            	if(targetFile.getAbsolutePath().equals(file.getAbsolutePath())){
            		Logger.getInstance().warning("The file "+file.getName()+" already exists.");
            		
            		int answear = JOptionPane.showOptionDialog(null, 
            				"File already exists. Do you want to import the file with another name?", 
            				"File already exists", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, 
            				null, null, null);
            		if(answear != JOptionPane.YES_OPTION){
            			return false;
            		}
            	}
            	FileUtils.copy(file, targetFile, false);
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            return false;
        }

        if (parent.isLeaf())
        {
            run((IProjectTreeNode) parent.getParent());
        }
        else
        {
            run(parent);
        }

        return true;
    }

    /**
     * This methode moves the given node to the new parent. This means the
     * diskfile which is represented by the given node will be copied into the
     * new folder and the old diskfile will be deleted.
     * 
     * @param node
     *            node which should be moved
     * @param target
     *            new parent for the element
     * @return true if move was successfull, false else
     */
    public boolean move(IProjectTreeNode node, IProjectTreeNode target)
    {
        boolean copy = copy(node, target);
        boolean delete = false;
        if (copy)
        {
            delete = delete(node);
        }
        return (copy && delete);
    }

    /**
     * The methode run runs the given node. This means if the node is a file it
     * will be opened in the defaulteditor, if the node is a folder the folder
     * will be updated.
     * 
     * @param node
     *            node which should be opened/updated
     */
    public void run(IProjectTreeNode node)
    {
        TreePath selectionPath = ProjectTreePane.getInstance().getTree().getSelectionPath();
        if (node.isProject())
        {
            IProject project = (IProject) getItemForNode(node, IProject.class);
            if (project != null)
            {
                project.updateFiles(true);
                if (project.getSize() > 0)
                {
                    // expand the node
                    if (!ProjectTreePane.getInstance().getTree().isExpanded(
                            selectionPath))
                    {
                        ProjectTreePane.getInstance().getTree().expandPath(
                                selectionPath);
                    }
                    else
                    {
                        ProjectTreePane.getInstance().getTree().collapsePath(
                                selectionPath);
                    }
                }
            }
        }
        else if (node.isFolder())
        {
            IAbstractFolder folder = (IAbstractFolder) getItemForNode(node,
                    IAbstractFolder.class);
            if (folder != null)
            {
                folder.update(true);
                if (folder.getSize() > 0)
                {
                    // expand the node
                    if (!ProjectTreePane.getInstance().getTree().isExpanded(
                            selectionPath))
                    {
                        ProjectTreePane.getInstance().getTree().expandPath(
                                selectionPath);
                    }
                    else
                    {
                        ProjectTreePane.getInstance().getTree().collapsePath(
                                selectionPath);
                    }
                }
            }
        }
        else
        {
            IAbstractFile file = (IAbstractFile) getItemForNode(node,
                    IAbstractFile.class);
            if (file != null)
            {
                try
                {
                    Workbench.getInstance().openFile(file);
                }
                catch (CannotOpenFileException exception)
                {
                    Workbench.getInstance().showExceptionInDialog(exception);
                }
            }
        }
    }

    /**
     * This mehtode returns the absolut path of the projectfolder to which the
     * given node belongs to.
     * 
     * @param node
     *            node from which the project is searched
     * @return path of the project-folder
     */
    public String getProjectPath(IProjectTreeNode node)
    {
        while (!node.isProject())
        {
            node = (IProjectTreeNode) node.getParent();
        }

        File folder = getFolderforNode(node);
        if (folder != null)
        {
            return folder.getAbsolutePath();
        }
        return null;
    }

    /**
     * This methode adds the file which is represented by the given node to the
     * blacklist of the project to which the given node belongs to.
     * 
     * @param node
     */
    public void addToBlacklist(IProjectTreeNode node)
    {
        IProject project = getProjectForNode(node);
        IAbstractFile file = (IAbstractFile) getItemForNode(node,
                IAbstractFile.class);
        if (file != null)
        {
            project.addFileToBlacklist(file);
        }
    }

    /**
     * This mehtode sends a close message to all editors which are registert at
     * the abstractfile which is represented by this node. This means all open
     * windows which show this file will be closed.
     * 
     * @param node
     */
    public void close(IProjectTreeNode node)
    {
        IAbstractFile file = (IAbstractFile) getItemForNode(node,
                IAbstractFile.class);
        if (file != null)
        {
            IEditor[] editors = Workbench.getInstance().findEditors(file);
            if (editors != null)
            {
                for (int i = 0; i < editors.length; i++)
                {
                    Workbench.getInstance().closeEditor(editors[i], false);
                }
            }
        }
    }

    /**
     * This methode closes the project to which the given node belongs. This
     * means all open editors which show files from this project will be closed.
     * 
     * @param node
     */
    public void closeProject(IProjectTreeNode node)
    {
        IProject project = getProjectForNode(node);
        closeRecursive(project.getFiles());
    }

    private void closeRecursive(IAbstractFile[] files)
    {
        for (int i = 0; i < files.length; i++)
        {
            if (files[i] instanceof IAbstractFolder)
            {
                IAbstractFile[] childs = ((IAbstractFolder) files[i]).getChilds();
                closeRecursive(childs);
            }
            else
            {
                IEditor[] editors = Workbench.getInstance().findEditors(
                        files[i]);
                for (int e = 0; e < editors.length; e++)
                {
                    Workbench.getInstance().closeEditor(editors[e], false);
                }
            }
        }
    }

    /**
     * This methode calls the editor who has the lock of the file which is
     * represented by the given node that changes should be saved.
     * 
     * @param node
     */
    public void save(IProjectTreeNode node)
    {
        IAbstractFile file = (IAbstractFile) getItemForNode(node,
                IAbstractFile.class);

        // TODO: change to locking editor when implemented
        IEditor[] editors = Workbench.getInstance().findEditors(file);
        if (editors.length > 0)
        {
            try
            {
                editors[0].save();
            }
            catch (EditorNotOpenedException exception)
            {
                Workbench.getInstance().showExceptionInDialog(exception);
            }
        }
    }

    private void deleteFolder(File folder)
    {
        File[] files = folder.listFiles();

        for (int i = 0; i < files.length; i++)
        {
            if (files[i].isFile())
            {
                if (!files[i].delete())
                {
                    files[i].deleteOnExit();
                }
            }
            else if (files[i].isDirectory())
            {
                deleteFolder(files[i]);
            }
        }

        if (!folder.delete())
        {
            folder.deleteOnExit();
        }
    }

    /**
     * This methode saves all changes on all open files which belongs to the
     * project the given node belongs to.
     * 
     * @param node
     */
    public void saveProject(IProjectTreeNode node)
    {
        IProject project = getProjectForNode(node);
        saveRecursive(project.getFiles());
    }

    /**
     * This methode runs recursive through the hole project and calls all open
     * editors that they should save.
     * 
     * @param files
     *            Array of files form project
     */
    private void saveRecursive(IAbstractFile[] files)
    {
        for (int i = 0; i < files.length; i++)
        {
            if (files[i] instanceof IAbstractFolder)
            {
                IAbstractFile[] childs = ((IAbstractFolder) files[i]).getChilds();
                saveRecursive(childs);
            }
            else
            {
                if (files[i].isLocked())
                {
                    try
                    {
                        if (Workbench.getInstance().findLockingEditor(files[i]) != null)
                        {
                            Workbench.getInstance().findLockingEditor(files[i]).save();
                        }
                    }
                    catch (EditorNotOpenedException exception)
                    {
                        Workbench.getInstance().showExceptionInDialog(exception);
                    }
                }
            }
        }
    }

    /**
     * This methode calls the Workspaceloader to save the current workspace.
     * 
     */
    public void saveWorkspace()
    {
        try
        {
            Workbench.getInstance().getCurrentWorkspace().save();
        }
        catch (FileInterpreterException exception)
        {
            Workbench.getInstance().showExceptionInDialog(exception);
        }
    }

    /**
     * This methode retruns the project to which the given node belongs to.
     * 
     * @param node
     *            node form the project
     * @return project to the node
     */
    public IProject getProjectToNode(IProjectTreeNode node)
    {
        return getProjectForNode(node);
    }

    // ---------------------------------------------------------------------------
    // implementation of the project-listener
    // ---------------------------------------------------------------------------

    /**
     * This methode will be called if a new file was added.
     */
    public void fileAdded(ProjectFileEvent event)
    {
        IAbstractFile file = event.getAbstractFile();
        IAbstractFolder folder = event.getAbstractFolder();
        IProject project = event.getProject();

        ProjectTreeNode node = new ProjectTreeNode(file.getName());
        if (file instanceof IAbstractFolder)
        {
            node.setFolder(true);
        }

        projectToNode_.put(file, node);
        nodeToProject_.put(node, file);
        file.addAbstractFileListener(this);

        if (folder.getFile().equals(
                project.getAbstractFile().getFile().getParentFile()))
        {
            ProjectTreeNode projectNode = (ProjectTreeNode) projectToNode_.get(project);
            projectNode.insert(node, 0);
        }
        else
        {
            ProjectTreeNode folderNode = (ProjectTreeNode) projectToNode_.get(folder);
            folderNode.insert(node, 0);
        }
    }

    /**
     * This methode will be called if a file was removed.
     */
    public void fileRemoved(ProjectFileEvent event)
    {
        IAbstractFile file = event.getAbstractFile();
        IAbstractFolder folder = event.getAbstractFolder();
        IProject project = event.getProject();

        ProjectTreeNode node = (ProjectTreeNode) projectToNode_.get(file);

        projectToNode_.remove(file);
        nodeToProject_.remove(node);
        file.removeAbstractFileListener(this);

        if (folder.getFile().equals(
                project.getAbstractFile().getFile().getParentFile()))
        {
            ProjectTreeNode parent = (ProjectTreeNode) projectToNode_.get(project);
            parent.remove(node);
        }
        else
        {
            ProjectTreeNode parent = (ProjectTreeNode) projectToNode_.get(folder);
            parent.remove(node);
        }
    }

    /**
     * This methode will be called if a node was changed.
     */
    public void fileChanged(ProjectFileEvent event)
    {
        Object oNode = null;
        if (event.getAbstractFile() != null)
        {
            oNode = projectToNode_.get(event.getAbstractFile());
        }
        else if (event.getAbstractFolder() != null)
        {
            oNode = projectToNode_.get(event.getAbstractFolder());
        }
        else if (event.getProject() != null)
        {
            oNode = projectToNode_.get(event.getProject());
        }

        if (oNode != null && oNode instanceof ProjectTreeNode)
        {
            ProjectTreeModel model = (ProjectTreeModel) ProjectTreePane.getInstance().getTree().getModel();
            model.updateNode((ProjectTreeNode) oNode);
        }
    }

    /**
     * This methode returns the Project of the selected element at the tree or
     * null if no element is selected
     * 
     * @return IProject selected Project
     */
    public IProject getSelectedProject()
    {
        TreePath path = ProjectTreePane.getInstance().getTree().getSelectionPath();
        if (path != null)
        {
            IProjectTreeNode node = (IProjectTreeNode) path.getLastPathComponent();
            IProject project = getProjectForNode(node);
            return project;
        }
        return null;
    }

    /**
     * This methode returns the Folder of the selected element at the tree or
     * null if no element is selected
     * 
     * @return IProjectFolder selected ProjectFolder
     */
    public IAbstractFolder getSelectedFolder()
    {
        TreePath path = ProjectTreePane.getInstance().getTree().getSelectionPath();
        if (path != null)
        {
            IProjectTreeNode node = (IProjectTreeNode) path.getLastPathComponent();
            Object obj = getProjectFolderforNode(node);
            if (obj instanceof IAbstractFolder)
            {
                return (IAbstractFolder) obj;
            }
            return null;
        }
        return null;
    }

    /**
     * This methode returns the selected element at the tree or null if no
     * element is selected
     * 
     * @return IProject selected Project
     */
    public IAbstractFile getSelectedElement()
    {
        TreePath path = ProjectTreePane.getInstance().getTree().getSelectionPath();
        if (path != null)
        {
            IProjectTreeNode node = (IProjectTreeNode) path.getLastPathComponent();
            IAbstractFile file = (IAbstractFile) getItemForNode(node,
                    IAbstractFile.class);
            return file;
        }
        return null;
    }

    public boolean isProject(IAbstractFile file)
    {
        return (projectToNode_.get(file) != null);
    }

    // ---------------------------------------------------------------------------
    // implementation of the workspace-listener
    // ---------------------------------------------------------------------------

    /**
     * This methode will be called is a project was added.
     */
    public void projectAdded(WorkspaceEvent event)
    {
        IProject project = event.getProject();
        ProjectTreeNode node = new ProjectTreeNode(project.toString(),
                ProjectTreeNode.PROJECT);
        ProjectTreeModel model = (ProjectTreeModel) ProjectTreePane.getInstance().getTree().getModel();

        projectToNode_.put(project, node);
        nodeToProject_.put(node, project);
        projectFileToNode_.put(project.getAbstractFile(), node);

        ((ProjectTreeNode) model.getRoot()).insert(node, 0);

        project.addProjectListener(this);
        project.updateFiles(true);
    }

    /**
     * This methode will be called if a project was removed from the workspace.
     */
    public void projectRemoved(WorkspaceEvent event)
    {
        IProject project = event.getProject();
        ProjectTreeModel model = (ProjectTreeModel) ProjectTreePane.getInstance().getTree().getModel();
        ProjectTreeNode node = (ProjectTreeNode) projectToNode_.get(project);

        model.removeNode(node);

        removeRecursiveElementsFromHashmap(project.getFiles());

        nodeToProject_.remove(node);
        projectToNode_.remove(project);
        projectFileToNode_.remove(project.getAbstractFile());
    }

    /**
     * This methode is used to remove the files which belongs to a remove
     * project from the maps.
     * 
     * @param files
     *            file-array form project
     */
    private void removeRecursiveElementsFromHashmap(IAbstractFile[] files)
    {
        for (int i = 0; i < files.length; i++)
        {
            if (files[i] instanceof IAbstractFolder)
            {
                IAbstractFile[] childs = ((IAbstractFolder) files[i]).getChilds();
                removeRecursiveElementsFromHashmap(childs);

                ProjectTreeNode node = (ProjectTreeNode) projectToNode_.get(files[i]);
                projectToNode_.remove(files[i]);
                nodeToProject_.remove(node);
            }
            else
            {
                ProjectTreeNode node = (ProjectTreeNode) projectToNode_.get(files[i]);
                projectToNode_.remove(files[i]);
                nodeToProject_.remove(node);
            }
        }
    }

    // ---------------------------------------------------------------------------
    // implementation of the workspace-listener
    // ---------------------------------------------------------------------------

    public void workspaceChanged(WorkspaceChangeEvent event)
    {
        ProjectTreePane.getInstance().resetTree(
                event.getCurrentWorkspace().toString());
        event.getCurrentWorkspace().addWorkspaceListener(this);

        try
        {
            event.getCurrentWorkspace().load();
        }
        catch (FileInterpreterException e)
        {
            Workbench.getInstance().showExceptionInDialog(e);
            e.printStackTrace();
            Workbench.getInstance().setCurrentWorkspace(
                    event.getPreviousWorkspace());
        }
    }

    public void fileLocked(IFileLocker locker)
    {
        ProjectTreePane.getInstance().repaint();
        if (locker instanceof IEditor)
        {
            MainFrame.getInstance().getExtendedPane().setForegroundForTab(
                    (IEditor) locker, Color.RED);
        }
    }

    public void fileReleased(IFileLocker locker, boolean modified)
    {
        ProjectTreePane.getInstance().repaint();
        if (locker instanceof IEditor)
        {
            MainFrame.getInstance().getExtendedPane().setForegroundForTab(
                    (IEditor) locker, Color.BLACK);
        }
    }
    
    /**
     * Adds the actions that are necessary to build the contextmenu.
     */
    private void addContextMenuActions() {
    	ContainerAction containerAction;
    	EmptyAction emptyAction;
    	DeleteElementAction deleteElementAction;
    	CloseElementAction closeElementAction;
    	SaveElementAction saveElementAction;
    	OpenElementAction openElementAction;
    	CopyElementAction copyElementAction;
    	PasteElementAction pasteElementAction;
    	RenameElementAction renameElementAction;

    	containerAction = new ContainerAction();
     	containerAction.putValue("Id", "file.openwith");
     	containerAction.putValue("Name", "Open with...");  	
 		ActionHandler.getInstance().addAction(containerAction); 
 		
 		emptyAction = new EmptyAction();
 		emptyAction.putValue("Id", "file.openwith.empty");
 		emptyAction.putValue("Name", "(empy)");  	
 		ActionHandler.getInstance().addAction(emptyAction); 
 		
 		deleteElementAction = new DeleteElementAction();
 		deleteElementAction.putValue("Id", "file.delete");
 		deleteElementAction.putValue("Name", "Delete");  	
 		deleteElementAction.putValue("ShortDescription", "Delete the selected file(s)");
 		deleteElementAction.putValue("LongDescription", "Delete the selected file(s) from the file system");
 		ActionHandler.getInstance().addAction(deleteElementAction); 
 		
 		closeElementAction = new CloseElementAction();
 		closeElementAction.putValue("Id", "cm.file.close");
 		closeElementAction.putValue("Name", "Close");  	
 		closeElementAction.putValue("InheritFrom", "file.close");
 		ActionHandler.getInstance().addAction(closeElementAction); 
 		
 		saveElementAction = new SaveElementAction();
 		saveElementAction.putValue("Id", "cm.file.save");
 		saveElementAction.putValue("Name", "Save");  	
 		saveElementAction.putValue("InheritFrom", "file.save");
 		ActionHandler.getInstance().addAction(saveElementAction); 
 		
 		openElementAction = new OpenElementAction();
 		openElementAction.putValue("Id", "file.open");
 		openElementAction.putValue("Name", "Open");  	
 		openElementAction.putValue("ShortDescription", "Open the selected file(s)");
 		openElementAction.putValue("LongDescription", "Open all selected files with their default editor.");
 		openElementAction.putValue("MnemonicKey", new Integer('O'));
 		ActionHandler.getInstance().addAction(openElementAction); 
 		
 		copyElementAction = new CopyElementAction();
 		copyElementAction.putValue("Id", "file.copy");
 		copyElementAction.putValue("Name", "Copy");  	
 		copyElementAction.putValue("ShortDescription", "Copy selected elements");
 		copyElementAction.putValue("LongDescription", "Copy the selected elements to the system clipboard");
 		copyElementAction.putValue("MnemonicKey", new Integer('C'));
 		ActionHandler.getInstance().addAction(copyElementAction); 
 		
 		pasteElementAction = new PasteElementAction();
 		pasteElementAction.putValue("Id", "file.paste");
 		pasteElementAction.putValue("Name", "Paste");  	
 		pasteElementAction.putValue("ShortDescription", "Paste elements form Clipboard");
 		pasteElementAction.putValue("LongDescription", "Paste all elements from system clipboard");
 		pasteElementAction.putValue("MnemonicKey", new Integer('P'));
 		ActionHandler.getInstance().addAction(pasteElementAction); 
 		
 		renameElementAction = new RenameElementAction();
 		renameElementAction.putValue("Id", "file.rename");
 		renameElementAction.putValue("Name", "Rename");  	
 		renameElementAction.putValue("ShortDescription", "Renames the selected element");
 		renameElementAction.putValue("LongDescription", "Renames the selected element");
 		renameElementAction.putValue("MnemonicKey", new Integer('R'));
 		ActionHandler.getInstance().addAction(renameElementAction); 

    }
}
