package viptool.impl.project.io;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import viptool.base.error.FileInterpreterException;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.io.FileInterpreter;
import viptool.base.io.IFileInterpreterProvider;
import viptool.impl.logger.Logger;
import viptool.impl.project.gui.treedialogs.ProjectDeleteDialog;
import viptool.logger.ILogger;
import viptool.project.FileSystem;
import viptool.project.IAbstractFile;
import viptool.project.IAbstractFolder;
import viptool.project.IProject;
import viptool.project.error.ChildNotAcceptedException;
import viptool.project.event.IAbstractFolderListener;
import viptool.project.event.IProjectListener;
import viptool.project.event.ProjectFileEvent;
import viptool.util.FileUtils;

/**
 * This is the base class for projects in viptool. A project can contain several
 * files and folders.
 */
public class ProjectFile extends FileInterpreter implements IProject,
        IAbstractFolderListener
{
    // set of all childrens
    private Set childrens_;

    // set of the listeners
    private Set listeners_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new project based on the given abstract file. You must
     * explicitly call load to load the project.
     * 
     * @param provider
     *            the provider that created the project
     * @param file
     *            the project file
     */
    public ProjectFile(IFileInterpreterProvider provider, IAbstractFile file)
    {
        super(provider, file);

        childrens_ = new HashSet();
        listeners_ = new HashSet();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Project listener support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Notifies all listeners that a file was added to the project.
     * 
     * @param file
     *            the file that was added
     */
    protected void fireFileAddedEvent(IAbstractFile file)
    {
        Iterator it;
        ProjectFileEvent event = new ProjectFileEvent(this,
                file.getParentFolder(), file);

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IProjectListener) it.next()).fileAdded(event);
        }
    }

    /**
     * Notifies all listeners that a file was removed from the project.
     * 
     * @param file
     *            the file that was removed
     */
    protected void fireFileRemovedEvent(IAbstractFile file)
    {
        Iterator it;
        ProjectFileEvent event = new ProjectFileEvent(this,
                file.getParentFolder(), file);

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IProjectListener) it.next()).fileRemoved(event);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // helper methods for loading and saving the project
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This methode helps to load the project.
     */
    private void analyze(Node node) throws FileInterpreterProviderException
    {
        if (node != null)
        {
            // delete list support not needed at the moment
            // if(node.getNodeName().equals("delete"))
            // {
            // if(node.getAttributes().getLength() > 0)
            // {
            // String path = node.getAttributes().item(0).getNodeValue();
            // File file = new File(path);
            // if(file != null && file.exists())
            // {
            // if(!file.delete())
            // {
            // addToDeletelist(file);
            // }
            // }
            // }
            // }
            // // black list support not implemented
            // else if(node.getNodeName().equals("black"))
            // {
            // if(node.getAttributes().getLength() > 0)
            // {
            // String path = node.getAttributes().item(0).getNodeValue();
            // File file = new File(path);
            // if(file != null && file.exists())
            // {
            // addToBlacklist(file);
            // }
            // }
            // }
            // rekursive analyze whole tree
            if (node.hasChildNodes())
            {
                // analyze all childnodes
                for (int i = 0, items = node.getChildNodes().getLength(); i < items; i++)
                {
                    // analyze node
                    analyze(node.getChildNodes().item(i));
                }
            }
        }

        updateFiles(true);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Overridden methods from Object
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see java.lang.Object */
    public String toString()
    {
        return getAbstractFile().getTitle();
    }

    /* @see java.lang.Object */
    public int hashCode()
    {
        return getAbstractFile().hashCode();
    }

    /* @see java.lang.Object */
    public boolean equals(Object obj)
    {
        if (obj instanceof ProjectFile)
        {
            return getAbstractFile().equals(
                    ((ProjectFile) obj).getAbstractFile());
        }

        return false;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IFileInterpreter interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.io.IFileInterpreter */
    public void load() throws FileInterpreterException
    {
        DocumentBuilder docBuilder;
        Document document;

        try
        {
            // create Parser
            docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            document = docBuilder.parse(getAbstractFile().getFile());

            // analyze document
            analyze(document);
        }
        catch (ParserConfigurationException exception)
        {
            throw new FileInterpreterException(this,
                    "The DOM parser cannot be build!", exception);
        }
        catch (SAXException exception)
        {
            throw new FileInterpreterException(this,
                    "The workspace file cannot be parsed!", exception);
        }
        catch (FileInterpreterProviderException exception)
        {
            throw new FileInterpreterException(this,
                    "A project cannot be loaded!", exception);
        }
        catch (IOException exception)
        {
            throw new FileInterpreterException(this,
                    "The workspace file cannot be parsed!", exception);
        }
    }

    /* @see viptool.base.io.IFileInterpreter */
    public void save() throws FileInterpreterException
    {
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IProject interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.project.IProject */
    public void delete()
    {
        ProjectDeleteDialog.showDialog(this);
    }

    /* @see viptool.base.project.IProject */
    public String getName()
    {
        return getAbstractFile().getTitle();
    }

    /* @see viptool.base.project.IProject */
    public IAbstractFile addFile(IAbstractFile file)
            throws ChildNotAcceptedException
    {
        if (file != null)
        {
            // if the file is not a directory do a test if a project is given.
            // project files must not be added to the project
            if (!file.getFile().isDirectory())
            {
                // don't add hidden files
                if (FileUtils.isHiddenFile(file.getFile()))
                {
                    return null;
                }
            }

            // add the files to the project
            if (childrens_.add(file))
            {
                fireFileAddedEvent(file);

                if (file instanceof IAbstractFolder)
                {
                    ((IAbstractFolder) file).addAbstractFolderListener(this);
                }
            }
            else
            {
                Iterator it;
                IAbstractFile oldFile;

                it = childrens_.iterator();
                while (it.hasNext())
                {
                    oldFile = (IAbstractFile) it.next();
                    if (file.equals(oldFile))
                    {
                        file = oldFile;
                        break;
                    }
                }
            }

        }
        else
        {
            Logger.getInstance().warning(
                    "addFile was called with null as argument! The operation"
                            + "is ignored!");
        }

        return file;
    }

    /* @see viptool.base.project.IProject */
    public void removeFile(IAbstractFile file)
    {
        if (childrens_.remove(file))
        {
            fireFileRemovedEvent(file);

            if (file instanceof IAbstractFolder)
            {
                ((IAbstractFolder) file).removeAbstractFolderListener(this);
            }
        }
    }

    /* @see viptool.base.project.IProject */
    public void addProjectListener(IProjectListener projectListener)
    {
        if (projectListener != null)
        {
            listeners_.add(projectListener);
        }
        else
        {
            // TODO: log event
        }
    }

    /* @see viptool.base.project.IProject */
    public void removeProjectListener(IProjectListener projectListener)
    {
        listeners_.remove(projectListener);
    }

    /* @see viptool.base.project.IProject */
    public Iterator getFileIterator()
    {
        return childrens_.iterator();
    }

    /* @see viptool.base.project.IProject */
    public IAbstractFile[] getFiles()
    {
        return (IAbstractFile[]) childrens_.toArray(new IAbstractFile[childrens_.size()]);
    }

    /* @see viptool.base.project.IProject */
    public IAbstractFile[] getFiles(Class filter)
    {
        HashSet files = new HashSet();
        Iterator it = getFileIterator();
        IAbstractFile file;

        while (it.hasNext())
        {
            file = (IAbstractFile) it.next();
            if (filter.isAssignableFrom(file.getClass()))
            {
                files.add(file);
            }
        }

        return (IAbstractFile[]) files.toArray(new IAbstractFile[files.size()]);
    }

    /* @see viptool.base.project.IProject */
    public int getSize()
    {
        return childrens_.size();
    }

    /* @see viptool.base.project.IProject */
    public void addBlacklistPattern(String pattern)
    {
        // TODO: implement
    }

    /* @see viptool.base.project.IProject */
    public void addFileToBlacklist(IAbstractFile file)
    {
        // TODO: implement
    }

    /* @see viptool.base.project.IProject */
    public boolean isFileInBlacklist(IAbstractFile file)
    {
        return false;
    }

    /* @see viptool.base.project.IProject */
    public void removeBlacklistPattern(String pattern)
    {
        // TODO: implement
    }

    /* @see viptool.base.project.IProject */
    public void removeFileFromBlacklist(IAbstractFile file)
    {
        // TODO: implement
    }

    /* @see viptool.base.project.IProject */
    public IAbstractFile[] updateFiles(boolean recursive)
    {
        int k;
        File[] files = getAbstractFile().getPathFile().listFiles();
        IAbstractFile[] newfiles = new IAbstractFile[files.length];
        IAbstractFile file;

        k = 0;
        for (int i = 0; i < files.length; i++)
        {
            file = null;
            if (files[i].isDirectory())
            {
                file = FileSystem.getInstance().createAbstractFolder(this,
                        files[i]);
                newfiles[k++] = file;
            }
            else if (files[i].isFile())
            {
                file = FileSystem.getInstance().createAbstractFile(this,
                        files[i]);
                newfiles[k++] = file;
            }

            try
            {
                if (file != null)
                {
                    file = addFile(file);

                    // update the subfolder if needed
                    if (recursive && (file instanceof IAbstractFolder))
                    {
                        ((IAbstractFolder) file).update(true);
                    }
                }
            }
            catch (ChildNotAcceptedException exception)
            {
                Logger.getInstance().log(
                        ILogger.WARRNING_LEVEL,
                        "The following file was not accepted by the project: "
                                + exception.getChild().getFullPath());
            }
        }

        return newfiles;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IAbstractFolderListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.project.event.IAbstractFolderListener */
    public void fileAdded(IAbstractFile file)
    {
        if (file instanceof IAbstractFolder)
        {
            ((IAbstractFolder) file).addAbstractFolderListener(this);
        }

        fireFileAddedEvent(file);
    }

    /* @see viptool.base.project.event.IAbstractFolderListener */
    public void fileRemoved(IAbstractFile file)
    {
        if (file instanceof IAbstractFolder)
        {
            ((IAbstractFolder) file).removeAbstractFolderListener(this);
        }

        fireFileRemovedEvent(file);
    }
}
