package viptool.impl.project.io;

import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import viptool.base.IWorkbench;
import viptool.base.editor.IEditor;
import viptool.base.editor.IEditorProvider;
import viptool.base.error.CannotOpenFileException;
import viptool.base.error.EditorNotOpenedException;
import viptool.base.error.FileInterpreterException;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.error.XMLException;
import viptool.base.io.FileInterpreter;
import viptool.base.io.IFileInterpreterProvider;
import viptool.impl.base.MainFrame;
import viptool.impl.base.Workbench;
import viptool.impl.base.io.FileInterpreterManager;
import viptool.impl.extension.extensionloader.ExtensionLoader;
import viptool.impl.logger.Logger;
import viptool.impl.project.ProjectGuiLinker;
import viptool.impl.project.gui.ProjectTreePane;
import viptool.impl.util.XMLUtils;
import viptool.logger.ILogger;
import viptool.project.FileSystem;
import viptool.project.IAbstractFile;
import viptool.project.IProject;
import viptool.project.IProjectTreeNode;
import viptool.project.IWorkspace;
import viptool.project.event.IWorkspaceListener;
import viptool.project.event.WorkspaceEvent;

/**
 * This is the base class for a workspace in viptool. A workspace can contain
 * several projects.
 */
public class WorkspaceFile extends FileInterpreter implements IWorkspace
{
    // set of all projects
    private Set projects_;

    // set of the listeners
    private Set listeners_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new workspace based on the given abstract file. You must
     * explicitly call load to load the workspace.
     * 
     * @param provider
     *            the provider that created the workspace
     * @param file
     *            the workspace file
     */
    public WorkspaceFile(IFileInterpreterProvider provider, IAbstractFile file)
    {
        super(provider, file);

        projects_ = new HashSet();
        listeners_ = new HashSet();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supported section
    // <><><><><><><><><><><><><><><><><><><><><>
    private void safeSave()
    {
        // save workspace
        try
        {
            save();
        }
        catch (FileInterpreterException exception)
        {
            Logger.getInstance().warning(
                    "The workspace file '" + getAbstractFile().getFullPath()
                            + "' couldn't be saved!. " + exception.getMessage());
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Workspace listener support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Notifies all listeners that a project was added to the workspace.
     * 
     * @param project
     *            the project that was added
     */
    protected void fireProjectAddedEvent(IProject project)
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IWorkspaceListener) it.next()).projectAdded(new WorkspaceEvent(
                    this, project));
        }
    }

    /**
     * Notifies all listeners that a project was removed from the workspace.
     * 
     * @param project
     *            the project that was removed
     */
    protected void fireProjectRemovedEvent(IProject project)
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IWorkspaceListener) it.next()).projectRemoved(new WorkspaceEvent(
                    this, project));
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // helper methods for loading and saving the workspace
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Analyze a node and sets the appropriate workspace settings.
     * 
     * @param node
     *            the node to analyze
     */
    private void analyze(Node node) throws FileInterpreterProviderException
    {
        String path;
        String clazz;
        IAbstractFile file;
        IProject project;
        Element element;
        IEditorProvider provider;
        IFileInterpreterProvider[] providers;

        if (node != null)
        {

            if (node.getNodeName().equals("project"))
            {
                try
                {
                    if (node.getAttributes().getLength() > 0)
                    {
                        path = node.getAttributes().item(0).getNodeValue();

                        file = FileSystem.getInstance().createAbstractFile(path);
                        if (file.getFile().exists())
                        {
                            project = (IProject) FileInterpreterManager.getInstance().getDefaultFileInterpreterProvider(
                                    file).createFromFile(file);

                            addProject(project);
                        }
                    }
                }
                catch (NullPointerException exception)
                {
                    // ignore. probably no provider was found
                }
            }
            else if (node.getNodeName().equals("editor"))
            {
                // open editors again
                element = (Element) node;

                path = element.getAttribute("path");
                clazz = element.getAttribute("class");

                file = FileSystem.getInstance().createAbstractFile(path);

                try
                {
                    if (clazz.length() > 0)
                    {
                        provider = Workbench.getInstance().getEditorManager().getEditorProvider(
                                ExtensionLoader.getInstance().getClass(clazz));

                        if (provider != null)
                        {
                            providers = provider.getAbstractFileProviders();
                            for (int i = 0; i < providers.length; i++)
                            {
                                if (providers[i].matchFormat(file))
                                {
                                    Workbench.getInstance().openFile(file,
                                            providers[i], provider);
                                    return;
                                }

                            }
                        }
                    }

                    Workbench.getInstance().openFile(
                            FileSystem.getInstance().createAbstractFile(path));
                    return;
                }
                catch (CannotOpenFileException exception)
                {
                    // ignore
                }
                catch (FileInterpreterProviderException exception)
                {
                    // ignore
                }
            }
            else if (node.getNodeName().equals("node"))
            {
                // expand items
                element = (Element) node;

                path = element.getAttribute("file");
                file = FileSystem.getInstance().createAbstractFile(path);

                ProjectGuiLinker.getInstance().expandItem(file);
            }
            else if (node.getNodeName().equals("mainframe"))
            {
                int state = Frame.NORMAL;
                Rectangle rc = new Rectangle();
                element = (Element) node;

                try
                {
                    rc.x = Integer.parseInt(element.getAttribute("left"));
                    rc.y = Integer.parseInt(element.getAttribute("top"));
                    rc.width = Integer.parseInt(element.getAttribute("width"));
                    rc.height = Integer.parseInt(element.getAttribute("height"));

                    state = Integer.parseInt(element.getAttribute("state"));
                }
                catch (NumberFormatException exception)
                {
                    rc.x = (Toolkit.getDefaultToolkit().getScreenSize().width - 1000) / 2;
                    rc.y = (Toolkit.getDefaultToolkit().getScreenSize().height - 535) / 2;
                    rc.width = 1000;
                    rc.height = 535;
                }

                MainFrame.getInstance().setBounds(rc);
                MainFrame.getInstance().setExtendedState(state);
            }
            else if (node.getNodeName().equals("extendedPane"))
            {
                MainFrame.getInstance().layoutMainFrame((Element) node);
            }
            else if (node.hasChildNodes())
            {
                // analyze all childnodes
                for (int i = 0, items = node.getChildNodes().getLength(); i < items; i++)
                {
                    // analyze node
                    analyze(node.getChildNodes().item(i));
                }
            }
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Overridden methods from Object
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see java.lang.Object */
    public String toString()
    {
        return getAbstractFile().getTitle();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IFileInterpreter interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.io.IFileInterpreter */
    public void load() throws FileInterpreterException
    {
        DocumentBuilder docBuilder;
        Document document;

        try
        {
            // create Parser
            docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            document = docBuilder.parse(getAbstractFile().getFile());

            // analyze document
            analyze(document);
        }
        catch (ParserConfigurationException exception)
        {
            throw new FileInterpreterException(this,
                    "The DOM parser cannot be build!", exception);
        }
        catch (SAXException exception)
        {
            throw new FileInterpreterException(this,
                    "The workspace file cannot be parsed!", exception);
        }
        catch (FileInterpreterProviderException exception)
        {
            throw new FileInterpreterException(this,
                    "A project cannot be loaded!", exception);
        }
        catch (IOException exception)
        {
            throw new FileInterpreterException(this,
                    "The workspace file cannot be parsed!", exception);
        }
    }

    /* @see viptool.base.io.IFileInterpreter */
    public void save() throws FileInterpreterException
    {
        Iterator it;
        Element root;
        Element element;
        Element editorElement;
        Element treeElement;
        IProject project;
        IWorkbench workbench;
        IEditor[] editors;
        List<IProjectTreeNode> nodes;
        Document document = null;
        DocumentBuilder docBuilder;

        // create the DOM tree
        try
        {
            // initialize DocumentBuilder
            docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();

            // create Document
            document = docBuilder.newDocument();

            // create root
            root = document.createElement("workspace");
            // add root to document
            document.appendChild(root);

            // add projects
            it = getProjectIterator();
            while (it.hasNext())
            {
                element = document.createElement("project");
                project = (IProject) it.next();
                element.setAttribute("path",
                        project.getAbstractFile().getFullPath());

                root.appendChild(element);
            }

            // add open editors
            editorElement = document.createElement("editors");
            root.appendChild(editorElement);

            workbench = Workbench.getInstance();
            editors = workbench.getOpenEditors();

            for (int i = 0; i < editors.length; i++)
            {
                element = document.createElement("editor");
                element.setAttribute("path",
                        editors[i].getAbstractFile().getFullPath());
                element.setAttribute("class",
                        editors[i].getProvider().getClass().getCanonicalName());

                editorElement.appendChild(element);
            }

            // add expanded tree elements
            treeElement = document.createElement("expandedNodes");
            root.appendChild(treeElement);

            nodes = ProjectTreePane.getInstance().getTreeModel().getExpandedNodes(
                    null);
            for (IProjectTreeNode node : nodes)
            {
                element = document.createElement("node");
                treeElement.appendChild(element);

                element.setAttribute(
                        "file",
                        ProjectGuiLinker.getInstance().getAbstractFile(node).getFullPath());
            }

            // store expanded pane layout
            MainFrame.getInstance().getExtendedPane().storeLayoutAsXML(
                    document, root);

            // store the main frame size
            element = document.createElement("mainframe");
            element.setAttribute("left",
                    String.valueOf(MainFrame.getInstance().getLocation().x));
            element.setAttribute("top",
                    String.valueOf(MainFrame.getInstance().getLocation().y));
            element.setAttribute("width",
                    String.valueOf(MainFrame.getInstance().getWidth()));
            element.setAttribute("height",
                    String.valueOf(MainFrame.getInstance().getHeight()));

            // store the window state
            element.setAttribute(
                    "state",
                    String.valueOf(MainFrame.getInstance().getWindow().getExtendedState()));

            root.appendChild(element);
        }
        catch (EditorNotOpenedException exception)
        {
            throw new FileInterpreterException(this,
                    "Cannot create the DOM builder!", exception);
        }
        catch (ParserConfigurationException exception)
        {
            throw new FileInterpreterException(this,
                    "Cannot create the DOM builder!", exception);
        }

        // write DOM tree to the file
        try
        {
            XMLUtils.storeXMLFile(getAbstractFile().getFile().toURI(), document);
        }
        catch (XMLException exception)
        {
            throw new FileInterpreterException(this, exception);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IWorkspace interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.project.IWorkspace */
    public void addProject(IProject project)
    {
        if (project != null)
        {
            if (projects_.add(project))
            {
                fireProjectAddedEvent(project);
            }
        }
        else
        {
            Logger.getInstance().log(
                    ILogger.WARRNING_LEVEL,
                    "addProject was called with null as argument! "
                            + "The operation is ignored!");
        }

        // safeSave();
    }

    /* @see viptool.base.project.IWorkspace */
    public void removeProject(IProject project)
    {
        if (projects_.remove(project))
        {
            fireProjectRemovedEvent(project);
        }

        safeSave();
    }

    /* @see viptool.base.project.IWorkspace */
    public void addWorkspaceListener(IWorkspaceListener workspaceListener)
    {
        if (workspaceListener != null)
        {
            listeners_.add(workspaceListener);
        }
        else
        {
            Logger.getInstance().log(
                    ILogger.WARRNING_LEVEL,
                    "addWorkspaceListener was called with null as argument! "
                            + "The operation is ignored!");
        }
    }

    /* @see viptool.base.project.IWorkspace */
    public void removeWorkspaceListener(IWorkspaceListener workspaceListener)
    {
        listeners_.remove(workspaceListener);
    }

    /* @see viptool.base.project.IWorkspace */
    public Iterator getProjectIterator()
    {
        return projects_.iterator();
    }

    /* @see viptool.base.project.IWorkspace */
    public IProject[] getProjects()
    {
        return (IProject[]) projects_.toArray(new IProject[projects_.size()]);
    }

    /* @see viptool.base.project.IWorkspace */
    public int getSize()
    {
        return projects_.size();
    }

    public void update()
    {
        Iterator it;
        IProject project;

        it = projects_.iterator();
        while (it.hasNext())
        {
            project = (IProject) it.next();
            project.updateFiles(true);
        }
    }
}
