package viptool.impl.util;

import java.awt.Component;
import java.awt.Container;
import java.util.HashMap;

import javax.swing.AbstractButton;
import javax.swing.JMenu;

/**
 * This class provides some functionality to manage components in a container
 * like in a file system. It allows to add components dynamicly at any position.
 * It uses a string that has the layout of a file path to position the item.
 * Every component (like a file in the file system) by insertComponent.
 * Containers that contains subelements must be registered by registerContainer.
 * <br>
 * Example:<br>
 * In a menu you can add the file menu by calling insertComponent("File", 0,
 * container). Then you have to register by calling registerContainer("File",
 * fileContainer). After that you can add the 'new' command by calling
 * insertComponent("File/", 0, newCommand). To add an 'open' item, call
 * insertComponent("File/New", 1, openCommand). See the detailed methods
 * descriptions too.
 */
public class ComponentManager
{
    // maps every path to its container. The map hosts only containers not
    // components
    // Key: java.lang.String
    // Value: java.awt.Container
    private HashMap   containers_;

    // Main Container
    private Container container_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructor
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new ComponentManager object to manages components like in a
     * file system.
     */
    public ComponentManager(Container container)
    {
        containers_ = new HashMap();
        container_ = container;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Container manipulation
    // <><><><><><><><><><><><><><><><><><><><><>
    public void registerContainer(String path, Container container)
    {
        if(!path.endsWith("/"))
        {
            path += "/";
        }
        
        containers_.put(path, container);
    }

    /**
     * Returns the container specified by the path. If the path points to a
     * component (does not end with /), the container of the component will be
     * returned. If the container cannot be found, null is returned.
     * 
     * @param path
     *            specifies which parent path is to search or null to get the
     *            main container
     * 
     * @return the parent container of the given path or null if the container
     *         was not found
     */
    public Container getContainer(String path)
    {
        if(path == null || getParentPath(path).length() == 0)
            return container_;
        
        if(!path.endsWith("/"))
        {
            path = getParentPath(path) + "/";
        }
        
        return (Container) containers_.get(path);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Component manipulation
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Inserts a component. The path and seek argument specifies the position of
     * the added component.
     * 
     * @param path
     *            a path where to add the component
     * @param seek
     *            a value how long to move from path before adding the
     *            component. Can be negative to move backward
     * @param component
     *            the component to add
     */
    public void insertComponent(String path, int seek, Component component)
    {
        int insert;
        Container parent = getContainer(path);

        if(parent == null)
            return;
        
        if(parent instanceof JMenu)
        {
            if(seek != 0)
            {
                insert = getComponentPosition(path, ((JMenu) parent)
                    .getMenuComponents());
            }
            else
            {
                insert = ((JMenu) parent).getMenuComponentCount(); 
            }
        }
        else
        {
            if(seek != 0)
            {
                insert = getComponentPosition(path, parent.getComponents());
            }
            else
            {
                insert = parent.getComponentCount();
            }
        }
        
        parent.add(component, insert + seek);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // request items
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * method has to be used if a Component registered in this manager is
     * requested.
     * 
     * @param path
     *            the path the wanted Component is registered in
     * 
     * @return the Component at the given path
     */
    public Component getComponent(String path)
    {
        Container container = getContainer(path);
        return container.getComponent(getComponentPosition(path, container
                .getComponents()));
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Path manipulation
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Extracts the name of the parent from the given path.
     * 
     * @param childPath
     *            a path to a component
     * 
     * @return path of the parent
     */
    public String getParentPath(String childPath)
    {
        String parent;
        int lastDivider = childPath.lastIndexOf('/');

        if(lastDivider >= 0)
            parent = childPath.substring(0, lastDivider);
        else
            parent = "";

        return parent;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the position of a component in a list of components.
     * 
     * @param path
     *            path of the component
     * @param components
     *            list of components in which to search the component
     * 
     * @return the index of the component or -1 if the component was not found
     */
    public int getComponentPosition(String path, Component[] components)
    {
        int insert;

        if(path.endsWith("/"))
        {
            return 0;
        }

        for (insert = 0; insert < components.length; insert++)
        {
            if(compareComponents(path, components[insert]))
            {
                return insert;
            }
        }

        return -1;
    }
    
    protected boolean compareComponents(String path, Component component)
    {
        if(path.equals(component.getName()))
        {
            return true;
        }

        if(component instanceof AbstractButton)
        {
            if(((AbstractButton) component).getText().equals(path))
            {
                return true;
            }
        }
        
        return false;
    }
}
