package viptool.job;

import viptool.job.error.JobRuntimeException;
import viptool.logger.ILogger;

/**
 * This is the interface for all jobs in viptool. Jobs are executed by the
 * IJobScheduler. Every job is running in is own thread. Jobs have to running
 * properties: priority and mode.<br>
 * The priority determines the priority of the thread. The mode is used by the
 * viptool: Only one foreground job can run at a time, but many background jobs
 * can ran simultaneous. It is also possible to run one foreground job while a
 * background job is running.<br>
 */
public interface IJob
{
    /** low priority flag for a job */
    public static final int PRIORITY_LOW    = 0;

    /** default priority flag for a job */
    public static final int PRIORITY_NORMAL = 1;

    /** high priority flag for a job */
    public static final int PRIORITY_HIGH   = 2;

    /** foreground mode for a job. Only one foreground job can run at a time */
    public static final int MODE_FOREGROUND = 0;

    /** background mode for a job. Multiple background jobs can run at a time */
    public static final int MODE_BACKGROUND = 1;

    /** flag that represents that a job has finished without beeing cancelled */
    public static final int JOB_FINISHED    = 0;

    /** flag that represents that a job could not finish and has been cancelled */
    public static final int JOB_CANCELLED   = 1;

    // <><><><><><><><><><><><><><><><><><><><><>
    // executing methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Starts the job. The run method is called in a own thread. Over the given
     * progress monitor the job can tell the viptool the progress of its work.<br>
     * It also can use the console to output user informations. The console is
     * only availabel if needConsole has retrurned true by a prior call.<br>
     * Remind that a call to cancel() of the job of this run method has to end
     * this run method as soon as possible!
     * 
     * @param logger
     *            a logger that is connected with a console for this job
     * @param monitor
     *            a progress monitor that should be used by the job to indicate
     *            its progress while working
     * 
     * @return has to be one of the final variables JOB_* defined in this class
     */
    public int run(ILogger logger, IProgressMonitor monitor)
            throws JobRuntimeException;

    /**
     * This method is called if the job has to be cancelled. The job should
     * rollback all changes he has made and then end up gracefully.
     */
    public void cancel();

    // <><><><><><><><><><><><><><><><><><><><><>
    // getter methods for job properties
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the name of the job. The name is displayed in the progress
     * window.
     * 
     * @return the name of the job
     */
    public String getName();

    /**
     * Returns a detailed description of what the job is doing.
     * 
     * @return a description of the job
     */
    public String getDescription();

    /**
     * Retunrs the mode of the job. This must be one of the MODE_... flags
     * defined in the IJob interface.
     * 
     * @return the running mode of the job
     */
    public int getMode();

    /**
     * Returns the priority of the job. This must be one of the PRIORITY_...
     * flags defined in the IJob interface.
     * 
     * @return the priority for the job
     */
    public int getPriority();

    /**
     * Returns true if the job want to have a logger console for output at
     * runtime. If there is no need for a console, false should be returned. If
     * true, run will get an instance of a logger
     * 
     * @return true if the job wants to have a logger console, false otherwise
     */
    public boolean needLoggerConsole();
}
