package viptool.job;

import java.util.Iterator;

import viptool.job.event.IJobSchedulerListener;

/**
 * This is the interface for the JobScheduler. The JobScheduler manages and
 * executes all jobs. It also controls the number of jobs, that are executed.
 * Remark that only on job can be running in the foreground, whereas background
 * jobs can be running parallel.
 */
public interface IJobScheduler
  {
    /**
     * Add a job to the scheduler. Remind to fire an event, if the job has been
     * added to the background waiting jobs.
     * 
     * @param job
     *          the job you want to add
     */
    public void addJob(IJob job);

    /**
     * this method will invoke the cancel() method of the given job. The job 
     * will be ended by this method of the job and deleted of all lists in the
     * scheduler when the run method will end (that it ends as soon as possible
     * has also to be managed by the cancel() method of the job).  
     * 
     * @param job
     *          the job that should be cancelled
     */
    public void cancelJob(IJob job);

    /**
     * method can be used to get an interator with which the list of all known
     * jobs can be iterated
     * 
     * @return an iterator over all known jobs
     */
    public Iterator getJobIterator();

    /**
     * Get all jobs the scheduler knows listed in an array.
     * 
     * @return all jobs in an array
     */
    public IJob[] getJobs();

    /**
     * Get all jobs, that are working. They are listed in an iterator.
     * 
     * @return all working jobs in an iterator
     */
    public Iterator getRunningJobIterator();

    /**
     * Get all jobs, that are working. They are listed in an array.
     * 
     * @return all working jobs in an array
     */
    public IJob[] getRunningJobs();

    /**
     * Get all jobs, that are waiting. They are listed in an iterator.
     * 
     * @return all waiting jobs in an iterator
     */
    public Iterator getWaitingJobIterator();

    /**
     * Get all jobs, that are waiting. They are listed in an array.
     * 
     * @return all waiting jobs in an array
     */
    public IJob[] getWaitingJobs();

    /**
     * Method returns the number of waiting background jobs, e.g. for giving
     * the possibility of informing the user about this number.
     * 
     * @return the number of waiting backgroud jobs
     */
    public int getNumberOfWaitingBackgroundJobs();    
    
    /**
     * Method returns the number of waiting foreground jobs, e.g. for giving
     * the possibility of informing the user about this number.
     * 
     * @return the number of waiting foreground jobs
     */
    public int getNumberOfWaitingForegroundJobs();
    
    /**
     * Add one more JobSchedulerListener. If the listener is null, nothing will
     * happen.
     * 
     * @param listener
     *          the listener, that should be added
     */
    public void addJobSchedulerListener(IJobSchedulerListener listener);

    /**
     * Remove a JobSchedulerListener.
     * 
     * @param listener
     *          the listener, that should be removed
     */
    public void removeJobSchedulerListener(IJobSchedulerListener listener);
  }
