package viptool.job.event;

import java.util.Date;
import java.util.EventObject;

import viptool.console.IConsole;
import viptool.job.IJob;
import viptool.job.IProgressMonitor;

/**
 * This class contains additional information about a job that has fired an
 * event. At the moment only the timestamp is availabel.
 */
public class JobEvent extends EventObject
{
    // the timestamp when the event has occurred
    private Date timestamp_;
    // the console that belongs to the underlying job - might be null
    private IConsole console_;
    // the progressMonitor displaying the progress of the underlying job
    private IProgressMonitor progressMonitor_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new job event object that contains additional information.
     * 
     * @param job
     *            the job that has sent the event
     * @param console
     *            the console that belongs to the underlying job - might be null
     *            if needConsole() from job returns false
     * @param progressMonitor
     *            the progressMonitor belonging to the underlying job
     */
    public JobEvent(IJob job, IConsole console, IProgressMonitor progressMonitor)
    {
        super(job);
        console_ = console;
        progressMonitor_ = progressMonitor;
        timestamp_ = new Date();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // getter methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * method returns the ProgressMonitor that displays the progress of the job
     * underlying this event.<br>
     * Be careful: If the parent JobEvent is no job starting event, the 
     * ProgressMonitor will be null!
     * 
     * @return the ProgressMonitor belonging to the job underlying this event or
     *         null, if the event is no job starting event
     */
    public IProgressMonitor getProgressMonitor()
      {
        return progressMonitor_;
      }
    
    /**
     * method returns the Console belonging to the job underlying this event.<br>
     * Be careful: If the parent JobEvent is no job starting event or the 
     * underlying job needs no console the console will be null!
     * 
     * @return the Console belonging to the job underlying this event or null,
     *         if the event is no job starting event or the underlying job needs
     *         no console
     */
    public IConsole getConsole()
      {
        return console_;
      }
    
    /**
     * Returns a time stamp showing when the event was fired.
     * 
     * @return a timestamp containing the fire date and time of the event
     */
    public Date getTimestamp()
    {
        return timestamp_;
    }

    /**
     * Returns the job that has fired the event.
     * 
     * @return the job that has fired the event
     */
    public IJob getJob()
    {
        return (IJob)getSource();
    }
}
