package viptool.logger;

import java.io.File;
import java.util.Vector;

import viptool.logger.event.ILoggerChangedListener;

/**
 * Inteface for all logger. A logger must have the possibility to store messages
 * with different levels locally. Also, must be possible to create a file to
 * store information on an output stream or append them on an existing one. In
 * addition, it is necessary to add listeners to your logger. These listeners
 * will be notified, if an object was added or removed from the logger.
 */
public interface ILogger
{
    public static final int DEBUG_LEVEL    = -1;

    public static final int DEFAULT_LEVEL  = 0;

    public static final int INFO_LEVEL     = 1;

    public static final int WARRNING_LEVEL = 2;

    public static final int ERROR_LEVEL    = 3;

    /**
     * This method will store all messages with level and date as a LogObject.
     * If there is an existing FileOutputStream it is possible to write the
     * information on it, too.
     * 
     * @param level
     *            the level of the message
     * @param msg
     *            the message, that will be stored
     */
    public void log(int level, String msg);

    /**
     * This method logs a default message into the logger.
     * 
     * @param msg
     *            the message
     */
    public void log(String msg);

    /**
     * This method logs a default message into the logger if the given condition
     * is true.
     * 
     * @param msg
     *            the message
     * 
     * @param condition
     *            a condition
     */
    public void log(String msg, boolean condition);

    /**
     * This method logs a debug message into the logger.
     * 
     * @param msg
     *            the message of the debug info
     */
    public void debug(String msg);

    /**
     * This method logs a debug message into the logger if the given condition
     * is true.
     * 
     * @param msg
     *            the message of the debug info
     * @param condition
     *            a condition
     */
    public void debug(String msg, boolean condition);

    /**
     * This method logs a warning into the logger.
     * 
     * @param msg
     *            the message of the warning
     */
    public void warning(String msg);

    /**
     * This method logs a warning into the logger if the given condition is
     * true.
     * 
     * @param msg
     *            the message of the warning
     * @param condition
     *            a condition
     */
    public void warning(String msg, boolean condition);

    /**
     * This method logs an information into the logger.
     * 
     * @param msg
     *            the message of the information
     */
    public void info(String msg);

    /**
     * This method logs an information into the logger if the given condition is
     * true.
     * 
     * @param msg
     *            the message of the information
     * @param condition
     *            a condition
     */
    public void info(String msg, boolean condition);

    /**
     * This method logs an error into the logger.
     * 
     * @param msg
     *            the message of the error
     */
    public void error(String msg);

    /**
     * This method logs an error into the logger if the given condition is true.
     * 
     * @param msg
     *            the message of the error
     * @param condition
     *            a condition
     */
    public void error(String msg, boolean condition);

    /**
     * Set a file for your logger. If there have been one,yet, append the new
     * information to the existing output stream.
     * 
     * @param file
     *            the file to be opened for writing
     */
    public void setFile(File file);

    /**
     * Add one more LoggerChangedListener to the HashSet with all
     * LoggerChangedListeners. If the listener you want to add is null, nothing
     * will happen.
     * 
     * @param listener
     *            the listener, that should be added
     */
    public void addLoggerChangedListener(ILoggerChangedListener listener);

    /**
     * Remove a LoggerChangedListener from the HashSet with all
     * LoggerChangedListeners.
     * 
     * @param listener
     *            the listener, that should be removed
     */
    public void removeLoggerChangedListener(ILoggerChangedListener listener);
    
    /**
     * Sets the log level for the logger. All message "lower" than the log 
     * level are not logged.
     * 
     *  @param level a new log level. See _LEVEL fields for possible values
     */
    public void setLogLevel(int level);

    public Vector getLogs();
}
