package viptool.preferences;

import java.awt.Color;
import java.util.HashMap;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * An interface for all PreferenceNodes. Those must have the possibility to
 * write preferences, that belong to to this node in a HashMap. Every 
 * preferences stored in this HashMap can be saved in a XML file. Of course it 
 * must be possible to load such files. Every PreferenceNode can have its own 
 * preferences.
 */
public interface IPreferenceNode {
	
	/**
	 * Get the name of the PreferenceNode
	 * 
	 * @return
	 * 	the name of the PreferenceNode
	 */
	public String getName();

	/**
	 * Add a new child to this PreferenceNode, that contains new preferences.
	 * 
	 * @param node
	 * 	the node you want to add to this PreferenceNode
	 */
	public void addPreferenceChildNode(IPreferenceNode node);
	
	/**
	 * Remove one child of this PreferenceNode from the HashMap with all those 
	 * nodes. If the key is not correct, nothing will happen.
	 * 
	 * @param key
	 * 	the name of the child, that should be removed
	 */
	public void removePreferenceChildNode(String key);
	
	/**
	 * Get the HashMap, that contains all childs of this PreferenceNode.
	 * key: name of the PreferenceNode
	 * value: the PreferenceNode
	 * 
	 * @return
	 * 	a HashMap with all childs
	 */
	public HashMap getPreferenceChildNodes();
	
	/**
	 * Get one child of this PreferncesNode, that belongs to the key. If this key 
	 * is not correct, a new PreferenceNode will be returned with this key as name.
	 * 
	 * @param key
	 * 	the name of the child, that should be returned
	 * @return
	 * 	the PreferenceNode belonging to the key or a new one
	 */
	public IPreferenceNode getPreferenceChildNode(String key);
	
	/**
	 * This will write an Integer object to the HashMap,that includes all
	 * prefernces.
	 * 
	 * @param key
	 * 	name of the preference
	 * @param value
	 * 	value of the preference
	 */
	public void writeInt(String key, int value);
	
	/**
	 * This will write a Double object to the HashMap, that includes all 
	 * preferences.
	 * 
	 * @param key
	 * 	name of the preference
	 * @param value
	 * 	value of the prefernce
	 */
	public void writeDouble(String key, double value);
	
	/**
	 * This will write a String object to the HashMap, that includes all
	 * preferences.
	 * 
	 * @param key
	 * 	name of the preference
	 * @param value
	 * 	value of the preference
	 */
	public void writeString(String key, String value);
	
	/**
	 * This will write a Boolean object to the HashMap, that includes all
	 * preferences.
	 * 
	 * @param key
	 * 	name of the preference
	 * @param value
	 * 	value of the preference
	 */
	public void writeBoolean(String key, boolean value);
	
	/**
	 * This will write a Color object to the HashMap, that includes all
	 * preferences.
	 * 
	 * @param key
	 * 	name of the preference
	 * @param value
	 * 	value of the preference
	 */
	public void writeColor(String key, Color value);
	
	/**
	 * This methode will return an object out of the Hash Map, that includes all
	 * preferences. The key must belong to an Integer object, so that it can be 
	 * returned. As a result you need not parse it on your own.
	 * Otherwise the defaultValue will be returned. This defaultValue will also be
	 * reurned, if the key is not correct.
	 * 
	 * @param key
	 * 	name of the preference
	 * @param defaultValue
	 * 	the value, that will be returned, if the key or the object is not correct
	 * @return
	 * 	an Integer object belonging to the key or the defaultValue, if something
	 * 	was not correct
	 */
	public int readInt(String key, int defaultValue);
	
	/**
	 * This methode will return an object out of the Hash Map, that includes all
	 * preferences. The key must belong to a Double object, so that it can be 
	 * returned. As a result you need not parse it on your own. 
	 * Otherwise the defaultValue will be returned. This defaultValue will also be reurned, 
	 * if the key is not correct.
	 * 
	 * @param key
	 * 	name of the preference
	 * @param defaultValue
	 * 	the value, that will be returned, if the key or the object is not correct
	 * @return
	 * 	a Double object belonging to the key or the defaultValue, if something
	 * 	was not correct
	 */
	public double readDouble(String key, double defaultValue);
	
	/**
	 * This methode will return an object out of the Hash Map, that includes all
	 * preferences. The key must belong to an object, that is included in the Hash Map.
	 * Otherwise the defaultValue will be returned.
	 * In the Hash Map all values of the objects are stored as a String. So this
	 * methode can return all entries.
	 * 
	 * @param key
	 * 	name of the preference
	 * @param defaultValue
	 * 	the value, that will be returned, if the key is not correct
	 * @return
	 * 	a String object belonging to the key or the defaultValue
	 */
	public String readString(String key, String defaultValue);
	
	/**
	 * This methode will return an object out of the Hash Map, that includes all
	 * preferences. The key must belong to a Boolean object, so that it can be 
	 * returned. As a result you need not parse it on your own. 
	 * Otherwise the defaultValue will be returned. This defaultValue will also be
	 * reurned, if the key is not correct.
	 * 
	 * @param key
	 * 	name of the preference
	 * @param defaultValue
	 * 	the value, that will be returned, if the key or the object is not correct
	 * @return
	 * 	a Boolean object belonging to the key or the defaultValue, if something
	 * 	was not correct
	 */
	public boolean readBoolean(String key, boolean defaultValue);
	
	/**
	 * This methode will return an object out of the Hash Map, that includes all
	 * preferences. The key must belong to a Color object, so that it can be 
	 * returned. As a result you need not parse it on your own. 
	 * Otherwise the defaultValue will be returned. This defaultValue will also be
	 * reurned, if the key is not correct.
	 * 
	 * @param key
	 * 	name of the preference
	 * @param defaultValue
	 * 	the value, that will be returned, if the key or the object is not correct
	 * @return
	 * 	a Color object belonging to the key or the defaultValue, if something was
	 * 	not correrct
	 */
	public Color readColor(String key, Color defaultValue);
	
	/**
	 * With this methode you can save all information from the PreferenceNode in
	 * a XML file. Those information will be appended to the given XML document.
	 * In this file they will be arranged under the given root node. After that
	 * this ducument will be returned
	 * 
	 * @param document
	 * 	the document, that gains tha new information
	 * @param root
	 * 	the parental node in the XML file
	 * @return
	 * 	the document with more information
	 */
	public Document save(Document document, Node root);
	
	/**
	 * With this methode you can load information from an existing XML file. You
	 * will save all information of child nodes of the given node. They all will 
	 * be stored in the corresponding HashMaps.
	 * 
	 * @param node
	 * 	all information of child nodes of this node will be saved
	 */
	public void loadPreferences(Node node);
}
