package viptool.preferences;

import java.util.HashMap;

/**
 * An interface for all PreferenceStores. Those must have the possibility to 
 * store PreferenceNodes, that gain the information about preferences. So you
 * will get a hierarichical structure. Of course, it must be possible to get 
 * those nodes back. In addition to that, it is necessary to save all
 * those information in an XML file and to load them from such a file.
 */
public interface IPreferenceStore {
	
	/**
	 * Add a new child to this PreferenceStore, that can contain new preferences.
	 * 
	 * @param node
	 * 	the node you want to add to this PreferenceNode
	 */
	public void addPreferenceChildNode(IPreferenceNode node);
	
	/**
	 * Remove one child of this PreferenceStore from the HashMap with all those 
	 * nodes. If the key is not correct, nothing will happen.
	 * 
	 * @param key
	 * 	the name of the child, that should be removed from the store
	 */
	public void removePreferenceChildNode(String key);
	
	/**
	 * Get the HashMap, that contains all childs of the PreferenceStore.
	 * key: name of the PreferenceNode
	 * value: the PreferenceNode
	 * 
	 * @return
	 * 	a HashMap with all childs
	 */
	public HashMap getPreferenceChildNodes();
	
	/**
	 * Get one child of the PreferncesStore, that belongs to the key. If this key 
	 * is not correct, a new PreferenceNode will be returned with this key as name.
	 * 
	 * @param key
	 * 	the name of the child, that should be returned
	 * @return
	 * 	the PreferenceNode belonging to the key or null
	 */
	public IPreferenceNode getPreferenceChildNode(String key);
	
	/**
     * Save all information about preferences in a XML file. They will be stored
     * at the default path.
     */
    public void savePreferences();
    
	/**
	 * Save all information about preferences in a XML file. Those information 
	 * must be stored in the HashMaps of the PreferenceStore and the 
	 * PreferenceNode. They will be stored at the given path. If you set path to 
	 * null, a default path should be used.
	 * 
	 * @param path
	 * 	the path, where you want to store the file or null if you want to store at
	 * 	a default path
	 */
	public void savePreferences(String path);
	
	/**
	 * Load all information about preferences from a XML file. You will load the
	 * file from the given path or from the default path, if you set it to null.
	 * Those information will be stored in the corresponding HashMaps of the
	 * PrefereneStore and the PreferenceNode.
	 * 
	 * @param path
	 * 	the path, from where the file will be loaded
   * 	if path = null the methode will try to load from a default path
	 */
	public void loadPreferences(String path);
}
