package viptool.project;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;

import javax.swing.JOptionPane;

import viptool.base.IWorkbench;
import viptool.base.editor.IEditor;
import viptool.base.error.EditorNotOpenedException;
import viptool.base.error.FileAlreadyLockedException;
import viptool.base.error.FileRenameException;
import viptool.base.error.VipToolException;
import viptool.base.io.IFileLocker;
import viptool.project.error.ChildNotAcceptedException;
import viptool.project.event.IAbstractFileListener;

/**
 * This is the implementation of the IAbstractFile interface and is used in the
 * viptool for file handling.
 * 
 * @see viptool.project.IAbstractFile
 */
public class AbstractFile extends File implements IAbstractFile
{
    private IProject                             project_       = null;

    private IAbstractFolder                      parent_        = null;

    private IFileLocker                          currentLocker_ = null;

    private long                                 lastModified_  = 0;

    private final HashSet<IAbstractFileListener> listeners_     = new HashSet<IAbstractFileListener>();

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs an abstract file that does not belong to any project and that
     * has no parent folder. This is necessary for workspace files for example.
     * 
     * @param pathname
     *            the file name
     */
    AbstractFile(String pathname)
    {
        super(pathname);

        parent_ = null;
        project_ = null;
    }

    /**
     * Constructs an abstract file that does not belong to any project and that
     * has no parent folder. This is necessary for workspace files for example.
     * 
     * @param file
     *            a file object
     */
    AbstractFile(File file)
    {
        this(file.getAbsolutePath());
    }

    /**
     * Constructs a new abstract file object. An abstract file must belong to a
     * project. So the project must not be null. This version creates an
     * abstract file directly under the project.
     * 
     * @param project
     *            the project to which the file belongs
     * @param pathname
     *            the file name
     */
    AbstractFile(IProject project, String pathname)
    {
        super(pathname);

        if (project == null)
        {
            throw new IllegalArgumentException("The project must not be null");
        }

        project_ = project;

        if (!project.getAbstractFile().getFile().getParentFile().getAbsolutePath().equals(
                pathname))
        {
            parent_ = new AbstractFolder(project, getParent());
        }
    }

    /**
     * Constructs a new abstract file object. An abstract file must belong to a
     * project. So the project must not be null. This version creates an
     * abstract file directly under the project.
     * 
     * @param project
     *            the project to which the file belongs
     * @param file
     *            a file object
     */
    AbstractFile(IProject project, File file)
    {
        this(project, file.getAbsolutePath());
    }

    /**
     * Constructs a new abstract file object. An abstract file must belong to a
     * project. So the project must not be null. This version creates an
     * abstract file under the given abstract folder.
     * 
     * @param project
     *            the project to which the file belongs
     * @param folder
     *            the parent abstract folder
     * @param pathname
     *            a file name
     */
    AbstractFile(IProject project, IAbstractFolder folder, String pathname)
    {
        this(project, pathname);

        parent_ = folder;
    }

    /**
     * Constructs a new abstract file object. An abstract file must belong to a
     * project. So the project must not be null. This version creates an
     * abstract file under the given abstract folder.
     * 
     * @param project
     *            the project to which the file belongs
     * @param folder
     *            the parent abstract folder
     * @param file
     *            a file onject
     */
    AbstractFile(IProject project, IAbstractFolder folder, File file)
    {
        this(project, folder, file.getAbsolutePath());
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Injection methods
    // <><><><><><><><><><><><><><><><><><><><><>
    void setProject(IProject project)
    {
        project_ = project;
    }

    void setParent(IAbstractFolder folder)
    {
        parent_ = folder;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // listener support
    // <><><><><><><><><><><><><><><><><><><><><>
    public void addAbstractFileListener(IAbstractFileListener listener)
    {
        if (listener != null)
        {
            listeners_.add(listener);
        }
    }

    public void removeAbstractFileListener(IAbstractFileListener listener)
    {
        listeners_.remove(listener);
    }

    protected void fireFileLockedEvent(IFileLocker locker)
    {
        for (IAbstractFileListener listener : listeners_)
        {
            listener.fileLocked(locker);
        }
    }

    protected void fireFileReleasedEvent(IFileLocker locker, boolean modified)
    {
        for (IAbstractFileListener listener : listeners_)
        {
            listener.fileReleased(locker, modified);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Overridden methods from Object
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see java.lang.Object */
    public int hashCode()
    {
        return super.hashCode();
    }

    /* @see java.lang.Object */
    public boolean equals(Object object)
    {
        boolean ret;

        try
        {
            if (object instanceof File || object instanceof AbstractFile)
            {
                ret = getCanonicalFile().equals(
                        ((File) object).getCanonicalFile());
            }
            else
            {
                ret = super.equals(object);
            }
        }
        catch (IOException exception)
        {
            ret = super.equals(object);
        }

        return ret;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IAbstractFile interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.project.IAbstractFile */
    public IAbstractFolder getParentFolder()
    {
        return parent_;
    }

    /* @see viptool.project.IAbstractFile */
    public IProject getProject()
    {
        return project_;
    }

    /* @see viptool.base.io.IAbstractFile */
    public File getFile()
    {
        return this;
    }

    /* @see viptool.base.io.IAbstractFile */
    public File getPathFile()
    {
        return getParentFile();
    }

    /* @see viptool.base.io.IAbstractFile */
    public String getFullPath()
    {
        return getAbsolutePath();
    }

    /* @see viptool.base.io.IAbstractFile */
    public String getTitle()
    {
        int index;
        String name = getName();

        index = name.lastIndexOf('.');
        if (index > 0)
        {
            name = name.substring(0, index);
        }

        return name;
    }

    /* @see viptool.base.io.IAbstractFile */
    public void setName(String name, IWorkbench workbench)
            throws FileRenameException
    {

        File file;
        IAbstractFile abstractFile;

        workbench.closeAllEditors(this, true);

        IFileLocker locker = getFileLocker();
        if (locker != null)
        {
            if (locker instanceof IEditor)
            {
                int key = JOptionPane.showConfirmDialog(null,
                        "Save changes before rename?", "File changed",
                        JOptionPane.WARNING_MESSAGE, JOptionPane.YES_NO_OPTION);
                if (key != JOptionPane.NO_OPTION)
                {
                    try
                    {
                        ((IEditor) getFileLocker()).save();
                    }
                    catch (EditorNotOpenedException e)
                    {
                        workbench.showExceptionInDialog(new VipToolException(e));
                    }
                }
            }
        }

        getParentFolder().remove(this);

        file = new File(getFile().getParentFile(), name);

        if (!renameTo(file))
        {
            throw new FileRenameException(this, name);
        }

        abstractFile = FileSystem.getInstance().createAbstractFile(
                getProject(), getParentFolder(), file);

        try
        {
            getParentFolder().add(abstractFile);

        }
        catch (ChildNotAcceptedException exception)
        {
            // can't be, but log it
            /**
             * <pre>
             * Logger.getInstance().log(
             *         ILogger.WARRNING_LEVEL,
             *         &quot;A ChildNotAcceptedException was thrown in the setName &quot;
             *                 + &quot;method of AbstractFile. Inform the developers!&quot;);
             * </pre>
             */
        }
    }

    /* @see viptool.base.io.IAbstractFile */
    public long getLastModified()
    {
        if (System.getProperty("os.name").startsWith("Windows"))
        {
            return 0;
        }

        return lastModified();
    }

    /* @see viptool.base.io.IAbstractFile */
    public boolean isLocked()
    {
        return (currentLocker_ != null);
    }

    /* @see viptool.base.io.IAbstractFile */
    public void releaseLock(IFileLocker locker)
    {
        if (locker == currentLocker_)
        {
            currentLocker_ = null;

            fireFileReleasedEvent(locker, (lastModified_ < getLastModified()));
            lastModified_ = getLastModified();
        }
    }

    /* @see viptool.base.io.IAbstractFile */
    public IFileLocker getFileLocker()
    {
        return currentLocker_;
    }

    /* @see viptool.base.io.IAbstractFile */
    public void requestLock(IFileLocker locker)
            throws FileAlreadyLockedException
    {
        if (currentLocker_ == locker)
        {
            // already locked
            return;
        }

        if (currentLocker_ == null)
        {
            currentLocker_ = locker;

            // save date to determine if file was changed on release
            lastModified_ = getLastModified();

            // notify listeners
            fireFileLockedEvent(locker);
        }
        else
        {
            throw new FileAlreadyLockedException("File is already locked.",
                    currentLocker_);
        }
    }

}
