package viptool.project;

import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import viptool.base.IWorkbench;
import viptool.base.editor.IEditor;
import viptool.base.error.FileAlreadyLockedException;
import viptool.base.error.FileRenameException;
import viptool.project.error.ChildNotAcceptedException;
import viptool.project.event.IAbstractFolderListener;

/**
 * This is the implementation of the IAbstractFolder interface and is used in
 * the viptool for folder handling.
 * 
 * @see viptool.project.IAbstractFolder
 */
public class AbstractFolder extends AbstractFile implements IAbstractFolder
{
    // a set of the childrens in the folder
    private Set childrens_ = new HashSet();

    // set of listeners
    private Set listeners_ = new HashSet();

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new abstract folder object. An abstract folder must belong
     * to a project. So the project must not be null. This version creates an
     * abstract folder directly under the project.
     * 
     * @param project
     *            the project to which the file belongs
     * @param pathname
     *            the pathname of the folder
     */
    AbstractFolder(IProject project, String pathname)
    {
        super(project, pathname);
    }

    /**
     * Constructs a new folder file object. An abstract folder must belong to a
     * project. So the project must not be null. This version creates an
     * abstract folder directly under the project.
     * 
     * @param project
     *            the project to which the file belongs
     * @param file
     *            a file object giving the path to the folder
     */
    AbstractFolder(IProject project, File file)
    {
        this(project, file.getAbsolutePath());
    }

    /**
     * * Constructs a new abstract folder object. An abstract folder must belong
     * to a project. So the project must not be null. This version creates an
     * abstract folder under the given abstract folder.
     * 
     * @param project
     *            the project to which the file belongs
     * @param folder
     *            the parent abstract folder
     * @param pathname
     *            the path to the folder
     */
    AbstractFolder(IProject project, IAbstractFolder folder, String pathname)
    {
        super(project, folder, pathname);
    }

    /**
     * * Constructs a new abstract folder object. An abstract file must belong
     * to a project. So the project must not be null. This version creates an
     * abstract folder under the given abstract folder.
     * 
     * @param project
     *            the project to which the file belongs
     * @param folder
     *            the parent abstract folder
     * @param file
     *            the file pointing to the path of the new folder
     */
    AbstractFolder(IProject project, IAbstractFolder folder, File file)
    {
        this(project, folder, file.getAbsolutePath());
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // listener support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Notifies all listeners that the given file was added to the folder.
     * 
     * @param file
     *            the file that was added
     */
    protected void fireFileAddEvent(IAbstractFile file)
    {
        Iterator it = listeners_.iterator();

        while (it.hasNext())
        {
            ((IAbstractFolderListener) it.next()).fileAdded(file);
        }
    }

    /**
     * Notifies all listeners that the given file was removed from the folder.
     * 
     * @param file
     *            the file that was removed
     */
    protected void fireFileRemoveEvent(IAbstractFile file)
    {
        Iterator it = listeners_.iterator();

        while (it.hasNext())
        {
            ((IAbstractFolderListener) it.next()).fileRemoved(file);
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IAbstractFolder interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.project.IAbstractFolder */
    public IAbstractFile add(IAbstractFile file)
            throws ChildNotAcceptedException
    {
        if (file != null)
        {
            if (childrens_.add(file))
            {
                fireFileAddEvent(file);
            }
            else
            {
                Iterator it;
                IAbstractFile oldFile;

                it = childrens_.iterator();
                while (it.hasNext())
                {
                    oldFile = (IAbstractFile) it.next();
                    if (file.equals(oldFile))
                    {
                        file = oldFile;
                        break;
                    }
                }
            }
        }
        else
        {
            // TODO: log message
        }

        return file;
    }

    /* @see viptool.project.IAbstractFolder */
    public Iterator getChildIterator()
    {
        return childrens_.iterator();
    }

    /* @see viptool.project.IAbstractFolder */
    public IAbstractFile[] getChilds()
    {
        return (IAbstractFile[]) childrens_.toArray(new IAbstractFile[childrens_.size()]);
    }

    /* @see viptool.project.IAbstractFolder */
    public IAbstractFile[] getChilds(Class filter)
    {
        HashSet childrens = new HashSet();
        Iterator it;
        IAbstractFile file;

        it = getChildIterator();
        while (it.hasNext())
        {
            file = (IAbstractFile) it.next();
            if (filter.isAssignableFrom(file.getClass()))
            {
                childrens.add(file);
            }
        }

        return (IAbstractFile[]) childrens.toArray(new IAbstractFile[childrens.size()]);
    }

    /* @see viptool.project.IAbstractFolder */
    public int getSize()
    {
        return childrens_.size();
    }

    /* @see viptool.project.IAbstractFolder */
    public void remove(IAbstractFile file)
    {
        if (childrens_.remove(file))
        {
            fireFileRemoveEvent(file);
        }
    }

    /* @see viptool.project.IAbstractFolder */
    public IAbstractFile[] update(boolean recursive)
    {
        int k;
        File[] files = getFile().listFiles();

        IAbstractFile[] newfiles = null;
        if (files == null)
        {
            return new IAbstractFile[0];
        }
        else
        {
            newfiles = new IAbstractFile[files.length];
        }
        IAbstractFile file;

        k = 0;
        for (int i = 0; i < files.length; i++)
        {
            file = null;
            if (files[i].isDirectory())
            {
                file = FileSystem.getInstance().createAbstractFolder(
                        getProject(), this, files[i]);
                newfiles[k++] = file;
            }
            else if (files[i].isFile())
            {
                file = FileSystem.getInstance().createAbstractFile(
                        getProject(), this, files[i]);
                newfiles[k++] = file;
            }

            try
            {
                if (file != null)
                {
                    file = add(file);
                    // update the subfolder if needed
                    if (recursive && (file instanceof IAbstractFolder))
                    {
                        ((IAbstractFolder) file).update(true);
                    }
                }
            }
            catch (ChildNotAcceptedException exception)
            {
                // TODO: log a message
            }
        }

        return newfiles;
    }

    /* @see viptool.project.IAbstractFolder */
    public void addAbstractFolderListener(IAbstractFolderListener listener)
    {
        if (listener != null)
        {
            listeners_.add(listener);
        }
        else
        {
            // TODO: log message
        }
    }

    /* @see viptool.project.IAbstractFolder */
    public void removeAbstractFolderListener(IAbstractFolderListener listener)
    {
        listeners_.remove(listener);
    }

    /* @see viptool.base.io.IAbstractFile */
    public void setName(String name, IWorkbench workbench)
            throws FileRenameException
    {
        File file;
        IAbstractFolder abstractFolder;

        boolean proj = false;
        if (getParentFolder().getParentFolder() == null)
        {
            proj = true;
        }

        if (!proj)
        {
            getParentFolder().remove(this);
        }
        else
        {
            getProject().removeFile(this);
        }

        file = new File(getFile().getParentFile(), name);

        if (!renameTo(file))
        {
            throw new FileRenameException(this, name);
        }

        abstractFolder = FileSystem.getInstance().createAbstractFolder(
                getProject(), getParentFolder(), file);

        try
        {
            if (!proj)
            {
                getParentFolder().add(abstractFolder);
            }
            else
            {
                getProject().addFile(abstractFolder);
            }
            abstractFolder.update(true);
        }
        catch (ChildNotAcceptedException exception)
        {
            // can't be, but log it
            /**
             * <pre>
             * Logger.getInstance().log(
             *         ILogger.WARRNING_LEVEL,
             *         &quot;A ChildNotAcceptedException was thrown in the setName &quot;
             *                 + &quot;method of AbstractFile. Inform the developers!&quot;);
             * </pre>
             */
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Useless methods for folders
    // <><><><><><><><><><><><><><><><><><><><><>

    /*
     * The following methods are not useful for folder So they generate only
     * warnings.
     */

    /* @see viptool.project.IAbstractFolder */
    public void addEditor(IEditor editor)
    {
        // TODO: log a message
    }

    /* @see viptool.project.IAbstractFolder */
    public IEditor getCurrentEditor()
    {
        // TODO: log a message
        return null;
    }

    /* @see viptool.project.IAbstractFolder */
    public IEditor[] getEditors()
    {
        // TODO: log a message
        return new IEditor[0];
    }

    /* @see viptool.project.IAbstractFolder */
    public boolean isLocked()
    {
        return false;
    }

    /* @see viptool.project.IAbstractFolder */
    public void releaseLock(IEditor editor)
    {
        // TODO: log a message
    }

    /* @see viptool.project.IAbstractFolder */
    public void removeEditor(IEditor editor)
    {
        // TODO: log a message
    }

    /* @see viptool.project.IAbstractFolder */
    public void requestLock(IEditor editor) throws FileAlreadyLockedException
    {
        // TODO: log a message
    }
}
