package viptool.project;

import java.io.File;
import java.util.HashMap;

public class FileSystem
{
    private static FileSystem                    instance_ = null;

    private final HashMap<String, IAbstractFile> files_;

    private FileSystem()
    {
        files_ = new HashMap<String, IAbstractFile>();
    }

    public static synchronized FileSystem getInstance()
    {
        if (instance_ == null)
        {
            instance_ = new FileSystem();
        }

        return instance_;
    }

    private IAbstractFile getAbstractFile(File file)
    {
        return files_.get(file.getAbsolutePath());
    }

    private IAbstractFolder getAbstractFolder(File file)
    {
        IAbstractFile abstractFile = getAbstractFile(file);

        if (file instanceof IAbstractFolder)
        {
            return (IAbstractFolder) abstractFile;
        }

        return null;
    }

    private IProject searchProject(File file)
    {
        IAbstractFile abstractFile;

        abstractFile = getAbstractFile(file);
        if (abstractFile == null || abstractFile.getProject() == null)
        {
            if (file.getParentFile() == null)
            {
                return null;
            }

            return searchProject(file.getParentFile());
        }

        return abstractFile.getProject();
    }

    private void addAbstractFile(File file, IAbstractFile abstractFile)
    {
        IProject project;

        // search the project
        project = abstractFile.getProject();
        if (project == null)
        {
            project = searchProject(file);
            ((AbstractFile) abstractFile).setProject(project);
        }

        // be sure that all parents exists (for project files)
        if (project != null
                && abstractFile.getParentFolder() == null
                && !file.getParentFile().equals(
                        project.getAbstractFile().getFile().getParentFile()))
        {
            if (file.getParentFile() == null)
            {
                System.out.println("WARNING: Filesystem Exception. Abstract "
                        + "file 'abstractFile' lies not in an project "
                        + "directory!");
            }

            ((AbstractFile) abstractFile).setParent(createAbstractFolder(
                    project, file.getParentFile()));
        }

        files_.put(file.getAbsolutePath(), abstractFile);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // abstract files
    // <><><><><><><><><><><><><><><><><><><><><>

    public IAbstractFile createAbstractFile(String pathname)
    {
        File file = new File(pathname);
        IAbstractFile abstractFile = getAbstractFile(file);

        if (abstractFile == null)
        {
            abstractFile = new AbstractFile(pathname);
            addAbstractFile(file, abstractFile);
        }

        return abstractFile;
    }

    public IAbstractFile createAbstractFile(File file)
    {
        IAbstractFile abstractFile = getAbstractFile(file);

        if (abstractFile == null)
        {
            abstractFile = new AbstractFile(file);
            addAbstractFile(file, abstractFile);
        }

        return abstractFile;
    }

    public IAbstractFile createAbstractFile(IProject project, String pathname)
    {
        File file = new File(pathname);
        IAbstractFile abstractFile = getAbstractFile(file);

        if (abstractFile == null)
        {
            abstractFile = new AbstractFile(project, pathname);
            addAbstractFile(file, abstractFile);
        }

        return abstractFile;
    }

    public IAbstractFile createAbstractFile(IProject project, File file)
    {
        IAbstractFile abstractFile = getAbstractFile(file);

        if (abstractFile == null)
        {
            abstractFile = new AbstractFile(project, file);
            addAbstractFile(file, abstractFile);
        }

        return abstractFile;
    }

    public IAbstractFile createAbstractFile(IProject project,
            IAbstractFolder folder, String pathname)
    {
        File file = new File(pathname);
        IAbstractFile abstractFile = getAbstractFile(file);

        if (abstractFile == null)
        {
            abstractFile = new AbstractFile(project, folder, pathname);
            addAbstractFile(file, abstractFile);
        }

        return abstractFile;
    }

    public IAbstractFile createAbstractFile(IProject project,
            IAbstractFolder folder, File file)
    {
        IAbstractFile abstractFile = getAbstractFile(file);

        if (abstractFile == null)
        {
            abstractFile = new AbstractFile(project, folder, file);
            addAbstractFile(file, abstractFile);
        }

        return abstractFile;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // abstract folders
    // <><><><><><><><><><><><><><><><><><><><><>

    public IAbstractFolder createAbstractFolder(IProject project,
            String pathname)
    {
        File file = new File(pathname);
        IAbstractFolder abstractFolder = getAbstractFolder(file);

        if (abstractFolder == null)
        {
            abstractFolder = new AbstractFolder(project, pathname);
            addAbstractFile(file, abstractFolder);
        }

        return abstractFolder;
    }

    public IAbstractFolder createAbstractFolder(IProject project, File file)
    {
        IAbstractFolder abstractFolder = getAbstractFolder(file);

        if (abstractFolder == null)
        {
            abstractFolder = new AbstractFolder(project, file);
            addAbstractFile(file, abstractFolder);
        }

        return abstractFolder;
    }

    public IAbstractFolder createAbstractFolder(IProject project,
            IAbstractFolder folder, String pathname)
    {
        File file = new File(pathname);
        IAbstractFolder abstractFolder = getAbstractFolder(file);

        if (abstractFolder == null)
        {
            abstractFolder = new AbstractFolder(project, folder, pathname);
            addAbstractFile(file, abstractFolder);
        }

        return abstractFolder;
    }

    public IAbstractFolder createAbstractFolder(IProject project,
            IAbstractFolder folder, File file)
    {
        IAbstractFolder abstractFolder = getAbstractFolder(file);

        if (abstractFolder == null)
        {
            abstractFolder = new AbstractFolder(project, folder, file);
            addAbstractFile(file, abstractFolder);
        }

        return abstractFolder;
    }
}
