package viptool.project;

import java.io.File;

import viptool.base.IWorkbench;
import viptool.base.error.FileAlreadyLockedException;
import viptool.base.error.FileRenameException;
import viptool.base.io.IFileLocker;
import viptool.project.event.IAbstractFileListener;

/**
 * An abstract file is a file with extended functionality such as editor support
 * and internal locking methods.<br>
 * An abstract file is interpreted by a file interpreter and shown by an editor. <br>
 * An editor that opens a file must add itself to the editor list of the file.
 * Otherwise unknown behaviour is the result.<br>
 * The interface also has a method to return the java.io.File object behind the
 * abstract file.
 */
public interface IAbstractFile
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // general methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Get the timestamp with the the last modification time of the file.
     * 
     * @return the last modification time
     */
    public long getLastModified();

    /**
     * Returns the parent folder of the file. If the file is a direct child of
     * the project, the folder of the project is returned. This is not the
     * project itself. It's just the path of the project files. <br>
     * The method returns null if and only if the method is called from the
     * project root, e.g. you have called getParentFolder() on a direct children
     * of a project and call getParentFolder on the result again.
     * 
     * @return the parent folder or null
     */
    public IAbstractFolder getParentFolder();

    /**
     * Returns the hosting project of the file. This method can also return null
     * if the file belongs to no project.
     * 
     * @return the project of the file or null
     */
    public IProject getProject();

    // <><><><><><><><><><><><><><><><><><><><><>
    // Locking support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Requests the file to lock it, so the file locker can change the file. If
     * the file is already locked an exception is thrown. After locking, the
     * file locker can change the file.
     * 
     * @param locker
     *            the file locker that want to lock the file
     * @throws FileAlreadyLockedException
     *             - if the file is already locked by another file locker
     */
    public void requestLock(IFileLocker locker)
            throws FileAlreadyLockedException;

    /**
     * Releases the lock on the file. These means the file can be edited by
     * other file lockers now. If the given file locker was not the locking file
     * locker the operation is ignored.<br>
     * If the file was not locked, nothing is done.
     * 
     * @param locker
     *            the file locker that initially locked the file
     */
    public void releaseLock(IFileLocker locker);

    /**
     * Returns the current file locker. If the file is not locked, null is
     * returned.
     * 
     * @return the current file locker or null if the file is not locked
     */
    public IFileLocker getFileLocker();

    /**
     * Returns the current lock state of the file. If the file is locked by a
     * file locker, true will be returned, otherwise false is returned.<br>
     * A file is locked by the time, a file locker want to change the file.
     * 
     * @return true if the file is locked, false otherwise
     */
    public boolean isLocked();

    public void addAbstractFileListener(IAbstractFileListener listener);

    public void removeAbstractFileListener(IAbstractFileListener listener);

    // <><><><><><><><><><><><><><><><><><><><><>
    // file naming and path support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the file object assigned to the abstracted file.
     * 
     * @return a reference to the file object
     */
    public File getFile();

    /**
     * Returns the path of the the file. This is the parent folder of the file.
     * 
     * @return the path of the file
     */
    public File getPathFile();

    /**
     * Returns the full path of the file. This path is unique in the current
     * file system.
     * 
     * @return the full path of the file (e.g. c:\tmp\example.txt)
     */
    public String getFullPath();

    /**
     * Returns the title of the file. The title is the file name without path
     * and extension.
     * 
     * @return the title of the file (e.g. example)
     */
    public String getTitle();

    /**
     * Returns the name of the file. The name is the file without path but with
     * extension.
     * 
     * @return the name of the file (e.g. example.txt)
     */
    public String getName();

    /**
     * Sets the name of the file. This will rename a file on the hard disk.
     * 
     * @param name
     *            the new name for the file
     * @throws FileRenameException
     *             - if the file cannot be renamed
     */
    public void setName(String name, IWorkbench workbench)
            throws FileRenameException;
}
