package viptool.project;

import java.util.Iterator;

import viptool.project.error.ChildNotAcceptedException;
import viptool.project.event.IAbstractFolderListener;

/**
 * This interface provides basic functionality for folders. A folder is a file
 * too. It comes with the basic functions add, remove and update. There are also
 * methods to get the child elements of the folder.
 */
public interface IAbstractFolder extends IAbstractFile
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // general methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds the given file to the folder. Since folders are also files, folders
     * can be added too. If the file already exists the existing instance of the
     * abstract file is returned. Otherwise the given file is returned.
     * 
     * @param file
     *            the file that should be added to the folder
     * @return the added file or the instance of the file that already exists
     * 
     * @param file
     *            the file that should be added to the folder.
     * @throws ChildNotAcceptedException -
     *             if the folder does not accept the file
     */
    public IAbstractFile add(IAbstractFile file)
            throws ChildNotAcceptedException;

    /**
     * Removes the given file from the folder. Since folders are also files,
     * folders can be removed too.
     * 
     * @param file
     *            the file to remove from the folder
     */
    public void remove(IAbstractFile file);

    /**
     * Reqeusts an update in the folder. The method will look for new files in
     * the folder and add them automatically.
     * 
     * @param recursive
     *            a boolean parameter to set the update mode for the method. If
     *            this parameter is set to true, subfolders are updated too.
     *            Otherwise subfolders are ignored.
     * @return an array that contains the added files
     */
    public IAbstractFile[] update(boolean recursive);

    /**
     * Returns an array containing all childs of the folder. If the folder
     * contains no childs, an empty array will be returned. Null is never
     * returned.
     * 
     * @return an array containing the folders childs or an empty array if the
     *         folder is empty
     */
    public IAbstractFile[] getChilds();

    /**
     * Returns an array containing all childrens that are instance of the given
     * filter class. If no child was found, an empty array is returned. Null is
     * never returned.
     * 
     * @param filter
     *            a class that serves as filter for selecting childrens
     * @return an array containing childrens of the given class
     */
    public IAbstractFile[] getChilds(Class filter);

    /**
     * Returns an iterator that is capable to iterate over all childrens of the
     * folder. The order of the elments is not defined.
     * 
     * @return an iterator that can be used to iterate all childrens of the
     *         folder
     */
    public Iterator getChildIterator();

    /**
     * Returns the size of the folder. The size of a folder is the count of
     * childrens that are in the folder.
     * 
     * @return the count of childrens
     */
    public int getSize();

    // <><><><><><><><><><><><><><><><><><><><><>
    // listern support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds the given abstract folder listener to the abstract folder. The
     * listener is notified about new files in the folder and when files were
     * removed.
     * 
     * @param listener
     *            the listener that should be added to the abstract folder
     */
    public void addAbstractFolderListener(IAbstractFolderListener listener);

    /**
     * Removes the given abstract folder listener from the abstract folder. The
     * listener will no longer be notified about file changes.
     * 
     * @param listener
     *            the listener that should be removed
     */
    public void removeAbstractFolderListener(IAbstractFolderListener listener);

}
