package viptool.project;

import java.util.Iterator;

import viptool.base.io.IFileInterpreter;
import viptool.project.error.ChildNotAcceptedException;
import viptool.project.event.IProjectListener;

/**
 * A project is a container for several files and folders. It also provides
 * blacklist supports, that means files and folders can be added explictly or
 * using a pattern with regular expressions.
 */
public interface IProject extends IFileInterpreter
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // General methods
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Return the name of the project. The name is typically the title of the
     * project file.
     * 
     * @return the name of the project
     */
    public String getName();
    
    /**
     * This method asks the user if the project should removed form workspace
     * or deleted and deletes the project if the user wants it.
     */
    public void delete();

    // <><><><><><><><><><><><><><><><><><><><><>
    // Project listener support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds the given project listener to the project. Project listeners are
     * notified if the project structure changed, this is when files are removed
     * or added.
     * 
     * @param listener
     *            the listener to add to the project
     */
    public void addProjectListener(IProjectListener listener);

    /**
     * Removes the given project listener form the projects. The listener will
     * no longer be notified about changes in the project structure.
     * 
     * @param listener
     *            the listener that should be removed from the project
     */
    public void removeProjectListener(IProjectListener listener);

    // <><><><><><><><><><><><><><><><><><><><><>
    // File support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds the given file to the project. Since folders are also files, folders
     * can be added too. If the file already exists the existing instance of the
     * abstract file is returned. Otherwise the given file is returned.
     * 
     * @param file
     *            the file that should be added to the project.
     * @return the added file or the instance of the file that already exists
     * 
     * @throws ChildNotAcceptedException -
     *             if the project does not accept the file, e.g. the file is in
     *             the black list
     */
    public IAbstractFile addFile(IAbstractFile file)
            throws ChildNotAcceptedException;

    /**
     * Removes the given file from the project. Since folders are also files,
     * folders can be removed too.
     * 
     * @param file
     *            the file to remove from the project
     */
    public void removeFile(IAbstractFile file);

    /**
     * Reqeusts an update in the project. The method will look for new files in
     * the project folder and add them automatically.
     * 
     * @param recursive
     *            a boolean parameter to set the update mode for the method. If
     *            this parameter is set to true, subfolders are updated too.
     *            Otherwise subfolders are ignored.
     * @return an array that contains the added files
     */
    public IAbstractFile[] updateFiles(boolean recursive);

    /**
     * Returns an array containing all files of the project. If the project
     * contains no files, an empty array will be returned. Null is never
     * returned.<br>
     * Note: Only the files on the first level are returned. Subfolders are not
     * searched. <br>
     * Note: Files that are in the blacklist are not returned, since these files
     * are not part of the project.
     * 
     * @return an array containing the projects files or an empty array if the
     *         project is empty
     */
    public IAbstractFile[] getFiles();

    /**
     * Returns an array containing all files of the project. If the project
     * contains no files, an empty array will be returned. Null is never
     * returned.<br>
     * Note: Only the files on the first level are returned. Subfolders are not
     * searched. <br>
     * Note: Files that are in the blacklist are not returned, since these files
     * are not part of the project.
     * 
     * @param filter
     *            a class that serves as filter for selecting files
     * @return an array containing files of the given class
     */
    public IAbstractFile[] getFiles(Class filter);

    /**
     * Returns an iterator that is capable to iterate over all files of the
     * project. The order of the elments is not defined.<br>
     * Note: Only the files on the first level are returned. Subfolders are not
     * searched. <br>
     * Note: Files that are in the blacklist are not returned, since these files
     * are not part of the project.
     * 
     * @return an iterator that can be used to iterate all files of the project
     */
    public Iterator getFileIterator();

    /**
     * Returns the size of the project. The size of a project is the count of
     * files that are in the project.<br>
     * Note: Only the first level is counted. Subfolders are not searched. <br>
     * Note: Files that are in the blacklist are not counted, since these files
     * are not part of the project.
     * 
     * @return the count of files
     */
    public int getSize();

    // <><><><><><><><><><><><><><><><><><><><><>
    // Blacklist support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds the specified file to the blacklist. This will remove the file
     * automatically from the project too. Further calls to addFile with this
     * file will end in a ChildNotAcceptedException.<br>
     * Unlike the remove method, this method stores the file so that the file
     * cannot be added in the future to the project (or by any calls to update),
     * unless it is removed from the blacklist again.<br>
     * If the file is folder, all subfiles are removed from the project too, but
     * are not added to blacklist.
     * 
     * @param file
     *            the file to add to the blacklist
     */
    public void addFileToBlacklist(IAbstractFile file);

    /**
     * Adds the specified pattern to the blacklist. These will add all files to
     * the blacklist, whose name matches the given pattern.
     * 
     * @param pattern
     *            a regular expression for adding files to the blacklist by name
     */
    public void addBlacklistPattern(String pattern);

    /**
     * Removes the given file from the blacklist. This will add the file to the
     * project automatically. If the file is a folder all files in the blacklist
     * are removed from blacklist too, excepting the ones that were explicitly
     * added to the blacklist.
     * 
     * @param file
     *            the file to remove from the blacklist
     */
    public void removeFileFromBlacklist(IAbstractFile file);

    /**
     * Removes the pattern from the blacklist. All files that were added to the
     * blacklist because of this pattern, are removed from the blacklist. Files
     * that were added explicitly with addFileToBlacklist are not removed, even
     * if there name matches the given pattern.
     * 
     * @param pattern
     *            the pattern to remove form the blacklist
     */
    public void removeBlacklistPattern(String pattern);

    /**
     * Determines whether the file is in the blacklist or not. Returns true if
     * the file is in the blacklist, and false otherwise.
     * 
     * @param file
     *            the file to test
     * @return true if the file is in the blacklist, false otherwise
     */
    public boolean isFileInBlacklist(IAbstractFile file);
}
