package viptool.project;

import java.util.Iterator;

import viptool.base.io.IFileInterpreter;
import viptool.project.event.IWorkspaceListener;

/**
 * This interface is the base of a workspace. A workspace is a folder that
 * contains projects. Since a workspace can only have projects as childs,
 * ChildNotAcceptedException is thrown when add is called with an object other
 * that IProject.
 */
public interface IWorkspace extends IFileInterpreter
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // workspace listener support
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds the workspace listener to the workspace. A workspace listener is
     * notified when projects are added to or removed from the workspace.
     * 
     * @param workspaceListener
     *            the workspace listener that should be added to the workspace
     */
    public void addWorkspaceListener(IWorkspaceListener workspaceListener);

    /**
     * Removes the given workspace listener from the workspace. The workspace
     * listener is no longer notified about projects that are added to or
     * removed from the workspace.
     * 
     * @param workspaceListener
     *            the listener that should be removed from the workspace
     */
    public void removeWorkspaceListener(IWorkspaceListener workspaceListener);

    // <><><><><><><><><><><><><><><><><><><><><>
    // Project support
    // <><><><><><><><><><><><><><><><><><><><><>
    
    
    /**
     * Adds the given project to the workspace.
     * 
     * @param project
     *            the project that should be added to the workspace
     */
    public void addProject(IProject project);

    /**
     * Removes the given project from the workspace.
     * 
     * @param project
     *            the project to remove from the workspace
     */
    public void removeProject(IProject project);

    /**
     * Returns an array containing all projects of the workspace. If the
     * workspace contains no projects, an empty array will be returned. Null is
     * never returned.<br>
     * 
     * @return an array containing the projects or an empty array if the
     *         workspace is empty
     */
    public IProject[] getProjects();

    /**
     * Returns an iterator that is capable to iterate over all projects of the
     * workspace. The order of the elments is not defined.<br>
     * 
     * @return an iterator that can be used to iterate all projects of the
     *         workspace
     */
    public Iterator getProjectIterator();

    /**
     * Returns the size of the workspace. The size of a workspace is the count
     * of projects that are in the workspace.<br>
     * 
     * @return the count of projects
     */
    public int getSize();

    /**
     * Updates the workspace. This means that a recursive update of all projects
     * in the workspace is made.
     */
    public void update();
}
