package viptool.project.wizard;

import java.awt.Image;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.JComponent;

import viptool.base.IWorkbench;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.gui.component.FileNamePanel;
import viptool.base.gui.component.FolderTreePanel.FolderTreeSelection;
import viptool.base.gui.component.event.IFileNamePanelListener;
import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.IStandardModalDialog;
import viptool.base.wizard.IWizardPage;
import viptool.base.wizard.WizardAdapter;
import viptool.project.FileSystem;
import viptool.project.IAbstractFolder;
import viptool.project.IAbstractFolderProvider;
import viptool.project.IProject;
import viptool.util.resources.ResourceConstants;

public class NewFolderWizard extends WizardAdapter
{
    // the wizard page
    protected IFolderPage             folderPage_ = null;

    // the provider for the folder
    protected IAbstractFolderProvider provider_   = null;

    // the created folder
    protected String                  folder_     = null;

    // the hosting standard dialog
    protected IStandardModalDialog    dialog_     = null;

    // a flag the wizard was finish correctly. Used in windowClosed to avoid
    // a call to cancel on windowClosed method.
    protected boolean                 finished_   = false;

    protected IWorkbench              workbench_  = null;

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>
    protected interface IFolderPage extends IWizardPage, IFileNamePanelListener
    {
        public String getEnteredName();

        public FolderTreeSelection getSelectedPath();

        public FileNamePanel getPanel();
    }

    /**
     * The workspace page for creating a new workspace. We have only one page
     * for entering name and path of the workspace.
     */
    protected class FolderPage implements IFolderPage
    {
        protected FileNamePanel panel_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        public FolderPage()
        {
            panel_ = new FileNamePanel(workbench_);
            panel_.addFileNamePanelListener(this);
            panel_.disableNewFolder();
        }

        public String getEnteredName()
        {
            return panel_.getEnteredName();
        }

        public FolderTreeSelection getSelectedPath()
        {
            return panel_.getSelectedPath();
        }

        public FileNamePanel getPanel()
        {
            return panel_;
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the WizardPage interface
        // <><><><><><><><><><><><><><><><><><><><><>
        /* @see viptool.gui.wizards.WizardPage */
        public boolean canGoToPrevoius()
        {
            // we have only one page
            return false;
        }

        /* @see viptool.gui.wizards.WizardPage */
        public JComponent getContent()
        {
            return panel_;
        }

        /* @see viptool.gui.wizards.WizardPage */
        public Image getImage()
        {
            return workbench_.getResourceLoader().getImage(
                    ResourceConstants.ICON_DEFAULT_FILE);
        }

        /* @see viptool.gui.wizards.WizardPage */
        public String getMainDescription()
        {
            return "Enter folder name...";
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of IFileNamePanelListener interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see viptool.base.gui.component.event.IFileNamePanelListener */
        public void focusGainedOnName()
        {
            dialog_.setDetailedDescription("Please enter the name "
                    + "for the new folder");
        }

        /* @see viptool.base.gui.component.event.IFileNamePanelListener */
        public void focusGainedOnPathTree()
        {
            dialog_.setDetailedDescription("Please select the parent folder");
        }

        /* @see viptool.base.gui.component.event.IFileNamePanelListener */
        public void focusLost()
        {
            dialog_.setDetailedDescription(null);
        }

        /* @see viptool.base.gui.component.event.IFileNamePanelListener */
        public void nameNotValid()
        {
            dialog_.setNotice("Please enter a name for the new folder!",
                    DialogConstants.TYPE_ERROR);
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, false);
        }

        /* @see viptool.base.gui.component.event.IFileNamePanelListener */
        public void validated()
        {
            dialog_.removeNotice();
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, true);
        }

    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    public NewFolderWizard(IWorkbench workbench,
            IAbstractFolderProvider provider)
    {
        workbench_ = workbench;
        provider_ = provider;
    }

    public String getCreatedFolder()
    {
        return folder_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the Wizard interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.wizards.Wizard */
    public void cancel()
    {
        // nothing to do here, the user has canceled the operation
    }

    /* @see viptool.wizards.Wizard */
    public void finish()
    {
        IAbstractFolder folder = null;
        FolderTreeSelection selection = folderPage_.getSelectedPath();

        try
        {
            // the parent is a project
            if (selection.getFolder() == null)
            {
                folder = FileSystem.getInstance().createAbstractFolder(
                        selection.getProject(),
                        selection.getProject().getAbstractFile().getPathFile().getAbsolutePath()
                                + File.separator + folderPage_.getEnteredName());
                provider_.createNewFile(folder);
            }
            else
            // the parent is a folder
            {
                folder = FileSystem.getInstance().createAbstractFolder(
                        selection.getProject(),
                        selection.getFolder(),
                        selection.getFolder().getFullPath() + File.separator
                                + folderPage_.getEnteredName());
                provider_.createNewFile(folder);
            }

            folder_ = folder.getFullPath();
            selection.getProject().updateFiles(true);

            if (folder != null)
            {
                workbench_.selectFile(folder);
            }
        }
        catch (FileInterpreterProviderException exception)
        {
            workbench_.showExceptionInDialog(exception);
        }

        finished_ = true;
    }

    /* @see viptool.wizards.Wizard */
    public synchronized IWizardPage getFirstPage(IStandardModalDialog dialog)
    {
        if (folderPage_ == null)
        {
            folderPage_ = new FolderPage();
        }

        dialog_ = dialog;
        return folderPage_;
    }

    /* @see viptool.wizards.Wizard */
    public IWizardPage getNextPage()
    {
        // we have only one page
        return null;
    }

    /* @see viptool.wizards.Wizard */
    public IWizardPage getPreviousPage()
    {
        // we have only one page
        return null;
    }

    /* @see viptool.wizards.Wizard */
    public String getTitle()
    {
        return "New Folder";
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the WindowListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see java.awt.event.WindowListener */
    public void windowActivated(WindowEvent event)
    {
        IProject project;
        IAbstractFolder folder;

        project = workbench_.getSelectedProject();
        folder = workbench_.getSelectedFolder();

        if (folder != null)
        {
            folderPage_.getPanel().initialize(folder, null);
        }
        else if (project != null)
        {
            folderPage_.getPanel().initialize(project, null);
        }
        else
        {
            folderPage_.getPanel().initialize(
                    workbench_.getCurrentWorkspace().getProjects()[0], null);
        }
    }

    /* @see java.awt.event.WindowListener */
    public void windowClosed(WindowEvent event)
    {
        // if finished was not called the user has cancelled the wizard
        if (!finished_)
        {
            cancel();
        }
    }
}
