package viptool.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.Vector;

import viptool.base.error.UnsupportedFileException;
import viptool.base.error.VipToolException;

/**
 * This Class represents a Java Class File. To create an instance call the static method getClassFile(File). 
 */
public class ClassFile{

	/**
	 * This method creates a new instance of ClassFile for the given file, if and only if the given file
	 * is a valid Java Class File. The full qualified class name and the name of the parent class will be
	 * read out from the bytecode of the given class file. 
	 * 
	 * @param file
	 * 		class file as File
	 * @return
	 * 		instance of ClassFile for the given file
	 * @throws UnsupportedFileException
	 * 		if the file is no class file or the file isn't readable this exeption will be thrown
	 */
	public static ClassFile getClassFile(File file) throws UnsupportedFileException{
		if(file == null){
			throw new UnsupportedFileException(file, "ClassFiles can not be created form null!", UnsupportedFileException.UNKOWNFORMAT);
		}
		try {
			//open file as stream
			FileInputStream stream = new FileInputStream(file);
			//map file as ByteBuffer
			ByteBuffer buffer = stream.getChannel().map(FileChannel.MapMode.READ_ONLY, 0, file.length());
			
			//test if file starts with the magic number
			if(buffer.getInt(0) != 0xCAFEBABE){
				throw new UnsupportedFileException(file, "File doesn't start with 0xCAFEBABE.", UnsupportedFileException.UNKOWNFORMAT);
			}
			
			//offset of constant pool
			int offset=10;
			//get name of class
			//get identifier of consant type
			byte b = buffer.get(offset);
			if(b != 0x07){
				throw new UnsupportedFileException(file, "unknown constant pool constant type", UnsupportedFileException.UNKOWNFORMAT);
			}
			//go to entry type identifier
			offset+=3;
			
			byte b2 = buffer.get(offset);
			if(b2 != 0x01){
				throw new UnsupportedFileException(file, "unknown constant pool entry type", UnsupportedFileException.UNKOWNFORMAT);
			}
			offset++;
			
			//get lenght of class name
			short len = buffer.getShort(14); 
			offset+=2;
			
			//get full qualified class name
			String name = "";
			int oldoffset = offset;
			for(; offset<len+oldoffset; offset++){
				int byte1 = (int)buffer.get(offset);
				//test if it is a one or two byte char
				if((byte1 & 0x80) == 0){
					//it is a one byte char
					name += (char)byte1;
				}else{
					//it is a two byte char
					offset++;
					int byte2 = (int)buffer.get(offset);
					name += (char)(((byte1 & 0x1f) << 6) + (byte2 & 0x3f));
				}
			}
			name = name.replaceAll("/", ".");
			
			//get name of parent class
			b = buffer.get(offset);
			if(b != 0x07){
				throw new UnsupportedFileException(file, "unknown constant pool constant type", UnsupportedFileException.UNKOWNFORMAT);
			}
			offset+=3;
			
			b2 = buffer.get(offset);
			if(b2 != 0x01){
				throw new UnsupportedFileException(file, "unknown constant pool entry type", UnsupportedFileException.UNKOWNFORMAT);
			}
			offset++;
			
			len = buffer.getShort(offset); 
			offset+=2;
			
			String supername = "";
			oldoffset = offset;
			for(; offset<len+oldoffset; offset++){
				int byte1 = (int)buffer.get(offset);
				if((byte1 & 0x80) == 0){
					supername += (char)byte1;
				}else{
					offset++;
					int byte2 = (int)buffer.get(offset);
					supername += (char)(((byte1 & 0x1f) << 6) + (byte2 & 0x3f));
				}
			}
			supername = supername.replaceAll("/", ".");
			
			return new ClassFile(file, name, supername);
			
		}catch(FileNotFoundException ex){
			throw new UnsupportedFileException(file, ex, UnsupportedFileException.FILENOTFOUND);
		} catch (IOException e) {
			throw new UnsupportedFileException(file, e, UnsupportedFileException.IOEXCEPTION);
		}
	}
	
	//full qualified class name
	private String clazz_;
	//full qualified name of the parent class
	private String superclazz_;
	//reference to file
	private File file_;
	//url usable for ExtensionLoader
	private URL classurl_ = null;
	
	private ClassFile(File file, String classname, String superclassname)
	{
		file_ = file;
		superclazz_ = superclassname;
		clazz_ = classname;
	}
	
	/**
	 * This method returns the File-Object to this ClassFile
	 * 
	 * @return
	 * 		file as File
	 */
	public File getFile()
	{
		return file_;
	}
	
	/**
	 * This method returns the full qualified class name of the class which is stored into this file.
	 * 
	 * @return
	 * 		full qualified class name
	 */
	public String getClassName()
	{
		return clazz_;
	}
	
	/**
	 * This method returns the parent class of the class which is stored into this file.
	 * 
	 * @return
	 * 		name of super class
	 */
	public String getParentClass()
	{
		return superclazz_;
	}
	
	/**
	 * This method returns an URL which can be used to load this class through an URLClassLoader.
	 * 
	 * @return
	 * 		ClassLoader URL
	 */
	public URL getClassURL() throws VipToolException
	{
		if(classurl_ == null)
		{
			String name = getClassName();
			int o = 0;
			Vector parts = new Vector();
			while(true){
				int newo = name.indexOf('.', o);
				if(newo == -1){
					break;
				}
				String part = name.substring(o, newo);
				o = newo+1;
				parts.add(part);
			}
			
			File dir = getFile().getParentFile();
			
			int count = parts.size()-1;
			for(; count>=0; count--){
				String part = (String)parts.get(count);
				String dirname = dir.getName();
				if(!dirname.equals(part)){
					throw new VipToolException("The path of "+getClassName()+" doesn't match the class packages!");
				}
				dir = dir.getParentFile();
			}
			try {
				classurl_ = dir.toURI().toURL();
			} catch (MalformedURLException e) {
				throw new VipToolException("The URL for "+getClassName()+" couldn't be created, so the class isn't loadabel!");
			}
		}
		return classurl_;
	}
	
	public String toString()
	{
		return file_.getAbsolutePath();
	}
}
