package viptool.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.CharBuffer;
import java.nio.channels.FileChannel;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import viptool.base.error.FileInterpreterProviderException;
import viptool.base.io.IFileInterpreterProvider;
import viptool.project.FileSystem;
import viptool.util.resources.ResourceConstants;

public class FileUtils
{
    /**
     * Creates the given file and write the content to it. If the file does
     * already exists an IOException is thrown.
     * 
     * @param file
     *            the file to create
     * @param content
     *            the content to write
     * @throws IOException
     *             - if the file exists or cannot be created.
     */
    public static void createFileWithContent(File file, String content)
            throws IOException
    {
        if (!file.createNewFile())
        {
            throw new IOException("The file already exists!");
        }

        FileOutputStream ostream = new FileOutputStream(file);
        OutputStreamWriter writer = new OutputStreamWriter(ostream);
        BufferedWriter out = new BufferedWriter(writer);

        out.write(content, 0, content.length());
        out.close();
        writer.close();
        ostream.close();
    }

    /**
     * Tests whether the given file is of the given type. The type is identified
     * by the first 1024 bytes. These bytes are read, and the given pattern is
     * searched. If the pattern is found, the method returns true, otherwise
     * false is returned.
     * 
     * @param file
     *            the file to test
     * @param pattern
     *            the pattern to search in the first 1024 bytes of the file
     * @return true if the file contains the pattern, false otherwise
     */
    public static boolean isFileOfType(File file, Pattern pattern)
            throws IOException
    {
        InputStreamReader reader;
        BufferedReader bufferedReader;
        CharBuffer buffer;
        char[] charBuffer = new char[1024];

        buffer = CharBuffer.allocate(1024);
        reader = new InputStreamReader(new FileInputStream(file));
        bufferedReader = new BufferedReader(reader);

        bufferedReader.read(charBuffer);

        buffer = CharBuffer.wrap(charBuffer);
        buffer.position(0);

        bufferedReader.close();
        reader.close();

        Matcher matcher = pattern.matcher(buffer);
        return matcher.matches();
    }

    /**
     * Copy the file named by source to the target file. You can specify whether
     * the method should overwrite existing files.
     * 
     * @param source
     *            the source file
     * @param target
     *            the target file
     * @param overwrite
     *            set to true to overwrite eexisting files
     * @throws IOException
     *             - if the file already exists or another IO error occurred
     * @throws FileNotFoundException
     *             - if the source file cannot be found
     */
    public static void copy(File source, File target, boolean overwrite)
            throws IOException, FileNotFoundException
    {
        long filesize;

        if (source == null)
        {
            return;
        }

        if (target.exists())
        {
            String newName = JOptionPane.showInputDialog(null,
                    "Please enter new file name for " + target.getName() + ":",
                    "Target file exists", JOptionPane.QUESTION_MESSAGE);
            
            if (newName == null || newName.length() == 0)
            {
                newName = "Copy_of_" + target.getName();
            }

            String[] parts = target.getName().split("[.]");
            if(newName.split("[.]").length > 1){
            	int answear = JOptionPane.showOptionDialog(null, 
            			"Do you want to change the file extension?", "File extension", 
            			JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, null, null);
            	if(answear != JOptionPane.YES_OPTION){
            		newName += "." + parts[parts.length - 1];
            	}
            }else{
	            if (parts.length > 1)
	            {
	                if (!newName.endsWith(parts[parts.length - 1]))
	                {
	                    if (!newName.endsWith("."))
	                    {
	                        newName += ".";
	                    }
	                    newName += parts[parts.length - 1];
	                }
	            }
            }
            
            target = new File(target.getParentFile(), newName);
        }

        if (!target.exists() || overwrite)
        {
            // open source file
            FileInputStream inputFile = new FileInputStream(source);
            FileChannel input = inputFile.getChannel();

            // open target file
            FileOutputStream outputFile = new FileOutputStream(target);
            FileChannel output = outputFile.getChannel();

            filesize = input.size();

            // copy
            input.transferTo(0, filesize, output);

            input.close();
            output.close();
        }
        else
        {
            throw new IOException("Target file already exists!");
        }
    }

    public static void recursiveCopy(File source, File target,
            IFileInterpreterProvider notthis) throws IOException
    {
    	if (source.isDirectory())
        {
    		
    		if(target.getAbsolutePath().startsWith(source.getAbsolutePath())){
    			System.err.println("FileUtils-Error: Recursive copy loop!");
    			return;
    		}
    		
            File newDir = new File(target, source.getName());
            boolean ex = false;
            if (newDir.exists())
            {
                ex = true;
                int ret = JOptionPane.showConfirmDialog(null,
                        "Integrate files into existing folder?",
                        "Folder already exists", JOptionPane.YES_NO_OPTION,
                        JOptionPane.WARNING_MESSAGE);
                if (ret == JOptionPane.YES_OPTION)
                {
                    // necessary to avoid that closing dialog by x is
                    // interpreted as yes
                }
                else
                {
                    return;
                }
            }
            if (ex || newDir.mkdir())
            {
                File[] files = source.listFiles();
                for (int i = 0; i < files.length; i++)
                {
                    if (files[i].isDirectory())
                    {
                        recursiveCopy(files[i], newDir, notthis);
                    }
                    else
                    {
                        if (notthis != null)
                        {
                            try
                            {
                                if (!notthis.matchFormat(FileSystem.getInstance().createAbstractFile(
                                        files[i])))
                                {
                                    copy(files[i], new File(newDir,
                                            files[i].getName()), false);
                                }
                            }
                            catch (FileInterpreterProviderException e)
                            {
                                e.printStackTrace();
                            }
                        }
                        else
                        {
                            copy(files[i],
                                    new File(newDir, files[i].getName()), false);
                        }
                    }
                }
            }
            else
            {
                throw new IOException("Directory " + newDir
                        + " couldn't be created!");
            }
        }
        else
        {
            copy(source, target, false);
        }
    }

    public static boolean isHiddenFile(File file)
    {
        String name = file.getName();

        if (ResourceConstants.HIDDENFILE_START_WITH.length != ResourceConstants.HIDDENFILE_END_WITH.length)
        {
            throw new IllegalStateException(
                    "Array HIDDENFILE_START_WITH and HIDDENFILE_END_WITH has different size!");
        }

        for (int i = 0; i < ResourceConstants.HIDDENFILE_START_WITH.length; i++)
        {
            if (name.startsWith(ResourceConstants.HIDDENFILE_START_WITH[i])
                    && name.endsWith(ResourceConstants.HIDDENFILE_END_WITH[i]))
            {
                return true;
            }
        }

        return false;
    }
}
