package viptool.util;

import java.awt.Color;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.CharBuffer;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.JOptionPane;

import viptool.base.IWorkbench;
import viptool.base.editor.IEditor;
import viptool.base.error.EditorNotOpenedException;
import viptool.project.IAbstractFile;
import viptool.util.event.CombinationListener;

/**
 * Class contains methods that are useful in different situations. This class
 * grows continuously...
 */
public class GeneralUtils
{
    static public long getUsedBytes()
    {
        Runtime runtime = Runtime.getRuntime();

        return runtime.totalMemory() - runtime.freeMemory();
    }

    static public long getMaxHeapSize()
    {
        return Runtime.getRuntime().maxMemory();
    }

    /**
     * method returns a string that can be used as id for the given object.
     * Every object will get its own unique string. The string follows the
     * pattern <br>
     * classname@object'sHashCode
     * 
     * @param object
     *            the object thats id is questioned
     * @return a string that can be used as id for the given object
     */
    static public String generateUID(Object object)
    {
        return object.getClass().getName() + "@" + object.hashCode();
    }

    /**
     * mehtod returns a string that represents the given color. The String
     * follows the pattern <br>
     * rgb(red, green, blue) <br>
     * which should be on possible representation for a color in Cascade Style
     * Sheet 2 (CSS2). This format is necessary for pnml files.
     * 
     * @param color
     *            the color that CSS2 string is questioned
     * @return a string representing the given color in a possible CSS2 form
     */
    static public String getRGBString(Color color)
    {
        String red;
        String green;
        String blue;

        red = Integer.toString(color.getRed());
        green = Integer.toString(color.getGreen());
        blue = Integer.toString(color.getBlue());

        return "rgb(" + red + ", " + green + ", " + blue + ")";
    }

    /**
     * With this method the user can test wether the given object is castable in
     * to the given class. If so, the object will be returned, if not, a
     * ClassCastException will be thrown.
     * 
     * @param object
     *            the object that should be casted
     * @param toCastIn
     *            the class the object should be casted to
     * @return the given object, if it can be casted. Otherwise a
     *         ClassCastException will be thrown.
     */
    @Deprecated
    static public Object tryCast(Object object, Class toCastIn)
    {
        if (toCastIn.isAssignableFrom(object.getClass()))
        {
            return object;
        }
        else
        {
            throw new ClassCastException(object + " is not of the type "
                    + toCastIn.getName());
        }
    }

    /**
     * This method creates a special MessageDialog (JOptionPane). This on will
     * be an error message whose parent component will be the MainFrame.
     * 
     * @param message
     *            the message that should be shown
     * @param title
     *            the title that should heading the dialog pane
     */
    static public void showErrorDialog(Object message, String title)
    {
        // TODO: think about
        // JOptionPane.showMessageDialog(MainFrame.getInstance(), message,
        // title,
        // JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Centers a component (dialog, window) relative to another component.
     * 
     * @param componentToCenter
     *            the component that should be centered
     * @param centerRelativeTo
     *            the component that should be the 'parent' while centering
     */
    static public void centerComponent(Component componentToCenter,
            Component centerRelativeTo)
    {
        Rectangle parentBounds;
        Rectangle componentBounds;

        if (centerRelativeTo == null)
        {
            parentBounds = new Rectangle();
            parentBounds.x = 0;
            parentBounds.y = 0;
            parentBounds.width = Toolkit.getDefaultToolkit().getScreenSize().width;
            parentBounds.height = Toolkit.getDefaultToolkit().getScreenSize().height;
        }
        else
        {
            parentBounds = centerRelativeTo.getBounds();
        }

        componentBounds = componentToCenter.getBounds();

        componentBounds.x = ((int) parentBounds.getCenterX())
                - (componentBounds.width / 2);
        componentBounds.y = ((int) parentBounds.getCenterY())
                - (componentBounds.height / 2);

        componentToCenter.setLocation(componentBounds.x, componentBounds.y);
    }

    /**
     * Replace a string in a text file. The string to replace is searched using
     * the given regular expression. If the file does not exists or cannot be
     * opened an exception is thrown.
     * 
     * @param file
     *            the file in which the string should be replaced
     * @param regex
     *            the regular expression to specify the search string
     * @param content
     *            the string that should be replace the found string
     * @throws IOException
     * @throws FileNotFoundException
     */
    static public void replaceInFile(File file, String regex, String content)
            throws IOException, FileNotFoundException
    {
        int read;
        InputStreamReader reader;
        BufferedReader bufferedReader;
        CharBuffer buffer;
        FileOutputStream ostream;
        OutputStreamWriter writer;
        BufferedWriter out;
        String text;
        File tmpFile = File.createTempFile("prefix", "suffix",
                file.getParentFile());
        char[] charBuffer = new char[1000];

        buffer = CharBuffer.allocate(1000);
        reader = new InputStreamReader(new FileInputStream(file));
        bufferedReader = new BufferedReader(reader);

        ostream = new FileOutputStream(tmpFile);
        writer = new OutputStreamWriter(ostream);
        out = new BufferedWriter(writer);

        while ((read = bufferedReader.read(charBuffer)) != -1)
        {
            buffer = CharBuffer.wrap(charBuffer, 0, read);
            buffer.position(0);

            text = buffer.toString();
            text = text.replaceAll(regex, content);

            out.write(text, 0, text.length());
        }

        out.close();
        bufferedReader.close();

        bufferedReader.close();
        reader.close();

        file.delete();
        tmpFile.renameTo(file);
        tmpFile.setLastModified(new Date().getTime());
    }

    /**
     * Test wether a propety contains a given flag. This is done by a simpe
     * 'and' operation. This means, the following code is executed:
     * 
     * <pre>
     * ((property &amp; flag) == flag)
     * </pre>
     * 
     * @param property
     *            the property to test
     * @param flag
     *            the flag that should be in the property
     * @return true if the property contains the flag, false otherwise
     */
    static public boolean containsFlag(int property, int flag)
    {
        return ((property & flag) == flag);
    }

    /**
     * Returns the home directory of the user. The directory ends with the file
     * separator.
     * 
     * @return home directory of the current user
     */
    static public String getUsersHomeDirectory()
    {
        String path = System.getProperty("user.home");

        if (!path.endsWith(File.separator))
        {
            path += File.separator;
        }

        return path;
    }

    /**
     * Returns the temporary directory of the system. The directory ends with
     * the file separator.
     * 
     * @return temporary directory of the system
     */
    static public String getTemporaryDirectory()
    {
        String path = System.getProperty("java.io.tmpdir");

        if (!path.endsWith(File.separator))
        {
            path += File.separator;
        }

        return path;
    }

    static public String convertIdToPath(String id)
    {
        int index;
        StringBuffer path;

        path = new StringBuffer(id.replace('.', '/'));

        index = path.lastIndexOf("/");
        path.setCharAt(index, '.');

        return path.toString();
    }

    static public String toString(Point2D point)
    {
        return "(" + ((int) point.getX()) + ", " + ((int) point.getY()) + ")";
    }

    static public boolean testForFileLocker(IWorkbench workbench,
            IAbstractFile[] files, String message)
    {
        for (int i = 0; i < files.length; i++)
        {
            if (!testForFileLocker(workbench, files[i], message))
            {
                return false;
            }
        }

        return true;
    }

    static public boolean testForFileLocker(IWorkbench workbench,
            IAbstractFile file, String message)
    {
        int key;
        IEditor editor;

        if ((editor = workbench.findLockingEditor(file)) != null)
        {
            key = JOptionPane.showConfirmDialog(workbench.getMainFrame(),
                    message.replace("{file}", file.getName()), "File '"
                            + file.getName() + "' locked",
                    JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (key == JOptionPane.OK_OPTION)
            {
                try
                {
                    editor.save();
                }
                catch (EditorNotOpenedException exception)
                {
                    workbench.showExceptionInDialog(exception);
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        return true;
    }

    public static <TYPE> void calculateCombinations(List<TYPE> elements,
            int min, int max, CombinationListener<TYPE> listener)
    {
        int index;
        ArrayList<TYPE> combination;

        combination = new ArrayList<TYPE>();
        for (int count = min; count <= max && count <= elements.size(); count++)
        {
            index = 0;
            while (index < elements.size())
            {
                fillList(elements, combination, index++, count, listener);
            }
        }
    }

    private static <TYPE> int fillList(List<TYPE> elements,
            List<TYPE> combination, int index, int count,
            CombinationListener<TYPE> listener)
    {
        if (count == 1)
        {
            combination.add(elements.get(index));
            listener.combination(combination);
            combination.remove(combination.size() - 1);
            return 0;
        }

        combination.add(elements.get(index++));
        while (index < elements.size())
        {
            fillList(elements, combination, index++, count - 1, listener);
        }
        combination.remove(combination.size() - 1);

        return 0;
    }

    public static int factorial(int number)
    {
        int result = 1;

        for (int i = number; i >= 2; i--)
        {
            result *= i;
        }

        return result;
    }

    public static int cumulativeSum(int number)
    {
        return ((number + 1) * number) / 2;
    }
}
