package viptool.util;

import java.awt.Point;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;

/**
 * This class provide some standard geometry calculation methods such as line
 * intersecting.
 */
public class GeometryUtils
{
    /**
     * Calculates the standard line equation for two points. This method returns
     * an array with three entries: a, b, and c where the line equation is: ax +
     * by + c = 0
     * 
     * @param x1
     *            x-coordinate of the first point
     * @param y1
     *            y-coordinate of the first point
     * @param x2
     *            x-coordinate of the second point
     * @param y2
     *            y-coordinate of the second point
     * @return an array with the three parameters a, b and c
     */
    public static double[] getLinearEquation(double x1, double y1, double x2,
            double y2)
    {
        double a;
        double b;
        double c;
        // The common line equation (ax + by + c = 0) is calculated with the
        // given points P1(x1, y1) and P2(x2, y2) as follows:
        // a = y2 - y1;
        // b = x1 - x2;
        // c = x2y1 - x1y2;

        // The points must not be the same
        if(x1 == x2 && y1 == y2)
            return null;

        a = y2 - y1;
        b = x1 - x2;
        c = (x2 * y1) - (x1 * y2);
        return new double[] {a, b, c};
    }

    /**
     * Calculates the intersection point between two lines that are given as
     * standard line equation. If the lines do not intersect, null is returned.
     * 
     * @param a1
     *            parameter of the first line: a1*x + b1*y + c1 = 0
     * @param b1
     *            parameter of the first line: a1*x + b1*y + c1 = 0
     * @param c1
     *            parameter of the first line: a1*x + b1*y + c1 = 0
     * @param a2
     *            parameter of the second line: a2*x + b2*y + c2 = 0
     * @param b2
     *            parameter of the second line: a2*x + b2*y + c2 = 0
     * @param c2
     *            parameter of the second line: a2*x + b2*y + c2 = 0
     * @return the intersection point or null, if the lines do not intersect
     */
    public static Point intersectLines(double a1, double b1, double c1,
            double a2, double b2, double c2)
    {
        // The intersection S(Sx, Sy) of two lines with their linear
        // equations a1x + b1y + c1 = 0 and a2x + b2y + c2 = 0 is calculated as
        // follows
        // Sx = |b1 c1, b2 c2| / |a1 b1, a2 b2|;
        // Sy = |c1 a1, c2 a2| / |a1 b1, a2 b2|;
        Point intersection = new Point();
        double determinant; // Determinante

        // |a1 b1, a2 b2|
        determinant = ((a1 * b2) - (a2 * b1));
        if(determinant == 0)
            return null;

        // x-coordinate
        intersection.x = (int)(((b1 * c2) - (b2 * c1)) / determinant);

        // |a1 b1, a2 b2|
        determinant = ((a1 * b2) - (a2 * b1));
        if(determinant == 0)
            return null;

        // y-coordinate
        intersection.y = (int)(((c1 * a2) - (c2 * a1)) / determinant);
        return intersection;
    }

    /**
     * Calculates the intersection point between two lines that are given
     * through two points each. If the lines do not intersect, null is returned.
     * 
     * @param gx1
     *            x-coordinate of the first point on the first line
     * @param gy1
     *            y-coordinate of the first point on the first line
     * @param gx2
     *            x-coordinate of the second point on the first line
     * @param gy2
     *            y-coordinate of the second point on the first line
     * @param hx1
     *            x-coordinate of the first point on the second line
     * @param hy1
     *            y-coordinate of the first point on the second line
     * @param hx2
     *            x-coordinate of the second point on the second line
     * @param hy2
     *            y-coordinate of the second point on the second line
     * @return the intersection point or null, if the lines do not intersect
     */
    public static Point intersectLines(double gx1, double gy1, double gx2,
            double gy2, double hx1, double hy1, double hx2, double hy2)
    {
        // parameter for the linear equations
        double line1[] = new double[3];
        double line2[] = new double[3];

        // get the linear equation of the first line
        if((line1 = getLinearEquation(gx1, gy1, gx2, gy2)) == null)
            return null;
        // get the linear equation of the second line
        if((line2 = getLinearEquation(hx1, hy1, hx2, hy2)) == null)
            return null;

        // calculate intersection
        return intersectLines(line1[0], line1[1], line1[2], line2[0], line2[1],
                line2[2]);
    }

    /**
     * Calculates the intersection point between a line and a circle. If line
     * and circle do not intersect, null is returned.
     * 
     * @param circle
     *            the circle to intersect
     * @param line
     *            the line to intersect
     * @return the intersection point or null if the line does not intersect the
     *         circle
     */
    public static Point intersectLineWithCircle(Ellipse2D circle, Line2D line)
    {
        double radius = circle.getWidth() / 2;

        double vectorX = ((circle.getCenterX() - line.getX1()) / Math.sqrt(Math
                .pow(line.getX1() - circle.getCenterX(), 2)
                + Math.pow(line.getY1() - circle.getCenterY(), 2)));

        double vectorY = ((circle.getCenterY() - line.getY1()) / Math.sqrt(Math
                .pow(line.getX1() - circle.getCenterX(), 2)
                + Math.pow(line.getY1() - circle.getCenterY(), 2)));
        
      
        return new Point((int)(circle.getCenterX() - radius * vectorX),
                (int)(circle.getCenterY() - radius * vectorY));
    }
    

    /**
     * Transform the given shape respecting the translate and scale values and
     * returns the transformed shape. This method creates an AffineTransform
     * object and calls createTransformedShape.
     * 
     * @param shape
     *            the shape to transform
     * @param translateX
     *            the distance by which coordinates are translated in the X axis
     *            direction
     * @param translateY
     *            the distance by which coordinates are translated in the Y axis
     *            direction
     * @param scaleX
     *            the factor by which coordinates are scaled along the X axis
     *            direction
     * @param scaleY
     *            the factor by which coordinates are scaled along the Y axis
     *            direction
     * @return the transformed shape
     */
    public static Shape transformShape(Shape shape, double translateX,
            double translateY, double scaleX, double scaleY)
    {
        AffineTransform affineTransform = new AffineTransform();

        affineTransform.translate(translateX, translateY);
        affineTransform.scale(scaleX, scaleY);

        return affineTransform.createTransformedShape(shape);
    }
    
    /**
     * Transform the given shape respecting the translate and scale values and
     * returns the transformed shape. This method creates an AffineTransform
     * object and calls createTransformedShape.
     * 
     * @param shape
     *            the shape to transform
     * @param translateX
     *            the distance by which coordinates are translated in the X axis
     *            direction
     * @param translateY
     *            the distance by which coordinates are translated in the Y axis
     *            direction
     * @param scale
     *            the factor by which coordinates are scaled along the X/Y axis
     *            direction
     * @return the transformed shape
     */
    public static Shape transformShape(Shape shape, double translateX,
            double translateY, double scale)
    {
        return transformShape(shape, translateX, translateY, scale, scale);
    }
    
    /**
     * Calculates the intersection points of a Bezier curve and a line.
     * The Bezier curve is given as a {@link CubicCurve2D#Double}.
     */
    public static Point[] intersectBezierAndLine(CubicCurve2D curve, Line2D line){
    	
        AffineTransform af = new AffineTransform();
    	
        Point2D.Double p1 = new Point2D.Double();
    	Point2D.Double p2 = new Point2D.Double();
    	
    	Point2D[] curvePointsTranslated = new Point2D.Double[4];
    	
    	Point2D[] curvePoints = new Point2D.Double[4];
    	curvePoints[0] = curve.getP1();
    	curvePoints[1] = curve.getCtrlP1();
    	curvePoints[2] = curve.getCtrlP2();
    	curvePoints[3] = curve.getP2();
    
    	double alpha;
    	
    	if(line.getX1() != line.getX2()){
    		alpha = Math.atan2(line.getY2()-line.getY1(), line.getX2()-line.getX1());
    	}else{
    		alpha = Math.toRadians(90);
    	}
    	
    	
    	af.translate(-line.getX1(), -line.getY1());
    	
    	af.transform(p1,p1);
    	af.transform(p2,p2);
    	af.transform(curvePoints, 0, curvePointsTranslated, 0, 4);
    	
    	af.setToIdentity();
    	
    	af.rotate(-alpha);
    	af.transform(line.getP1(),p1);
    	af.transform(line.getP2(),p2);
    	af.transform(curvePointsTranslated, 0, curvePointsTranslated, 0, 4);
    	
    	for(int i = 0; i<4; i++){
    		System.out.println("curvePointsTranslated "+ i + ": " +curvePointsTranslated[i].getX() + " " + curvePointsTranslated[i].getY());
    	}
    	
    	double d = - curvePointsTranslated[0].getY() 
    			+ 3.0d * curvePointsTranslated[1].getY()
    			- 3.0d * curvePointsTranslated[2].getY()
    			+ curvePointsTranslated[3].getY();
    	
    	double a = 3.0d * curvePointsTranslated[0].getY()
    			- 6.0d * curvePointsTranslated[1].getY()
    			+ 3.0d * curvePointsTranslated[2].getY();
    	
    	double b = - 3.0d * curvePointsTranslated[0].getY()
    			+ 3.0d * curvePointsTranslated[1].getY();
    	
    	double c = curvePointsTranslated[0].getY();
    	
    	double[] eqn = new double[4];
    	eqn[0] = c;
    	eqn[1] = b;
    	eqn[2] = a;
    	eqn[3] = d;
    
    	double[] solution = new double[3];
    	
    	int numRoots = CubicCurve2D.solveCubic(eqn,solution);
    	
    	d = - curvePointsTranslated[0].getX() 
    			+ 3.0d * curvePointsTranslated[1].getX()
    			- 3.0d * curvePointsTranslated[2].getX()
    			+ curvePointsTranslated[3].getX();
    	
    	a = 3.0d * curvePointsTranslated[0].getX()
    			- 6.0d * curvePointsTranslated[1].getX()
    			+ 3.0d * curvePointsTranslated[2].getX();
    	
    	b = - 3.0d * curvePointsTranslated[0].getX()
    			+ 3.0d * curvePointsTranslated[1].getX();
    	
    	c = curvePointsTranslated[0].getX();
    	
    	Point [] intersectionPoints = new Point[3];
    	
    	for(int i = 0; i<numRoots; i++){
    		
    		if(solution[i] >= (0 - 5E-10) && solution[i] <= (1 + 5E-10)){
    			double x_t = d * Math.pow(solution[i] , 3) + a * Math.pow(solution[i] , 2) + b * Math.pow(solution[i] , 1) + c; 
    	    	Point intersectionPoint = new Point();
    	    	intersectionPoint.setLocation(x_t,0);
    	    	intersectionPoints[i] = intersectionPoint;
    		}
    	}
    	
    	for(int i = 0; i<3;i++){
    		if(intersectionPoints[i]!=null){
    			
    			af.setToIdentity();
    			af.rotate(alpha);
    	    	af.transform(intersectionPoints[i], intersectionPoints[i]);
    			
    			
    			af.setToIdentity();
    			af.translate(line.getX1(), line.getY1());
    	    	af.transform(intersectionPoints[i], intersectionPoints[i]);
    			
    		}
    	}
    	
    	return intersectionPoints;

	}
    
    
}
