package viptool.util;

import java.awt.Component;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;
import javax.swing.text.BadLocationException;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoableEdit;
import javax.swing.undo.UndoableEditSupport;

import viptool.base.IWorkbench;

/**
 * This class provides a rename process of every object that implements the
 * renameable interface. The created object of this class will be set as a
 * temporary glass pane on which the textfield for the process will be
 * displayed. While the process has not finished, the end-user will not be able
 * to perform any other action in the window, only the window head will be
 * active. Every rename operation will be able to be re-/undone. Therefor the
 * UndoableEdit interface is implemented.
 */
public class Renamer extends JComponent implements FocusListener, KeyListener,
        MouseListener, UndoableEdit
{
    // the current workbench
    private IWorkbench workbench_;
    
    // the text component which is needed to get an input of the requested new
    // name
    private ExtendedJTextPane   inputField_;

    // the object which should be renamed
    private Renameable          object_;

    // the displayed name of the object before it has been renamed
    private String              oldName_;

    // the new name of the object
    private String              newName_;

    // the glass pane of the window will be replaced for the duration of the
    // process. The old one will be restored afterwards and therefore must be
    // stored before replacing
    private Component           glassPane_;

    // necessary for posting undo/redo objects
    private UndoableEditSupport undoableEditSupport_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * a new renamer object is created. This includes that another glass pane is
     * set for the duration of this process on which the edit textfield will be
     * displayed. To avoid complication the resizable property of the main frame
     * will be forbidden (function not active at the moment) and no action can
     * be performed as long as the new name equals an empty string.
     * 
     * @param workbench
     *            the current workbench
     * @param rectangle
     *            this object has to contain the height, width and position
     *            (relative to the component relativeTo) of the object that
     *            shows the name at the moment
     * @param object
     *            the object that should be renamed. It is necessary that the
     *            class of this object implements the interface renameable
     * @param relativeTo
     *            the component the rectangles position is relative to, e.g. the
     *            rectangle displaying the name of a tab is relative to the
     *            father tabbedPane
     * @param inputField
     *            a JTextComponent which should be used as editing field
     * @param undoableEditSupport
     *            object for posting undo/redo objects
     */
    public Renamer(IWorkbench workbench, Rectangle rectangle,
            Renameable object, Component relativeTo,
            ExtendedJTextPane inputField,
            UndoableEditSupport undoableEditSupport)
    {
        Font objectFont;

        // store the object that should be renamed
        object_ = object;
        oldName_ = object_.getName();
        undoableEditSupport_ = undoableEditSupport;
        workbench_ = workbench;

        // set us as new glass pane and store the old one
        glassPane_ = workbench_.getMainFrame().getRootPane().getGlassPane();
        workbench_.getMainFrame().getRootPane().setGlassPane(this);

        // convert the rectangle coordinates to the glass pane, so it can be
        // displayed at the right position
        rectangle = SwingUtilities
                .convertRectangle(relativeTo, rectangle, this);

        // Set our inputField to the given input field, set its starting value
        // and the caret position. Also select the whole text in the text field
        // when it is first displayed.
        inputField_ = inputField;
        objectFont = object.getFont();
        inputField_.setFont(objectFont);
        inputField_.setText(oldName_);
        inputField_.setCaretPosition(0);
        inputField_.moveCaretPosition(oldName_.length());
        add(inputField_);

        // the size of the editfield should fit the size of the given rectangle
        inputField_.setSize(rectangle.width - 2 + 15, rectangle.height - 2 + 20);
        inputField_.setLocation(rectangle.x + 1, rectangle.y + 1);

        // show our new glass pane with the textfield on it
        setVisible(true);

        // get focus for the textfield so that the edit can begin immediately
        inputField_.requestFocus();
        // add key- and focus listener so that an end of the rename process can
        // be notified
        inputField_.addFocusListener(this);
        inputField_.addKeyListener(this);

        // add these listeners also to the renamer because the click has not to
        // be performed on the textfield
        addFocusListener(this);
        addKeyListener(this);
        // add also a mouse listener so that we are able to forbid all action in
        // the window as long as the new name has the length 0
        addMouseListener(this);

        // forbid the resizable property of the frame, otherwise there might
        // occur problems with the position of the textfield inside the frame
        // MainFrame.getInstance().setResizable(false);

        // updateLength();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // End of rename process
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * this method manages all actions that have to be performed if the rename
     * process is ended. It sets the new name to the renamed object, restores
     * the old glass pane and enables the resizable property of the main frame
     * (last function not activated at the moment).
     */
    private void endRename()
    {
        // end only if the new name is valid and assure that it is not empty
        if((object_.isValidName(inputField_.getText()) && (inputField_
                .getText().length() > 0)))
        {
            newName_ = inputField_.getText();
            // set the new name to the object
            object_.setName(newName_);
            // hide our glass pane and restore the old one
            setVisible(false);
            workbench_.getMainFrame().getRootPane().setGlassPane(glassPane_);
            // enable the resizable property of the mainframe because rename
            // process is over now
            // MainFrame.getInstance().setResizable(true);

            // notify all listeners of UndoableEditSupport, that a new edit
            // occured
            if(undoableEditSupport_ != null)
            {
                undoableEditSupport_.postEdit(this);
            }
        }
        else
        {
            // if the new name is not accepted the focus should stay at the text
            // field
            inputField_.requestFocus();
        }
    }

    /*
     * private void updateLength() { TextLayout textLayout; Rectangle
     * fieldBounds; Rectangle2D textBounds; Graphics2D graphics2D =
     * (Graphics2D)inputField_.getGraphics();
     * 
     * if((graphics2D != null) && (inputField_.getText().length() > 0)) {
     * textLayout = new TextLayout(inputField_.getText(), inputField_
     * .getFont(), graphics2D.getFontRenderContext());
     * 
     * fieldBounds = inputField_.getBounds(); textBounds =
     * textLayout.getBounds();
     * 
     * fieldBounds.width = (int)(textBounds.getWidth() + 15); fieldBounds.height =
     * (int)textBounds.getHeight() + 12;
     * 
     * inputField_.setBounds(fieldBounds); } }
     */

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the FocusListener interface
    // <><><><><><><><><><><><><><><><><><><><><>
    public void focusGained(FocusEvent e)
    {
    }

    public void focusLost(FocusEvent e)
    {
        // if the text field or the glass pane lost the focus the end of the
        // rename process is avoked
        endRename();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the KeyListener interface
    // <><><><><><><><><><><><><><><><><><><><><>
    public void keyPressed(KeyEvent e)
    {
        // if the text field or the glass pane recognizes a press of the enter
        // button the end of the rename process is avoked
        if(e.getKeyCode() == KeyEvent.VK_ENTER)
        {
            if(e.isShiftDown())
            {
                try
                {
                    inputField_.getStyledDocument().insertString(
                            inputField_.getCaretPosition(), "\n", null);
                }
                catch (BadLocationException ex)
                {
                    ex.printStackTrace();
                }
            }
            else
            {
                // don't add a new line to the end of the name before ending
                // renaming
                e.consume();
                
                // end renaming
                endRename();
            }
        }

        // updateLength();
    }

    public void keyReleased(KeyEvent e)
    {
    }

    public void keyTyped(KeyEvent e)
    {
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the MouseListener interface
    // <><><><><><><><><><><><><><><><><><><><><>
    public void mouseClicked(MouseEvent mouseEvent)
    {
        // if the glass pane recognizes a mouseclick outside the textfield the
        // end of the rename process is avoked
        endRename();
    }

    public void mouseEntered(MouseEvent mouseEvent)
    {
    }

    public void mouseExited(MouseEvent mouseEvent)
    {
    }

    public void mousePressed(MouseEvent mouseEvent)
    {
    }

    public void mouseReleased(MouseEvent mouseEvent)
    {
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the UndoableEdit interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see javax.swing.undo */
    public boolean addEdit(UndoableEdit arg0)
    {
        return false;
    }

    /* @see javax.swing.undo */
    public boolean canRedo()
    {
        return true;
    }

    /* @see javax.swing.undo */
    public boolean canUndo()
    {
        return true;
    }

    /* @see javax.swing.undo */
    public void die()
    {

    }

    /* @see javax.swing.undo */
    public String getPresentationName()
    {
        return "rename " + object_;
    }

    /* @see javax.swing.undo */
    public String getRedoPresentationName()
    {
        return getPresentationName();
    }

    /* @see javax.swing.undo */
    public String getUndoPresentationName()
    {
        return getPresentationName();
    }

    /* @see javax.swing.undo */
    public boolean isSignificant()
    {
        return true;
    }

    /* @see javax.swing.undo */
    public void redo() throws CannotRedoException
    {
        if(canRedo())
        {
            object_.setName(newName_);
        }
    }

    /* @see javax.swing.undo */
    public boolean replaceEdit(UndoableEdit arg0)
    {
        return false;
    }

    /* @see javax.swing.undo */
    public void undo() throws CannotUndoException
    {
        if(canUndo())
        {
            object_.setName(oldName_);
        }
    }
}
