package viptool.util.resources;

import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.net.MalformedURLException;
import java.net.URI;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import viptool.base.error.ResourceLocateException;

public abstract class AbstractResourceLoader implements IResourceLoader
{
    /**
     * Creates and returns a Cursor Object, which is named cursorName. It may
     * contain a different sized Pic on different systems.
     * 
     * @param cursorName
     *            has to be the Filename of the requested Pic without
     *            size+".gif". The created Object can be called by cursorName
     * 
     * @return a Cursor Object
     */
    public Cursor getCursor(String name)
    {
        // necessary to react on different System abilties of creating a Cursor
        int height = Toolkit.getDefaultToolkit().getBestCursorSize(16, 16).height;
        int width = Toolkit.getDefaultToolkit().getBestCursorSize(16, 16).width;

        // cursor should have his hotspot in its midth
        return getCursor(name, new Point(width / 2, height / 2));
    }

    /**
     * Creates and returns a Cursor Object, which is named cursorName. It may
     * contain a different sized Pic on different systems. Also the hotspot may
     * be choosen.
     * 
     * @param cursorName
     *            has to be the Filename of the requested Pic without
     *            size+".gif". The created Object can be called by cursorName
     * 
     * @param hotSpot
     *            the preferred hotspot in relation to the cursor object
     * 
     * @return a Cursor Object
     */
    public Cursor getCursor(String name, Point hotspot)
    {
        Image cursorPic;

        // necessary to react on different System abilties of creating a Cursor
        int height = Toolkit.getDefaultToolkit().getBestCursorSize(16, 16).height;
        int width = Toolkit.getDefaultToolkit().getBestCursorSize(16, 16).width;

        // if the System is able to create a cursor by the size 16x16 it shall
        // do it
        // Otherwise another Pic is loaded which has an invisible border around
        // the prefered Size (e.g. in windows)
        if (height <= 16 && width <= 16)
        {
            cursorPic = getImage(name + "16");
        }
        else
        {
            cursorPic = getImage(name + "32");
        }

        // creates the cursor Object
        return getCursor(cursorPic, hotspot);
    }

    public Cursor getCursor(Image image)
    {
        // necessary to react on different System abilties of creating a Cursor
        int height = Toolkit.getDefaultToolkit().getBestCursorSize(16, 16).height;
        int width = Toolkit.getDefaultToolkit().getBestCursorSize(16, 16).width;

        // cursor should have his hotspot in its midth
        return getCursor(image, new Point(width / 2, height / 2));
    }

    public Cursor getCursor(Image image, Point hotspot)
    {
        // creates the cursor Object
        return Toolkit.getDefaultToolkit().createCustomCursor(image, hotspot,
                "customCursor");
    }

    /**
     * Returns an Image, which is created by the specified Filename. Notice that
     * the format .gif is added automatically
     * 
     * @param name
     *            has to be the Filename without .gif
     * 
     * @return the image
     */
    public Image getImage(String name)
    {
        Image image;
        URI resource;

        try
        {
            name = name + ".gif";

            resource = getResource(name);
            if (resource == null)
            {
                throw new ResourceLocateException("Image not found: " + name);
            }

            image = Toolkit.getDefaultToolkit().getImage(resource.toURL());
        }
        catch (MalformedURLException exception)
        {
            throw new ResourceLocateException(exception);
        }

        return image;
    }

    /**
     * Returns an Icon, which is created by the specified Filename. Notice that
     * the format .gif is added automatically
     * 
     * @param name
     *            has to be the Filename without .gif
     * 
     * @return the Icon
     */
    public Icon getIcon(String name)
    {
        URI resource;

        try
        {
            name = name + ".gif";
            resource = getResource(name);
            if (resource == null)
            {
                throw new ResourceLocateException("Icon not found: " + name);
            }

            return new ImageIcon(Toolkit.getDefaultToolkit().getImage(
                    resource.toURL()));

        }
        catch (MalformedURLException exception)
        {
            throw new ResourceLocateException(exception);
        }
    }
}
